@@ -90,6 +90,9 @@ struct linsys  *S;
 REAL *w; /* in */ 
 REAL *v; /* out */
 { int row,col,i;
+
+  PROF_START(hessian_mul);
+
   memset((char*)v,0,S->N*sizeof(REAL));
   for ( row = 0 ; row < S->N ; row++ )
   { for ( i = S->IA[row]-A_OFF ; i < S->IA[row+1]-A_OFF ; i++ )
@@ -106,6 +109,8 @@ REAL *v; /* out */
      for ( i = 0 ; i < S->N ; i++ ) v[i] -= tempv[i];
      temp_free((char*)tempv);
   }
+
+  PROF_FINISH(hessian_mul);
 } /* end bk_mul() */
 
 
@@ -197,11 +202,11 @@ int *P;       /* preallocated for return of permutation */
   for ( i = 0, Bptr = xlist ; i < n ; i++,Bptr++ ) Bptr->inx = i;
   FOR_ALL_VERTICES(v_id)
   { struct hess_verlist *v;      /* current  vertex */
-    v = verlist + ordinal(v_id);
+    v = verlist + loc_ordinal(v_id);
     Bptr = xlist + v->rownum;
     for ( j = 0 ; j < v->freedom ; j++, Bptr++ )
     {
-      Bptr->ord = ordinal(v->v_id);
+      Bptr->ord = loc_ordinal(v->v_id);
       Bptr->deg = j;
       Bptr->x = get_coord(v->v_id)[0];
     }
@@ -234,10 +239,11 @@ struct linsys *S;  /* pointer to empty structure */
   int *ptr,*cptr,*dptr;
   int isize,dsize;
 
+  PROF_START(hessian_AIJ_setup);
 
   S->flags &= ~ S_ODRV_REORDERED; 
   S->flags |= S_JA_INCLUDED;  /* S->JA not separately allocated */.
-  if ( augmented_hessian_flag )
+  if ( augmented_hessian_mode )
     N = total_rows;
   S->N = N;
 
@@ -329,6 +335,7 @@ struct linsys *S;  /* pointer to empty structure */
     S->A[i] = 1e30; 
   }
 
+  PROF_FINISH(hessian_AIJ_setup);
 #endif
 } /* end bk_AIJ_setup() */
 
@@ -346,10 +353,17 @@ int concount;  /* number of constraints */
 struct linsys *S; 
 { int i,j;
  
+  PROF_START(hessian_constraint_setup);
+
   if ( S->CN == 0 ) 
   { S->N = S->A_rows; goto debug_print; }
 
-  if ( augmented_hessian_flag )
+  if ( S->A_rows < S->CN )
+  { sprintf(errmsg,"Degrees of freedom, %d, fewer than constraints, %d.\n",
+       S->A_rows,S->CN);
+    kb_error(3382,errmsg,RECOVERABLE);
+  }
+  if ( augmented_hessian_mode )
   { /* Keeping constraints in sparse augmented hessian. */
     /* So not much to do, except eliminate unused constraints,  */
     /* building constraint conversion index in the process,   */
@@ -424,8 +438,8 @@ struct linsys *S;
                   optparamcount + i;
         if ( valid_id(vh->v_id) )
           sprintf(errmsg,
-          "Internal error: hessian setup skipped fixed body %d.\n",
-                ordinal(vh->v_id)+1);
+          "Internal error: hessian setup skipped fixed body %s.\n",
+                ELNAME(vh->v_id));
         else 
           sprintf(errmsg,
           "Internal error: hessian setup skipped fixed quantity %s.\n",
@@ -437,13 +451,13 @@ struct linsys *S;
                   optparamcount + i;
         if ( valid_id(vh->v_id) )
           sprintf(errmsg,
-            "Fixed body %d is not connected to any mobile vertices.\n",
-                ordinal(vh->v_id)+1);
+            "Fixed body %s is not connected to any mobile vertices.\n",
+                ELNAME(vh->v_id));
         else 
           sprintf(errmsg,
             "Fixed quantity %s is not connected to any mobile vertices.\n",
                 GEN_QUANT(vh->v_id)->name);
-        kb_error(2462,errmsg,RECOVERABLE);
+        kb_error(3362,errmsg,RECOVERABLE);
       }
     }
     /* Strip out empty constraints */
@@ -477,7 +491,8 @@ struct linsys *S;
 debug_print:
   /* some debug printing */
   if ( hess_debug )
-  { 
+  { vertex_id v_id;
+  
     printf("N = %d\n",S->N);
     printf("IA: ");
     for ( i = 0 ; i <= S->N ; i++ ) printf(" %d",S->IA[i]);
@@ -485,15 +500,16 @@ debug_print:
     for ( i = 0 ; i < S->IA[S->N]-A_OFF ; i++ ) printf(" %d",S->JA[i]);
     printf("\n");
     /* column labels */
-    for ( i = 0 ; i <= web.skel[VERTEX].max_ord ; i++ )
-    { for ( j = 0 ; j < vhead[i].freedom ; j++ )
-      { sprintf(msg,"v%d.%d",i+1,j+1); 
+    MFOR_ALL_VERTICES(v_id)
+    { struct hess_verlist *vh = get_vertex_vhead(v_id);
+      for ( j = 0 ; j < vh->freedom ; j++ )
+      { sprintf(msg,"v%s.%d",ELNAME(v_id),j+1); 
         printf("%10s",msg);
       }
     }
     for ( i = 0 ; i < optparamcount ; i++ )
       printf("%10s",globals(optparam[i].pnum)->name);
-    if ( augmented_hessian_flag )
+    if ( augmented_hessian_mode )
     { if ( !everything_quantities_flag )
       { for ( i = 0 ; i <= web.skel[BODY].max_ord ; i++ )
           if ( S->coninx[i] >= 0 )
@@ -520,7 +536,7 @@ debug_print:
         printf("\n");
      }
 
-    if ( !augmented_hessian_flag )
+    if ( !augmented_hessian_mode )
     {
     printf("Constraints:\n");
     if ( sparse_constraints_flag )
@@ -542,6 +558,8 @@ debug_print:
     }
     }
    } /* end hess_debug */
+
+  PROF_FINISH(hessian_constraint_setup);
 } /* end bk_constraint_setup */
 
 /*********************************************************************
@@ -556,10 +574,12 @@ debug_print:
 void BK_hess_project_setup(S)
 struct linsys *S;
 { int i;
-  int con_index;
+  int con_index=0;
 
-  if ( S->CN == 0 ) return;
-  if ( augmented_hessian_flag ) goto adjust_index;
+  PROF_START(hessian_project_setup);
+
+  if ( S->CN == 0 ) goto set_counts;
+  if ( augmented_hessian_mode ) goto adjust_index;
 
   if ( !sparse_constraints_flag )
   {
@@ -638,7 +658,7 @@ struct linsys *S;
 
 adjust_index:
   /* adjust index for constraints */
-  if ( augmented_hessian_flag )
+  if ( augmented_hessian_mode )
   { S->neg -= (S->CN - S->degencon);
     S->pos -= (S->CN - S->degencon);
     S->zero -= S->degencon;
@@ -647,8 +667,10 @@ adjust_index:
   { S->neg -= con_index;
     S->pos -= S->CN - con_index;
   }
+set_counts:
   eigen_neg = S->neg; eigen_pos = S->pos;
 
+  PROF_FINISH(hessian_project_setup);
 } /* end BK_hess_project_setup */
 
 /*************************************************************************
@@ -688,11 +710,16 @@ REAL lowest_eigenpair(S,v)
 struct linsys *S;
 REAL *v; /* eigenvector, preallocated */
 { REAL lo;
+  int old_quiet;
+
+#ifdef XXX
   REAL old_ev,new_ev;
   int i;
   int krydim = 20;
   int nlook = 2;
   REAL evalues[5];
+  REAL *vtmp = (REAL*)temp_calloc(S->N,sizeof(REAL));
+#endif
 
   /* find lower bound on lowest eigenvalue */
   lo = -0.01;
@@ -705,7 +732,15 @@ REAL *v; /* eigenvector, preallocated */
     (*sp_factor_func)(S);
     (*sp_hess_project_setup_func)(S);
   }
-  lanczos(S,krydim,evalues,nlook);
+  
+  old_quiet = quiet_flag;
+  quiet_flag = 1;
+  do_ritz(S,S->lambda,1,&v);
+  quiet_flag = old_quiet;
+  return last_eigenvalue;
+  
+  #ifdef XXX
+  //lanczos(S,krydim,evalues,nlook);
 
   /* inverse iteration to find eigenvector */
   S->lambda = evalues[0] - .0001;
@@ -714,16 +749,18 @@ REAL *v; /* eigenvector, preallocated */
   for ( i = 0 ; i < S->N ; i++ ) v[i] = drand48();
   old_ev = new_ev = 11231.0;  /* weird number, nonzero */
   do
-  { sp_hessian_solve(S,v,v,NO_SET_PRESSURE);
+  { sp_hessian_solve(S,v,vtmp,NO_SET_PRESSURE);
     old_ev = new_ev;
-    new_ev = sqrt(dot(v,v,S->N));
-    for ( i = 0 ; i < S->N ; i++ ) v[i] /= new_ev;
+    new_ev = sqrt(dot(vtmp,vtmp,S->N));
+    for ( i = 0 ; i < S->N ; i++ ) v[i] = vtmp[i]/new_ev;
   } while ( fabs(1/old_ev-1/new_ev) > 1000*machine_eps );
+  temp_free((char*)vtmp);
 
   if ( S->zero != 0 ) last_eigenvalue = S->lambda;
   else
   last_eigenvalue = S->lambda + 1/new_ev;
   return S->lambda + 1/new_ev;
+  #endif
 } /* end lowest_eigenpair */
 
 
@@ -743,18 +780,18 @@ struct linsys *S;
 REAL *x; /* eigenvector, preallocated */
 { REAL norm_inv;
   int i;
-  REAL *h; /* search direction */
-  REAL *ah; 
-  REAL *f,*If; /* gradient of XAX, form and vector */
-  REAL *mx; /* MX */
-  REAL *mh; /* MH */
+  REAL *h=NULL; /* search direction */
+  REAL *ah=NULL; 
+  REAL *f=NULL,*If=NULL; /* gradient of XAX, form and vector */
+  REAL *mx=NULL; /* MX */
+  REAL *mh=NULL; /* MH */
   REAL cgamma; /* cg coefficient */
   REAL xax,old_xax;
   int count;
   int maxcount;
   char response[100];
-  REAL **CCinv;
-  REAL *Cf,*Gf;
+  REAL **CCinv=NULL;
+  REAL *Cf=NULL,*Gf=NULL;
 
   h = (REAL *)temp_calloc(S->N,sizeof(REAL));
   ah = (REAL *)temp_calloc(S->N,sizeof(REAL));
@@ -1145,19 +1182,19 @@ REAL **x; /* eigenvectors, preallocated */
 REAL *ev; /* eigenvalues, preallocated */
 { 
   int i,j,k;
-  REAL **h; /* search direction */
-  REAL **ah; 
-  REAL **f,**If; /* gradient of XAX, form and vector */
-  REAL **mx; /* MX */
-  REAL **mh; /* MH */
+  REAL **h=NULL; /* search direction */
+  REAL **ah=NULL; 
+  REAL **f=NULL,**If=NULL; /* gradient of XAX, form and vector */
+  REAL **mx=NULL; /* MX */
+  REAL **mh=NULL; /* MH */
   REAL cgamma; /* cg coefficient */
-  REAL **xax;
+  REAL **xax=NULL;
   REAL trxax,old_trxax; /* objective */
   int count;
   int maxcount;
   char response[100];
-  REAL **CCinv;
-  REAL **Cf,**Gf;
+  REAL **CCinv=NULL;
+  REAL **Cf=NULL,**Gf=NULL;
   struct linsys *M;  /* Hessian metric to use */
   REAL **L,**Linv; /* for LQ decomp */
   REAL **Q;
@@ -1390,9 +1427,16 @@ int set_pressure_flag; /* whether to set body pressures */
   body_id b_id;
   struct gen_quant*q;
  
+  if ( (S->CN > 0) && augmented_hessian_mode )
+    { /* Kludge since volume deltas got in with wrong sign */
+      for ( i = S->N - S->CN ; i < S->N ; i++ )
+        rhs[i] = - rhs[i];
+    }
+
   (*sp_solve_func)(S,rhs,X);
 
   /* check decrease in residual */
+  if ( itdebug || hess_debug )
   { REAL *xrhs = (REAL *)temp_calloc(S->maxN,sizeof(REAL));
     REAL rhsmag = 0.0;
     REAL resmag = 0.0;
@@ -1407,10 +1451,11 @@ int set_pressure_flag; /* whether to set body pressures */
              sqrt(rhsmag),sqrt(resmag));
       kb_error(2561,errmsg,WARNING);
     }
+    temp_free((char*)xrhs);
   }
 
   if ( S->CN > 0 ) /* have constraints */ 
-  { if ( augmented_hessian_flag )
+  { if ( augmented_hessian_mode )
     { /* remap Lagrange multipliers*/
       for ( i = 0 ; i < S->concount ; i++ )
         if ( S->coninx[i] >= 0 )
@@ -1433,7 +1478,7 @@ int set_pressure_flag; /* whether to set body pressures */
      for ( i = 0 ; i < S->CN ; i++ ) con_rhs[i] = -T2[i];
      
      /* remap Lagrange multipliers*/
-     for ( i = 0 ; i <= S->concount ; i++ )
+     for ( i = 0 ; i < S->concount ; i++ )
         if ( S->coninx[i] >= 0 )
           pressures[i] = con_rhs[S->coninx[i]];
      temp_free((char*)T1);
@@ -1445,13 +1490,13 @@ int set_pressure_flag; /* whether to set body pressures */
     if ( set_pressure_flag == SET_PRESSURE )
     {  if ( ! everything_quantities_flag )
         FOR_ALL_BODIES(b_id)
-        { REAL p = get_body_pressure(b_id)+pressures[ordinal(b_id)];
+        { REAL p = get_body_pressure(b_id)+pressures[loc_ordinal(b_id)];
           set_body_pressure(b_id,p);
         }
        for ( i = 0 ; i < gen_quant_count ; i++ )
        { q = GEN_QUANT(i);
          if ( q->flags & (Q_FIXED|Q_CONSERVED) )
-		 { q->pressure += pressures[everything_quantities_flag?i:web.skel[BODY].max_ord+1+i];
+         { q->pressure += pressures[everything_quantities_flag?i:web.skel[BODY].max_ord+1+i];
            if ( valid_id(q->b_id) )
            set_body_pressure(q->b_id,q->pressure);
          }
@@ -1459,12 +1504,20 @@ int set_pressure_flag; /* whether to set body pressures */
     }
   }
   if ( hess_debug )
-  { for ( i = 0 ; i <= web.skel[VERTEX].max_ord ; i++ )
-    { for ( j = 0 ; j < vhead[i].freedom ; j++ )
-      { ii = vhead[i].rownum+j;
-        printf("    %f  X[%d] v%d.%d\n",(DOUBLE)X[ii],ii,i+1,j+1);
+  { 
+    #ifdef MPI_EVOLVER
+    for ( ii = 0 ; ii < S->N ; ii++ )
+      printf("    %20.15f  X[%d] \n",(DOUBLE)X[ii],ii);
+    #else
+    vertex_id v_id;
+    MFOR_ALL_VERTICES(v_id)
+    { struct hess_verlist *vh = get_vertex_vhead(v_id);
+      for ( j = 0 ; j < vh->freedom ; j++ )
+      { ii = vh->rownum+j;
+        printf("    %20.15f  X[%d] v%d.%d\n",(DOUBLE)X[ii],ii,ordinal(v_id)+1,j+1);
       } 
     }
+    #endif
     for ( i = 0 ; i < optparamcount ; i++ )
     { ii = optparam[i].pnum;
       jj = optparam[i].rownum;
@@ -1637,16 +1690,21 @@ afterits:
 *
 * purpose;  Set up and solve sparse Hessian with given right hand side.
 */
-void sp_Hessian_solver(S,rhs,X)
+void sp_Hessian_solver(S,rhs,Xptr)
 struct linsys *S;  /* system */
 REAL *rhs; /* right side */
-REAL *X; /* returned vector */
+REAL **Xptr; /* returned vector */
 {
+  REAL *X;
+  
+  if ( *Xptr == NULL )
+    *Xptr = (REAL*)temp_calloc(S->N,sizeof(REAL));
+  X = *Xptr;
 
   (*sp_AIJ_setup_func)(S->A_rows,S);
-  if ( sp_ordering_func ) (*sp_ordering_func)(S);
   (*sp_constraint_setup_func)
      (web.skel[BODY].max_ord+1 + gen_quant_count,S);
+  if ( sp_ordering_func ) (*sp_ordering_func)(S);
   (*sp_factor_func)(S);
   (*sp_hess_project_setup_func)(S);
 
@@ -1702,3 +1760,55 @@ struct linsys *M;  /* the metric */
   return sum;
 } /* end sparse_metric_dot() */
 
+
+/*************************************************************************
+*
+* function: sp_CHinvC()
+* 
+* purpose: Find inner product of matrix using hessian inverse as metric.
+*          C stored sparsely in S.
+*
+*          Want to use sparsity in case of large CN.  So solve H X = C
+*          for one constraint at a time, and sparse-multiply the result
+*          by the rest of the constraint rows.  Avoids forming full
+*          CN x N matrix.
+*/
+
+void sp_CHinvC(S)
+struct linsys *S; /* factored system */
+{
+  int n; /* row index */
+  int i,j,k;
+  REAL *BB,*Y;
+
+
+  if ( S->P == NULL )
+   kb_error(2530,"Internal error: Must call sp_factor_func before sp_CHinvC.\n",
+     RECOVERABLE);
+
+  BB = (REAL*)temp_calloc(S->N,sizeof(REAL));  /* expanded rhs */
+  Y = (REAL*)temp_calloc(S->N,sizeof(REAL));  /* solution */
+
+  for ( i = 0 ; i < S->CN ; i++ )
+  {
+    /* solve H Y = C */
+    memset(BB,0,S->N*sizeof(REAL));
+    for ( n = S->CIA[i] ; n < S->CIA[i+1] ; n++ ) 
+      BB[S->CJA[n]] = S->CA[n]; 
+    (*sp_solve_func)(S,BB,Y);
+
+    /* multiply by rows of C */
+    for ( j = 0 ; j <= i ; j++ )
+    { REAL sum;
+      for ( k = S->CIA[j] , sum = 0.0 ; k < S->CIA[j+1] ; k++ )
+        sum += S->CA[k]*Y[S->CJA[k]];
+      S->CHinvCinv[i][j] = sum;
+      if ( ! blas_flag )
+        S->CHinvCinv[j][i] = sum; /* symmetric */
+    }
+  }
+
+  temp_free((char*)BB);
+  temp_free((char*)Y);
+}
+
