@@ -14,6 +14,7 @@
 
 #include "include.h"
 
+#ifdef THREADS
 /******************************************************************
 *
 * function: thread_calc_facet_energy()
@@ -22,10 +23,15 @@
 */
 
 void thread_calc_facet_energy()
-{ facet_id f_id;
-
-    THREAD_FOR_ALL_FACETS(f_id)
-      (*calc_facet_energy)(f_id,ALL_ENERGIES);
+{ 
+#ifdef OLDTHNEXT
+  facet_id f_id;
+  THREAD_FOR_ALL_FACETS(f_id)
+     (*calc_facet_energy)(f_id,ALL_ENERGIES);
+#else
+  THREAD_FOR_ALL_NEW(FACET, {(*calc_facet_energy)(*idptr,ALL_ENERGIES);});
+#endif
+  
 }
 
 /******************************************************************
@@ -36,30 +42,60 @@ void thread_calc_facet_energy()
 */
 
 void thread_calc_facet_forces()
-{ facet_id f_id;
-
-    THREAD_FOR_ALL_FACETS(f_id)
-    {  
-       (*calc_facet_forces)(f_id);
-    }
+{ 
+  
+#ifdef OLDTHNEXT
+  facet_id f_id;
+  THREAD_FOR_ALL_FACETS(f_id)
+  {  
+     (*calc_facet_forces)(f_id);
+  }
+#else
+  THREAD_FOR_ALL_NEW(FACET,(*calc_facet_forces)(*idptr))
+#endif
 }
+#endif
+ /* THREADS */
 
 /*******************************************************************
 *
 *  Function: calc_energy()
 *
+*  Purpose:  Wrapper for total energy calculation.
+*    
+* 
+*/
+
+void calc_energy() 
+{
+#ifdef MPI_EVOLVER
+  mpi_calc_energy();
+#else
+  local_calc_energy();
+#endif
+}
+
+/*******************************************************************
+*
+*  Function: local_calc_energy()
+*
 *  Purpose: Finds total energy of configuration.
 *           Requires volumes set to calculate pressure energy.
 *
 */
 
-void calc_energy() 
+void local_calc_energy() 
 {
+  int i;
   body_id    b_id;
   edge_id e_id;
 
   web.total_energy = 0.0;
   web.total_area    = 0.0;
+  for ( i = 0 ; i < MAXADDENDS ; i++ ) 
+  { web.total_area_addends[i] = 0.0;
+    web.total_energy_addends[i] = 0.0;
+  }
   web.spring_energy = 0.0;
   euclidean_area = 0.0;
 
@@ -182,28 +218,48 @@ quantities_only:
       && !quantities_only_flag )
     {
       if ( !equal_id(b_id,web.outside_body) )
-      { web.total_energy += -web.pressure*(fix*log(vol/fix)-(vol-fix));
+      { binary_tree_add(web.total_energy_addends,
+           -web.pressure*(fix*log(vol/fix)-(vol-fix)));
         if ( valid_id(web.outside_body) )
-             web.total_energy += web.pressure*vol;
+          binary_tree_add(web.total_energy_addends,web.pressure*vol);
         set_body_pressure(b_id,web.pressure*fix/vol);     
       }
     }
     else if ( (get_battr(b_id) & PRESSURE) && !quantities_only_flag )
-       web.total_energy -= get_body_pressure(b_id)*vol;
+       binary_tree_add(web.total_energy_addends,-get_body_pressure(b_id)*vol);
   }
 
   if ( gen_quant_count )
      web.total_energy += calc_quants(Q_ENERGY);
 
+  /* finalize web.total_area */
+  for ( i = 0 ; i < MAXADDENDS ; i++ )
+  { web.total_area += web.total_area_addends[i];
+    web.total_energy += web.total_energy_addends[i];
+  }
+
   extrap_val[reflevel] = web.total_energy;
 
-#if defined(PVM) || defined(MPI)
-  web_checksum = kb_checksum((char*)&web,sizeof(web));
-  dymem_checksum = kb_checksum((char*)dymem,dymemsize);
-#endif
-
 } /* end calc_energy() */
 
+
+/****************************************************************
+*
+*  Function: calc_force()
+*
+*  Purpose: Calls proper energy gradient calculator.
+*
+*/
+
+void calc_force()  
+{
+  #ifdef MPI_EVOLVER
+  mpi_calc_force();
+  #else
+  local_calc_force();
+  #endif
+}
+
 /****************************************************************
 *
 *  Function: calc_force()
@@ -211,10 +267,9 @@ quantities_only:
 *  Purpose: calculates net force at triangulation vertices due
 *           to surface tension and constraints
 *
-*  
 */
 
-void calc_force()  
+void local_calc_force()  
 {
   vertex_id v_id;
   facet_id f_id;
@@ -222,7 +277,7 @@ void calc_force()
   int i;
 
   /* zero out vertex cumulative quantities */
-  FOR_ALL_VERTICES(v_id)
+  MFOR_ALL_VERTICES(v_id)
   { REAL *f = get_force(v_id);
     for ( i = 0 ; i < SDIM ; i ++ ) f[i] = 0.0;
     set_vertex_star(v_id,0.0);             
@@ -389,11 +444,6 @@ grad_quantities_only:
      unsave_coords(&csaved,SAVE_SEPARATE);
   }
 
-#if defined(PVM) || defined(MPI)
-  web_checksum = kb_checksum((char*)&web,sizeof(web));
-  dymem_checksum = kb_checksum((char*)dymem,dymemsize);
-#endif
-
 } /* end calc_force() */
 
 /*********************************************************************
@@ -508,7 +558,7 @@ void force_normalization()
            area /= sqrt(6.);
       }
       if ( area == 0.0 )  
-      { sprintf(errmsg,"Zero area around vertex %d.\n",ordinal(v_id)+1);
+      { sprintf(errmsg,"Zero area around vertex %s.\n",ELNAME(v_id));
         kb_error(1453,errmsg,RECOVERABLE);
       }
       for ( i = 0 ; i < SDIM ; i++ )
@@ -541,29 +591,174 @@ void force_normalization()
 *
 *  Function: calc_content()
 *
-*  Purpose: calculates body volumes 
+*  Purpose: wrapper for local_calc_content,  and
+*       calculates body volumes diffs
+*
 *  Return value: total difference from fixed value constraints
 */
-
-REAL  calc_content(mode)  
-int mode; /* Q_FIXED | Q_INFO | Q_RENORMALIZE */
-{
-  facet_id  f_id;
-  body_id    b_id,bi_id;
-  facetedge_id fe_id;
+REAL calc_content(mode)
+int mode;
+{ body_id b_id,bi_id;
   int k;
   REAL diff = 0.0;
   struct gen_quant *gq;
+  facet_id f_id;
+
+  
+  /* starting body volumes */
+  FOR_ALL_BODIES(b_id)
+  { int attr = get_battr(b_id);
+    if ( !everything_quantities_flag || ((attr & FIXEDVOL) && (mode & Q_FIXED))
+       || ((attr & PRESSURE) && (mode & (Q_ENERGY|Q_INFO)) )
+       || (!(attr & (FIXEDVOL|PRESSURE)) && (mode & Q_INFO)) )
+     { set_body_volume(b_id,get_body_volconst(b_id),NOSETSTAMP);
+       if ( web.representation == STRING )
+       { /* kludge for calculating separate facet areas */
+         facetedge_id fe = get_body_fe(b_id);
+         facet_id start_f;
+         if ( valid_id(fe) )
+         { start_f = get_fe_facet(fe);
+           f_id = start_f;
+           do
+           { set_facet_area(get_fe_facet(fe),0.0);
+             f_id = get_next_body_facet(f_id);
+           } while ( !equal_id(f_id,start_f));
+         } 
+       }
+     }
+  }
+
+  #ifdef MPI_EVOLVER
+  mpi_calc_content(mode);
+  #else 
+  local_calc_content(mode);
+  #endif 
+
+
+  if ( web.torus_flag )
+  { /* do volume adjust to get continuity with old volume, or fixed volume */
+     FOR_ALL_BODIES(b_id)
+     { REAL vol = get_body_volume(b_id);
+       REAL dvol = 0.0;
+       REAL new_vc;
+       if ( (get_battr(b_id) & FIXEDVOL) && !web.pressure_flag )
+       { REAL ovol = get_body_fixvol(b_id);
+         dvol = web.torusv*floor((ovol-vol)/web.torusv+0.5);
+         add_body_volume_plain(b_id,dvol);   
+         save_body_volume(b_id);
+       }
+       else if (mode & Q_RENORMALIZE) /* set between 0 and 1 */
+       { dvol = -web.torusv*floor(vol/web.torusv);
+         add_body_volume_plain(b_id,dvol);
+         save_body_volume(b_id);  
+       }
+       else  /* enforce continuity */
+       { REAL ovol = get_body_oldvolume(b_id);
+         dvol = web.torusv*floor((ovol-vol)/web.torusv+0.5);
+         add_body_volume_plain(b_id,dvol); 
+       }
+       new_vc = get_body_volconst(b_id)+dvol;
+       set_body_volconst(b_id,new_vc);
+  
+       if ( (web.representation == STRING) && (new_vc != 0.0) )
+       { /* apply volconst to most worthy facet */
+         facetedge_id fe = get_body_fe(b_id);
+         facet_id f_id = get_fe_facet(fe);
+         facet_id start_f = f_id;
+         REAL worst_a = new_vc > 0.0 ? 1e30 : -1e30;
+         facet_id best_f = NULLID;
+         if ( valid_id(fe) && valid_id(f_id) )
+         do
+         { REAL a = get_facet_area(f_id);
+           if ( inverted(f_id) ) 
+             a = -a;
+           if ( ((new_vc > 0.0) && (a < worst_a))
+              || ((new_vc < 0.0) && (a > worst_a)) )
+           { best_f = f_id;
+             worst_a = a;
+           }
+           f_id = get_next_body_facet(f_id);
+         } while ( !equal_id(f_id,start_f) );
+         if ( valid_id(best_f) )
+           add_facet_area(best_f,new_vc);
+       }
+     }
+  }
+
+  if ( mode & Q_RENORMALIZE )
+  { /* take care of bodies with declared actual volume */
+    FOR_ALL_BODIES(b_id)
+      if ( get_battr(b_id) & ACTUALVOL )
+      { REAL calcvol = get_body_volume(b_id);
+        REAL actual  = get_body_actualvolume(b_id);
+        set_body_volume(b_id,actual,SETSTAMP);
+        set_body_volconst(b_id,get_body_volconst(b_id)+actual-calcvol); 
+        save_body_volume(b_id);
+      }
+  }
+
+  web.vol_flag = 1;
+
+
+  /* get total deviation from constraints */
+  if ( !everything_quantities_flag ) 
+    FOR_ALL_BODIES(bi_id)
+    { REAL absvol = get_body_abstotal(b_id);
+      REAL target = get_body_fixvol(bi_id);
+      REAL actual = get_body_volume(bi_id);
+
+      if ( !(get_battr(bi_id) & FIXEDVOL) ) continue;
+      diff += fabs(target - actual)/
+                        (web.target_tolerance*(absvol?absvol:1.0));
+    }
+ 
+  for ( k = 0 ; k < gen_quant_count ; k++ )
+  { gq = GEN_QUANT(k);
+    if ( !(gq->flags & Q_FIXED) ) continue;
+    if ( gq->tolerance > 0.0 )
+      diff += fabs(gq->target - gq->value)/(gq->tolerance*gq->abstotal);
+    else
+      diff += fabs(gq->target - gq->value)/(web.target_tolerance*gq->abstotal);
+  }
+  
+  if ( mode & Q_FIXED )
+    fixed_volume_timestamp = global_timestamp;
+  if ( mode & Q_INFO )
+    info_volume_timestamp = global_timestamp;
+  return diff;
+
+}
+
+/*************************************************************
+*
+*  Function: local_calc_content()
+*
+*  Purpose: calculates body volumes  on local piece of surface.
+*
+*/
+
+void  local_calc_content(mode)  
+int mode; /* Q_FIXED | Q_INFO | Q_RENORMALIZE */
+{
+  facet_id  f_id;
+  facetedge_id fe_id;
+  body_id b_id;
+
+  FOR_ALL_BODIES(b_id)
+  { /* initialize binary tree add */
+    struct body *ptr;
+	int k;
+
+	ptr = bptr(b_id);
+    for ( k = 0 ; k < MAXADDENDS ; k++ )
+       ptr->volume_addends[k] = 0;
+  }
 
   if ( gen_quant_count )
      calc_quants(mode);
 
-  if ( quantities_only_flag ) goto do_diffs;
-  if ( web.bodycount == 0 ) goto do_diffs;
-
-  /* starting body volumes */
-  FOR_ALL_BODIES(b_id)
-     set_body_volume(b_id,get_body_volconst(b_id));
+  if ( quantities_only_flag ) return;
+  if ( web.bodycount == 0 ) return;
 
   if ( web.representation == SIMPLEX )
   { FOR_ALL_FACETS(f_id)
@@ -599,68 +794,18 @@ int mode; /* Q_FIXED | Q_INFO | Q_RENORMALIZE */
       }
     }
   }
-do_diffs:
-  if ( web.torus_flag )
-  { /* do volume adjust to get continuity with old volume, or fixed volume */
-     FOR_ALL_BODIES(b_id)
-     { REAL vol = get_body_volume(b_id);
-       REAL dvol = 0.0;
-       if ( (get_battr(b_id) & FIXEDVOL) && !web.pressure_flag )
-       { REAL ovol = get_body_fixvol(b_id);
-         dvol = web.torusv*floor((ovol-vol)/web.torusv+0.5);
-         add_body_volume(b_id,dvol);   save_body_volume(b_id);
-       }
-       else if (mode & Q_RENORMALIZE) /* set between 0 and 1 */
-       { dvol = -web.torusv*floor(vol/web.torusv);
-         add_body_volume(b_id,dvol);
-         save_body_volume(b_id);  save_body_volume(b_id);
-       }
-       else  /* enforce continuity */
-       { REAL ovol = get_body_oldvolume(b_id);
-         dvol = web.torusv*floor((ovol-vol)/web.torusv+0.5);
-         add_body_volume(b_id,dvol); 
-       }
-       set_body_volconst(b_id,get_body_volconst(b_id)+dvol);
-     }
+
+  /* finish binary tree accumulation */
+  FOR_ALL_BODIES(b_id)
+  { REAL sum = 0.0; 
+    struct body *ptr = bptr(b_id);
+	int k;
+    for ( k = 0 ; k < MAXADDENDS ; k++ )
+      sum += ptr->volume_addends[k];
+    add_body_volume_plain(b_id,sum);
   }
 
-  if ( mode & Q_RENORMALIZE )
-  { /* take care of bodies with declared actual volume */
-    FOR_ALL_BODIES(b_id)
-      if ( get_battr(b_id) & ACTUALVOL )
-      { REAL calcvol = get_body_volume(b_id);
-        REAL actual  = get_body_actualvolume(b_id);
-        set_body_volume(b_id,actual);
-        set_body_volconst(b_id,get_body_volconst(b_id)+actual-calcvol); 
-        save_body_volume(b_id);
-      }
-  }
-
-  web.vol_flag = 1;
-
-
-
-  /* get total deviation from constraints */
-  if ( !everything_quantities_flag )
-    FOR_ALL_BODIES(bi_id)
-    {
-      if ( !(get_battr(bi_id) & FIXEDVOL) ) continue;
-      diff += (fabs(get_body_fixvol(bi_id) - get_body_volume(bi_id)))/
-                        (web.target_tolerance*get_body_abstotal(bi_id));
-    }
- 
-  for ( k = 0 ; k < gen_quant_count ; k++ )
-  { gq = GEN_QUANT(k);
-    if ( !(gq->flags & Q_FIXED) ) continue;
-    if ( gq->tolerance > 0.0 )
-      diff += fabs(gq->target - gq->value)/(gq->tolerance*gq->abstotal);
-    else
-      diff += fabs(gq->target - gq->value)/(web.target_tolerance*gq->abstotal);
-  }
-  volume_timestamp = global_timestamp;
-  return diff;
-
-} /* end calc_content() */
+} /* end local_calc_content() */
 
 /*************************************************************
 *
@@ -687,7 +832,7 @@ void calc_pressure()
     { if ( get_body_volume(b_id) > 0.0 )
         p = web.pressure*get_body_fixvol(b_id)/get_body_volume(b_id);
       else 
-      { sprintf(errmsg,"Body %d has volume %f\n",1+ordinal(b_id),
+      { sprintf(errmsg,"Body %s has volume %f\n",ELNAME(b_id),
                                (DOUBLE)get_body_volume(b_id));
         kb_error(1776,errmsg,WARNING);
         p = 0.0;
