@@ -14,8 +14,9 @@
 #include "include.h"
 
 /* maximum errors of any type before aborting */
-#define MAXERR 10
+#define MAXERR 20
 int wrap_check ARGS((void));
+int check_body_facets ARGS((void));
 
 /*********************************************************************
 *
@@ -37,6 +38,11 @@ int run_checks()
     }
   numerr += method_check();
   if ( web.symmetry_flag ) numerr += wrap_check(); 
+
+  #ifdef MPI_EVOLVER
+  numerr += mpi_node_checks();
+  #endif
+
   check_count = numerr;
   return numerr;
 } /* end run_checks() */
@@ -64,8 +70,8 @@ int method_check()
        for ( k = 0; k < (int)e_ptr->method_count ; k++ )
          for ( n = k+1; n < (int)e_ptr->method_count ; n++ )
            if ( abs(instlist[k]) == abs(instlist[n]) )
-           { sprintf(msg,"%s %d has method %s twice.\n",
-               typenames[type],ordinal(id)+1,METH_INSTANCE(abs(instlist[k]))->name);
+           { sprintf(msg,"%s %s has method %s twice.\n",
+               typenames[type],ELNAME(id),METH_INSTANCE(abs(instlist[k]))->name);
              outstring(msg);
              count++;
            }
@@ -89,7 +95,7 @@ int list_check()
   element_id id,prev_id;
   int numerr = 0;
 
-  free_discards(DISCARDS_ALL);
+  /* free_discards(DISCARDS_ALL); */
   for ( type = 0 ; type < 5 ; type++ )
   { 
     int freecount,maxfree;
@@ -129,6 +135,7 @@ int list_check()
       erroutstring(msg);
     }
 
+#ifndef MPI_EVOLVER
     /* check used list */
     maxused = web.skel[type].count;
     id = web.skel[type].used;
@@ -179,8 +186,22 @@ int list_check()
     { sprintf(msg,"Type %d usedlist last id is non-null: %lX\n",type,id);
       erroutstring(msg);
     }
+#endif
   } /* end for loop */
 
+  #ifdef MPI_EVOLVER
+  for ( type = VERTEX ; type < NUMELEMENTS ; type++ )
+  { int k;
+    for ( k = 0 ; k < web.skel[type].maxcount ; k++ )
+      if ( (web.skel[type].ibase[k]->local_id & OFFSETMASK) != k )
+      { sprintf(msg,"Task %d: local_id is %08X on %s ibase[0x%X], self id %08X\n",
+          this_machine,web.skel[type].ibase[k]->local_id,typenames[type],k,
+          web.skel[type].ibase[k]->self_id);
+        erroutstring(msg);
+      }
+  }
+  #endif
+  
   return numerr;
 } /* end list_check() */
 
@@ -207,77 +228,77 @@ int flag;  /* PRELIMCHECK for check before subdivision, else REGCHECK */
   /* check facetedge chain consistencies */
   if ( numerr >= MAXERR ) numerr = 0;
   count = 0;
-  FOR_ALL_FACETEDGES(fe_id)
+  MFOR_ALL_FACETEDGES(fe_id)
   { facetedge_id fe;
 
     f_id = get_fe_facet(fe_id);
     if ( valid_id(f_id) && !valid_element(f_id) )
-     { sprintf(msg,"Facetedge  %d links to invalid facet %08lX.\n",
-          ordinal(fe_id)+1,f_id);
+     { sprintf(msg,"Facetedge  %s links to invalid facet %08lX.\n",
+          ELNAME(fe_id),f_id);
         erroutstring(msg);
      }
     if ( valid_id(f_id) && !valid_element(get_facet_fe(f_id)) )
-     { sprintf(msg,"Facetedge %d links to facet %d with no facetedge.\n",
-        ordinal(fe_id)+1,ordinal(f_id)+1);
-        erroutstring(msg);
+     { sprintf(msg,"Facetedge %s links to facet %s with no facetedge.\n",
+         ELNAME(fe_id),ELNAME1(f_id));
+       erroutstring(msg);
      }
     e_id = get_fe_edge(fe_id);
     if ( valid_id(e_id) && !valid_element(e_id) )
-     { sprintf(msg,"Facetedge %d links to invalid edge %08lX.\n",
-             ordinal(fe_id)+1,e_id);
+     { sprintf(msg,"Facetedge %s links to invalid edge %08lX.\n",
+             ELNAME(fe_id),e_id);
         erroutstring(msg);
      }
     if ( valid_id(e_id) && !valid_element(get_edge_fe(e_id)) )
-     { sprintf(msg,"Facetedge %d links to edge %d with no facetedge.\n",
-         ordinal(fe_id)+1,ordinal(e_id)+1);
+     { sprintf(msg,"Facetedge %s links to edge %s with no facetedge.\n",
+         ELNAME(fe_id),ELNAME1(e_id));
         erroutstring(msg);
      }
     fe = get_prev_edge(fe_id);
     if ( valid_id(fe) && !equal_id(get_next_edge(fe),fe_id) )
-      { sprintf(msg,"Facetedge %d has bad prev edge link\n",ordinal(fe_id)+1);
+      { sprintf(msg,"Facetedge %s has bad prev edge link\n",ELNAME(fe_id));
         erroutstring(msg);
         if ( ++numerr > MAXERR )
               {erroutstring("Too many errors.\n"); return numerr;}
       }
     fe = get_next_edge(fe_id);
     if ( valid_id(fe) && !equal_id(get_prev_edge(fe),fe_id) )
-      { sprintf(msg,"Facetedge %d has bad next edge link\n",ordinal(fe_id)+1);
+      { sprintf(msg,"Facetedge %s has bad next edge link\n",ELNAME(fe_id));
         erroutstring(msg);
         if ( ++numerr > MAXERR )
               {erroutstring("Too many errors.\n"); return numerr;}
       }
     fe = get_next_facet(fe_id);
     if ( valid_id(fe) && !equal_id(get_prev_facet(fe),fe_id) )
-      { sprintf(msg,"Facetedge %d has bad next facet link\n",ordinal(fe_id)+1);
+      { sprintf(msg,"Facetedge %s has bad next facet link\n",ELNAME(fe_id));
         erroutstring(msg);
         if ( ++numerr > MAXERR )
               {erroutstring("Too many errors.\n"); return numerr;}
       }
     fe = get_prev_facet(fe_id);
     if ( valid_id(fe) && !equal_id(get_next_facet(fe),fe_id) )
-      { sprintf(msg,"Facetedge %d has bad prev facet link\n",ordinal(fe_id)+1);
+      { sprintf(msg,"Facetedge %s has bad prev facet link\n",ELNAME(fe_id));
         erroutstring(msg);
         if ( ++numerr > MAXERR )
               {erroutstring("Too many errors.\n"); return numerr;}
       }
     fe = get_next_edge(fe_id);
     if ( valid_id(fe) && !equal_id(get_fe_headv(fe_id),get_fe_tailv(fe)) )
-      { sprintf(msg,"Facetedge %d head vertex disagrees with next tail.\n",
-        ordinal(fe_id)+1);
+      { sprintf(msg,"Facetedge %s head vertex disagrees with next tail.\n",
+        ELNAME(fe_id));
         erroutstring(msg);
         if ( ++numerr > MAXERR )
               {erroutstring("Too many errors.\n"); return numerr;}
       }
     fe = get_prev_edge(fe_id);
     if ( valid_id(fe) && !equal_id(get_fe_tailv(fe_id),get_fe_headv(fe)) )
-      { sprintf(msg,"Facetedge %d tail vertex disagrees with prev head.\n",
-        ordinal(fe_id)+1);
+      { sprintf(msg,"Facetedge %s tail vertex disagrees with prev head.\n",
+        ELNAME(fe_id));
         erroutstring(msg);
         if ( ++numerr > MAXERR )
               {erroutstring("Too many errors.\n"); return numerr;}
       }
     count++;
-     }
+  }
   if ( count != web.skel[FACETEDGE].count )
     { sprintf(msg,"Only %d facetedges out of %ld generated.\n",
          count,web.skel[FACETEDGE].count);
@@ -290,14 +311,14 @@ int flag;  /* PRELIMCHECK for check before subdivision, else REGCHECK */
     e_id = get_vertex_edge(v_id);
     if ( !valid_id(e_id) ) continue; 
     if ( !valid_element(e_id) ) 
-     { sprintf(errmsg,"Vertex %d has invalid edge link.\n",
-          ordinal(v_id)+1);
+     { sprintf(errmsg,"Vertex %s has invalid edge link.\n",
+          ELNAME(v_id)+1);
        kb_error(1829,errmsg,WARNING);
        continue;
      }
     else if (get_vattr(v_id) & Q_MIDPOINT)
      { if ( !equal_id(v_id,get_edge_midv(e_id)) )
-        { sprintf(errmsg,"Vertex %d has bad midpoint edge link.\n",ordinal(v_id)+1);
+        { sprintf(errmsg,"Vertex %s has bad midpoint edge link.\n",ELNAME(v_id));
           kb_error(1830,errmsg,WARNING);
           continue;
         }
@@ -308,13 +329,13 @@ int flag;  /* PRELIMCHECK for check before subdivision, else REGCHECK */
       for ( i = 0 ; i <= web.lagrange_order ; i++ )
          if ( equal_id(v_id,v[i]) ) break;
       if ( i > web.lagrange_order )
-      { sprintf(errmsg,"Vertex %d has bad edge link.\n",ordinal(v_id)+1);
+      { sprintf(errmsg,"Vertex %s has bad edge link.\n",ELNAME(v_id));
         kb_error(1831,errmsg,WARNING);
       }
     }
     else if (get_vattr(v_id) & Q_MIDFACET) { }
     else if ( !equal_id(v_id,get_edge_tailv(e_id)) )
-     { sprintf(errmsg,"Vertex %d has bad edge link.\n",ordinal(v_id)+1);
+     { sprintf(errmsg,"Vertex %s has bad edge link.\n",ELNAME(v_id));
        check_vertex_fe(v_id);
        kb_error(1832,errmsg,WARNING);
        continue;
@@ -326,25 +347,27 @@ int flag;  /* PRELIMCHECK for check before subdivision, else REGCHECK */
 
     v_id = get_edge_tailv(e_id);
     n = 0; ee_id = e_id;
-    do { ee_id = get_next_tail_edge(ee_id); 
-         if ( ++n > 2*web.skel[EDGE].count )
-          { sprintf(errmsg,"Vertex %d has bad edge loop from edge %d.\n",
-            ordinal(v_id)+1,ordinal(e_id)+1);
-            kb_error(1833,errmsg,WARNING);
-            break;
-          }
-       }
+    do 
+    { ee_id = get_next_tail_edge(ee_id); 
+      if ( ++n > 2*web.skel[EDGE].count )
+      { sprintf(errmsg,"Vertex %s has bad edge loop from edge %s.\n",
+            ELNAME(v_id),ELNAME1(e_id));
+        kb_error(1833,errmsg,WARNING);
+        break;
+      }
+    }
     while ( !equal_id(ee_id,e_id) );
     v_id = get_edge_headv(e_id);
     n = 0; ee_id = e_id;
-    do { ee_id = get_next_head_edge(ee_id); 
-         if ( ++n > 2*web.skel[EDGE].count )
-          { sprintf(errmsg,"Vertex %d has bad edge loop from edge %d.\n",
-            ordinal(v_id)+1,ordinal(e_id)+1);
-            kb_error(1834,errmsg,WARNING);
-            break;
-          }
-       }
+    do 
+    { ee_id = get_next_head_edge(ee_id); 
+      if ( ++n > 2*web.skel[EDGE].count )
+      { sprintf(errmsg,"Vertex %s has bad edge loop from edge %s.\n",
+            ELNAME(v_id),ELNAME1(e_id));
+        kb_error(1834,errmsg,WARNING);
+        break;
+      }
+    }
     while ( !equal_id(ee_id,e_id) );
   } /* end edges */
 
@@ -356,8 +379,8 @@ int flag;  /* PRELIMCHECK for check before subdivision, else REGCHECK */
     facetedge_id first_fe;
 
     if ( get_vattr(v_id) & AXIAL_POINT )
-      { sprintf(msg,"Vertex %d is axial and not first vertex of edge %d.\n",
-            ordinal(v_id)+1,ordinal(e_id)+1);
+      { sprintf(msg,"Vertex %s is axial and not first vertex of edge %s.\n",
+            ELNAME(v_id),ELNAME1(e_id));
         erroutstring(msg);
       }
     last_fe = NULLID;
@@ -365,17 +388,16 @@ int flag;  /* PRELIMCHECK for check before subdivision, else REGCHECK */
     if ( valid_id(fe_id) ) do
     { edge_id ee_id = get_fe_edge(fe_id);
       if ( !equal_id(e_id,ee_id) )
-         { sprintf(msg,"Facetedge %d on edge %d instead of %d.\n",
-             ordinal(fe_id)+1,(inverted(ee_id)?-1:1)*(ordinal(ee_id)+1),
-                 (inverted(e_id)?-1:1)*(ordinal(e_id)+1));
+         { sprintf(msg,"Facetedge %s on edge %s instead of %s.\n",
+             ELNAME(fe_id),SELNAME1(ee_id), SELNAME2(e_id));
            erroutstring(msg);
            if ( ++numerr > MAXERR )
               {erroutstring("Too many errors.\n"); goto facet_check;}
          }
       count++;
       if ( count > web.skel[FACETEDGE].count )
-      { sprintf(msg,"Bad chain of facetedges around edge %d.\n",
-        ordinal(e_id)+1);
+      { sprintf(msg,"Bad chain of facetedges around edge %s.\n",
+          ELNAME(e_id));
         erroutstring(msg);
         if ( ++numerr > MAXERR )
               {erroutstring("Too many errors.\n"); goto facet_check;}
@@ -386,91 +408,100 @@ int flag;  /* PRELIMCHECK for check before subdivision, else REGCHECK */
     if ( last_fe == NULLID )
      { 
        if ( (web.representation == SOAPFILM) && (flag == REGCHECK) ) 
-       {
+       { edge_id orig;
          bare_edge_count++;
          if ( get_eattr(e_id) & BARE_NAKED ) continue;
-         sprintf(msg,"Edge %d has no facets.\n",ordinal(e_id)+1);
+         sprintf(msg,"Edge %s has no facets.\n",ELNAME(e_id));
          erroutstring(msg);
-         if ( get_original(e_id)  != ordinal(e_id)+1 )
-            { sprintf(msg,"      (originally edge %d)\n",get_original(e_id));
+         orig = get_original(e_id);
+         if ( valid_id(orig) && !equal_element(orig,e_id) )
+            { sprintf(msg,"      (originally edge %s)\n",ELNAME(e_id));
               erroutstring(msg);
             }
        }
      }
     else if ( !equal_id(get_edge_fe(e_id),get_next_facet(last_fe)) )
-     { sprintf(msg,"Facets around edge %d do not link up.\n",
-          ordinal(e_id)+1);
+     { edge_id orig;
+       sprintf(msg,"Facets around edge %s do not link up.\n",
+          ELNAME(e_id));
        erroutstring(msg);
-       if ( get_original(e_id)  != ordinal(e_id)+1 )
-          { sprintf(msg,"      (originally edge %d)\n",get_original(e_id));
+       orig = get_original(e_id);
+       if ( valid_id(orig) && !equal_element(orig,e_id) )
+          { sprintf(msg,"      (originally edge %s)\n",ELNAME(e_id));
             erroutstring(msg);
           }
      }
   }
+#ifndef MPI_EVOLVER
   if ( count != web.skel[FACETEDGE].count )
-     { sprintf(msg,"Edges have %d facetedges out of %ld used.\n",
-          count,web.skel[FACETEDGE].count);
-       erroutstring(msg);
-       ++numerr;
-     }
+  { sprintf(msg,"Edges have %d facetedges out of %ld used.\n",
+        count,web.skel[FACETEDGE].count);
+    erroutstring(msg);
+    ++numerr;
+  }
+#endif
 
-facet_check:
+facet_check: 
   if ( numerr >= MAXERR ) numerr = 0;
   if ( web.representation == SOAPFILM ) 
   { int i;
     count = 0;
-    FOR_ALL_FACETS(f_id)
-    {
+    MFOR_ALL_FACETS(f_id)
+    { facetedge_id first_fe;
       int thiscount = 0;
+      
       last_fe = NULLID;
-      fe_id = get_facet_fe(f_id);
+      fe_id = first_fe = get_facet_fe(f_id);
       for ( i = 0 ; i < FACET_EDGES ; i++, fe_id = get_next_edge(fe_id) )
       { vertex_id v_id = get_fe_tailv(fe_id);
         facet_id ff_id;
         if ( (thiscount != 0) && (get_vattr(v_id) & AXIAL_POINT))
-        { sprintf(msg,
-            "Vertex %d is axial and not the first vertex of facet %d (original %d).\n",
-              ordinal(v_id)+1,ordinal(f_id)+1,get_original(f_id)+1);
-          erroutstring(msg);
+        { set_facet_fe(f_id,fe_id);       
         }
         ff_id = get_fe_facet(fe_id);
         if ( !equal_id(f_id,ff_id) )
-         { sprintf(msg,"Facetedge %d on facet %d instead of %d.\n",
-             ordinal(fe_id)+1,(inverted(ff_id)?-1:1)*(ordinal(ff_id)+1),
-               ordinal(f_id)+1);
-              erroutstring(msg);
+        { sprintf(msg,"Facetedge %s on facet %s instead of %s.\n",
+             ELNAME(fe_id),SELNAME1(ff_id), SELNAME2(f_id));
+           erroutstring(msg);
            if ( ++numerr > MAXERR )
               {erroutstring("Too many errors.\n"); break;}
          }
          count++;
          if ( ++thiscount > web.skel[FACETEDGE].count )
-         { sprintf(msg,"Facetedge loop not closed on facet %d.\n",
-             ordinal(f_id)+1);
+         { facet_id orig;
+           sprintf(msg,"Facetedge loop not closed on facet %s.\n",
+             ELNAME(f_id));
            erroutstring(msg);
-           if ( get_original(f_id) != ordinal(f_id)+1  )
-             { sprintf(msg,"      (originally facet %d)\n",get_original(f_id));
+           orig = get_original(f_id);
+           if ( valid_id(orig) && !equal_element(orig,f_id)  )
+             { sprintf(msg,"      (originally facet %s)\n", ELNAME(orig));
                 erroutstring(msg);
              }
            break;
          }
        last_fe = fe_id;
      } /* end while */
-     if ( !equal_id(get_facet_fe(f_id),get_next_edge(last_fe)) )
-     { sprintf(msg,"Edges around facet %d do not link up.\n",
-          ordinal(f_id)+1);
+     fe_id = get_next_edge(last_fe);
+     if ( !equal_id(first_fe,fe_id) )
+     { facet_id orig;
+       sprintf(msg,"Edges around facet %s do not link up.\n",
+          ELNAME(f_id));
        erroutstring(msg);
-       if ( get_original(f_id) != ordinal(f_id)+1  )
-          { sprintf(msg,"      (originally facet %d)\n",get_original(f_id));
+       orig = get_original(f_id);
+       if ( valid_id(orig) &&  !equal_element(orig,f_id) )
+          { sprintf(msg,"      (originally facet %s)\n", ELNAME(orig));
             erroutstring(msg);
           }
        if ( ++numerr > MAXERR )
               {erroutstring("Too many errors.\n"); break;}
      }
      if ( (thiscount != 3) && (flag == REGCHECK)  )
-     { sprintf(msg,"Facet %d has %d edges.\n",ordinal(f_id)+1,thiscount);
+     { facet_id orig;
+       sprintf(msg,"Facet %s has %d edges.\n",ELNAME(f_id),thiscount);
        erroutstring(msg);
-       if ( get_original(f_id) != ordinal(f_id)+1 )
-          { sprintf(msg,"      (originally facet %d)\n",get_original(f_id));
+       orig = get_original(f_id);
+       if ( valid_id(orig) && !equal_element(orig,f_id) )
+          { sprintf(msg,"      (originally facet %s)\n", ELNAME(orig));
             erroutstring(msg);
           }
        if ( ++numerr > MAXERR )
@@ -478,7 +509,7 @@ facet_check:
      }
     }
     if ( count != web.skel[FACETEDGE].count )
-     { sprintf(msg,"Facets have %d facetedges out of %ld used.\n",
+     { sprintf(msg,"Facets have %d facet-edges out of %ld used.\n",
           count,web.skel[FACETEDGE].count);
        erroutstring(msg);
        ++numerr;
@@ -494,6 +525,7 @@ facet_check:
 *
 *  Purpose:  Check whether adjacent facets have same bodies.
 *        Should be used only for complete bodies.
+*     Also checks body facet lists.
 */
 
 int facet_body_check()
@@ -502,24 +534,23 @@ int facet_body_check()
   facet_id f_id;
   body_id b_id;
   int numerr = 0;
-  int orig;
+  element_id orig;
       
 
   FOR_ALL_BODIES(b_id)
   { fe = get_body_fe(b_id);
     if ( !valid_id(fe) ) continue;
     if ( !valid_element(fe) )
-    { sprintf(msg,"Body %d has invalid facetedge link to edge %d facet %d.\n",
-       ordinal(b_id)+1,ordinal(get_fe_edge(fe))+1,ordinal(get_fe_facet(fe))+1);
+    { sprintf(msg,"Body %s has invalid facetedge link to edge %s facet %s.\n",
+       ELNAME(b_id),ELNAME1(get_fe_edge(fe)),ELNAME2(get_fe_facet(fe)));
       erroutstring(msg);
       numerr++;
       continue;
      }
      f_id = get_fe_facet(fe);
      if ( !equal_id(b_id,get_facet_body(f_id) ) )
-     { sprintf(msg,"Body %d has link to facet %d, which is on body %d.\n",
-         ordinal(b_id)+1,inverted(f_id)?-(ordinal(f_id)+1):ordinal(f_id)+1,
-         ordinal(get_facet_body(f_id))+1);
+     { sprintf(msg,"Body %s has link to facet %s, which is on body %s.\n",
+         ELNAME(b_id),SELNAME1(f_id),ELNAME2(get_facet_body(f_id)));
        erroutstring(msg);
        numerr++;
      }
@@ -542,29 +573,30 @@ int facet_body_check()
       ff_id = get_fe_facet(fe_inverse(get_prev_facet(fe)));
       bb_id = get_facet_body(ff_id);
       e_id = get_fe_edge(fe);
-      if ( !equal_id(b_id,bb_id) 
+      if ( /* !equal_id(b_id,bb_id)  */
+           (ordinal(b_id) != ordinal(bb_id))  /* for MPI, temp kludge */
              && !(get_attr(e_id)&(CONSTRAINT|FIXED|BOUNDARY)) )
-      { sprintf(msg,"Inconsistent bodies for facets on edge %d.",
-              ordinal(e_id)+1 );
+      { sprintf(msg,"Inconsistent bodies for facets on edge %s.",
+              ELNAME(e_id) );
         erroutstring(msg);
         numerr++;
         orig = get_original(e_id);
-        if ( orig && (orig != ordinal(e_id)+1) && (orig != -1) )
-        { sprintf(msg,"      (originally edge %d)",orig);
+        if ( valid_id(orig) && !equal_element(orig,e_id) )
+        { sprintf(msg,"      (originally edge %s)",ELNAME(orig));
           erroutstring(msg);
         }
         sprintf(msg,"\n     facet %d",oid(f_id));
         erroutstring(msg);
         orig = get_original(f_id);
-        if ( orig && (orig != ordinal(f_id)+1) )
-        { sprintf(msg," (orig. %d)",orig); erroutstring(msg); }
+        if ( valid_id(orig) && !equal_element(orig,f_id) )
+        { sprintf(msg," (orig. %s)",ELNAME(orig)); erroutstring(msg); }
         if ( valid_id(b_id) )
         {
-          sprintf(msg," has body %d",ordinal(b_id)+1);
+          sprintf(msg," has body %s",ELNAME(b_id));
             erroutstring(msg);
           orig = get_original(b_id);
-          if ( orig && (orig != ordinal(b_id)+1) )
-             { sprintf(msg," (orig. %d)",orig);
+          if ( valid_id(orig) && !equal_element(orig,b_id) )
+             { sprintf(msg," (orig. %s)",ELNAME(orig));
                erroutstring(msg);
              }
         }
@@ -572,17 +604,17 @@ int facet_body_check()
         sprintf(msg,"; facet %d",oid(ff_id));
         erroutstring(msg);
         orig = get_original(ff_id);
-        if ( orig && (orig != ordinal(ff_id)+1) )
-        { sprintf(msg," (orig. %d)",orig);
+        if ( valid_id(orig) && !equal_element(orig,ff_id) )
+        { sprintf(msg," (orig. %s)",ELNAME(orig));
           erroutstring(msg);
         }
         if ( valid_id(bb_id) )
         {
-          sprintf(msg," has body %d",ordinal(bb_id)+1);
+          sprintf(msg," has body %s",ELNAME(bb_id));
           erroutstring(msg);
           orig = get_original(bb_id);
-          if ( orig && (orig != ordinal(bb_id)+1) )
-             { sprintf(msg," (orig. %d)",orig);
+          if ( valid_id(orig) && !equal_element(orig,bb_id) )
+             { sprintf(msg," (orig. %s)",ELNAME(orig));
                erroutstring(msg);
              }
         }
@@ -592,9 +624,52 @@ int facet_body_check()
       }
     }
    } /* end FACETEDGES */
+  numerr += check_body_facets();
   return numerr;
 } /* end facet_body_check() */
 
+/************************************************************************
+*
+* function: check_body_facets()
+*
+* purpose:  check body facet lists 
+*/
+
+int check_body_facets()
+{ int numerr = 0;
+  body_id b_id;
+
+  FOR_ALL_BODIES(b_id)
+  { facet_id  f_id,start_f,next_f;
+    facetedge_id fe = get_body_fe(b_id);
+    if ( !valid_id(fe) ) continue;
+    start_f = f_id = get_fe_facet(fe); 
+    do 
+    { if ( !valid_element(f_id) )
+      { sprintf(msg,"Invalid facet %s on body facet list of body %s.\n",
+              ELNAME(f_id),ELNAME1(b_id));
+        erroutstring(msg);
+        numerr++;
+      }        
+      if ( !equal_id(b_id,get_facet_body(f_id)) )
+      { sprintf(msg,"Facet %s on body %s facet list, but is on body %s.\n",
+          SELNAME(f_id),ELNAME1(b_id),ELNAME2(get_facet_body(f_id))); 
+        erroutstring(msg);
+        numerr++;
+      }
+            next_f = get_next_body_facet(f_id);  
+      if ( !equal_id(f_id,get_prev_body_facet(next_f)) )
+      { sprintf(msg,"Facet %s next body facet has bad prev_body_facet.\n",
+         SELNAME(f_id));
+        erroutstring(msg);
+        numerr++;
+        break;
+      }
+      f_id = next_f;
+    } while ( !equal_id(f_id,start_f) );
+  }
+  return numerr;
+}
 
 /**********************************************************************
 *
@@ -609,8 +684,8 @@ int vvvvcomp(a,b)
 struct vvvv *a,*b;
 { int i;
   for ( i = 0 ; i < 3 ; i++ )
-  { if ( a->vord[i] < b->vord[i] ) return -1;
-    if ( a->vord[i] > b->vord[i] ) return 1;
+  { if ( a->v[i] < b->v[i] ) return -1;
+    if ( a->v[i] > b->v[i] ) return 1;
   }
   return 0;
 }
@@ -620,11 +695,12 @@ int collapse_check()
   struct vvvv *vvlist,*current;
   facet_id f_id;
   edge_id e_id;
-  int ord1,ord2,ord3,tmp;
+  vertex_id tmp;
+  vertex_id v1,v2,v3;
   int i;
   int count;
   int numerr = 0;
-  int orig;
+  element_id orig;
 
   /* check edges */
   if ( web.skel[EDGE].count == 0 ) return numerr;
@@ -634,34 +710,34 @@ int collapse_check()
   count = 0;
   FOR_ALL_EDGES(e_id)
   { 
-    ord1 = ordinal(get_edge_tailv(e_id));
-    ord2 = ordinal(get_edge_headv(e_id));
-    if ( ord1 == ord2 )
-    { sprintf(msg,"Edge %d",ordinal(e_id)+1);
+    v1 = get_edge_tailv(e_id);
+    v2 = get_edge_headv(e_id);
+    if ( equal_element(v1,v2) && !(get_vattr(v1) & AXIAL_POINT) )
+    { sprintf(msg,"Edge %s",ELNAME(e_id));
       erroutstring(msg);
       orig = get_original(e_id);
-      if ( orig && (orig != ordinal(e_id)+1) )
-        { sprintf(msg," (orig. %d)\n",orig);
-          erroutstring(msg);
-        }
-      sprintf(msg," is loop on vertex %d",ord1+1);
+      if ( valid_id(orig) && !equal_element(orig,e_id) )
+      { sprintf(msg," (orig. %s)",ELNAME(orig));
+        erroutstring(msg);
+      }
+      sprintf(msg," is loop on vertex %s",ELNAME(v1));
       erroutstring(msg);
       orig = get_original(get_edge_tailv(e_id));
-      if ( orig && (orig != ord1+1) )
-        { sprintf(msg," (orig. %d)\n",orig);
-          erroutstring(msg);
-        }
+      if ( valid_id(orig) && !equal_element(orig,v1) )
+      { sprintf(msg," (orig. %s)",ELNAME(orig));
+        erroutstring(msg);
+      }
       erroutstring(".\n");
     }
-    if ( ord1 <= ord2 )
+    if ( v1 <= v2 )
       { current->id = e_id;
-        current->vord[0] = ord1;
-        current->vord[1] = ord2;
+        current->v[0] = v1;
+        current->v[1] = v2;
       }
     else
       { current->id = inverse_id(e_id);
-        current->vord[0] = ord2;
-        current->vord[1] = ord1;
+        current->v[0] = v2;
+        current->v[1] = v1;
       }
     if ( ++count > web.skel[EDGE].count )
       { erroutstring("Edge count disagrees with supposed value.");
@@ -677,9 +753,9 @@ int collapse_check()
    { if ( vvvvcomp(vvlist+i,vvlist+i+1) == 0 )
      if ( web.symmetry_flag &&
         (get_edge_wrap(vvlist[i].id) == get_edge_wrap(vvlist[i+1].id)) )
-      { sprintf(msg,"Edges %d and %d have same endpoints: %d %d.\n",
-         ordinal(vvlist[i].id)+1,ordinal(vvlist[i+1].id)+1,vvlist[i].vord[0]+1,
-         vvlist[i].vord[1]+1);
+      { sprintf(msg,"Edges %s and %s have same endpoints: %s %s.\n",
+         ELNAME(vvlist[i].id),ELNAME1(vvlist[i+1].id),
+           ELNAME2(vvlist[i].v[0]), ELNAME3(vvlist[i].v[1]));
          erroutstring(msg);
       }
    }
@@ -697,20 +773,20 @@ int collapse_check()
   { 
     facetedge_id fe;
     fe = get_facet_fe(f_id);
-    ord1 = ordinal(get_fe_tailv(fe));
-    ord2 = ordinal(get_fe_headv(fe));
-    ord3 = ordinal(get_fe_headv(get_next_edge(fe)));
+    v1 = positive_id(get_fe_tailv(fe));
+    v2 = positive_id(get_fe_headv(fe));
+    v3 = positive_id(get_fe_headv(get_next_edge(fe)));
     /* bubble sort */
-    if ( ord1 > ord2 )
-      { tmp = ord1; ord1 = ord2; ord2 = tmp; invert(f_id); }
-    if ( ord2 > ord3 )
-      { tmp = ord2; ord2 = ord3; ord3 = tmp; invert(f_id); }
-    if ( ord1 > ord2 )
-      { tmp = ord1; ord1 = ord2; ord2 = tmp; invert(f_id); }
+    if ( v1 > v2 )
+      { tmp = v1; v1 = v2; v2 = tmp; invert(f_id); }
+    if ( v2 > v3 )
+      { tmp = v2; v2 = v3; v3 = tmp; invert(f_id); }
+    if ( v1 > v2 )
+      { tmp = v1; v1 = v2; v2 = tmp; invert(f_id); }
     current->id = f_id;
-    current->vord[0] = ord1;
-    current->vord[1] = ord2;
-    current->vord[2] = ord3;
+    current->v[0] = v1;
+    current->v[1] = v2;
+    current->v[2] = v3;
     if ( ++count > web.skel[FACET].count )
       { erroutstring("Facet count disagrees with supposed value.");
         count--;
@@ -724,12 +800,16 @@ int collapse_check()
 
   /* scan list for duplicates */
   for ( i = 0 ; i < count-1 ; i++ )
-  { if ( vvvvcomp(vvlist+i,vvlist+i+1) == 0 )
-      { sprintf(msg,"Facets %d and %d have same vertices: %d %d %d.\n",
-        ordinal(vvlist[i].id)+1,ordinal(vvlist[i+1].id)+1,vvlist[i].vord[0]+1,
-        vvlist[i].vord[1]+1,vvlist[i].vord[2]+1);
-        erroutstring(msg);
-      }
+  { if ( (vvvvcomp(vvlist+i,vvlist+i+1) == 0) 
+          && !(get_vattr(vvlist[i].v[0]) & AXIAL_POINT)
+          && !(get_vattr(vvlist[i].v[1]) & AXIAL_POINT)
+          && !(get_vattr(vvlist[i].v[2]) & AXIAL_POINT)
+       )
+    { sprintf(msg,"Facets %s and %s have same vertices: %s %s %s.\n",
+         ELNAME(vvlist[i].id),ELNAME1(vvlist[i+1].id), ELNAME2(vvlist[i].v[0]),
+         ELNAME3(vvlist[i].v[1]),ELNAME4(vvlist[i].v[2]));
+      erroutstring(msg);
+    }
   }
   temp_free((char*)vvlist);
   return numerr;
@@ -771,16 +851,16 @@ REAL normal_change_check()
     for ( ii = 0 ; ii < FACET_EDGES ; ii++, fe = get_next_edge(fe) )
     {
       v_id = get_fe_tailv(fe);
-      ord_v = ordinal(v_id);
+      ord_v = loc_ordinal(v_id);
       if ( get_vattr(v_id) & BOUNDARY )     
       { bdry = get_boundary(v_id);
         for ( i = 0 ; i < SDIM ; i++ )
-          x[n][i] = eval(bdry->coordf[i],saved.coord[ord_v],v_id,NULL);
+          x[ii][i] = eval(bdry->coordf[i],saved.coord[ord_v],v_id,NULL);
       }
       else
       {    
         for ( i = 0 ; i < SDIM ; i++ )
-           x[n][i] = saved.coord[ord_v][i];
+           x[ii][i] = saved.coord[ord_v][i];
       }
     }
     for ( i = 0 ; i < SDIM ; i++ )
@@ -820,29 +900,57 @@ REAL normal_change_check()
 * function: wrap_check()
 *
 * purpose: check wraps around facet make identity, except for axial
-*         point.
+*         point.  And checks torus wraps for validity.
 *
 * return: number of bad facets.
 */
 
 int wrap_check()
-{
+{ edge_id e_id;
   facet_id f_id;
   int numerr = 0;
+  int count = 0;
+  int i;
+
+  if ( web.torus_flag )
+  FOR_ALL_EDGES(e_id)
+  { WRAPTYPE wrap = get_edge_wrap(e_id);
+    for ( i = 0 ; i < SDIM ; i++, wrap >>= TWRAPBITS )
+      switch ( wrap & WRAPMASK  )
+      {
+          case  NEGWRAP :  break;
+          case  0       :  break;
+          case  POSWRAP :  break;
+          default : 
+              sprintf(errmsg,"Big wrap %d on edge %s period %d\n",
+                  WRAPNUM(wrap&WRAPMASK),ELNAME(e_id),i+1);
+              erroutstring(errmsg);
+              numerr++;
+            break;
+      }
+  }
 
   FOR_ALL_FACETS(f_id)
   {
      facetedge_id fe,start_fe;
      int wrap = 0;
      fe = start_fe = get_facet_fe(f_id);
+     count = 0;
      do
      { if ( get_vattr(get_fe_tailv(fe)) & AXIAL_POINT )
        { wrap = 0; break; }
        wrap = (*sym_compose)(wrap,get_fe_wrap(fe));
        fe = get_next_edge(fe);
+       count++;
+       if ( count > 2*web.skel[EDGE].count ) 
+       { sprintf(errmsg,"Facet %s has unclosed edge loop.\n",ELNAME(f_id));
+         erroutstring(errmsg);
+         numerr++;
+         break;
+       }
      } while ( valid_id(fe) && !equal_id(fe,start_fe) );
      if ( valid_id(fe) && (wrap != 0) )
-       { sprintf(errmsg,"Wraps around facet %d not consistent.\n",ordinal(f_id)+1);
+       { sprintf(errmsg,"Wraps around facet %s not consistent.\n",ELNAME(f_id));
          kb_error(2000,errmsg,WARNING);
          numerr++;
        }
