@@ -22,7 +22,7 @@
 
 void recalc()
 { 
-
+  global_timestamp++;  /* so INFO quantities will recalc later */
   if ( need_fe_reorder_flag )
   { /* straighten out facet order around edges */
     if ( web.representation == SOAPFILM )
@@ -38,11 +38,16 @@ void recalc()
     transform_gen_expr(transform_expr);
   recalc_verts();
   update_display();
-  calc_content(Q_FIXED);
-  /* if ( web.torus_flag ) fix_volconst(); */
-  calc_pressure();
-  calc_energy();
-  change_flag = 0;
+  #ifdef MPI_EVOLVER
+  if ( this_machine == 0 )
+  #endif
+  {
+    calc_content(Q_FIXED);
+    /* if ( web.torus_flag ) fix_volconst(); */
+    calc_pressure();
+    calc_energy();
+    change_flag = 0;
+  }
   reset_conj_grad();
   if ( normal_motion_flag ) begin_normal_motion();
 } /* end recalc() */
@@ -67,7 +72,7 @@ void reset_conj_grad()
   if ( conj_grad_flag && ( ribiere_flag ) )
   { int r_attr = find_attribute(VERTEX,RIBIERE_ATTR_NAME);
      if ( r_attr == -1 )
-     { add_attribute(VERTEX,RIBIERE_ATTR_NAME,REAL_ATTR,1,&size,0,NULL);
+     { add_attribute(VERTEX,RIBIERE_ATTR_NAME,REAL_TYPE,1,&size,0,NULL);
        r_attr = find_attribute(VERTEX,RIBIERE_ATTR_NAME);
      }
      FOR_ALL_VERTICES(v_id)
@@ -84,13 +89,18 @@ void reset_conj_grad()
 *  purpose: Handle one command.  Useful for event loopers.
 */
 
-void old_menu(text)
+int old_menu(text)
 char *text;
-{
-  if ( text[0] == '!' ) old_history(text); /* history list */
-  else command(text);
+{ int retval = 0;
+  if ( text[0] == '!' ) 
+   retval = old_history(text); /* history list */
+  else 
+   retval = command(text,ADD_TO_HISTORY);
+
   if ( change_flag )
        recalc();
+
+  return retval;
 }
 
 /*************************************************************
@@ -101,6 +111,8 @@ char *text;
 *
 */
 
+void report_times ARGS((void));
+
 void letter_command(c)
 int c;
 {
@@ -149,9 +161,9 @@ int c;
               { int j;
                 sprintf(msg,"%9s %32s %8s %3d    %3d  ",typenames[i],
                 EXTRAS(i)[k].name,
-                attr_type_name[EXTRAS(i)[k].type],               
+                datatype_name[EXTRAS(i)[k].type],               
                 EXTRAS(i)[k].offset,
-                attr_type_size[EXTRAS(i)[k].type]*EXTRAS(i)[k].datacount);
+                datatype_size[EXTRAS(i)[k].type]*EXTRAS(i)[k].datacount);
                 for ( j = 0 ; j < EXTRAS(i)[k].adim ; j++ )
                   sprintf(msg+strlen(msg),"[%d]",EXTRAS(i)[k].sizes[j]);
                 if ( EXTRAS(i)[k].adim == 0 ) strcat(msg," scalar ");
@@ -205,12 +217,13 @@ int c;
            if ( get_battr(b_id) & PRESSURE )
            {
              pp = get_body_pressure(b_id);
-             sprintf(msg,"Body %d. Current pressure %f.  Enter new: ",
-                         ordinal(b_id)+1,(DOUBLE)pp);
+             sprintf(msg,"Body %s. Current pressure %f.  Enter new: ",
+                         ELNAME(b_id),(DOUBLE)pp);
              prompt(msg,response,sizeof(response));
              if ( logfd ) fprintf(logfd,"%s\n",response);
              if ( const_expr(response,&pp) > 0 )
              { set_body_pressure(b_id,pp);
+               reset_conj_grad();
                if ( everything_quantities_flag )
                { struct gen_quant *q = GEN_QUANT(bptr(b_id)->volquant);
                  if ( pp == 0.0 )
@@ -229,13 +242,14 @@ int c;
           else /* edit volumes */
           { 
             pp = get_body_fixvol(b_id);
-            sprintf(msg,"Body %d. Current target volume %g.  Enter new: ",
-                        ordinal(b_id)+1,(DOUBLE)pp);
+            sprintf(msg,"Body %s. Current target volume %g.  Enter new: ",
+                        ELNAME(b_id),(DOUBLE)pp);
             prompt(msg,response,sizeof(response));
             if ( logfd ) fprintf(logfd,"%s\n",response);
             cmdptr = response;
             if ( const_expr(response,&pp) > 0 )
             { set_body_fixvol(b_id,pp);
+              reset_conj_grad();
               if ( pp == 0.0 )
               { if ( get_attr(b_id) & FIXEDVOL)
                 { unset_attr(b_id,FIXEDVOL);
@@ -395,6 +409,10 @@ int c;
                 report_quantities();
                 break;
 
+          case 'T': /* profiling times */
+                report_times();
+                break;
+
           case 'W': /* homothety toggle */
                 web.homothety = !web.homothety;
                 sprintf(msg,"Homothety adjustment is %s.\n",
@@ -414,7 +432,7 @@ int c;
          case 'g' :  /* one gradient descent iteration */
                  if ( breakflag ) break;
                  iterate();
-                temp_free_all();
+             
                  break;
 
         case 'j' : /* jiggling */
@@ -437,7 +455,8 @@ int c;
                 }
                 for (;;)
                 {
-                  prompt("Enter minimum acute angle desired(h for histogram): ",response,sizeof(response));
+                  prompt("Enter minimum acute angle desired(h for histogram): ",
+                       response,sizeof(response));
                   if ( logfd ) fprintf(logfd,"%s\n",response);
                   if ( tolower(response[0]) == 'h' )
                      skinny_histogram();
@@ -445,8 +464,11 @@ int c;
                 }
                 if ( const_expr(response,&val) > 0 )
                 { 
-                  sprintf(msg,"New edges: %d\n",
-                     web.refine_count = skinny(val));
+                  if ( web.counts_reported & facet_refine_count_bit )
+                     web.facet_refine_count = 0;
+                  sprintf(msg,"Edges refined: %d\n",
+                     web.facet_refine_count += n = skinny(val));
+                  web.counts_reported |= facet_refine_count_bit; 
                   outstring(msg);
                   recalc();
                 }
@@ -464,10 +486,13 @@ int c;
                }
               if ( const_expr(response,&val) > 0 )
                { web.max_len = val;
-                 sprintf(msg,"New edges: %d\n",
-                     web.refine_count = articulate(web.max_len));
+                 if ( web.counts_reported & edge_refine_count_bit )
+                   web.equi_count = 0;
+                 sprintf(msg,"Edges refined: %d\n",
+                     web.edge_refine_count += n = articulate(web.max_len));
                  outstring(msg);
-                 recalc();
+                 web.counts_reported |= edge_refine_count_bit;
+                 if ( n > 0 ) recalc();
                }
               break;
 
@@ -489,7 +514,7 @@ int c;
                   if ( logfd ) fprintf(logfd,"%s\n",response);
                   if ( tolower(response[0]) == 'h' )
                     { if ( web.representation == STRING )
-                        command("histogram(vertex,dihedral)");
+                        command("histogram(vertex,dihedral)",NO_HISTORY);
                       else ridge_histogram();
                     }
                   else break;
@@ -499,15 +524,19 @@ int c;
                     { sprintf(msg,
                         "refine edge ee where max(ee.vertex,dihedral) > %f",
                            (DOUBLE)val);
-                      command(msg); web.notch_count = web.where_count;
+                      command(msg,NO_HISTORY); 
+					  web.notch_count = web.where_count;
                     }
                     else
                     { web.max_angle = val;
                       if ( web.max_angle <= 0.0 ) break;
+                      if ( web.counts_reported & notch_count_bit )
+                        web.equi_count = 0;
                       sprintf(msg,"Number of edges notched: %d\n",
-                      web.notch_count = ridge_notcher(web.max_angle));
+                      web.notch_count += n = ridge_notcher(web.max_angle));
                       outstring(msg);
-                      recalc();
+                      web.counts_reported |= notch_count_bit;
+                      if ( n > 0 ) recalc();
                     }
                   }
                 break;
@@ -519,8 +548,21 @@ int c;
                   break;
                 }
                 if ( web.representation == STRING )
-                   web.vertex_pop_count += n = verpop_str();
-                else web.edge_pop_count += n = edgepop_film();
+                { 
+                  if ( web.counts_reported & vertex_pop_count_bit )
+                    web.vertex_pop_count = 0;
+                  web.vertex_pop_count += n = verpop_str();
+                  sprintf(msg,"Vertices popped: %d\n",web.vertex_pop_count);
+                  web.counts_reported |= vertex_pop_count_bit;
+                }
+                else
+                { 
+                  if ( web.counts_reported & edge_pop_count_bit )
+                    web.edge_pop_count = 0;
+                  web.edge_pop_count = n = edgepop_film();
+                  sprintf(msg,"Edges popped: %d\n",web.edge_pop_count);
+                  web.counts_reported |= edge_pop_count_bit;
+                }
                 if ( n > 0 ) recalc();
                 break;
 
@@ -531,9 +573,14 @@ int c;
                   break;
                 }
                 if ( web.representation == STRING )
-                  web.vertex_pop_count += n = verpop_str();
+                  web.vertex_pop_count = n = verpop_str();
                 else
-                  web.vertex_pop_count += n = popfilm();
+                  web.vertex_pop_count = n = popfilm();
+                if ( web.counts_reported & vertex_pop_count_bit )
+                  web.edge_pop_count = 0;
+                sprintf(msg,"Vertices popped: %d\n",web.vertex_pop_count);
+                outstring(msg);
+                web.counts_reported |= vertex_pop_count_bit;
                 if ( n > 0 ) recalc();
                 break;
 
@@ -559,10 +606,13 @@ int c;
                   web.min_length = val;
                else break;
                if ( web.min_length <= 0.0 ) break;
+               if ( web.counts_reported & edge_delete_count_bit )
+                   web.edge_delete_count = 0;
                sprintf(msg,"Deleted edges: %d\n",
-               web.refine_count = edgeweed(web.min_length));
+                  web.edge_delete_count += n = edgeweed(web.min_length));
                outstring(msg);
-               if ( web.refine_count ) recalc();
+               web.counts_reported |= edge_delete_count_bit;
+               if ( n > 0 ) recalc();
                break;
 
          case 'U' : /* conjugate gradient */
@@ -587,10 +637,13 @@ int c;
                break;
 
          case 'u' : /* equiangulate */
+                if ( web.counts_reported & equi_count_bit )
+                   web.equi_count = 0;
                 sprintf(msg,"Edges switched in equiangulation: %d\n",
-                  web.equi_count = equiangulate() );
+                  web.equi_count += n = equiangulate() );
+                web.counts_reported |= equi_count_bit;
                 outstring(msg);
-                if ( web.equi_count > 0 )
+                if ( n > 0 )
                 { recalc();
                   if ( web.torus_flag )
                     fix_volconst();
@@ -623,10 +676,13 @@ int c;
                   web.min_area = val;
                 else break; 
                 if ( web.min_area <= 0.0 ) break;
+                if ( web.counts_reported & facet_delete_count_bit )
+                   web.facet_delete_count = 0;
                 sprintf(msg,"Skinny triangles weeded: %d\n",
-                 web.delete_count = areaweed(web.min_area));
+                   web.facet_delete_count += n = areaweed(web.min_area));
                 outstring(msg);
-                recalc();
+                web.counts_reported |= facet_delete_count_bit;
+                if ( n > 0 ) recalc();
                 break;
 
          case 'y' : /* torus duplication */
@@ -652,7 +708,7 @@ int c;
                 break;            
                 
          case 'Z' : /* zooming in on vertex */
-            znum = ordinal(web.zoom_v) + 1;
+            znum = loc_ordinal(web.zoom_v) + 1;
             sprintf(msg,"Enter zoom vertex number (%d): ",znum);
             prompt(msg,response,sizeof(response));
             if ( logfd ) fprintf(logfd,"%s\n",response);
@@ -661,7 +717,7 @@ int c;
             { vertex_id v_id;
               int found = 0;
               FOR_ALL_VERTICES(v_id)
-              { if ( znum == (ordinal(v_id)+1) )
+              { if ( znum == (loc_ordinal(v_id)+1) )
                 { web.zoom_v = v_id; found = 1; break; }
               }
               if ( !found )
@@ -700,6 +756,8 @@ int c;
                 break;
 
          case 's' : /* Show image */ 
+                if ( !go_display_flag && !dont_resize_flag )
+                  resize();
                 go_display_flag = 1;
                 do_show();
                 break;
@@ -736,6 +794,10 @@ int c;
                  logfd = NULL;
                  outstring("Command logging OFF.\n");
                }
+               if ( subshell_flag )
+               { exit_flag = 1; 
+                 break;
+               } 
                startup(NULL);
                longjmp(jumpbuf,1); /* kludge, but current command
                                       list just got deallocated */
@@ -775,9 +837,9 @@ void extrapolate()
   for ( m = 0 ; m <= reflevel ; m++ )
   { 
 #ifdef LONGDOUBLE
-    sprintf(msg,"refinement: %3d  energy: %*.*Lg  ",m,DWIDTH,DPREC,extrap_val[m]);
+    sprintf(msg,"refinement: %1d  energy: %*.*Lg  ",m,DWIDTH,DPREC,extrap_val[m]);
 #else
-    sprintf(msg,"refinement: %3d  energy: %20.15f  ",m,extrap_val[m]);
+    sprintf(msg,"refinement: %1d  energy: %19.15f  ",m,extrap_val[m]);
 #endif
     outstring(msg);
     if ( m > 1 ) /* can extrapolate */
@@ -787,7 +849,7 @@ void extrapolate()
 #ifdef LONGDOUBLE
          sprintf(msg,"extrapolation: %*.*Lg\n",DWIDTH,DPREC,ext);
 #else
-         sprintf(msg,"extrapolation: %20.15f\n",ext);
+         sprintf(msg,"extrapolation: %19.15f\n",ext);
 #endif
          outstring(msg);
       }
@@ -862,9 +924,9 @@ void information()
          outstring(msg);
       }
 #ifdef LONGDOUBLE
-    sprintf(msg,"Total %s:    %*.*Lg\n",areaname,DWIDTH,DPREC,web.total_area);
+    sprintf(msg,"Total %s:   %*.*Lg\n",areaname,DWIDTH,DPREC,web.total_area);
 #else
-    sprintf(msg,"Total %s:    %17.15g\n",areaname,web.total_area);
+    sprintf(msg,"Total %s:   %17.15g\n",areaname,web.total_area);
 #endif
     outstring(msg);
     if ( web.conformal_flag )
@@ -1012,92 +1074,98 @@ void information()
 */
 
 void show_volumes()
-{  body_id b_id;
-    int k;
-    struct gen_quant *quan;
-    int headerflag = 0;
+{ body_id b_id;
+  int k;
+  struct gen_quant *quan;
+  int headerflag = 0;
 
-    if ( volume_timestamp < global_timestamp ) calc_content(Q_FIXED|Q_INFO);
-    if ( (web.bodycount == 0) && !gen_quant_count)
-      { outstring("No bodies or quantities.\n");
-        return;
-      }
-    for ( k=0 ; k < gen_quant_count; k++ )
-    { quan = GEN_QUANT(k);
-      if ( (quan->timestamp < graph_timestamp) 
+  if ( (fixed_volume_timestamp < global_timestamp)
+      || ( info_volume_timestamp < global_timestamp) )
+     calc_content(Q_FIXED|Q_INFO|Q_ENERGY);
+  if ( (web.bodycount == 0) && !gen_quant_count)
+  { outstring("No bodies or quantities.\n");
+    return;
+  }
+  for ( k=0 ; k < gen_quant_count; k++ )
+  { quan = GEN_QUANT(k);
+    if ( (quan->flags & DEFAULT_QUANTITY) && !show_all_quantities )
+         continue;
+    if (quan->flags & Q_DELETED ) continue;
+    if ( (quan->timestamp < graph_timestamp) 
                  || (quan->timestamp<web_timestamp) )
-      { outstring("recalculating...\n");
-        calc_quants(Q_INFO|Q_ENERGY|Q_FIXED);
-      }
+    { outstring("recalculating...\n");
+      calc_quants(Q_INFO|Q_ENERGY|Q_FIXED);
     }
-    if ( web.bodycount )
-    {
-      outstring("Body             target volume        actual volume        pressure\n");
-      FOR_ALL_BODIES(b_id)
-      { if ( equal_id(b_id,web.outside_body) ) continue;
+  }
+  if ( web.bodycount )
+  {
+    outstring("Body             target volume        actual volume        pressure\n");
+    FOR_ALL_BODIES(b_id)
+    { if ( equal_id(b_id,web.outside_body) ) continue;
 #ifdef LONGDOUBLE
-        if ( get_battr(b_id) & FIXEDVOL )
-          sprintf(msg,"%3d  %*.*Lg      %*.*Lg    %*.*Lg\n",
-             1+ordinal(b_id),DWIDTH,DPREC,get_body_fixvol(b_id),
+      if ( get_battr(b_id) & FIXEDVOL )
+        sprintf(msg,"%3s  %*.*Lg      %*.*Lg    %*.*Lg\n",
+             ELNAME(b_id),DWIDTH,DPREC,get_body_fixvol(b_id),
             DWIDTH,DPREC, get_body_volume(b_id),DWIDTH,DPREC,get_body_pressure(b_id));
-        else if ( get_battr(b_id) & PRESSURE )
-          sprintf(msg,"%3d         ------------     %*.*Lg    %*.*Lg\n",
-             1+ordinal(b_id),
+      else if ( get_battr(b_id) & PRESSURE )
+        sprintf(msg,"%3s         ------------     %*.*Lg    %*.*Lg\n",
+             ELNAME(b_id),
              DWIDTH,DPREC,get_body_volume(b_id),DWIDTH,DPREC,get_body_pressure(b_id));
-        else /* not a constraint */
-          sprintf(msg,"%3d         ------------     %*.*Lg \n",
-             1+ordinal(b_id), DWIDTH,DPREC,get_body_volume(b_id));
+      else /* not a constraint */
+          sprintf(msg,"%3s         ------------     %*.*Lg \n",
+             ELNAME(b_id), DWIDTH,DPREC,get_body_volume(b_id));
 #else
-        if ( get_battr(b_id) & FIXEDVOL )
-          sprintf(msg,"%3d         %17.15g      %17.15g    %17.15g\n",
-             1+ordinal(b_id),get_body_fixvol(b_id),
+      if ( get_battr(b_id) & FIXEDVOL )
+        sprintf(msg,"%3s         %17.15g      %17.15g    %17.15g\n",
+             ELNAME(b_id),get_body_fixvol(b_id),
              get_body_volume(b_id),get_body_pressure(b_id));
-        else if ( get_battr(b_id) & PRESSURE )
-          sprintf(msg,"%3d            ------------     %17.15g    %17.15g\n",
-             1+ordinal(b_id),
+      else if ( get_battr(b_id) & PRESSURE )
+        sprintf(msg,"%3s            ------------     %17.15g    %17.15g\n",
+             ELNAME(b_id),
              get_body_volume(b_id),get_body_pressure(b_id));
-        else /* not a constraint */
-          sprintf(msg,"%3d            ------------     %17.15g \n",
-             1+ordinal(b_id), get_body_volume(b_id));
+      else /* not a constraint */
+        sprintf(msg,"%3s            ------------     %17.15g \n",
+             ELNAME(b_id), get_body_volume(b_id));
 #endif
-        outstring(msg);
-      }
+      outstring(msg);
     }
-    if (  gen_quant_count )
-      {
-        for ( k=0 ; k < gen_quant_count; k++ )
-        { quan = GEN_QUANT(k);
-          if ( (quan->flags & DEFAULT_QUANTITY) && !show_all_quantities )
-             continue;
-          if ( !headerflag )
-      outstring(
+  }
+  if (  gen_quant_count )
+  {
+     for ( k=0 ; k < gen_quant_count; k++ )
+     { quan = GEN_QUANT(k);
+       if ( (quan->flags & DEFAULT_QUANTITY) && !show_all_quantities )
+         continue;
+       if ( quan->flags & Q_DELETED ) continue;
+       if ( !headerflag )
+         outstring(
   "          Quantity      target value     actual value          pressure\n");
-          headerflag = 1;
+       headerflag = 1;
 #ifdef LONGDOUBLE
-          if ( quan->flags & Q_CONSERVED )
-              sprintf(msg,"%20s    (conserved)      ----------------- %*.*Lg\n",
+       if ( quan->flags & Q_CONSERVED )
+         sprintf(msg,"%20s    (conserved)      ----------------- %*.*Lg\n",
               quan->name, DWIDTH,DPREC,quan->pressure);
-          else if ( quan->flags & Q_FIXED )
-              sprintf(msg,"%20s  %*.*Lg  %*.*Lg  %*.*Lg\n",
+       else if ( quan->flags & Q_FIXED )
+         sprintf(msg,"%20s  %*.*Lg  %*.*Lg  %*.*Lg\n",
               quan->name,DWIDTH,DPREC,quan->target,DWIDTH,DPREC,
               quan->value,DWIDTH,DPREC,quan->pressure);
-          else
-              sprintf(msg,"%20s         ---------  %*.*Lg\n",
-                quan->name,DWIDTH,DPREC,quan->value);
+       else
+         sprintf(msg,"%20s         ---------  %*.*Lg\n",
+               quan->name,DWIDTH,DPREC,quan->value);
 #else
-          if ( quan->flags & Q_CONSERVED )
-              sprintf(msg,"%20s     (conserved)     ----------------   %17.15g\n",
+       if ( quan->flags & Q_CONSERVED )
+         sprintf(msg,"%20s     (conserved)     ----------------   %17.15g\n",
               quan->name,quan->pressure);
-          else if ( quan->flags & Q_FIXED )
-              sprintf(msg,"%20s  %17.15g  %17.15g  %17.15g\n",
+       else if ( quan->flags & Q_FIXED )
+         sprintf(msg,"%20s  %17.15g  %17.15g  %17.15g\n",
               quan->name,quan->target,quan->value,quan->pressure);
-          else
-              sprintf(msg,"%20s         ---------  %17.15g\n",
+       else
+         sprintf(msg,"%20s         ---------  %17.15g\n",
                 quan->name,quan->value);
 #endif
-          outstring(msg);
-        }
-    }
+       outstring(msg);
+     }
+   }
 }  /* end show_volumes() */
 
 /*****************************************************************
@@ -1236,6 +1304,7 @@ void report_quantities()
      if ( (q->timestamp < graph_timestamp) || (q->timestamp<web_timestamp) )
          calc_quants(Q_INFO|Q_ENERGY|Q_FIXED);
      if ( (q->flags & DEFAULT_QUANTITY) && !show_all_quantities ) continue;
+     if ( q->flags & Q_DELETED ) continue;
 
      /* name and value */
 #ifdef LONGDOUBLE
@@ -1284,7 +1353,7 @@ void report_quantities()
      /* instances */
      for ( j = 0 ; j < q->method_count ; j++ )
      { struct method_instance *mi = METH_INSTANCE(q->meth_inst[j]);
-       if ( mi->flags & IMPLICIT_INSTANCE )
+       if ( !verbose_flag && (mi->flags & (IMPLICIT_INSTANCE|Q_DELETED)) )
           continue;  /* don't clutter up things */
 
        /* name and value */
@@ -1304,9 +1373,109 @@ void report_quantities()
       sprintf(msg,"    %31.31s  %#21.15g\n","modulus",mi->modulus);
 #endif
       outstring(msg);
-    
+#ifdef PROFILING_ENABLED
+      /* timings */
+      if ( verbose_flag )
+      { sprintf(msg,"    %31.31s  %#21.15f  %10d calls\n",
+        "value elapsed time, sec",mi->value_elapsed_time,mi->value_call_count);
+        outstring(msg);
+        sprintf(msg,"    %31.31s  %#21.15f  %10d calls\n",
+        "gradient elapsed time, sec",mi->grad_elapsed_time,mi->grad_call_count);
+        outstring(msg);
+        sprintf(msg,"    %31.31s  %#21.15f  %10d calls\n",
+        "hessian elapsed time, sec",mi->hess_elapsed_time,mi->hess_call_count);
+        outstring(msg);
+      }
+#endif
     }
   }
 
 }
+ 
+/**************************************************************************
+*
+* function: report_times()
+*
+* purpose: report profiling times on systems with that enabled.
+*/
+
+void report_times()
+{
+#ifdef PROFILING_ENABLED
+    sprintf(msg,"Method instance element setup time:   %15.10f\n",
+		     PROF_ELAPSED(element_setup));
+    outstring(msg);
+
+    sprintf(msg,"Overall calc_quants time:             %15.10f\n",
+		     PROF_ELAPSED(calc_quants));
+    outstring(msg);
+
+    sprintf(msg,"Overall calc_quant_grads time:        %15.10f\n",
+		     PROF_ELAPSED(calc_quant_grads));
+    outstring(msg);
+
+    sprintf(msg,"Overall calc_quant_hess time:         %15.10f\n",
+		     PROF_ELAPSED(calc_quant_hess));
+    outstring(msg);
+
+    sprintf(msg,"Hessian solution time:                %15.10f\n",
+		     PROF_ELAPSED(hessian_AIJ_setup)
+		   + PROF_ELAPSED(hessian_constraint_setup)
+		   + PROF_ELAPSED(hessian_project_setup)
+		   + PROF_ELAPSED(hessian_factor)
+		   + PROF_ELAPSED(hessian_CHinvC)
+		   + PROF_ELAPSED(hessian_solve)
+		   + PROF_ELAPSED(hessian_mul)
+                     );
+    outstring(msg);
+
+    if ( verbose_flag )
+    {
+      sprintf(msg,"    Hessian AIJ setup time:               %15.10f\n",
+		     PROF_ELAPSED(hessian_AIJ_setup));
+      outstring(msg);
+      sprintf(msg,"    Hessian constraint setup time:        %15.10f\n",
+		     PROF_ELAPSED(hessian_constraint_setup));
+      outstring(msg);
+      sprintf(msg,"    Hessian project setup time:           %15.10f\n",
+		     PROF_ELAPSED(hessian_project_setup));
+      outstring(msg);
+      sprintf(msg,"    Hessian factor time:                  %15.10f\n",
+		     PROF_ELAPSED(hessian_factor));
+      outstring(msg);
+      sprintf(msg,"    Hessian CHinvC time:                  %15.10f\n",
+		     PROF_ELAPSED(hessian_CHinvC));
+      outstring(msg);
+      sprintf(msg,"    Hessian solve time:                   %15.10f\n",
+		     PROF_ELAPSED(hessian_solve));
+      outstring(msg);
+      sprintf(msg,"    Hessian multiplication time:          %15.10f\n",
+		     PROF_ELAPSED(hessian_mul));
+      outstring(msg);
+    }
+
+
+    sprintf(msg,"exparse time:                         %15.10f\n",
+		     PROF_ELAPSED(exparse));
+    outstring(msg);
+
+    sprintf(msg,"yyparse time:                         %15.10f\n",
+		     PROF_ELAPSED(yyparse));
+    outstring(msg);
+
+    sprintf(msg,"yylex+kblex total time:               %15.10f\n",
+		     PROF_ELAPSED(yylex));
+    outstring(msg);
+
+    sprintf(msg,"kblex time:                           %15.10f\n",
+		     PROF_ELAPSED(kblex));
+    outstring(msg);
+
+    sprintf(msg,"Using processor speed of %g Hz\n",cpu_speed);
+    outstring(msg);
+   
+#else
+  outstring("Profiling not enabled on this system.\n");
+#endif
+}
 
