@@ -10,9 +10,6 @@
 *  Purpose:  Overall control of graphics file output.
 */
 
-#ifdef WIN32
-#include <windows.h>
-#endif
 #include "include.h"
 
 
@@ -48,12 +45,13 @@ int arg; /* menu choice; -1 to invoke user dialog */
      outstring("3. PostScript file\n");
      outstring("4. Triangle file \n");
      outstring("5. Softimage file \n");
+     outstring("6. OFF file \n");
 #if defined(OOGL) && !defined(WIN32)
      outstring("8. Start simultaneous geomview  \n");
      outstring("9. End simultaneous geomview  \n");
+#endif
      outstring("A. Start OOGL pipe (you must start reader)  \n");
      outstring("B. End OOGL pipe\n");
-#endif
      outstring("0. cancel  \n");
      prompt("Choice: ",response,sizeof(response));
      if ( response[0] == '3' )
@@ -182,6 +180,7 @@ char *fname;
        graph_end = painter_end;
        need_bounding_box = 1;
        edgewidths = pswidths;
+       graph_capabilities = GC_ARCS;
 
        /* do output */
        graphgen();
@@ -196,6 +195,7 @@ char *fname;
        graph_facet = old_gfacet;
        graph_edge  = old_gedge;
        need_bounding_box = 0;
+       graph_capabilities = 0;
 
        break;
  
@@ -248,13 +248,24 @@ char *fname;
        softimage();
        break;
 
-    case '6': /* interactive Minneview */
-       Begin_OOGL();
-       go_display_flag = 1;  /* default autodisplay */
-       break;
+    case '6': /* OFF format file */
+       old_start = graph_start;
+       old_end    = graph_end;
+       old_gfacet = graph_facet;
+       old_gedge = graph_edge;
+       graph_start = OFF_start;
+       graph_edge = OFF_edge;
+       graph_facet = OFF_facet;
+       graph_end    = OFF_end;
 
-    case '7': /* end MinneView */
-       if ( OOGL_flag ) End_OOGL();
+       /* do output */
+       graphgen();
+
+       /* restore old graphics */
+       graph_start = old_start;
+       graph_end    = old_end;
+       graph_facet = old_gfacet;
+       graph_edge = old_gedge;
        break;
 
     case '8': /* interactive geomview */
@@ -411,10 +422,13 @@ same: ;
 *  Expression syntax:
 *      G is nonterminal, represents set of transforms
 *      G ::=  letter - corresponding generator, a - z, G = {I,a}
+*                       but a! generates only {a}
 *      G ::=  GG      - all ordered products
 *      G ::=  G|G     - union
 *      G ::=  nG      - all ordered n-fold products
 *      G ::=  (G)     - for precedence, same set
+*      Leading ! in string means do not include identity transform, even
+*             if generated by other transforms.
 *          
 *  Sets global variable view_transforms to matrix list allocated with dmatrix3()
 *  Sets global variable transform_count
@@ -465,8 +479,10 @@ char *expr;
     REAL ***old_mats;
     REAL ***id;
     MAT2D(temp_mat,MAXCOORD+1,MAXCOORD+1);
+    int no_identity = 0;
 
-    calc_view_transform_gens();  /* re-evaluate expressions */
+   
+     
 
     if ( expr[0] == 0 ) /* empty transform string */
     { stacktop = 1 ; stack[1].count = 0; 
@@ -477,6 +493,8 @@ char *expr;
     stacktop = 0;
     stack[stacktop].type = G_START;
     c =  expr;
+    if ( *c == '!' ) 
+    { no_identity = 1; c++; }
     while ( *c || (stacktop > 1) )
     { /* test first for reduction */
       if ( stack[stacktop].type == G_SET )
@@ -582,10 +600,17 @@ char *expr;
         }
         /* set up set */
         stack[++stacktop].type = G_SET;
-        stack[stacktop].count = 2;
-        stack[stacktop].mats = dmatrix3(2+1,SDIM+1,SDIM+1);
-        matcopy(stack[stacktop].mats[0],identmat,SDIM+1,SDIM+1);
-        matcopy(stack[stacktop].mats[1],view_transform_gens[n],SDIM+1,SDIM+1);
+        if ( c[1] == '!' ) /* exclude identity */
+        { stack[stacktop].count = 1;
+          stack[stacktop].mats = dmatrix3(1+1,SDIM+1,SDIM+1);
+          matcopy(stack[stacktop].mats[0],view_transform_gens[n],SDIM+1,SDIM+1);
+          c++;
+        } else /* include identity */
+        { stack[stacktop].count = 2;
+          stack[stacktop].mats = dmatrix3(2+1,SDIM+1,SDIM+1);
+          matcopy(stack[stacktop].mats[0],identmat,SDIM+1,SDIM+1);
+          matcopy(stack[stacktop].mats[1],view_transform_gens[n],SDIM+1,SDIM+1);
+        }
 
         if ( transform_gen_swap[n] )
           for ( i = 0 ; i <= SDIM ; i++ )
@@ -621,22 +646,28 @@ char *expr;
   strncpy(transform_expr,expr,sizeof(transform_expr));
 
 skip_stuff:
-  /* put identity at the head of the list */
+  /* put identity at the head of the list, if not excluded */
   id = (REAL***)bsearch((char*)&identmat,(char*)stack[1].mats,stack[1].count,
             sizeof(REAL**), FCAST  matcomp);
-  if ( id && (*id != stack[1].mats[0]) )
-  { REAL **mat = stack[1].mats[0];
-    stack[1].mats[0] = *id;
-    *id = mat;
+  if ( no_identity )
+  { if ( id )
+      *id = stack[1].mats[--stack[1].count];
   }
-  else if ( !id )
-  {
-    for ( i = 0 ; i <= SDIM ; i++ )
-      for ( j = 0 ; j <= SDIM ; j++ )
-      { stack[1].mats[stack[1].count][i][j] = stack[1].mats[0][i][j];
-        stack[1].mats[0][i][j] = ( i==j ) ? 1.0 : 0.0;
-      }
-    stack[1].count++;
+  else
+  { if ( id && (*id != stack[1].mats[0]) )
+    { REAL **mat = stack[1].mats[0];
+      stack[1].mats[0] = *id;
+      *id = mat;
+    }
+    else if ( !id )
+    {
+      for ( i = 0 ; i <= SDIM ; i++ )
+        for ( j = 0 ; j <= SDIM ; j++ )
+        { stack[1].mats[stack[1].count][i][j] = stack[1].mats[0][i][j];
+          stack[1].mats[0][i][j] = ( i==j ) ? 1.0 : 0.0;
+        }
+      stack[1].count++;
+    }
   }
   
   ENTER_GRAPH_MUTEX;
