@@ -60,6 +60,9 @@ element_id q_id; /* reference element, if any */
     switch ( node->type )
     {
 
+      case EXPRLIST_:
+             break;  /* leave expression on stack */
+ 
       case FUNCTION_CALL_:
       { struct eval_frame this_frame;
         REAL fake_stack[20],value;
@@ -67,12 +70,12 @@ element_id q_id; /* reference element, if any */
         this_frame.basenode = &node;  
         this_frame.parent = NULL;
 
-        for ( i = 0 ; i < node->op2.intval ; i++ )
-        fake_stack[i] = stacktop[i-node->op2.intval+1].value;
-        this_frame.stacktop = fake_stack+node->op2.intval-1;
-        value = eval(&globals(node->op1.intval)->value.proc,
+        for ( i = 0 ; i < node->op2.argcount ; i++ )
+          fake_stack[i] = stacktop[i-node->op2.argcount+1].value;
+        this_frame.stacktop = fake_stack+node->op2.argcount-1;
+        value = eval(&globals(node->op1.name_id)->value.proc,
            NULL,NULLID,&this_frame);
-        stacktop -= node->op2.intval; /* pop arguments */
+        stacktop -= node->op2.argcount; /* pop arguments */
         (++stacktop)->value = value;
         FIRST = 0.0;
         break;
@@ -84,7 +87,7 @@ element_id q_id; /* reference element, if any */
       case ARRAY_HEAD_: break;  /* let indices accumulate */
 
       case ARRAYEVAL:
-      { struct array *a = globals(node->op2.intval)->value.arrayptr;
+      { struct array *a = globals(node->op2.name_id)->value.arrayptr;
         REAL value;
         int i,offset;
         void *lvalue;
@@ -92,14 +95,14 @@ element_id q_id; /* reference element, if any */
         { int k = (int)stacktop[i+1-a->dim].value;
           if ( k < 1 )
           { sprintf(errmsg,"Array index %d of array %s is %d. Indexes start at 1.",
-               i+1,globals(node->op2.intval)->name,k);
+               i+1,globals(node->op2.name_id)->name,k);
             sprintf(errmsg+strlen(errmsg),"(source file %s, line %d)\n",
                  file_names[node->file_no],node->line_no);
             kb_error(2533,errmsg,RECOVERABLE);
           }
           if ( k > a->size[i] )
           { sprintf(errmsg,"Array index %d of array %s is %d; exceeds bound of %d.",
-               i+1,globals(node->op2.intval)->name,k,a->size[i]);
+               i+1,globals(node->op2.name_id)->name,k,a->size[i]);
             sprintf(errmsg+strlen(errmsg),"(source file %s, line %d)\n",
                  file_names[node->file_no],node->line_no);
             kb_error(2536,errmsg,RECOVERABLE);
@@ -112,8 +115,24 @@ element_id q_id; /* reference element, if any */
         stacktop -= a->dim;
         lvalue = ((char *)a) + a->datastart + offset*a->itemsize;
         switch ( a->datatype )
-        { case REAL_TYPE_: value = *(REAL*)(lvalue);    break;
-          case INTEGER_TYPE_: value = *(int*)(lvalue);    break;
+        { case REAL_TYPE: value = *(REAL*)(lvalue);    break;
+          case INTEGER_TYPE: value = *(int*)(lvalue);    break;
+          case UINT_TYPE: value = *(unsigned int*)(lvalue);    break;
+          case CHAR_TYPE: value = *(char*)(lvalue);    break;
+          case UCHAR_TYPE: value = *(unsigned char*)(lvalue);    break;
+          case SHORT_TYPE: value = *(short int*)(lvalue);    break;
+          case USHORT_TYPE: value = *(unsigned short int*)(lvalue);    break;
+          case LONG_TYPE: value = *(long*)(lvalue);    break;
+          case ULONG_TYPE: value = *(unsigned long*)(lvalue);    break;
+          case PTR_TYPE: value = (unsigned long)*(char**)(lvalue);    break;
+          case VERTEX_TYPE:
+          case EDGE_TYPE:
+          case FACET_TYPE:
+          case BODY_TYPE:
+          case FACETEDGE_TYPE:
+          case ELEMENTID_TYPE: value = *(element_id*)(lvalue); break;
+          default: value = *(int*)(lvalue); break;
+
         }
         (++stacktop)->value = value;
         FIRST=0.0;
@@ -123,10 +142,10 @@ element_id q_id; /* reference element, if any */
       case BACKQUOTE_START_:
         { struct expnode bqnode = *ex;
           bqnode.start = node;
-          bqnode.root = node+node->op1.intval;
+          bqnode.root = node+node->op1.skipsize;
           bqnode.locals = localbase;
           eval(&bqnode,params,q_id,NULL);
-          node += node->op1.intval-1;
+          node += node->op1.skipsize-1;
         }
         break;
       case BACKQUOTE_END_ : break;  /* just a placeholder */
@@ -134,7 +153,7 @@ element_id q_id; /* reference element, if any */
            break;
 
        case SET_GLOBAL_:
-         { struct global *g = globals(node->op1.intval);
+         { struct global *g = globals(node->op1.name_id);
            g->value.real = stacktop->value;
            if ( g->attr.varstuff.gradhess == NULL )
              g->attr.varstuff.gradhess = (REAL*)mycalloc(MAXCOORD*(MAXCOORD+1), sizeof(REAL));
@@ -161,6 +180,14 @@ element_id q_id; /* reference element, if any */
            FIRST = 0.0;
            break;
 
+       case GET_MPI_TASK_:
+           if ( node->op1.localnum ) 
+              id = *(element_id*)get_localp(node->op1.localnum);
+           else id = q_id;
+           (++stacktop)->value = id_machine(id);
+           FIRST = 0.0;
+           break;
+
        case REPLACECONST:
            for ( i = 0 ; i < pcount ; i++ )
               stacktop->deriv[i] = 0.0;
@@ -170,13 +197,13 @@ element_id q_id; /* reference element, if any */
      case COND_TEST_:
            if ( (stacktop--)->value == 0. )
            { /* jump */
-             node += node->op1.intval;
+             node += node->op1.skipsize;
            }
            break;
 
      case COND_EXPR_:
            /* did first command, so skip second */
-           node += node->op1.intval;
+           node += node->op1.skipsize;
            break;
 
      case COND_ELSE_:
@@ -196,13 +223,13 @@ element_id q_id; /* reference element, if any */
            break;
 
      case TOGGLEVALUE:
-        (++stacktop)->value = (REAL)get_toggle_value(node->op1.intval);
+        (++stacktop)->value = (REAL)get_toggle_value(node->op1.toggle_state);
         FIRST = 0.0;
         break;
 
      case SIZEOF_:
         (++stacktop)->value = (REAL) 
-            EXTRAS(node->op2.intval)[node->op1.intval].datacount;
+            EXTRAS(node->op2.eltype)[node->op1.extranum].datacount;
         FIRST = 0.0;
         break;
 
@@ -222,7 +249,7 @@ element_id q_id; /* reference element, if any */
 
      case PUSHGLOBAL_:
      case PUSHADJUSTABLE:
-           { struct global *g = globals(node->op1.intval);
+           { struct global *g = globals(node->op1.name_id);
              if ( g->flags & FILE_VALUES )
              (++stacktop)->value = g->value.file.values[int_val];
              else if ( g->flags & STRINGVAL )
@@ -237,9 +264,9 @@ element_id q_id; /* reference element, if any */
            break;
 
      case PUSHPARAM:
-        (++stacktop)->value = params[node->op1.intval];
+        (++stacktop)->value = params[node->op1.coordnum];
         for ( i = 0 ; i < pcount ; i++ )
-             if ( i == node->op1.intval )
+             if ( i == node->op1.coordnum )
                 stacktop->deriv[i] = 1.0;
              else stacktop->deriv[i] = 0.0;
         break;
@@ -248,49 +275,49 @@ element_id q_id; /* reference element, if any */
         if ( node->op1.localnum ) 
            id = *(element_id*)get_localp(node->op1.localnum);
         else id = q_id;
-        (++stacktop)->value = get_param(id)[node->op2.intval];
+        (++stacktop)->value = get_param(id)[node->op2.coordnum];
         for ( i = 0 ; i < SDIM ; i++ )
-             if ( i == node->op2.intval )
+             if ( i == node->op2.coordnum )
                 stacktop->deriv[i] = 1.0;
              else stacktop->deriv[i] = 0.0;
         break;
 
 
      case PUSHQPRESSURE_:
-         (++stacktop)->value = GEN_QUANT(node->op1.intval)->pressure;
+         (++stacktop)->value = GEN_QUANT(node->op1.quant_id)->pressure;
          FIRST = 0.0;
          break;
 
      case PUSHQTARGET_:
-          (++stacktop)->value = GEN_QUANT(node->op1.intval)->target;
+          (++stacktop)->value = GEN_QUANT(node->op1.quant_id)->target;
           FIRST = 0.0;
           break;
 
      case PUSHQMODULUS_:
-          (++stacktop)->value = GEN_QUANT(node->op1.intval)->modulus;
+          (++stacktop)->value = GEN_QUANT(node->op1.quant_id)->modulus;
           FIRST = 0.0;
           break;
 
      case PUSHQTOLERANCE_:
-          (++stacktop)->value = GEN_QUANT(node->op1.intval)->tolerance;
+          (++stacktop)->value = GEN_QUANT(node->op1.quant_id)->tolerance;
           FIRST = 0.0;
           break;
 
      case PUSHMMODULUS_:
           (++stacktop)->value
-             = METH_INSTANCE(node->op1.intval)->modulus;
+             = METH_INSTANCE(node->op1.quant_id)->modulus;
           FIRST = 0.0;
           break;
 
      case PUSHQVALUE_:
-          { struct gen_quant *q = GEN_QUANT(node->op1.intval);
+          { struct gen_quant *q = GEN_QUANT(node->op1.quant_id);
             (++stacktop)->value = q->value;
             FIRST = 0.0;
           }
           break;
 
      case PUSHMVALUE_:
-          { struct method_instance *mi = METH_INSTANCE(node->op1.intval);
+          { struct method_instance *mi = METH_INSTANCE(node->op1.meth_id);
             (++stacktop)->value = mi->value;
             if ( (mi->flags & Q_COMPOUND) && (mi->stamp == comp_quant_stamp) )
             { for ( i = 0 ; i < pcount ; i++ )
@@ -302,20 +329,20 @@ element_id q_id; /* reference element, if any */
 
      case PUSHQVOLCONST_:
           (++stacktop)->value
-             = GEN_QUANT(node->op1.intval)->volconst;
+             = GEN_QUANT(node->op1.quant_id)->volconst;
           FIRST = 0.0;
           break;
 
  
      case PUSH_NAMED_QUANTITY:
      case PUSH_METHOD_INSTANCE_:
-          (++stacktop)->value = (REAL)node->op1.intval;
+          (++stacktop)->value = (REAL)node->op1.quant_id;
           FIRST = 0.0;
           break;
 
 
      case GET_INTERNAL_:
-           (++stacktop)->value = get_internal_variable(node->op1.intval);
+           (++stacktop)->value = get_internal_variable(node->op1.name_id);
            FIRST = 0.0;
            break;
   
@@ -326,15 +353,15 @@ element_id q_id; /* reference element, if any */
      case USERFUNC:
            stacktop++;
            stacktop->value =
-             (*userfunc_deriv[node->op1.intval])(params,stacktop->deriv);
+             (*userfunc_deriv[node->op1.userfunc])(params,stacktop->deriv);
            break;
 
      case INDEXED_ELEMENT_:
          { int ord = (int)((stacktop--)->value);
-           element_id id = get_ordinal_id(node->op1.intval,abs(ord)-1);
+           element_id id = get_ordinal_id(node->op1.eltype,abs(ord)-1);
            if ( !valid_id(id) ) 
            { sprintf(errmsg,"%s index %d is not valid.\n",
-                 typenames[node->op1.intval],ord);
+                 typenames[node->op1.eltype],ord);
              sprintf(errmsg+strlen(errmsg),"(source file %s, line %d)\n",
                  file_names[node->file_no],node->line_no);
              kb_error(1289,errmsg,RECOVERABLE);
@@ -386,19 +413,28 @@ element_id q_id; /* reference element, if any */
            FIRST = 0.0;
            break;
 
-     case AND_:
-           stacktop--;
-           stacktop[0].value = (REAL)(stacktop[0].value && stacktop[1].value);
-           FIRST = 0.0;
+     case AND_: /* short-circuit */
+	   if ( stacktop->value == 0.0 )
+	   { (++stacktop)->value = 0.0;
+             FIRST = 0.0;
+             node += node->op1.skipsize;
+	   } 
            break;
 
      case CONJUNCTION_END: 
-           if ( stacktop->value != 0.0 ) stacktop->value = 1.0; break;
+	   /* short-circuiting results in second arg being answer */
+	   stacktop--;
+	   *stacktop = stacktop[1];
+           if ( stacktop->value != 0.0 ) 
+             stacktop->value = 1.0;
+	   break;
 
      case OR_:
-           stacktop--;
-           stacktop[0].value = (REAL)(stacktop[0].value || stacktop[1].value);
-           FIRST = 0.0;
+	   if ( stacktop->value != 0.0 )
+	   { (++stacktop)->value = 1.0;
+             FIRST = 0.0;
+             node += node->op1.skipsize;
+	   } 
            break;
 
      case NOT_:
@@ -485,31 +521,31 @@ element_id q_id; /* reference element, if any */
            break;
 
      case INTPOW:
-         if ( node->op1.intval == 0 )
+         if ( node->op1.intpow == 0 )
            { stacktop->value = 1.0;
              for ( i = 0 ; i < pcount ; i++ )
                 stacktop->deriv[i] = 0.0;
            }
-         else if ( node->op1.intval == 1 )
+         else if ( node->op1.intpow == 1 )
            { /* no action necessary */
            }
          else
            { x = stacktop->value;
-             if ( node->op1.intval > 1 ) /* get n-1 power first */ 
-                for ( n = 1 ; n < node->op1.intval-1 ; n++ )
+             if ( node->op1.intpow > 1 ) /* get n-1 power first */ 
+                for ( n = 1 ; n < node->op1.intpow-1 ; n++ )
                    stacktop->value *= x;
-             else if ( node->op1.intval < 0 ) 
+             else if ( node->op1.intpow < 0 ) 
              { if ( x == 0.0 ) 
-                { sprintf(errmsg,"Negative power (%d) of zero.\n",node->op1.intval);
+                { sprintf(errmsg,"Negative power (%d) of zero.\n",node->op1.intpow);
                    kb_error(1023,errmsg,RECOVERABLE);
                 }
                 stacktop->value = 1/x;
-                for ( n = 0 ; n < -node->op1.intval ; n++ )
+                for ( n = 0 ; n < -node->op1.intpow ; n++ )
                    stacktop->value *= 1/x;
              }
              for ( i = 0 ; i < pcount ; i++ )
                 stacktop->deriv[i] = 
-                    node->op1.intval*stacktop->value*stacktop->deriv[i];
+                    node->op1.intpow*stacktop->value*stacktop->deriv[i];
              stacktop->value *= x;
            }
            break;
@@ -609,6 +645,16 @@ element_id q_id; /* reference element, if any */
              stacktop->deriv[i] /= 2*stacktop->value;
         break;
 
+     case CEIL_:
+          stacktop->value = ceil(stacktop->value);
+          FIRST = 0;
+          break;
+
+     case FLOOR_:
+          stacktop->value = floor(stacktop->value);
+          FIRST = 0;
+          break;
+
      case ABS:
            if ( stacktop->value < 0.0 )
            {
@@ -765,6 +811,14 @@ element_id q_id; /* reference element, if any */
            FIRST = 0.0;
            break;
 
+     case GET_MEANCURV_:
+           if ( node->op1.localnum ) 
+              id = *(element_id*)get_localp(node->op1.localnum);
+           else id = q_id;
+           stacktop->value = vertex_mean_curvature(id);
+           FIRST = 0.0;
+           break;
+
      case LENGTH_:
      case GET_LENGTH_:
            if ( node->op1.localnum ) 
@@ -873,7 +927,7 @@ element_id q_id; /* reference element, if any */
               id = *(element_id*)get_localp(node->op1.localnum);
            else id = q_id;
            ++stacktop;
-           stacktop->value = quantity_attribute(id,node->op2.intval);
+           stacktop->value = quantity_attribute(id,node->op2.quant_id);
            FIRST = 0.0;
            break;
 
@@ -882,7 +936,7 @@ element_id q_id; /* reference element, if any */
               id = *(element_id*)get_localp(node->op1.localnum);
            else id = q_id;
            ++stacktop;
-           stacktop->value = instance_attribute(id,node->op2.intval);
+           stacktop->value = instance_attribute(id,node->op2.meth_id);
            FIRST = 0.0;
            break;
 
@@ -958,7 +1012,7 @@ element_id q_id; /* reference element, if any */
               id = *(element_id*)get_localp(node->op1.localnum);
            else id = q_id;
            ++stacktop;
-           stacktop->value = (REAL)get_original(id);
+           stacktop->value = valid_id(id) ? (REAL)ordinal(get_original(id))+1:0;
            FIRST = 0.0;
            break;
 
@@ -1060,11 +1114,11 @@ element_id q_id; /* reference element, if any */
            int spot,k;
           
          
-         n = node->op3.intval[0];
+         n = node->op3.extranum;
          if ( node->op1.localnum ) 
             id = *(element_id*)get_localp(node->op1.localnum);
          else id = q_id;
-         ext = EXTRAS(node->op2.intval) + n;
+         ext = EXTRAS(node->op2.eltype) + n;
          /* get index */
          spot = 0;
          for ( k = 0 ; k < ext->adim ; k++ )
@@ -1079,15 +1133,15 @@ element_id q_id; /* reference element, if any */
            spot += (int)(stacktop[-ext->adim+k+1].value) - 1;
          }
          stacktop -= ext->adim;
-         if ( id_type(id) != node->op2.intval )
-         { if ( (id_type(id)==EDGE) && (node->op2.intval==VERTEX) && params )
+         if ( id_type(id) != node->op2.eltype )
+         { if ( (id_type(id)==EDGE) && (node->op2.eltype==VERTEX) && params )
            { ext = EXTRAS(VERTEX) + n;
              (++stacktop)->value = interp_edge_attribute(id,ext,spot,(int)params[2*SDIM]);
              FIRST = 0.0;
              break;
            }
            else 
-           if ( (id_type(id)==FACET) && (node->op2.intval==VERTEX) && params )
+           if ( (id_type(id)==FACET) && (node->op2.eltype==VERTEX) && params )
            { ext = EXTRAS(VERTEX) + n;
              (++stacktop)->value = interp_facet_attribute(id,ext,spot,(int)params[2*SDIM]);
              FIRST = 0.0;
@@ -1096,8 +1150,8 @@ element_id q_id; /* reference element, if any */
            else 
            { sprintf(errmsg,
                "Attribute %s is %s attribute, not %s attribute (in %s).\n",
-               EXTRAS(node->op2.intval)[n].name,
-                 typenames[node->op2.intval], typenames[id_type(id)],ex->name);
+               EXTRAS(node->op2.eltype)[n].name,
+                 typenames[node->op2.eltype], typenames[id_type(id)],ex->name);
              kb_error(2537,errmsg,RECOVERABLE);
            }
          }
@@ -1108,17 +1162,49 @@ element_id q_id; /* reference element, if any */
            autorecalc_flag = oldflag;
           }
          switch ( ext->type )
-         { case REAL_ATTR: (++stacktop)->value = ((REAL*)get_extra(id,n))[spot]; 
+         { case REAL_TYPE: 
+            (++stacktop)->value = ((REAL*)get_extra(id,n))[spot]; 
+            break;
+           case INTEGER_TYPE: 
+           case CONSTRAINT_TYPE: 
+           case BOUNDARY_TYPE: 
+           case QUANTITY_TYPE: 
+           case INSTANCE_TYPE: 
+           case PROCEDURE_TYPE: 
+            (++stacktop)->value = (REAL)((int*)get_extra(id,n))[spot];
+            break;
+           case UINT_TYPE: 
+            (++stacktop)->value = (REAL)((unsigned int*)get_extra(id,n))[spot];
+            break;
+           case USHORT_TYPE: 
+            (++stacktop)->value = (REAL)((unsigned short*)get_extra(id,n))[spot];
+            break;
+           case SHORT_TYPE: 
+            (++stacktop)->value = (REAL)((short*)get_extra(id,n))[spot];
+            break;
+           case ULONG_TYPE: 
+            (++stacktop)->value = (REAL)((unsigned long*)get_extra(id,n))[spot];
+            break;
+           case LONG_TYPE: 
+            (++stacktop)->value = (REAL)((long*)get_extra(id,n))[spot];
+            break;
+           case UCHAR_TYPE: 
+            (++stacktop)->value = (REAL)((unsigned char*)get_extra(id,n))[spot];
               break;
-           case INTEGER_ATTR: 
-             (++stacktop)->value = (REAL)((int*)get_extra(id,n))[spot];
+           case CHAR_TYPE: 
+            (++stacktop)->value = (REAL)((char*)get_extra(id,n))[spot];
               break;
-           case ULONG_ATTR: 
-             (++stacktop)->value = (REAL)((unsigned long*)get_extra(id,n))[spot];
-              break;
-           case UCHAR_ATTR: 
-             (++stacktop)->value = (REAL)((unsigned char*)get_extra(id,n))[spot];
+           case PTR_TYPE: 
+            (++stacktop)->value = (REAL)(unsigned long int)((char**)get_extra(id,n))[spot];
               break;
+           case ELEMENTID_TYPE: 
+           case VERTEX_TYPE: 
+           case EDGE_TYPE: 
+           case FACET_TYPE: 
+           case BODY_TYPE: 
+           case FACETEDGE_TYPE: 
+            (++stacktop)->value = (REAL)((element_id*)get_extra(id,n))[spot];
+            break;
          }
        }
            FIRST = 0.0;
@@ -1203,3 +1289,6 @@ element_id q_id; /* reference element, if any */
 } /* end eval_all() */
 
 
+
+
+
