@@ -80,6 +80,8 @@ element_id q_id;    /* reference element, if any */
    {
      switch ( node->type )
      {
+       case EXPRLIST_:
+             break;  /* leave expression on stack */
 
       case FUNCTION_CALL_:
       { struct eval_frame this_frame;
@@ -88,12 +90,12 @@ element_id q_id;    /* reference element, if any */
         this_frame.basenode = &node;  
         this_frame.parent = NULL;
 
-        for ( i = 0 ; i < node->op2.intval ; i++ )
-        fake_stack[i] = stacktop[i-node->op2.intval+1].value;
-        this_frame.stacktop = fake_stack+node->op2.intval-1;
-        value = eval(&globals(node->op1.intval)->value.proc,
+        for ( i = 0 ; i < node->op2.argcount ; i++ )
+          fake_stack[i] = stacktop[i-node->op2.argcount+1].value;
+        this_frame.stacktop = fake_stack+node->op2.argcount-1;
+        value = eval(&globals(node->op1.name_id)->value.proc,
            NULL,NULLID,&this_frame);
-        stacktop -= node->op2.intval; /* pop arguments */
+        stacktop -= node->op2.argcount; /* pop arguments */
         (++stacktop)->value = value;
         FIRST = 0.0;
         SECOND = 0.0;
@@ -105,22 +107,22 @@ element_id q_id;    /* reference element, if any */
       case ARRAY_HEAD_: break;  /* let indices accumulate */
 
       case ARRAYEVAL:
-      { struct array *a = globals(node->op2.intval)->value.arrayptr; 
-        REAL value;
+      { struct array *a = globals(node->op2.name_id)->value.arrayptr; 
+        REAL value=0.0;
         int i,offset;
         void *lvalue;
         for ( i = 0 ; i < a->dim ; i++ )
         { int k = (int)stacktop[i+1-a->dim].value;
           if ( k < 1 )
           { sprintf(errmsg,"Array index %d of array %s is %d. Indexes start at 1.",
-               i+1,globals(node->op2.intval)->name,k);
+               i+1,globals(node->op2.name_id)->name,k);
             sprintf(errmsg+strlen(errmsg),"(source file %s, line %d)\n",
                  file_names[node->file_no],node->line_no);
             kb_error(2534,errmsg,RECOVERABLE);
           }
           if ( k > a->size[i] )
           { sprintf(errmsg,"Array index %d of array %s is %d; exceeds bound of %d.",
-               i+1,globals(node->op2.intval)->name,k,a->size[i]);
+               i+1,globals(node->op2.name_id)->name,k,a->size[i]);
             sprintf(errmsg+strlen(errmsg),"(source file %s, line %d)\n",
                  file_names[node->file_no],node->line_no);
             kb_error(2535,errmsg,RECOVERABLE);
@@ -133,8 +135,24 @@ element_id q_id;    /* reference element, if any */
         stacktop -= a->dim;
         lvalue = ((char *)a) + a->datastart + offset*a->itemsize;
         switch ( a->datatype )
-        { case REAL_TYPE_: value = *(REAL*)(lvalue);    break;
-          case INTEGER_TYPE_: value = *(int*)(lvalue);    break;
+        { case REAL_TYPE: value = *(REAL*)(lvalue);    break;
+          case INTEGER_TYPE: value = *(int*)(lvalue);    break;
+          case UINT_TYPE: value = *(unsigned int*)(lvalue);    break;
+          case CHAR_TYPE: value = *(char*)(lvalue);    break;
+          case UCHAR_TYPE: value = *(unsigned char*)(lvalue);    break;
+          case SHORT_TYPE: value = *(short int*)(lvalue);    break;
+          case USHORT_TYPE: value = *(unsigned short int*)(lvalue);    break;
+          case LONG_TYPE: value = *(long*)(lvalue);    break;
+          case ULONG_TYPE: value = *(unsigned long*)(lvalue);    break;
+          case PTR_TYPE: value = (unsigned long)*(char**)(lvalue);    break;
+          case VERTEX_TYPE:
+          case EDGE_TYPE:
+          case FACET_TYPE:
+          case BODY_TYPE:
+          case FACETEDGE_TYPE:
+          case ELEMENTID_TYPE: value = *(element_id*)(lvalue); break;
+          default: value = *(int*)(lvalue); break;
+        
         }
         (++stacktop)->value = value;
         FIRST=0.0;
@@ -145,10 +163,10 @@ element_id q_id;    /* reference element, if any */
       case BACKQUOTE_START_:
         { struct expnode bqnode = *ex;
           bqnode.start = node;
-          bqnode.root = node+node->op1.intval;
+          bqnode.root = node+node->op1.skipsize;
           bqnode.locals = localbase;
           eval(&bqnode,params,q_id,NULL);
-          node += node->op1.intval-1;
+          node += node->op1.skipsize-1;
         }
         break;
       case BACKQUOTE_END_ : break;  /* just a placeholder */
@@ -156,7 +174,7 @@ element_id q_id;    /* reference element, if any */
            break;
 
        case SET_GLOBAL_:
-         { struct global *g = globals(node->op1.intval);
+         { struct global *g = globals(node->op1.name_id);
            g->value.real = stacktop->value;
            if ( g->attr.varstuff.gradhess == NULL )
               g->attr.varstuff.gradhess = (REAL*)mycalloc(MAXCOORD*(MAXCOORD+1), sizeof(REAL));
@@ -177,13 +195,13 @@ element_id q_id;    /* reference element, if any */
        case COND_TEST_:
           if ( (stacktop--)->value == 0. )
              { /* jump */
-               node += node->op1.intval;
+               node += node->op1.skipsize;
              }
           break;
 
        case COND_EXPR_:
           /* did first command, so skip second */
-          node += node->op1.intval;
+          node += node->op1.skipsize;
           break;
 
        case COND_ELSE_:
@@ -202,13 +220,13 @@ element_id q_id;    /* reference element, if any */
              break;
 
        case TOGGLEVALUE:
-          (++stacktop)->value = (REAL)get_toggle_value(node->op1.intval);
+          (++stacktop)->value = (REAL)get_toggle_value(node->op1.toggle_state);
           zero_seconds(pcount,stacktop);
           break;
 
        case SIZEOF_:
           (++stacktop)->value = 
-              (REAL)EXTRAS(node->op2.intval)[node->op1.intval].datacount;
+              (REAL)EXTRAS(node->op2.eltype)[node->op1.extranum].datacount;
           zero_seconds(pcount,stacktop);
           break;
 
@@ -226,7 +244,7 @@ element_id q_id;    /* reference element, if any */
 
        case PUSHGLOBAL_:
        case PUSHADJUSTABLE:
-             { struct global *g = globals(node->op1.intval);
+             { struct global *g = globals(node->op1.name_id);
                if ( g->flags & FILE_VALUES )
                (++stacktop)->value = g->value.file.values[int_val];
                else if ( g->flags & STRINGVAL )
@@ -245,9 +263,9 @@ element_id q_id;    /* reference element, if any */
              break;
 
        case PUSHPARAM:
-          (++stacktop)->value = params[node->op1.intval];
+          (++stacktop)->value = params[node->op1.coordnum];
           for ( i = 0 ; i < pcount ; i++ )
-             if ( i == node->op1.intval )
+             if ( i == node->op1.coordnum )
                 stacktop->deriv[i] = 1.0;
              else stacktop->deriv[i] = 0.0;
           SECOND = 0.0;
@@ -257,41 +275,41 @@ element_id q_id;    /* reference element, if any */
         if ( node->op1.localnum ) 
            id = *(element_id*)get_localp(node->op1.localnum);
         else id = q_id;
-        (++stacktop)->value = get_param(id)[node->op2.intval];
+        (++stacktop)->value = get_param(id)[node->op2.coordnum];
         for ( i = 0 ; i < SDIM ; i++ )
-             if ( i == node->op2.intval )
+             if ( i == node->op2.coordnum )
                 stacktop->deriv[i] = 1.0;
              else stacktop->deriv[i] = 0.0;
         SECOND = 0;
         break;
 
       case PUSHQPRESSURE_:
-           (++stacktop)->value = GEN_QUANT(node->op1.intval)->pressure;
+           (++stacktop)->value = GEN_QUANT(node->op1.quant_id)->pressure;
            zero_seconds(pcount,stacktop);
            break;
 
        case PUSHQTARGET_:
-           (++stacktop)->value = GEN_QUANT(node->op1.intval)->target;
+           (++stacktop)->value = GEN_QUANT(node->op1.quant_id)->target;
            zero_seconds(pcount,stacktop);
            break;
 
        case PUSHQMODULUS_:
-           (++stacktop)->value = GEN_QUANT(node->op1.intval)->modulus;
+           (++stacktop)->value = GEN_QUANT(node->op1.quant_id)->modulus;
            zero_seconds(pcount,stacktop);
            break;
 
        case PUSHQTOLERANCE_:
-           (++stacktop)->value = GEN_QUANT(node->op1.intval)->tolerance;
+           (++stacktop)->value = GEN_QUANT(node->op1.quant_id)->tolerance;
            zero_seconds(pcount,stacktop);
            break;
 
        case PUSHMMODULUS_:
-           (++stacktop)->value = METH_INSTANCE(node->op1.intval)->modulus;
+           (++stacktop)->value = METH_INSTANCE(node->op1.meth_id)->modulus;
            zero_seconds(pcount,stacktop);
            break;
 
        case PUSHQVALUE_:
-         { struct gen_quant *q = GEN_QUANT(node->op1.intval);
+         { struct gen_quant *q = GEN_QUANT(node->op1.quant_id);
            if ((q->timestamp<graph_timestamp) || (q->timestamp<web_timestamp))
                       calc_quants(q->flags&(Q_INFO|Q_ENERGY|Q_FIXED));
            (++stacktop)->value = q->value;
@@ -300,7 +318,7 @@ element_id q_id;    /* reference element, if any */
          break;
 
        case PUSHMVALUE_:
-         { struct method_instance *mi = METH_INSTANCE(node->op1.intval);
+         { struct method_instance *mi = METH_INSTANCE(node->op1.meth_id);
            if ( mi->flags & Q_COMPOUND )
            { if ( compound_hess_flag == CH_GRADS )
              { volgrad *vgptri = get_vertex_vgrad(comp_quant_vi);
@@ -337,20 +355,20 @@ element_id q_id;    /* reference element, if any */
          break;
 
        case PUSHQVOLCONST_:
-           (++stacktop)->value = GEN_QUANT(node->op1.intval)->volconst;
+           (++stacktop)->value = GEN_QUANT(node->op1.quant_id)->volconst;
            zero_seconds(pcount,stacktop);
            break;
 
 
        case PUSH_NAMED_QUANTITY:
        case PUSH_METHOD_INSTANCE_:
-             (++stacktop)->value = (REAL)node->op1.intval;
+             (++stacktop)->value = (REAL)node->op1.quant_id;
              zero_seconds(pcount,stacktop);
              break;
 
 
        case GET_INTERNAL_:
-             (++stacktop)->value = get_internal_variable(node->op1.intval);
+             (++stacktop)->value = get_internal_variable(node->op1.name_id);
              zero_seconds(pcount,stacktop);
              break;
 
@@ -362,17 +380,17 @@ element_id q_id;    /* reference element, if any */
           { MAT2D(tmp,MAXCOORD,MAXCOORD);
             stacktop++;
             stacktop->value =
-              (*userfunc_seconds[node->op1.intval])(params,stacktop->deriv,tmp);
+              (*userfunc_seconds[node->op1.userfunc])(params,stacktop->deriv,tmp);
             SECOND = tmp[i][j];
           }
           break;
 
          case INDEXED_ELEMENT_:
            { int ord = (int)((stacktop--)->value);
-             id = get_ordinal_id(node->op1.intval,abs(ord)-1);
+             id = get_ordinal_id(node->op1.eltype,abs(ord)-1);
              if ( !valid_id(id) ) 
              { sprintf(errmsg,"%s index %d is not valid.\n",
-                   typenames[node->op1.intval],ord);
+                   typenames[node->op1.eltype],ord);
                sprintf(errmsg+strlen(errmsg),"(source file %s, line %d)\n",
                  file_names[node->file_no],node->line_no);
                kb_error(1201,errmsg,RECOVERABLE);
@@ -423,20 +441,29 @@ element_id q_id;    /* reference element, if any */
              zero_seconds(pcount,stacktop);
              break;
 
-       case CONJUNCTION_END:
-             if ( stacktop->value != 0.0 ) stacktop->value = 1.0; break;
-
-       case AND_:
-             stacktop--;
-             stacktop[0].value = (REAL)(stacktop[0].value && stacktop[1].value);
+     case AND_: /* short-circuit */
+	   if ( stacktop->value == 0.0 )
+	   { (++stacktop)->value = 0.0;
              zero_seconds(pcount,stacktop);
-             break;
+             node += node->op1.skipsize;
+	   } 
+           break;
 
-       case OR_:
-             stacktop--;
-             stacktop[0].value = (REAL)(stacktop[0].value || stacktop[1].value);
+     case CONJUNCTION_END: 
+	   /* short-circuiting results in second arg being answer */
+	   stacktop--;
+	   *stacktop = stacktop[1];
+           if ( stacktop->value != 0.0 ) 
+             stacktop->value = 1.0;
+	   break;
+
+     case OR_:  /* short-circuit */
+	   if ( stacktop->value != 0.0 )
+	   { (++stacktop)->value = 1.0;
              zero_seconds(pcount,stacktop);
-             break;
+             node += node->op1.skipsize;
+	   } 
+           break;
 
        case NOT_:
              stacktop[0].value = (REAL)(!stacktop[0].value);
@@ -533,35 +560,35 @@ element_id q_id;    /* reference element, if any */
              break;
 
        case INTPOW:
-           if ( node->op1.intval == 0 )
+           if ( node->op1.intpow == 0 )
               { stacktop->value = 1.;
                  zero_seconds(pcount,stacktop);
               }
-           else if ( node->op1.intval == 1 )
+           else if ( node->op1.intpow == 1 )
               { /* no action necessary */
               }
            else
              { REAL f1,f2=1.0;
                x = stacktop->value;
-               if ( node->op1.intval > 1 ) /* get n-2 power first */ 
-                  for ( n = 1 ; n < node->op1.intval-1 ; n++ )
+               if ( node->op1.intpow > 1 ) /* get n-2 power first */ 
+                  for ( n = 1 ; n < node->op1.intpow-1 ; n++ )
                      f2 *= x;
-               else if ( node->op1.intval < 0 ) 
+               else if ( node->op1.intpow < 0 ) 
                { if ( x == 0.0 ) 
                  { sprintf(errmsg,"Negative power zero.\n");
                    sprintf(errmsg+strlen(errmsg),"(source file %s, line %d)\n",
                       file_names[node->file_no],node->line_no);
                    kb_error(1015,errmsg,RECOVERABLE);
                  }
-                 for ( f2 = 1/x, n = 0 ; n <= -node->op1.intval ; n++ )
+                 for ( f2 = 1/x, n = 0 ; n <= -node->op1.intpow ; n++ )
                      f2 /= x;
                }
                f1 = f2*x;
                stacktop->value = f1*x;
-               SECOND = node->op1.intval*(node->op1.intval-1)*f2
+               SECOND = node->op1.intpow*(node->op1.intpow-1)*f2
                          *stacktop->deriv[i]*stacktop->deriv[j] +
-                      node->op1.intval*f1*stacktop->second[i][j];
-               FIRST = node->op1.intval*f1*stacktop->deriv[i];
+                      node->op1.intpow*f1*stacktop->second[i][j];
+               FIRST = node->op1.intpow*f1*stacktop->deriv[i];
              }
            break;
 
@@ -645,6 +672,18 @@ element_id q_id;    /* reference element, if any */
              stacktop->value = x;
              break;
 
+     case CEIL_:
+          stacktop->value = ceil(stacktop->value);
+          FIRST = 0;
+          SECOND = 0;
+          break;
+
+     case FLOOR_:
+          stacktop->value = floor(stacktop->value);
+          FIRST = 0;
+          SECOND = 0;
+          break;
+
        case ABS:
              if ( stacktop->value < 0.0 )
              {
@@ -888,6 +927,14 @@ element_id q_id;    /* reference element, if any */
              zero_seconds(pcount,stacktop);
              break;
 
+       case GET_MEANCURV_:
+             if ( node->op1.localnum ) 
+                id = *(element_id*)get_localp(node->op1.localnum);
+             else id = q_id;
+             stacktop->value = vertex_mean_curvature(id);
+             zero_seconds(pcount,stacktop);
+             break;
+
        case LENGTH_:
        case GET_LENGTH_:
              if ( node->op1.localnum ) 
@@ -1000,7 +1047,7 @@ element_id q_id;    /* reference element, if any */
                 id = *(element_id*)get_localp(node->op1.localnum);
              else id = q_id;
              ++stacktop;
-             stacktop->value = quantity_attribute(id,node->op2.intval);
+             stacktop->value = quantity_attribute(id,node->op2.quant_id);
              zero_seconds(pcount,stacktop);
              break;
 
@@ -1009,7 +1056,7 @@ element_id q_id;    /* reference element, if any */
                 id = *(element_id*)get_localp(node->op1.localnum);
              else id = q_id;
              ++stacktop;
-             stacktop->value = instance_attribute(id,node->op2.intval);
+             stacktop->value = instance_attribute(id,node->op2.meth_id);
              zero_seconds(pcount,stacktop);
              break;
 
@@ -1087,6 +1134,13 @@ element_id q_id;    /* reference element, if any */
              zero_seconds(pcount,stacktop);
              break;
 
+          case GET_MPI_TASK_:
+             if ( node->op1.localnum ) 
+                id = *(element_id*)get_localp(node->op1.localnum);
+             else id = q_id;
+             (++stacktop)->value = id_machine(id);
+             break;
+
           case ID_:
           case GET_ID_:
           case GET_OID_:
@@ -1106,7 +1160,7 @@ element_id q_id;    /* reference element, if any */
                 id = *(element_id*)get_localp(node->op1.localnum);
              else id = q_id;
              ++stacktop;
-             stacktop->value = (REAL)get_original(id);
+             stacktop->value = valid_id(id)?(REAL)ordinal(get_original(id))+1:0;
              zero_seconds(pcount,stacktop);
              break;
 
@@ -1208,11 +1262,11 @@ element_id q_id;    /* reference element, if any */
            int spot,k;
           
          
-         n = node->op3.intval[0];
+         n = node->op3.extranum;
          if ( node->op1.localnum ) 
             id = *(element_id*)get_localp(node->op1.localnum);
          else id = q_id;
-         ext = EXTRAS(node->op2.intval) + n;
+         ext = EXTRAS(node->op2.eltype) + n;
          /* get index */
          spot = 0;
          for ( k = 0 ; k < ext->adim ; k++ )
@@ -1229,15 +1283,15 @@ element_id q_id;    /* reference element, if any */
            spot += (int)(stacktop[-ext->adim+k+1].value) - 1;
          }
          stacktop -= ext->adim;
-         if ( id_type(id) != node->op2.intval )
-         { if ( (id_type(id)==EDGE) && (node->op2.intval==VERTEX) && params )
+         if ( id_type(id) != node->op2.eltype )
+         { if ( (id_type(id)==EDGE) && (node->op2.eltype==VERTEX) && params )
            { ext = EXTRAS(VERTEX) + n;
              (++stacktop)->value = interp_edge_attribute(id,ext,spot,(int)params[2*SDIM]);
              FIRST = 0.0; zero_seconds(pcount,stacktop);
              break;
            }
            else 
-           if ( (id_type(id)==FACET) && (node->op2.intval==VERTEX) && params )
+           if ( (id_type(id)==FACET) && (node->op2.eltype==VERTEX) && params )
            { ext = EXTRAS(VERTEX) + n;
              (++stacktop)->value = interp_facet_attribute(id,ext,spot,(int)params[2*SDIM]);
              FIRST = 0.0; zero_seconds(pcount,stacktop);
@@ -1246,8 +1300,8 @@ element_id q_id;    /* reference element, if any */
            else 
            { sprintf(errmsg,
                "Attribute %s is %s attribute, not %s attribute (in %s).\n",
-               EXTRAS(node->op2.intval)[n].name,
-                 typenames[node->op2.intval], typenames[id_type(id)],ex->name);
+               EXTRAS(node->op2.eltype)[n].name,
+                 typenames[node->op2.eltype], typenames[id_type(id)],ex->name);
              sprintf(errmsg+strlen(errmsg),"(source file %s, line %d)\n",
                  file_names[node->file_no],node->line_no);
              kb_error(2538,errmsg,RECOVERABLE);
@@ -1260,17 +1314,46 @@ element_id q_id;    /* reference element, if any */
            autorecalc_flag = oldflag;
           }
          switch ( ext->type )
-         { case REAL_ATTR: (++stacktop)->value = ((REAL*)get_extra(id,n))[spot]; 
+         { case REAL_TYPE: 
+            (++stacktop)->value = ((REAL*)get_extra(id,n))[spot]; 
+            break;
+           case INTEGER_TYPE: 
+           case CONSTRAINT_TYPE: 
+           case BOUNDARY_TYPE: 
+           case QUANTITY_TYPE: 
+           case INSTANCE_TYPE: 
+           case PROCEDURE_TYPE: 
+            (++stacktop)->value = (REAL)((int*)get_extra(id,n))[spot];
+            break;
+           case UINT_TYPE: 
+            (++stacktop)->value = (REAL)((unsigned int*)get_extra(id,n))[spot];
+            break;
+           case USHORT_TYPE: 
+            (++stacktop)->value = (REAL)((unsigned short*)get_extra(id,n))[spot];
+            break;
+           case SHORT_TYPE: 
+            (++stacktop)->value = (REAL)((short*)get_extra(id,n))[spot];
+            break;
+           case ULONG_TYPE: 
+            (++stacktop)->value = (REAL)((unsigned long*)get_extra(id,n))[spot];
+            break;
+           case LONG_TYPE: 
+            (++stacktop)->value = (REAL)((long*)get_extra(id,n))[spot];
+            break;
+           case UCHAR_TYPE: 
+            (++stacktop)->value = (REAL)((unsigned char*)get_extra(id,n))[spot];
               break;
-           case INTEGER_ATTR: 
-             (++stacktop)->value = (REAL)((int*)get_extra(id,n))[spot];
-              break;
-           case ULONG_ATTR: 
-             (++stacktop)->value = (REAL)((unsigned long*)get_extra(id,n))[spot];
-              break;
-           case UCHAR_ATTR: 
-             (++stacktop)->value = (REAL)((unsigned char*)get_extra(id,n))[spot];
+           case CHAR_TYPE: 
+            (++stacktop)->value = (REAL)((char*)get_extra(id,n))[spot];
               break;
+           case ELEMENTID_TYPE: 
+           case VERTEX_TYPE: 
+           case EDGE_TYPE: 
+           case FACET_TYPE: 
+           case BODY_TYPE: 
+           case FACETEDGE_TYPE: 
+            (++stacktop)->value = (REAL)((element_id*)get_extra(id,n))[spot];
+            break;
          }
        }
            FIRST = 0.0; zero_seconds(pcount,stacktop);
@@ -1362,3 +1445,6 @@ element_id q_id;    /* reference element, if any */
   return;
 } /* end eval_second() */
 
+
+
+
