@@ -11,6 +11,14 @@
 */
 
 /* node types, numbered above 1512 to avoid yacc token numbers */
+#define GEN_ERROR_TOKEN 10000
+
+#define V_WINDOW_ASPECT_RATIO 2277
+#define PRINT_SINGLE_LETTER_ 2276
+#define PDELTA_LVALUE_   2275
+#define PSCALE_LVALUE_   2274
+
+
 #define INIT_VERTEX_EDGE_  1513
 #define INIT_VERTEX_FACET_  1514
 #define INIT_VERTEX_BODY_ 1515
@@ -67,7 +75,6 @@
 #define SET_COORD_8      1566
 #define SET_PROCEDURE_      1567
 #define SET_PROC_END_      1568
-#define SET_QUANTITY_      1569
 #define SET_AUTOCHOP_      1570
 #define SET_GAP_CONSTANT_      1571
 #define SET_AMBIENT_PRESSURE_      1572
@@ -425,7 +432,6 @@
 #define PRINT_PROCEDURE_ 1965
 #define SHOW_END_ 1966
 #define AMBIENT_PRESSURE_ 1967
-#define TOGGLE_QUANTITY_ 1968
 #define INTERP_NORMALS_ 1969
 #define MEAN_CURV_INT_ 1970
 #define COND_TEST_ 1971
@@ -493,7 +499,7 @@
 #define GET_NONCONTENT_ 2033
 #define BACKQUOTE_START_ 2034
 #define BACKQUOTE_END_   2035
-#define DEFINE_INDEX_  2036
+#define DEFINE_EXTRA_INDEX_  2036
 #define FOR_END_ 2037
 #define FOR_HEAD_ 2038
 #define FOR_TOP_  2039
@@ -533,45 +539,254 @@
 #define SIZEOF_ARRAY_ 2073
 #define SIZEOF_STRING_ 2074
 #define BEZIER_BASIS_ 2075
+#define GET_MEANCURV_ 2076
+#define DEFINE_EXTRA_ 2077
+#define SET_BOUNDARY_ 2078
+#define GET_HIT_PARTNER_ 2079
+#define SET_HIT_PARTNER_ 2080
+#define UNSET_HIT_PARTNER_ 2081
+#define DEFINE_QUANTITY_ 2082
+#define DEFINE_METHOD_INSTANCE_ 2083
+#define PUSHQFIXED_  2084
+#define PUSHQENERGY_  2085
+#define PUSHQINFO_ONLY_  2086
+#define PUSHQCONSERVED_  2087
+#define SMOOTH_GRAPH_  2088
+#define MPI_DEBUG_  2089
+#define GET_MPI_TASK_  2090
+#define POP_DISJOIN_  2091
+#define DEFINE_CONSTRAINT_ 2092
+#define DEFINE_BOUNDARY_ 2093
+#define V_DIFFUSION_  2094
+#define V_VERTEX_DISSOLVE_COUNT 2095
+#define V_EDGE_DISSOLVE_COUNT 2096
+#define V_FACET_DISSOLVE_COUNT 2097
+#define V_BODY_DISSOLVE_COUNT 2098
+#define V_EDGE_REFINE_COUNT 2099
+#define V_FACET_REFINE_COUNT 2100
+#define V_VERTEX_POP_COUNT  2101
+#define V_EDGE_POP_COUNT 2102
+#define V_POP_TRI_TO_EDGE_COUNT 2104
+#define V_POP_EDGE_TO_TRI_COUNT 2105
+#define V_POP_QUAD_TO_QUAD_COUNT 2106
+#define V_EDGESWAP_COUNT 2107
+#define V_T1_EDGESWAP_COUNT 2108
+#define V_SKINNY_REFINE_COUNT 2109
+#define V_VERTEX_DELETE_COUNT 2110
+#define V_EDGE_DELETE_COUNT 2111
+#define V_FACET_DELETE_COUNT 2112
+#define V_BODY_DELETE_COUNT 2113
+#define V_FIX_COUNT 2114
+#define V_UNFIX_COUNT 2115
+#define V_PS_LABELSIZE_ 2116
+#define V_CPU_COUNTER 2117
+#define GET_MID_EDGE_ 2118
+#define GET_MID_FACET_ 2119
+#define INIT_FACETEDGE_EDGE_ 2120
+#define INIT_FACETEDGE_FACET_ 2121
+#define NEXT_FACETEDGE_EDGE_ 2122
+#define NEXT_FACETEDGE_FACET_ 2123
+
+/* stuff transferred over from lex.h */
+#define  NO_TOKEN        0
+#define  WULFF_        2257
+#define  PERIODS_     2256
+#define  TORUS_        2252
+#define  TORUS_FILLED_  2251
+#define  SOAPFILM_    2249
+#define  MOBILITY_    2248
+#define  MOBILITY_TENSOR_  2247
+#define  ENVECT_      2245
+#define  CONVECT_     2244
+#define  MERITFACTOR_    2243
+#define  GRAV_CONST_     2242
+#define  SPRING_CONSTANT_ 2241
+#define  TEMPERATURE_      2239
+#define  FACES_          2237
+#define  CONVEX_         2230
+#define  NONNEGATIVE_      2229
+#define  NONPOSITIVE_      2228
+#define  PARAMETERS_    2227
+#define  CONTENT_        2222
+#define  BCOORD_         2219
+#define  GLOBAL_         2217
+#define  SURFACE_ENERGY_  2216
+#define  SYMMETRIC_CONTENT_ 2215
+#define  SCALE_LIMIT_      2214
+#define  CONSTRAINT_TOLERANCE_ 2211
+#define  ZOOM_VERTEX_      2210
+#define  ZOOM_RADIUS_      2209
+#define  QVECT_          2206
+#define  SPACE_DIMENSION_ 2205
+#define  SURFACE_DIMENSION_ 2204
+#define  SIMPLEX_REP_      2203
+#define  METRIC_         2202
+#define  SYMMETRY_GROUP_  2201
+#define  UNKNOWN         2199
+#define  CONFORMAL_     2198
+#define  SQUARE_CURVATURE_ 2197
+#define  PARAMETER_FILE_    2196
+#define  TOTAL_TIME_     2191
+#define  PHASEFILE_      2190
+#define  KLEIN_METRIC_      2187
+#define  GLOBAL_METHOD_     2185
+#define  EFIXED_        2184
+#define  VIEW_TRANSFORM_GENS_ 2182
+#define  GAUSS_CURVATURE_     2181
+#define  INSULATING_KNOT_ENERGY_ 2179
+#define  CONDUCTING_KNOT_ENERGY_ 2178
+#define  METHOD_     2177
+#define  NONWALL_        2176
+#define  SCALAR_INTEGRAND_    2175
+#define  VECTOR_INTEGRAND_    2174
+#define  FORM_INTEGRAND_    2173
+#define  PARAMETER_1_    2172
+#define  OPTIMIZING_PARAMETER_ 2171
+#define  K_VEC_ORDER_ 2170
+#define  LAGRANGE_ORDER_ 2169
+#define  HESSIAN_DOUBLE_NORMAL_ 2168
+#define  INTERP_BDRY_PARAM_ 2167
+#define  HESSIAN_NORMAL_ 2166
+#define  HESSIAN_NORMAL_ONE_ 2165
+#define  HESSIAN_NORMAL_PERP_ 2164
+#define  HESSIAN_SPECIAL_NORMAL_ 2163
+#define  LOAD_LIBRARY_ 2162
+#define  IGNORE_CONSTRAINTS_ 2161
+#define  VERSION_ 2160
+#define  KEEP_MACROS_ 2159
+#define  LAGRANGE_MULTIPLIER_ 2158
+#define  SWAP_COLORS_ 2157
+#define  STRING_TYPE_ 2156
+#define  IGNORE_FIXED_ 2155
+#define  KEEP_ORIGINALS_ 2154
+#define  ELEMENT_MODULUS_ 2153
+#define  VOLUME_METHOD_NAME_ 2152
+#define  DIRICHLET_MODE_ 2151
+#define  SOBOLEV_MODE_ 2150
+#define  KRAYNIKPOPVERTEX_FLAG_ 2148
+#define  KRAYNIKPOPEDGE_FLAG_ 2147
+#define  VERSIONTOKEN_ 2146
+#define  HESSIAN_SPECIAL_NORMAL_VECTOR_ 2145
+#define  RGB_COLORS_FLAG_ 2144
+#define  CIRCULAR_ARC_DRAW_ 2143
+#define  VISIBILITY_TEST_ 2142
+#define  SPARSE_CONSTRAINTS_ 2141
+#define  BLAS_FLAG_ 2140
+#define  AUGMENTED_HESSIAN_ 2139
+#define  AREA_METHOD_NAME_ 2138
+#define  LENGTH_METHOD_NAME_ 2137
+#define  BREAK_AFTER_WARNING_ 2136
+#define  PARTNER_HITTING_    2135
+#define  INTEGER_AT_         2134
+#define  LEAD_INTEGER_AT_     2133
+#define  DISPLAY_PERIODS_     2132
+/* end stuff transferred over from lex.h */
+
+#define FULL_BOUNDING_BOX_ 2258
+#define LIST_CONSTRAINT_   2259
+#define LIST_BOUNDARY_     2260
+#define LIST_QUANTITY_     2261
+#define LIST_METHOD_INSTANCE_   2262
+#define INIT_EDGE_FACETEDGE_ 2263
+#define NEXT_EDGE_FACETEDGE_ 2264
+#define POP_TO_FACE_         2265
+#define POP_TO_EDGE_         2266
+#define LINE_CONTINUATION    2267
+#define V_MINDEG_DEBUG_LEVEL 2268
+#define V_MINDEG_MARGIN      2269
+#define V_MINDEG_MIN_REGION_SIZE      2270
+#define SET_ELEMENT_GLOBAL_  2271
+#define SINGLE_ELEMENT_EXPR_ 2272
+#define UNPUTTED_            2273
+
 
 /* for BREAK and CONTINUE */
 extern int loopdepth;
 
 /* tree node for expression trees */
 struct treenode 
-            { int left;    /* left subexpression index offset  */
-              int right;   /* right subexpression index offset */
-              int type;    /* type of node                     */
-              int line_no; /* line number of source file       */
-              int file_no; /* number of source file            */
-              int flags;
-              union { int intval;            /* misc. integer data */
-                      REAL  real;            /* constant value     */
-                      struct sym *symptr;    /* symbol table ptrs  */
-                      int localnum;          /* where element id stored*/
-                      char *string;          /* string value       */
-                      struct expnode enode;  /* for expression     */
-                      dll_func_type funcptr; /* DLL function       */
-                    } op1;    /* operand 1*/
-              union { int intval;            /* misc. integer data */
-                      struct sym *symptr;    /* symbol table ptrs  */
-                      char *string;
-                      int localnum;          /* where loop element id stored*/
-                    } op2;    /* operand  2 */
-              struct { int intval[2]; /* misc. integer data */
-                       struct sym *symptr;   /* symbol table ptrs  */
-                       char *string;
-                       struct locallist_t *locals; /* for procedures */
-                     } op3;
-              int stack_delta;  /* what node does to runtime stack */
+{
+    int type;    /* type of node                     */
+    int left;    /* left subexpression index offset  */
+    int right;   /* right subexpression index offset */
+    int line_no; /* line number of source file       */
+    int file_no; /* number of source file            */
+    int datatype; /* type of expression value        */
+    int flags;
+    union { int intval;            /* misc. integer data */
+            int skipsize;          /* nodes to skip over */
+            int indexcount;        /* number of indices  */
+            int argcount;          /* number of function arguments */
+            int assigntype;        /* for assignments    */
+            int aggrtype;          /* aggregate type     */
+            int eltype;            /* element type       */
+            int maxsteps;          /* for burchard()     */
+            int coordnum;
+            REAL  real;            /* constant value     */
+            struct sym *symptr;    /* symbol table ptrs  */
+            int localnum;          /* where element id stored*/
+            int extranum;          /* number of extra attr */
+            int name_id;           /* name identifier    */
+            int letter;            /* for redefine       */
+            int quant_id;          /* named quantity id  */
+            int meth_id;           /* named method id    */
+            int con_id;            /* number of constraint */
+            int bdry_id;           /* number of boundary */
+            int toggle_id;
+            int toggle_state;      /* ON_ or OFF_        */ 
+            int bool;              /* 0 or 1             */
+            int intpow;            /* integer power      */
+            int wherecount;
+            int userfunc;
+            char *string;          /* string value       */
+            struct expnode enode;  /* for expression     */
+            dll_func_type funcptr; /* DLL function       */
+          } op1;    /* operand 1*/
+    union { int intval;            /* misc. integer data */
+            int eltype;            /* element type       */
+            int valtype;           /* data type          */
+            int breakdepth;        /* number of loop to break out of */
+            struct sym *symptr;    /* symbol table ptrs  */
+            char *string;
+            int localnum;          /* where loop element id stored*/
+            int coordnum;
+            int attr_kind;
+            int extranum;          /* number of extra attr */
+            int jumpsize;
+            int assigntype;
+            int argcount;
+            int indexcount;
+            int name_id;
+            int quant_id;
+            int meth_id;
+          } op2;    /* operand  2 */
+    union { int intval[2]; /* misc. integer data */
+            int breakjump;
+            int argcount;
+            int argtype;
+            int extra_info;       /* extra attr eltype and number */
+            int extranum;
+            int connum;           /* constraint number */
+            int bdrynum;
+            int name_id;
+           } op3;
+    union { int contjump;
+             int argtype;
+             int ret_type;
+           } op4;
+    union { struct sym *symptr;   /* symbol table ptrs  */
+            char *string;
+            struct locallist_t *locals; /* for procedures */
+          } op5;
+    int stack_delta;  /* what node does to runtime stack */
 #ifdef _DEBUG
-              int stack_spot;  /* where stack should be after evaluation
+    int stack_spot;  /* where stack should be after evaluation
                                   of this node. */
 #endif
-              int stackpos; /* local variable on stack for storing stack
-                      position for chopping stack after break or continue */
-            };
-/* flags */
+    int stackpos; /* local variable on stack for storing stack
+                     position for chopping stack after break or continue */
+};
+/* flags, also used for expnode */
 #define LOCAL_VAR_REF 4
 #define LOCAL_VAR_REF_2 8
 #define LOCAL_VAR_REF_3 0x10
@@ -582,10 +797,13 @@ struct treenode
 /* for some bit packing */
 #define ESHIFT 12
 
+/* for some type and number packing */
+#define YYTYPESHIFT 25
+#define YYSHIFTMASK (((1<<YYTYPESHIFT)-1))
+
 struct eval_frame {  /* for access to parent eval's */
    struct treenode **basenode;  /* for permload kludge */
    struct expnode *base_ex;     /* for permload kludge */
    struct eval_frame *parent;   /* up the frame chain */
    REAL *stacktop;              /* top of caller's stack, for passing args */
 };
-
