@@ -8,7 +8,10 @@
 *  File:  extern.h
 *
 *  Purpose:  declare global variables for Evolver program.
-*/
+*/  
+
+/* element id hashing */
+extern element_id (*id_hash) ARGS((element_id));
 
 #define DEFAULT_EDGE_COLOR BLACK
 #define DEFAULT_FACET_COLOR WHITE
@@ -29,6 +32,10 @@ extern char *cmdfilename; /* for saving command line read file */
 extern char *current_prompt; /* prompt string being displayed */
 extern int echo_flag; /* whether to echo stdin; for piped input */
 extern int auto_convert_flag; /* whether to automatically convert_to_quantities */
+extern int exit_flag;  /* whether to end this command interpreter */
+#define END_COMMANDS 1234551234
+extern int subshell_flag; /* whether in subshell, so can't reload */
+
 #define DEFAULT_BRIGHTNESS 0.65
 extern REAL brightness; /* midlevel gray for screen display */
 extern int markedgedrawflag; /* for single-drawing edges */
@@ -39,16 +46,41 @@ extern jmp_buf graphjumpbuf;  /* for errors during MS graphing */
 #ifdef PTHREADS
 extern pthread_t draw_thread_id; /* for graphics thread */
 #else
-extern int draw_thread_id; /* for graphics thread */
+extern unsigned int draw_thread_id; /* for graphics thread */
 #endif
 extern char *warning_messages;  /* for storing warning messages */
 extern int warning_messages_new; /* number of new warning messages since last looked */
-extern int warning_messages_max; /* allocated bytes for warning_messages */
+extern size_t warning_messages_max; /* allocated bytes for warning_messages */
 extern int shadow_warn_flag; /* warn if locals shadow existing names */
 extern int file_no; /* number of current source file */
 extern int file_no_used; /* number in list */
 extern int file_no_max; /* number allocated */
 extern char **file_names; /* names of source files */
+extern int function_kludge_flag; /* handling errors in function def in datafile */
+
+/* my own ctypes */
+extern char kb_upper_array[256];
+extern char kb_lower_array[256];
+
+extern int this_machine; /* machine identifier for MPI; really task number */
+extern int mpi_debug;
+extern int mpi_show_corona_flag; /* whether to display imported elements */
+
+/************************************************************/
+/* Unified lists of imported elements for MPI version       */
+/* On each node, there is list for each other node that     */
+/* shares elements.  Should be only 3 or 4 nontrivial       */
+/* neighbors.                                               */
+/* These are sparse lists to be used for locating elements  */
+/* by id. For compact lists of imported and exported ids,   */
+/* see import_elements and export_elements.                 */
+struct remote_list_t {
+     int *count;  /* used entries for each task */
+     int *max;   /* pointers allocated for each task */
+     struct element ***ibase;  /* task, ordinal */
+};
+extern struct remote_list_t remote_elements[NUMELEMENTS];
+
 
 /* types of element storage expansion */
 #define EXTEND_BATCH 1
@@ -64,6 +96,7 @@ struct optparam_t {
  REAL old_value;  /* for restoring */
  REAL oldgrad;  /* for conjugate gradient */
  int  rownum;  /* row in Hessian */
+ int  vhead_index;
  };
 extern struct optparam_t optparam[MAXOPTPARAM];
 extern REAL **optparam_congrads;  /* constraint gradients */
@@ -133,6 +166,7 @@ extern int everything_quantities_flag;  /* for pure quantity version, after conv
 extern int show_all_quantities;  /* to display default quantities also */
 extern int random_seed;  /* seed for random number generators */
 extern int option_q;  /* record command line option */
+extern int dont_resize_flag; /* set if datafile specified view matrix */
 
 extern char *VERSION;
 extern char *evolver_version;  /* for version checking */
@@ -166,17 +200,19 @@ extern char *typenames[NUMELEMENTS];
 #define BOUNDARY     (ATTR)0x0080
 #define NEGBOUNDARY  (ATTR)0x0100
 #define BDRY_ENERGY  (ATTR)0x0200
-#define BDRY_CONTENT (ATTR)0x1000
 #define CONSTRAINT   (ATTR)0x0400
-#define SURF_ENERGY  (ATTR)0x8000
-#define SURF_QUANTITY (ATTR)0x1000
 #define PRESSURE      (ATTR)0x0800
-#define NEGCONSTRAINT (ATTR)0x4000
+#define BDRY_CONTENT  (ATTR)0x1000
 #define HIT_WALL      (ATTR)0x2000
+#define NEGCONSTRAINT (ATTR)0x4000
+#define HIT_PARTNER   (ATTR)0x8000
+#define HAVE_CENTEROFMASS (ATTR)0x8000
 #define BARE_NAKED    (ATTR)0x10000
 #define Q_MIDPOINT    (ATTR)0x20000
 #define TETRA_PT      (ATTR)0x40000
 #define TRIPLE_PT     (ATTR)0x80000
+#define DID_BODYFRONTFACET (ATTR)0x80000
+#define DID_BODYBACKFACET (ATTR)0x100000
 #define HIT_ONE_SIDED (ATTR)0x100000
 #define Q_MIDFACET    (ATTR)0x200000
 #define Q_MIDEDGE     (ATTR)0x400000
@@ -187,6 +223,7 @@ extern char *typenames[NUMELEMENTS];
 #define ACTUALVOL     (ATTR)0x8000000
 #define REDUNDANT_BIT (ATTR)0x10000000
 #define NONCONTENT    (ATTR)0x20000000
+
 #define NO_ORIGINAL   (-1)
 
 /* modes for move_vertices() and hessian_move() */
@@ -199,8 +236,16 @@ extern int  dymemsize; /* size of dynamic memory region */
 extern struct constraint null_constraint; /* non-constraint placeholder */
 #define GETCONSTR(n) (web.constraints+(n))
 
+/* compare_vertex_edge_attr results */
+#define INCOMPARABLE    0
+#define A_SUB_B   1
+#define A_EQ_B    2
+#define A_SUPER_B 3
+
 extern char *areaname;  /* length or area */
 extern int read_command_flag;  /* whether commands at end of datafile */
+extern int err_tok_gen_flag;  /* whether -E option in effect */
+extern int token_count;  /* how many tokens parsed for -E option */
 extern int verb_flag;    /* set if lex looking for a verb */
 extern int cond_expr_flag; /* set if parser looking for ':' in conditional expr */
 extern int read_wrap_flag; /* set when reading wraps in datafile */
@@ -221,7 +266,7 @@ extern FILE *keylogfilefd;
 
 /* structure used in check.c */
 struct vvvv { element_id id;  /* which element    */
-              int vord[3];    /* id's of vertices */
+              int v[3];    /* id's of vertices */
             };
 extern int check_count;      /* number of errors returned by run_checks() */
 extern int gocount;          /* number of iterations left */ 
@@ -230,6 +275,7 @@ extern int box_flag;  /* whether or not to show outline box */
 extern int quiet_flag; /* whether to display normal output */
 extern int quiet_go_flag; /* whether to display normal g output */
 extern int shading_flag;  /* for facet shading by orientation */
+extern int smooth_graph_flag; /* whether to smoothly plot in lagrange mode */
 extern int visibility_test; /* whether to do after depth sort */
 extern int visdebuglevel;  /* visibility debug level */
 extern int color_flag;    /* facet coloring by user */
@@ -261,19 +307,25 @@ extern long top_timestamp;    /* timestamp for topology changes */
 extern long vedge_timestamp; /* for vertex edgelist currency */
 extern long vfacet_timestamp; /* for vertex facetlist currency */
 extern long bfacet_timestamp; /* for body facetlist currency */
-extern long volume_timestamp; /* for volume calculation */
+extern long fixed_volume_timestamp; /* for volume calculation */
+extern long info_volume_timestamp; /* for volume calculation */
 extern long reset_timestamp;  /* when surface loaded */
+#define NOSETSTAMP 0
+#define SETSTAMP   1
+
 extern int need_fe_reorder_flag; /* setting causes recalc to fe_reorder */
 extern int ackerman_flag;  /* whether doing phase space motion */
 extern char pix_file_name[150]; /* for P 2 */
 extern int labelflag; /* whether ps doing labels */
 extern int gridflag;  /* whether ps doing gridlines */
 extern int ps_colorflag; /* whether ps doing color */
+extern int full_bounding_box_flag; /* for window bounding box */
 extern int crossingflag; /* whether ps doing crossings */
 
 extern char ps_file_name[1000]; /* ps output file */
 
 /* Following PostScript line width variables relative to page size */
+extern REAL ps_labelsize;  /* relative size of labels */
 extern REAL ps_stringwidth;  /* default edge width */
 extern REAL ps_fixededgewidth;
 extern REAL ps_tripleedgewidth;
@@ -323,21 +375,6 @@ extern  struct dll dll_list[MAX_DLL];
 #define FUNC_DERIV  2
 #define FUNC_SECOND 3
 
-/* Type defines */
-/* Low, unsigned values to fit in struct global args */
-#define TYPE_UNSET     0
-#define TYPE_VOID      1
-#define TYPE_INTEGER   2
-#define TYPE_REAL      3
-#define TYPE_STRING    4
-#define TYPE_VERTEX    5
-#define TYPE_EDGE      6
-#define TYPE_FACET     7
-#define TYPE_BODY      8
-#define TYPE_FACETEDGE 9
-#define TYPE_BOOLEAN   10
-   
-
 
 /* global variable list */
 #define GLOBAL_NAME_SIZE 31
@@ -357,6 +394,7 @@ struct global
             int offset; /* local variable position on stack */
             void *dataptr;  /* for permanent internal variables */
             struct array *arrayptr;
+            element_id id;
           } value;
     union {
       struct {
@@ -375,9 +413,11 @@ struct global
        } varstuff;
     } attr;
     int  flags;     /* see defines below */
+    int  type;      /* datatype for variables */
   };
-extern struct locallist_t query_locals;
-extern struct locallist_t *local_scope_bases[]; /* for parsing */
+extern struct locallist_t *query_locals;
+#define LOCALSCOPEMAX 100
+extern struct locallist_t *local_scope_bases[LOCALSCOPEMAX]; /* for parsing */
 extern int local_nest_depth;  /* for local_scope_bases */
 
 /* for global identifier id */
@@ -417,6 +457,7 @@ extern int dy_global_hash_used;     /* entries actually in use */
 #define METHODNAME    0x40000000
 #define QUANTITYNAME  0x60000000
 #define PERM_NAME     0x80000000
+
 /* Hash table actions */
 #define HASH_LOOK 0
 #define HASH_ADD 1
@@ -443,6 +484,10 @@ extern struct locallist_t *localbase;
 #define LL_HAS_ARRAY 1
 #define LL_PERMANENT 2
 
+/* type of listing desired */
+#define LIST_PROTO  1
+#define LIST_FULL   2
+
 extern int proc_timestamp; /* for ordering procedure definitions */
 extern int old_global_count; /* for error recovery */
 extern int old_perm_global_count; /* for error recovery */
@@ -479,12 +524,12 @@ extern REAL **ritzvecs;
 #define QUANTITY_MODULUS  0x40  /* multiplier for quantity */
 #define QUANTITY_NAME  0x80  /* actual value for quantity */
 #define QUANTITY_TARGET  0x100  /* target value for quantity */
-#define QUANTITY_TYPE (QUANTITY_MODULUS|QUANTITY_NAME|QUANTITY_TARGET)
+#define QUANTITY_TYPES (QUANTITY_MODULUS|QUANTITY_NAME|QUANTITY_TARGET)
 #define METHOD_MODULUS 0x200
 #define METHOD_NAME 0x400
 #define ARRAY_PARAM 0x800
-#define METHOD_TYPE (METHOD_MODULUS|METHOD_NAME)
-#define ANY_TYPE  (FILE_VALUES|SUBROUTINE|QUANTITY_TYPE|METHOD_TYPE)
+#define METHOD_TYPES (METHOD_MODULUS|METHOD_NAME)
+#define ANY_TYPE  (FILE_VALUES|SUBROUTINE|QUANTITY_TYPES|METHOD_TYPES)
 #define STRINGVAL 0x2000  /* string */
 #define LEFTOVER  0x4000  /* if permanent left over from prev file */
 #define OPTIMIZING_PARAMETER 0x8000  /* variable during optimization */
@@ -500,6 +545,7 @@ extern REAL **ritzvecs;
 #define REALVAL 0x2000000
 
 extern struct expnode torus_period_expr[MAXCOORD][MAXCOORD];
+extern struct expnode torus_display_period_expr[MAXCOORD][MAXCOORD];
 
 /* for calc_periods() */
 #define NO_ADJUST_VOLUMES 0
@@ -520,6 +566,7 @@ extern int torus_display_mode;  /* default, raw, connected, clipped */
 extern int view_matrix_global; /* global id number */
 extern int torus_periods_global;
 extern int inverse_periods_global;
+extern REAL window_aspect_ratio;
 
 extern int read_depth,include_depth;
 extern char datafilename[PATHSIZE];  /* current datafile name */
@@ -535,8 +582,14 @@ extern struct cmdfile {
             int datafile_flag;       /* whether datafile    */
             int file_no;
          } cmdfile_stack[NESTDEPTH],datafile_stack[NESTDEPTH];
+
 extern int datafile_flag;  /* 1 for datafile, 0 for command so expression
                                         parser knows what's up */
+extern int datafile_input_flag; /* whether further input available from datafile */
+#define NOT_DATAFILE 0
+#define IN_DATAFILE 1
+#define PRETEND_DATAFILE 2
+
 extern int topflag;  /* 1 while in datafile top section */
 extern int backquote_flag; /* 1 while in backquoted command; kludge */
 extern int lists_flag;  /* set when parsing space-separated lists */
@@ -556,6 +609,18 @@ extern int estimate_flag;    /* for toggling estimate of energy decrease */
 extern REAL estimated_change; /* stored result */
 extern int autorecalc_flag; /* for toggling autorecalc after variable assign  */
 extern int autopop_flag;     /* whether to do autopopping */
+extern int pop_disjoin_flag;  /* whether cones to be disjoined rather than merged */
+extern int pop_to_edge_flag;  /* control which way popping goes */
+extern int pop_to_face_flag;  /* control which way popping goes */
+/* some popping modes for various popping functions */
+#define POP_TO_BETTER    0
+#define POP_TO_OPEN   1
+#define POP_TO_TRIPLE 2
+#define POP_TO_TWIST  3
+#define POP_TO_TRIANGLE 4
+#define POP_TO_EDGE   5
+#define POP_TO_FACE   6
+
 extern int autochop_flag;     /* whether to do autochopping */
 extern REAL autochop_size;  /* max edge length for autochop */
 extern int autopop_count;  /* number of edges found */
@@ -599,7 +664,8 @@ extern vertex_id comp_quant_vj; /* during calc_quant_hess */
 
 extern REAL *f_sums;  /* facet_knot_energy, for sums to all other vertices */
 
-extern char yytext[];
+/*extern char yytext[];*/
+extern char *yytext;
 extern char *inputbuffer;
 extern int inputbufferspot;
 extern int inputsave_flag;
@@ -616,6 +682,7 @@ extern int _CrtCheckMemory(void);
 #define HEAPCHECK
 #endif
 extern int tok;
+extern int unput_tok_count;  /* number of pushed-back tokens */
 extern int int_val;
 extern REAL real_val;
 extern int attr_etype; /* for ATTRIBUTE parse */
@@ -630,9 +697,14 @@ extern char *history_space;
 extern int  history_number;    /* number of current command */
 extern int history_count;  /* number in list */
 extern int  history_offsets[MAXHISTORY];
+
+/* following two are for mode argument of command() */
+#define NO_HISTORY 0
+#define ADD_TO_HISTORY 1
+
 #define MAXCMDSIZE 2000 
 extern char fulltext[MAXCMDSIZE+5]; /* for full text of commands */
-extern int  fulltextsize;              /* length of command */
+extern size_t  fulltextsize;              /* length of command */
 extern int aggrtype;  /* aggregate type being parsed */
 extern int aggregate_depth; /* nesting depth of aggregate loops */
 extern int attr_kind; /* kind of attribute being parsed */
@@ -655,10 +727,10 @@ extern struct sym *elsym;  /* name of element during parsing */
 extern struct sym *yysym;  /* name of identifier from lex */
 extern struct sym symtable[];    /* symbol table */
 struct array { int dim; 
-               int datatype;   /* REAL_TYPE_ or INTEGER_TYPE_ */
+               int datatype;   /* REAL_TYPE_, INTEGER_TYPE_, etc. */
                int itemsize;   /* bytes per item */
                int datasize;   /* total data elements */
-               int datastart;  /* byte offset of start of data */
+               size_t datastart;  /* byte offset of start of data */
                int size[1];    /* arbitrary length, followed by data */
              };
                
@@ -683,12 +755,14 @@ extern int h0_attr;  /* attribute number for h_zero, if any */
 #define PLAIN_SQ  1
 #define EFF_SQ     2
 #define NORMAL_SQ 3
+#define PERP_SQ   4
 extern int sqgauss_param;    /* which parameter for modulus */
 extern int normal_sq_mean_curvature_mi;
 extern int eff_area_sq_mean_curvature_mi;
 extern int sq_mean_curvature_mi;
 extern int mix_sq_mean_curvature_mi;
 extern int star_normal_sq_mean_curvature_mi;
+extern int star_perp_sq_mean_curvature_mi;
 extern int star_eff_area_sq_mean_curvature_mi;
 extern int star_sq_mean_curvature_mi;
 extern REAL target_length;  /* for string model */
@@ -699,6 +773,7 @@ extern int div_normal_curvature_flag; /* choice of curvature formula */
 extern int marked_edge_attr;
 #define MARKED_EDGE_ATTR_NAME "sqcurve_string_mark"
 extern int circular_arc_flag; /* whether to draw edges as arcs */
+extern int spherical_arc_flag; /* whether to draw edges as spherical arcs */
 
 /* flag bits to say if should be evaluated since modulus nonzero */
 #define EVALUATE 0x0002
@@ -785,9 +860,6 @@ struct divedge { int endpt[2];  /* endpoints */
 extern struct treenode *list;    /* tree */
 extern int listtop;  /* first spot reserved for root */
 extern int listmax;  /* allocated spots */
-/* flags for permanence of list */
-#define USERCOPY 1
-#define NOUSERCOPY 0
 
 /* type of checking to do */
 #define PRELIMCHECK  1
@@ -833,12 +905,16 @@ extern int breakflag;      /* set by user interrupt */
 #define BREAKFULL 1
 #define BREAKLOOP 2 /* unused */
 #define BREAKREPEAT 3
+#define BREAKAFTERWARNING 4
 extern int iterate_flag;  /* so handler knows when iteration in progress */
 extern int bare_edge_count;  /* edges without facets */
 
 struct oldcoord {  
     REAL  (*coord)[MAXCOORD];  /* allocated space for old coordinates */
     REAL  energy;
+    REAL *bod;
+    REAL *meth;
+    REAL *quant;
     REAL optparam_values[MAXOPTPARAM];
     int  size; /* bytes allocated for coord */
     } ;
@@ -875,6 +951,10 @@ struct bodyface { facet_id f_id;
                   WRAPTYPE wrap;  /* wraps of base vertex */
                   int        wrapflag; /* whether wrap done */
                 };
+/* wrapflag values */
+#define NOT_WRAPPED 0
+#define IS_WRAPPED  1
+#define BAD_BAD_BAD 2
 
 /* structure for depth sorting triangles */
 struct tlink { int node; /* index in tlist */
@@ -906,10 +986,17 @@ extern struct tsort {
           int from;                       /* for loop detection */
 #endif
        } *trilist;
-/* flag is EDGE or FACET in lower bits, and LABEL_EDGE, and */
+/* flag is EDGE or FACET in lower bits, and LABEL_EDGE, and ... */
 #define FLIPPED_FACET 0x1000
 #define WAS_BACKMOST  0x2000
 #define VISIBLE       0x4000
+#define ARC_EDGE      0x8000
+#define SPHERE_EDGE  0x10000
+
+extern int graph_capabilities;  /* bits to describe what current graphics 
+                                   device can do */
+#define GC_ARCS  0x1
+#define GS_ARCS  0x2
 
 /* bounding box, for PostScript */
 extern REAL bbox_minx,bbox_miny,bbox_maxx,bbox_maxy;
@@ -934,6 +1021,8 @@ struct graphdata { REAL  x[MAXCOORD+1];  /* homogeneous coordinates */
 /* SIMPLE_FACET and LIST_FACET are used in geomgraph.c */
 #define SIMPLE_FACET  1
 #define LIST_FACET     2
+#define EDGE_ARC 0x8000
+#define SPHERE_ARC 0x10000
 /* edge types */
 #define  INVISIBLE_EDGE 0
 #define  REGULAR_EDGE  1
@@ -944,10 +1033,13 @@ struct graphdata { REAL  x[MAXCOORD+1];  /* homogeneous coordinates */
 #define  CONSTRAINT_EDGE 0x20
 #define  BARE_EDGE      0x40
 #define  SPLITTING_EDGE 0x80
+#define  EBITS          0xFF
+
 #define  LABEL_EDGE     0x100
 #define  LABEL_HEAD     0x200
 #define  LABEL_TAIL     0x400
 #define  LABEL_FACET    0x800
+#define  LABEL_REVERSED 0x1000
 
 /* for extracting color bits from int color */
 #define GET_RED_BITS(c)  (((unsigned int)(c)>>24) & 0xFF)
@@ -1090,12 +1182,20 @@ struct hess_verlist {
             int flags;       /* for marking if done */
             int freedom;     /* degrees of freedom */
             int rownum;      /* starting row in sparse matrix  */
+#ifdef MPI_EVOLVER
+            int global_rownum;
+#endif
             REAL **proj;     /* projection matrix to constraints */
             REAL ***conhess; /* for constraint hessian */
             REAL slant;      /* cos of angle of normal from constraint */
           };
 /* flag bits */
 #define HV_DONE 1
+#define SUPERNODE_DONE 2
+extern int vhead_attr;  /* number of vertex attribute for vhead index */
+extern int bhead_attr;  /* number of body attribute for vhead index */
+#define VHEAD_ATTR_NAME "__vhead_index"
+#define BHEAD_ATTR_NAME "__bhead_index"
 
 /* sparse hessian matrix entry */
 struct hess_entry { 
@@ -1170,7 +1270,7 @@ struct SepNodeType {
       REAL subopc;   /* The opcount for the subtree of this node */
         } opc;  /* original metis fields */
         struct {
-             int size;   /* size of matrix */
+             size_t size;   /* size of matrix */
              int *vlist; /* list of variable numbers */
              REAL *mat;  /* triangular matrix */
         } info;  /* stuff useful in factoring */
@@ -1183,6 +1283,8 @@ struct sp_entry { int row,col;
                 };
 extern int blas_flag; /* whether to use BLAS */
 extern int augmented_hessian_flag; /* whether to use augmented Hessian */
+                                   /* -1 for unset                     */
+extern int augmented_hessian_mode; /* set during factoring */
 extern int sparse_constraints_flag; /* whether to store constraint gradients
                in sparse format */
 extern int BK_flag; /*for enabling Bunch-Kaufman version of sparse factoring*/
@@ -1254,6 +1356,7 @@ struct linsys { int flags;  /* status bits, see below */
 #define S_ODRV_REORDERED 0x10
 #define S_JA_INCLUDED 0x20
 #define S_ORDERFOUND  0x40
+#define S_USE_ELEMENTS 0x80
 
 /* psize values */
 /* size of diagonal elements, 1 for 1x1, 2 for first of 2x2, 
@@ -1273,6 +1376,11 @@ extern int ysmp_flag;  /* set if doing Yale Sparse Matrix version */
 #define YSMP_FACTORING 1
 #define METIS_FACTORING 2
 
+/* for mindeg control */
+extern int mindeg_debug_level;
+extern int mindeg_margin;
+extern int mindeg_min_region_size;
+
 /* multiply vector by original sparse matrix */
 extern void (*sp_mul_func)ARGS((struct linsys *, REAL*,REAL*));
 
@@ -1382,6 +1490,8 @@ extern REAL **gpoly;    /* interpolation polynomial value at integration point k
 extern REAL ***gpolypartial; /* partials of interpolation polynomials */
                          /* gausspt,dim,ctrlpt */
 
+#define MAXLAGRANGE 20
+
 /* new gaussian-lagrange structure */
 #define MAXGAUSSORDER 40
 struct gauss_lag { int lagrange_order;  /* computed for */
@@ -1402,6 +1512,13 @@ extern struct gauss_lag *gauss_lagrange[MAXCOORD];
 extern int maxgaussorder[MAXCOORD]; /* allocated for each dimension */
 
 extern int bezier_flag; /* whether to do Bezier basis polynomials in Lagrange */
+extern REAL **bezier1invert[MAXLAGRANGE];
+extern REAL **bezier1revert[MAXLAGRANGE];
+extern REAL **bezier_refine_1d[MAXLAGRANGE];
+extern REAL **bezier2invert[MAXLAGRANGE];
+extern REAL **bezier2revert[MAXLAGRANGE];
+extern REAL **bezier_refine_2d[MAXLAGRANGE];
+
 extern REAL **metric;  /* metric values at a point */
 extern REAL ***metric_partial; /* partial derivatives of metric */
 extern REAL **det_array;    /* tangent vector dot products */
@@ -1501,6 +1618,7 @@ struct sym_registry {
 #define WRAPMASK  037
 #define ALLWRAPMASK 03737373737
 #define NEGWRAP    WRAPMASK
+#define WRAPNUM(x) ( (x)>(1<<(TWRAPBITS-2)) ? (x)-(1<<(TWRAPBITS-1)) : (x))
 
 /* for symmetry group "rotate" */
 extern int rotorder;
@@ -1529,6 +1647,8 @@ extern int transform_colors_flag;
 #define CLEAR (-1)
 /* color for unshown facet */
 #define UNSHOWN (-2)
+/* color for saying color is not set */
+#define NOT_A_COLOR (-3)
 
 /* from borlandc/graphics.h */
 #if      !defined(__COLORS)
@@ -1584,29 +1704,60 @@ extern int ptop[MAXPROCS];    /* used per calculator */
 extern int pmax[MAXPROCS];    /* available per calculator */
 
 /* Graphing mutual exclusion for multi-threaded version */
+#define IMMEDIATE_TIMEOUT 0
+#define LONG_TIMEOUT 100000
 #ifdef WIN32
 extern void * graphmutex;
 extern void * mem_mutex;
-#define ENTER_GRAPH_MUTEX { MsgWaitForMultipleObjects(1,&graphmutex,0,100000,0); }
-#define TRY_GRAPH_MUTEX ( MsgWaitForMultipleObjects(1,&graphmutex,0,0,0)!=WAIT_TIMEOUT)
-#define LEAVE_GRAPH_MUTEX  { ReleaseMutex(graphmutex);  }
+extern unsigned int locking_thread; /* so we can tell who has it */
+/* Note ENTER_GRAPH_MUTEX and LEAVE_GRAPH_MUTEX form a {} block! */
+#define ENTER_GRAPH_MUTEX { int did_graphlock_here;\
+     if ( locking_thread != GetCurrentThreadId() ) \
+   { MsgWaitForMultipleObjects(1,&graphmutex,0,100000,0); \
+     locking_thread = GetCurrentThreadId(); \
+     did_graphlock_here = 1; } \
+    else did_graphlock_here = 0; 
+#define TRY_GRAPH_MUTEX(timeout) ( \
+   (MsgWaitForMultipleObjects(1,&graphmutex,0,timeout,0)!=WAIT_TIMEOUT) && \
+     (locking_thread = GetCurrentThreadId()) )
+#define END_TRY_GRAPH_MUTEX    \
+     {locking_thread = 0; ReleaseMutex(graphmutex);}
+#define LEAVE_GRAPH_MUTEX   if ( did_graphlock_here ) \
+     {locking_thread = 0; ReleaseMutex(graphmutex); did_graphlock_here = 0; } }
+#define ABORT_GRAPH_MUTEX  \
+    if ( GetCurrentThreadId() == locking_thread )\
+     {locking_thread = 0; ReleaseMutex(graphmutex); }
 #define ENTER_MEM_MUTEX { MsgWaitForMultipleObjects(1,&mem_mutex,0,100000,0); }
 #define LEAVE_MEM_MUTEX  { ReleaseMutex(mem_mutex);  }
+
 #elif defined(PTHREADS)
 extern pthread_mutex_t graphmutex;
-#define ENTER_GRAPH_MUTEX ;pthread_mutex_lock(&graphmutex);
-#define TRY_GRAPH_MUTEX (pthread_mutex_trylock(&graphmutex)!=EBUSY)
-#define LEAVE_GRAPH_MUTEX  pthread_mutex_unlock(&graphmutex); 
+extern pthread_t locking_thread; /* so we can tell who has it */
+#define ENTER_GRAPH_MUTEX {int did_graphlock_here;\
+    if ( locking_thread != pthread_self() ) \
+     { pthread_mutex_lock(&graphmutex);locking_thread=pthread_self();\
+       did_graphlock_here = 1;}\
+    else did_graphlock_here = 0;
+#define TRY_GRAPH_MUTEX(timeout) ((pthread_mutex_trylock(&graphmutex)!=EBUSY) &&\
+               (locking_thread=pthread_self()))
+#define LEAVE_GRAPH_MUTEX  if ( did_graphlock_here ) \
+        {locking_thread=0;pthread_mutex_unlock(&graphmutex);} }
+#define END_TRY_GRAPH_MUTEX \
+        {locking_thread=0;pthread_mutex_unlock(&graphmutex);} 
+#define ABORT_GRAPH_MUTEX  if ( locking_thread == pthread_self() ) \
+        {locking_thread=0;pthread_mutex_unlock(&graphmutex);} 
 #define ENTER_GRAPH_MUTEX_PR  printf("Lock %s:%d\n",__FILE__,__LINE__);pthread_mutex_lock(&graphmutex);
 #define LEAVE_GRAPH_MUTEX_PR printf("Unlock %s:%d\n",__FILE__,__LINE__); pthread_mutex_unlock(&graphmutex); 
 extern pthread_mutex_t mem_mutex;
 #define ENTER_MEM_MUTEX  pthread_mutex_lock(&mem_mutex);
 #define LEAVE_MEM_MUTEX  pthread_mutex_unlock(&mem_mutex); 
 #else
-#define ENTER_GRAPH_MUTEX 
-#define LEAVE_GRAPH_MUTEX  
-#define ENTER_MEM_MUTEX 
-#define LEAVE_MEM_MUTEX  
+#define ENTER_GRAPH_MUTEX {}
+#define LEAVE_GRAPH_MUTEX  {}
+#define ENTER_MEM_MUTEX {}
+#define LEAVE_MEM_MUTEX  {}
+#define TRY_GRAPH_MUTEX(timeout)  1
+#define ABORT_GRAPH_MUTEX {} 
 #endif
 
 /* Multithreading with worker threads */
@@ -1629,6 +1780,7 @@ extern int element_unlocks;
 #define TH_MULTI_QUANT_GRADS   6
 #define TH_MULTI_QUANT_HESS    7
 #define TH_FIX_GRADS           8
+#define TH_MOVE_VERTICES       9
 
 #ifdef PTHREAD_LOG
 struct thread_event { /* for tracking thread events */
@@ -1679,6 +1831,10 @@ struct thread_data {  /* per thread data, pointed to by thread_data_key */
          element_id iteration_id;
          REAL total_energy;
          REAL total_area;
+#ifdef _MSC_VER
+     __int64 stagestart[MAXTHREADSTAGES];
+     __int64 stageend[MAXTHREADSTAGES];
+#endif
 #ifdef PTHREAD_LOG
          int eventcount;  /* for timestamping events */
          struct thread_event events[MAXTHREADEVENTS];
@@ -1740,7 +1896,8 @@ extern int m_type;
 extern int m_mode;
 extern REAL *m_rhs;
 
-#ifdef _MSC_VER
+extern  double   cpu_speed;
+#if defined(_MSC_VER) && !defined(__BORLANDC__) && !(_IA64_==1)
 /***************************************************************************
   Some macros for cycle-counting profiling.
   Usage: Put PROF_START(f) at start of function f, PROF_FINISH(f) at end,
@@ -1750,23 +1907,145 @@ __int32 f_elapsed_time[2];
   PROF_PRINT(f);
   Make sure all return paths go through end of function. 
 ***************************************************************************/
-
-#define CPU_SPEED (1200e6)
-#define PROF_START(f)  __asm rdtsc __asm sub f##_elapsed_time,eax      \
-        __asm sbb f##_elapsed_time[4],edx   
-#define PROF_FINISH(f) 	__asm rdtsc __asm add f##_elapsed_time,eax   \
-	__asm adc f##_elapsed_time[4],edx
-#define PROF_ELAPSED(f) (((double)*(__int64*)f##_elapsed_time)/CPU_SPEED)
-#define PROF_RESET(f) f##_elapsed_time[0]=f##_elapsed_time[1]=0;:w
+/* always do PROF_NOW, so can get CPU counter even if not profiling */
+#define PROF_NOW(fullname) {  __asm {rdtsc} __asm {mov fullname,eax}      \
+        __asm { mov fullname[4],edx}   }
+#define PROF_CYCLES(fullname) ((double)*(__int64*)fullname)
+#ifdef PROFILING
+#define PROFILING_ENABLED
+#define PROF_START(f) { __asm {rdtsc} __asm {sub f##_elapsed_time,eax}      \
+        __asm {sbb f##_elapsed_time[4],edx}   }
+#define PROF_FINISH(f)  { __asm {rdtsc} __asm {add f##_elapsed_time,eax}   \
+    __asm {adc f##_elapsed_time[4],edx} }
+#define PROF_ELAPSED(f) (((REAL)*(__int64*)f##_elapsed_time)/cpu_speed)
+#define PROF_RESET(f) f##_elapsed_time[0]=f##_elapsed_time[1]=0;
 #define PROF_PRINT(f) { fprintf(stderr,"%22s: %13.0f\n",#f,\
     (double)*(__int64*)f##_elapsed_time); \
    f##_elapsed_time[0]=f##_elapsed_time[1]=0;}
+#define METHOD_PROFILING_START(mi,mode) {PROF_RESET(mode);PROF_START(mode);}
+#define METHOD_PROFILING_END(mi,mode) {PROF_FINISH(mode);mi->mode##_elapsed_time += PROF_ELAPSED(mode);mi->mode##_call_count++;}
+#endif
+#elif defined(__i386) && defined(__GNUC__) && !defined(__STRICT_ANSI__)
+  /* operands are in source-destination order in gcc asm */
+#define PROF_NOW(fullname) { \
+  asm("rdtsc" : : ); \
+  asm("movl %%eax,%0" : "=m"(fullname) : ); \
+  asm("movl %%edx,%0" : "=m"(fullname[1]) : ); }
+#define PROF_CYCLES(fullname) ((double)*(unsigned long long*)fullname)
+#ifdef PROFILING
+#define PROFILING_ENABLED
+#define PROF_START(f) { \
+  asm("push %%eax" : :); \
+  asm("push %%edx" : :); \
+  asm("rdtsc" : : ); \
+  asm("sub %%eax,%0" : "=m"(f##_elapsed_time) : ); \
+  asm("sbb %%edx,%0" : "=m"(f##_elapsed_time[1]) : ); \
+  asm("pop %%edx" : :); \
+  asm("pop %%eax" : :); } 
+#define PROF_FINISH(f) { \
+  asm("push %%eax" : :); \
+  asm("push %%edx" : :); \
+  asm("rdtsc" : : ); \
+  asm("add %%eax,%0" : "=m"(f##_elapsed_time) : ); \
+  asm("adc %%edx,%0" : "=m"(f##_elapsed_time[1]) : ); \
+  asm("pop %%edx" : :); \
+  asm("pop %%eax" : :); } 
+#define PROF_ELAPSED(f) (((double)*(unsigned long long*)f##_elapsed_time)/cpu_speed)
+#define PROF_RESET(f) f##_elapsed_time[0]=f##_elapsed_time[1]=0;
+#define PROF_PRINT(f) { fprintf(stderr,"%22s: %13.0f\n",#f,\
+    (double)*(unsigned long long*)f##_elapsed_time); \
+   f##_elapsed_time[0]=f##_elapsed_time[1]=0;}
+#define METHOD_PROFILING_START(mi,mode) {PROF_RESET(mode);PROF_START(mode);}
+#define METHOD_PROFILING_END(mi,mode) { PROF_FINISH(mode); \
+  mi->mode##_elapsed_time += PROF_ELAPSED(mode);mi->mode##_call_count++;}
+#endif
+#endif
+
+#ifndef PROF_NOW
+#define PROF_NOW(f)  f[0] = f[1] = 0;
+#define PROF_CYCLES(f) 0.0
+#endif
+
+#ifndef PROFILING_ENABLED
+#define PROF_START(f)   
+#define PROF_FINISH(f)  
+#define PROF_ELAPSED(f) 
+#define PROF_RESET(f)   
+#define PROF_PRINT(f)   
+#define METHOD_PROFILING_START(mi,mode) 
+#define METHOD_PROFILING_END(mi,mode) 
+#endif
+
+
+extern int element_setup_elapsed_time[2];
+extern int calc_quants_elapsed_time[2];
+extern int calc_quant_grads_elapsed_time[2];
+extern int calc_quant_hess_elapsed_time[2];
+extern int exparse_elapsed_time[2];
+extern int yyparse_elapsed_time[2];
+extern int yylex_elapsed_time[2];
+extern int kblex_elapsed_time[2];
+extern int hessian_solve_elapsed_time[2];
+extern int hessian_mul_elapsed_time[2];
+extern int hessian_AIJ_setup_elapsed_time[2];
+extern int hessian_constraint_setup_elapsed_time[2];
+extern int hessian_project_setup_elapsed_time[2];
+extern int hessian_factor_elapsed_time[2];
+extern int hessian_CHinvC_elapsed_time[2];
+
+
+#ifdef _MSC_VER
+/* Elaborate macro to make element loop fast and local.
+   Usage:  THREAD_FOR_ALL_NEW(elementtype,action)
+   where action is executable code.  action should refer to
+   the element id as *idptr.
+   action can be extensive code block, but do not use
+   comma in declaration lists; comma ok in argument lists,
+   but if not in nested parentheses, then looks like end
+   of action argument.
+*/
+
+#define THREAD_FOR_ALL_NEW(elementtype,action)                       \
+{ element_id *idptr;                                                 \
+  int proc,nextproc;                                                 \
+  struct thread_stages_data *th;                                     \
+  struct thread_data *data;                                          \
+  int maxstage;                                                      \
+  int elnum,maxelnum;                                                \
+  __int32 now[2];                                                    \
+                                                                     \
+  data = GET_THREAD_DATA;                                            \
+  proc = data->worker_id;                                            \
+  nextproc = (proc == nprocs - 1) ? 0 : proc+1;                      \
+  maxstage = (proc == 0) ? max_thread_stages+1 : max_thread_stages;  \
+  th = thread_stages + proc;                                         \
+                                                                     \
+  for ( th->stage = 0 ; th->stage < maxstage ; th->stage++ )         \
+  { /* wait for nextproc to reach same stage */                      \
+    while ( thread_stages[nextproc].stage < th->stage ) ;            \
+                                                                     \
+    PROF_NOW(now);                                                   \
+    data->stagestart[th->stage] = *(__int64*)now;                    \
+                                                                     \
+    /* do all elements in this stage */                              \
+    idptr = th->blocks[elementtype][th->stage];                      \
+    maxelnum = th->counts[elementtype][th->stage];                   \
+    for ( elnum = 0 ; elnum < maxelnum ; elnum++,idptr++ )           \
+     action;                                                         \
+                                                                     \
+    PROF_NOW(now);                                                   \
+    data->stageend[th->stage] = *(__int64*)now;                      \
+  }                                                                  \
+}           
+
+/* end THREAD_FOR_ALL_NEW */
+
+extern __int64 thread_launch_start;
+extern __int64 thread_launch_end;
+
 #else
-#define PROF_START(f)	
-#define PROF_FINISH(f)	
-#define PROF_ELAPSED(f)	
-#define PROF_RESET(f)	
-#define PROF_PRINT(f)	
+#define THREAD_FOR_ALL_NEW(elementtype,action) 
 #endif
 /* profiling storage */
 extern int q_facet_setup_elapsed_time[2];
+
