@@ -36,7 +36,7 @@ int mode; /* AREA_ONLY or ALL_ENERGY */
   REAL unwrap_x[FACET_VERTS][MAXCOORD];
   REAL z[FACET_VERTS];  /* for calculating average z*z */
   REAL zz;     /* for average z*z, for gravity */
-  REAL u;      /* gravitaional energy */
+  REAL u;      /* gravitational energy */
   facetedge_id fe_id;
   vertex_id v_id[FACET_VERTS];
   REAL ss,st,tt;
@@ -44,7 +44,7 @@ int mode; /* AREA_ONLY or ALL_ENERGY */
 #ifdef THREADS
   struct thread_data *data = GET_THREAD_DATA;
 #endif
-  int_val = get_original(f_id);  /* for eval  of file parameters */
+  int_val = ordinal(get_original(f_id))+1;  /* for eval  of file parameters */
 
   /* get side vectors */
   fe_id = get_facet_fe(f_id);
@@ -103,7 +103,7 @@ skip_from_metric:
     data->total_area    = data->total_area + energy; /* Borland C bug */
   else
 #endif
-  web.total_area    = web.total_area + energy; /* Borland C bug */
+  binary_tree_add(web.total_area_addends,energy);
 
   if ( get_fattr(f_id) & DENSITY )
      energy = energy * get_facet_density(f_id);  /* Borland C bug */
@@ -127,7 +127,7 @@ skip_from_metric:
     data->total_energy = data->total_energy + energy; /* Borland C bug */
   else
 #endif
-  web.total_energy = web.total_energy + energy; /* Borland C bug */
+  binary_tree_add(web.total_energy_addends,energy); 
 
 } /* end facet_energy_l() */
  
@@ -163,7 +163,7 @@ facet_id f_id;
   REAL ss,st,tt;
 
   memset((char*)forces,0,sizeof(forces));  /* set to 0 */
-  int_val = get_original(f_id);  /* for eval  of file parameters */
+  int_val = ordinal(get_original(f_id))+1;  /* for eval  of file parameters */
   /* get side vectors */
   fe_id = get_facet_fe(f_id);
   for ( i = 0 ; i < FACET_EDGES ; i++ )
@@ -215,8 +215,7 @@ facet_id f_id;
   { facetedge_id ffe;
       
     ffe = fe_id;
-    sprintf(errmsg,"WARNING! Zero area for facet %d.\n",
-         ordinal(f_id)+1);
+    sprintf(errmsg,"WARNING! Zero area for facet %s.\n",ELNAME(f_id));
     outstring(errmsg);
     if ( itdebug )
     {
@@ -303,17 +302,13 @@ end_euclidean:
 
     force = get_force(v_id[i]);
     if ( web.symmetry_flag )
-     { (*sym_form_pullback)(get_coord(v_id[i]),wforce,forces[i],wraps[i]);
-        LOCK_ELEMENT(v_id[i]);
-        for ( j = 0 ; j < SDIM ; j++ )
-          force[j] += wforce[j];
-        UNLOCK_ELEMENT(v_id[i]);
-     }
-    else
-    { LOCK_ELEMENT(v_id[i]);
+    { (*sym_form_pullback)(get_coord(v_id[i]),wforce,forces[i],wraps[i]);
       for ( j = 0 ; j < SDIM ; j++ )
-          force[j] += forces[i][j];
-      UNLOCK_ELEMENT(v_id[i]);
+        force[j] += wforce[j];
+    }
+    else
+    { for ( j = 0 ; j < SDIM ; j++ )
+        force[j] += forces[i][j];
     }
   }
 
@@ -338,7 +333,7 @@ facet_id f_id;
      
   if ( get_fattr(f_id) & NONCONTENT ) return;
 
-  int_val = get_original(f_id);  /* for eval  of file parameters */
+  int_val = ordinal(get_original(f_id))+1;  /* for eval  of file parameters */
   b_id0 = get_facet_body(f_id);
   b_id1 = get_facet_body(facet_inverse(f_id));
   if ( !valid_id(b_id0) && !valid_id(b_id1) ) return;
@@ -377,6 +372,8 @@ facet_id f_id;
 *          Also fixed quantity gradients.
 */
 
+#define NEWTORVOL
+
 void film_grad_l()
 {
   body_id bi_id;  /* identifier for body i */
@@ -393,18 +390,60 @@ void film_grad_l()
   REAL *x[FACET_VERTS];  /* coordinates of vertices */
   REAL unwrap_x[FACET_VERTS][MAXCOORD];
 
+#ifdef NEWTORVOL
+  struct qinfo f_info; /* for calling q_facet_torus_volume */
+  if ( web.torus_flag )
+    q_info_init(&f_info,METHOD_GRADIENT);
+#endif
+
+
   if ( sym_flags & NEED_FORM_UNWRAPPING )
      kb_error(1035,"Have to do convert_to_quantities with this symmetry group.\n",RECOVERABLE);
 
   FOR_ALL_FACETS(f_id)
   { 
     WRAPTYPE wraps[3];
-    int_val = get_original(f_id);  /* for eval  of file parameters */
+    int_val = ordinal(get_original(f_id))+1;  /* for eval  of file parameters */
     bi_id = get_facet_body(f_id);
     bj_id = get_facet_body(facet_inverse(f_id));
+    if ( (!valid_id(bi_id) || !(get_battr(bi_id) & (FIXEDVOL|PRESSURE)) )
+     && (!valid_id(bj_id) || !(get_battr(bj_id) & (FIXEDVOL|PRESSURE)) ) )
+      continue;
 
     if ( web.torus_flag )
-    { /* kludge copy from torvol_project, so could ditch torvol_project */
+    { 
+#ifdef NEWTORVOL
+      int n;
+      vertex_id v_ids[FACET_EDGES];
+      
+      /* get basic info */ 
+      fe = get_facet_fe(f_id);
+      for ( i = 0 ; i < FACET_EDGES ; i ++ )
+      { v_ids[i] = get_fe_tailv(fe);
+        fe = get_next_edge(fe);
+      }
+
+      f_info.id = f_id;
+      q_facet_setup(NULL,&f_info,NEED_SIDE|TORUS_MODULO_MUNGE|ORIENTABLE_METHOD);
+      q_facet_torus_volume_grad(&f_info);
+
+      if ( valid_id(bi_id) && (get_battr(bi_id) & (FIXEDVOL|PRESSURE)) )
+        for ( i = 0 ; i < FACET_VERTS ; i++ )
+        { vgptr = get_bv_new_vgrad(get_body_fixnum(bi_id),v_ids[i]);
+          vgptr->bb_id = bi_id;
+          for ( n = 0 ; n < SDIM ; n++ )
+             vgptr->grad[n] += f_info.grad[i][n];
+        }
+
+       if ( valid_id(bj_id) && (get_battr(bj_id) & (FIXEDVOL|PRESSURE)) )
+        for ( i = 0 ; i < FACET_VERTS ; i++ )
+        { vgptr = get_bv_new_vgrad(get_body_fixnum(bj_id),v_ids[i]);
+          vgptr->bb_id = bj_id;
+          for ( n = 0 ; n < SDIM ; n++ )
+            vgptr->grad[n] -= f_info.grad[i][n];
+        }
+#else
+      /* kludge copy from torvol_project, so could ditch torvol_project */
       REAL *v[FACET_VERTS];  /* pointers to three vertex coordinates */
       int j;
       REAL adj[FACET_EDGES][MAXCOORD];  /* torus wrap adjustments for edge */ 
@@ -412,6 +451,7 @@ void film_grad_l()
       int n;
       REAL g[MAXCOORD];
 
+
       /* get basic info */
       fe = get_facet_fe(f_id);
       for ( i = 0 ; i < FACET_EDGES ; i ++ )
@@ -426,6 +466,7 @@ void film_grad_l()
       {
          int m;
          REAL ga[MAXCOORD],gb[MAXCOORD],gc[MAXCOORD],gd[MAXCOORD],
+
             ge[MAXCOORD],gf[MAXCOORD]; /* gradient parts */
          j = (i+1)%FACET_EDGES;
          k = (i+2)%FACET_EDGES;
@@ -463,6 +504,7 @@ void film_grad_l()
               vgptr->grad[n] -= g[n];
           }
       }
+#endif
     }
     else
     {
@@ -562,6 +604,11 @@ void film_grad_l()
     } /* end torus/nontorus */
 
   } /* end facet loop */
+  
+#ifdef NEWTORVOL
+  if ( web.torus_flag )
+    q_info_free(&f_info);
+#endif
 } /* end film_grad_l() */
 
 /*******************************************************************
@@ -602,7 +649,7 @@ void film_constr_grad()
     tailv = get_edge_tailv(e_id);
     tcoord = get_coord(tailv);
     hcoord = get_coord(headv);
-    int_val = get_original(e_id);  /* for eval  of file parameters */
+    int_val = ordinal(get_original(e_id))+1;  /* for eval  of file parameters */
     for ( j = 0 ; j < SDIM ; j++ )
          side[j] = hcoord[j] - tcoord[j];
     if ( !(attr & BDRY_CONTENT) ) continue;
