@@ -194,7 +194,7 @@ skip_from_metric:
 
   if ( mode == AREA_ONLY ) return;
 
-  web.total_area    = web.total_area + energy;
+  binary_tree_add(web.total_area_addends,energy);
      
   /* apportion area to vertices and midpoints to scale motion */
   /* Note apportionment ratios depend on equivalent test        */
@@ -520,10 +520,9 @@ struct qinfo *f_info;
       proc_total_area[GET_THREAD_ID] += value;
      else
 #endif
-     web.total_area += value;
+     binary_tree_add(web.total_area_addends,value);
   }
-  if ( basic_gen_methods[METH_INSTANCE(f_info->method)->gen_method].spec_flags 
-                & SPEC_USE_DENSITY )
+  if ( METH_INSTANCE(f_info->method)->flags & USE_DENSITY )
       value *= get_facet_density(f_info->id);
   return value;
 
@@ -546,8 +545,7 @@ struct qinfo *f_info;
   REAL st,ss,tt;
   REAL density,fudge;
 
-  if ( basic_gen_methods[METH_INSTANCE(f_info->method)->gen_method].spec_flags 
-                & SPEC_USE_DENSITY )
+  if ( METH_INSTANCE(f_info->method)->flags & USE_DENSITY )
       density = get_facet_density(f_info->id);
   else density = 1.0;
 
@@ -600,8 +598,7 @@ struct qinfo *f_info;
   MAT4D(aa,FACET_CTRL,MAXCOORD,2,2);
   REAL density,fudge;
 
-  if ( basic_gen_methods[METH_INSTANCE(f_info->method)->gen_method].spec_flags 
-                & SPEC_USE_DENSITY )
+  if ( METH_INSTANCE(f_info->method)->flags & USE_DENSITY )
       density = get_facet_density(f_info->id);
   else density = 1.0;
 
@@ -697,11 +694,10 @@ struct qinfo *f_info;
       proc_total_area[GET_THREAD_ID] += value;
      else
 #endif
-     web.total_area += value;
+     binary_tree_add(web.total_area_addends,value);
   }
 
-  if ( basic_gen_methods[METH_INSTANCE(f_info->method)->gen_method].spec_flags 
-                & SPEC_USE_DENSITY )
+  if ( METH_INSTANCE(f_info->method)->flags & USE_DENSITY )
       value *= get_facet_density(f_info->id);
 
   return value; 
@@ -725,8 +721,7 @@ struct qinfo *f_info;
   REAL detgrad[FACET_CTRL][MAXCOORD];
   REAL density = 1.0;
 
-  if ( basic_gen_methods[METH_INSTANCE(f_info->method)->gen_method].spec_flags 
-                & SPEC_USE_DENSITY )
+  if ( METH_INSTANCE(f_info->method)->flags & USE_DENSITY )
       density = get_facet_density(f_info->id);
 
   for ( k = 0 ; k < FACET_CTRL ; k++ )
@@ -781,8 +776,7 @@ struct qinfo *f_info;
   REAL dethess[FACET_CTRL][FACET_CTRL][MAXCOORD][MAXCOORD];
   REAL density = 1.0;
 
-  if ( basic_gen_methods[METH_INSTANCE(f_info->method)->gen_method].spec_flags 
-                & SPEC_USE_DENSITY )
+  if ( METH_INSTANCE(f_info->method)->flags & USE_DENSITY )
       density = get_facet_density(f_info->id);
      
   for ( k = 0 ; k < FACET_CTRL ; k++ )
