@@ -19,7 +19,6 @@ static  int maxquants;  /* total number of constraints */
 REAL *vpressures = NULL; /* so global pressures not messed */
 static  int fixcount = 0;  /* number of constrained quantities */
 /* for numbering various types of constraints */
-static  int bodystart = 0;
 static  int gen_quant_start;
 static  struct linsys LS;  /* for sparse constraints */
 
@@ -28,7 +27,44 @@ REAL **vgef = NULL;  /* form vol grads, for approx curvature */
 
 /*************************************************************************
 *
-*  Function:  calc_volgrads()
+* Function: calc_volgrads()
+*
+* purpose: wrapper for local_calc_volgrads()
+*
+*/
+void calc_volgrads(mode)
+int mode; 
+{
+   if ( itdebug)
+    outstring("Calculating volgrads.\n");
+ 
+   
+  #ifdef MPI_EVOLVER
+   mpi_calc_volgrads(mode);
+  #else
+   local_calc_volgrads(mode);
+  #endif
+
+#ifdef _DEBUGXXX
+  if ( itdebug ) /* dump vgrads */
+  { vertex_id v_id;
+    FOR_ALL_VERTICES(v_id)
+    { volgrad  *vgptr;
+
+      vgptr = get_vertex_vgrad(v_id);
+      while ( vgptr )
+      { printf("v %3s   b %3s  grad %f %f %f\n",ELNAME(v_id),
+         ELNAME1(vgptr->bb_id),vgptr->grad[0],vgptr->grad[1],vgptr->grad[2]);
+        vgptr = vgptr->chain;
+      }
+    }
+  }
+#endif
+}
+
+/*************************************************************************
+*
+*  Function:  local_calc_volgrads()
 *
 *  purpose: calculate gradients of constrained volumes and quantities.
 *          Leaves them in vgptr... array.
@@ -36,7 +72,7 @@ REAL **vgef = NULL;  /* form vol grads, for approx curvature */
 *          Also does variable_parameter constraint gradients.
 */
 
-void calc_volgrads(mode)
+void local_calc_volgrads(mode)
 int mode; /* DO_OPTS or NO_OPTS */
 {
   body_id bi_id;  /* identifier for body i */
@@ -46,8 +82,7 @@ int mode; /* DO_OPTS or NO_OPTS */
   int qfixed = 0;
   struct gen_quant *gq;
   MAT2D(a,MAXCOORD,MAXCOORD);
-
-
+ 
   /* for numbering various types of constraints */
    gen_quant_start = web.skel[BODY].max_ord + 1;
  
@@ -63,8 +98,9 @@ int mode; /* DO_OPTS or NO_OPTS */
        { fixed_constraint_flag = 1; qfixed++;
          gq->vol_number = gen_quant_start + k++;
        }
-       else if ( web.pressure_flag )
-       gq->vol_number = gen_quant_start + k++;
+       else 
+         if ( web.pressure_flag )
+           gq->vol_number = gen_quant_start + k++;
      }
   }
   maxquants = gen_quant_start + k;
@@ -99,7 +135,7 @@ int mode; /* DO_OPTS or NO_OPTS */
      if ( !web.pressure_flag && !everything_quantities_flag )
       FOR_ALL_BODIES(bi_id)
         if ( get_battr(bi_id) & FIXEDVOL ) 
-          convalues[ordinal(bi_id)][0] = get_body_volume(bi_id);
+          convalues[loc_ordinal(bi_id)][0] = get_body_volume(bi_id);
      for ( k = n = 0 ; n < gen_quant_count ; n++ )
      { gq = GEN_QUANT(n);
        if ( gq->flags & Q_FIXED )
@@ -119,7 +155,7 @@ int mode; /* DO_OPTS or NO_OPTS */
        if (!web.pressure_flag &&  !everything_quantities_flag )
          FOR_ALL_BODIES(bi_id)
            if ( get_battr(bi_id) & FIXEDVOL ) 
-             convalues[ordinal(bi_id)][1] = get_body_volume(bi_id);
+             convalues[loc_ordinal(bi_id)][1] = get_body_volume(bi_id);
        for ( k = n = 0 ; n < gen_quant_count ; n++ )
        { gq = GEN_QUANT(n);
          if ( gq->flags & Q_FIXED )
@@ -137,7 +173,7 @@ int mode; /* DO_OPTS or NO_OPTS */
        if (!web.pressure_flag &&  !everything_quantities_flag )
          FOR_ALL_BODIES(bi_id)
            if ( get_battr(bi_id) & FIXEDVOL ) 
-             convalues[ordinal(bi_id)][2] = get_body_volume(bi_id);
+             convalues[loc_ordinal(bi_id)][2] = get_body_volume(bi_id);
        for ( k = n = 0 ; n < gen_quant_count ; n++ )
        { gq = GEN_QUANT(n);
          if ( gq->flags & Q_FIXED )
@@ -150,8 +186,9 @@ int mode; /* DO_OPTS or NO_OPTS */
        if (!web.pressure_flag &&  !everything_quantities_flag )
          FOR_ALL_BODIES(bi_id)
            if ( get_battr(bi_id) & FIXEDVOL ) 
-             optparam_congrads[i][ordinal(bi_id)] =
-             (convalues[ordinal(bi_id)][1]-convalues[ordinal(bi_id)][2])/2/dp;
+             optparam_congrads[i][loc_ordinal(bi_id)] =
+               (convalues[loc_ordinal(bi_id)][1] -
+                  convalues[loc_ordinal(bi_id)][2])/2/dp;
        for ( k = n = 0 ; n < gen_quant_count ; n++ )
        { gq = GEN_QUANT(n);
          if ( gq->flags & Q_FIXED )
@@ -164,7 +201,7 @@ int mode; /* DO_OPTS or NO_OPTS */
       if (!web.pressure_flag &&  !everything_quantities_flag )
       FOR_ALL_BODIES(bi_id)
         if ( get_battr(bi_id) & FIXEDVOL ) 
-           set_body_volume(bi_id, convalues[ordinal(bi_id)][0]);
+           set_body_volume(bi_id, convalues[loc_ordinal(bi_id)][0],SETSTAMP);
       for ( k = n = 0 ; n < gen_quant_count ; n++ )
       { gq = GEN_QUANT(n);
         if ( gq->flags & Q_FIXED )
@@ -310,7 +347,7 @@ int mode; /* CALC_FORCE and/or CALC_VOLGRADS */
   FOR_ALL_VERTICES(v_id)
   {
     volgrad *vgptr;
-    int ord = ordinal(v_id);
+    int ord = loc_ordinal(v_id);
     REAL *f;
     conmap_t * conmap = get_v_constraint_map(v_id);
     int oncount = 0;
@@ -380,7 +417,7 @@ int mode; /* CALC_FORCE and/or CALC_VOLGRADS */
 */
 
 int find_fixed()
-{ int i;
+{ int i,k;
   struct gen_quant *q;
   int redundant_bi = -1; /* for torus_filled */
   body_id b_id;
@@ -388,7 +425,7 @@ int find_fixed()
   /* figure out which quantities are fixed */
   fixcount = 0;
   if ( !everything_quantities_flag )
-    FOR_ALL_BODIES(b_id)
+    MFOR_ALL_BODIES(b_id)
       { if ( get_battr(b_id) & (FIXEDVOL|PRESSURE) ) 
         { if ( web.full_flag && (redundant_bi < 0) )
           { set_body_fixnum(b_id,-1);
@@ -401,7 +438,7 @@ int find_fixed()
         } 
         else set_body_fixnum(b_id,-1);
       }
-   for ( i = 0 ; i < gen_quant_count ; i++ )
+   for ( i = 0,k=0 ; i < gen_quant_count ; i++ )
    { q = GEN_QUANT(i);
       if ( q->flags & (Q_FIXED|Q_CONSERVED) )
       { if ( web.full_flag && valid_id(q->b_id) && redundant_bi < 0 )
@@ -409,10 +446,15 @@ int find_fixed()
           redundant_bi = 1;
         }
         else
-        q->fixnum = fixcount++;
+        { q->fixnum = fixcount++;
+          k++;
+        }
       }
       else q->fixnum = -1;
    }
+  gen_quant_start = web.skel[BODY].max_ord + 1;
+  maxquants = gen_quant_start + k;
+  fixed_constraint_flag = fixcount;
   return fixcount;
 }
 
@@ -422,60 +464,32 @@ int find_fixed()
 *
 * purpose: set up DV^T DV
 */
+static int degfree; /* for check there is sufficient degrees of freedom */
+static int calc_leftside_hash_count; /* for estimating need */
 
 void calc_leftside()
 { int i,j,k;
   int bi,bj;
-  vertex_id v_id;
+
+  degfree = 0;
 
   /* set up matrices for DV^T DV */
   if ( sparse_constraints_flag )
   { memset(&LS,0,sizeof(LS));
-    sp_hash_init(&LS,0);
+    sp_hash_init(&LS,calc_leftside_hash_count);
   }
   else
   {
     rleftside = dmatrix(0,fixcount,0,fixcount);
   }
 
-  /* generate  DV^T DV */
-  if ( !approx_curve_flag )
-    FOR_ALL_VERTICES(v_id)
-    {
-      volgrad *vgptri,*vgptrj;
-      ATTR attr = get_vattr(v_id);
-
-      if ( attr & FIXED ) continue;
-
-      for ( vgptri = get_vertex_vgrad(v_id); vgptri ; vgptri = vgptri->chain )
-      { REAL tmp;
-        bi = vgptri->fixnum;
-        if ( (bi < 0) || (bi >= fixcount) ) continue;
-        tmp = SDIM_dot(vgptri->velocity,vgptri->grad);
-/*      if ( !(attr & HIT_WALL) )    */
-        { if ( sparse_constraints_flag )
-          { sp_hash_search(&LS,bi,bi,tmp); }
-          else
-            rleftside[bi][bi] += tmp;
-        }
-        for ( vgptrj = vgptri->chain ; vgptrj ; vgptrj = vgptrj->chain )
-        { tmp = SDIM_dot(vgptri->grad,vgptrj->velocity);
-          bj = vgptrj->fixnum;
-          if ( (bj < 0) || (bj >= fixcount) ) continue;
-/*        if ( !(attr & HIT_WALL) ) */
-          {
-            if ( sparse_constraints_flag )
-            { if ( bi < bj ) sp_hash_search(&LS,bi,bj,tmp);
-              else sp_hash_search(&LS,bj,bi,tmp);
-            }
-            else
-            { rleftside[bi][bj] += tmp;
-              rleftside[bj][bi] += tmp;
-            }
-          }
-        }
-      }
-    }
+  #ifdef MPI_EVOLVER
+  if ( sparse_constraints_flag ) 
+    kb_error(3381,"Cannot do sparse constraints yet in MPI.\n",RECOVERABLE);
+  mpi_calc_leftside(fixcount);
+  #else
+  local_calc_leftside();
+  #endif
 
   if ( approx_curve_flag )
   { int NV = SDIM*(1+web.skel[VERTEX].max_ord);
@@ -505,7 +519,7 @@ void calc_leftside()
          }
        }
      }
-
+     degfree += NV;
   }  /* end approx_curve_flag */
 
   /* variable_parameter contributions */
@@ -531,20 +545,23 @@ void calc_leftside()
       }
     }
   }
- 
-#ifdef xxx
-printf("Rleftside:\n");
-for ( i = 0  ; i < fixcount ; i++ )
-{ for ( j = 0 ; j < fixcount ; j++ )
-    printf("%9.6f ",rleftside[i][j]);
-  printf("\n");
-}
-#endif
-
+  degfree += optparamcount;
+  
+  #ifndef MPI_EVOLVER
+  if ( degfree < fixcount ) 
+  { sprintf(errmsg,
+       "Degrees of freedom, %d, is less than number of constraints, %d\n",
+       degfree,fixcount);
+    if ( degfree == 0 )
+      strcat(errmsg,"Perhaps constraint is not applied to any elements?\n");
+    kb_error(3004,errmsg,RECOVERABLE);
+  }
+  #endif
+  
   /* solve for coefficients */
   if ( sparse_constraints_flag )
   { REAL old_hessian_epsilon = hessian_epsilon;
-    sp_hash_end(&LS,fixcount,fixcount,A_OFF);
+    calc_leftside_hash_count = sp_hash_end(&LS,fixcount,fixcount,A_OFF);
     LS.N = fixcount; 
     hessian_epsilon = 0.0;
     ysmp_factor(&LS);  /* since mindeg uses vertex info at the moment */
@@ -562,6 +579,64 @@ for ( i = 0  ; i < fixcount ; i++ )
 
 } /* end calc_leftside() */
 
+/*************************************************************************
+*
+* function: local_calc_leftside()
+*
+* purpose: Do local dot products of constraint gradients
+*/
+
+void local_calc_leftside()
+{ vertex_id v_id;
+  int bi,bj;
+  
+  /* generate  DV^T DV */
+  if ( !approx_curve_flag )
+    FOR_ALL_VERTICES(v_id)
+    {
+      volgrad *vgptri,*vgptrj;
+      ATTR attr = get_vattr(v_id);
+
+      if ( attr & FIXED ) continue;
+
+      for ( vgptri = get_vertex_vgrad(v_id); vgptri ; vgptri = vgptri->chain )
+      { REAL tmp;
+        bi = vgptri->fixnum;
+        if ( (bi < 0) || (bi >= fixcount) ) continue;
+        degfree++;
+        tmp = SDIM_dot(vgptri->velocity,vgptri->grad);
+/*        
+       if ( itdebug )
+         printf("%s (%f,%f,%f)*(%f,%f,%f)=%f\n",ELNAME(v_id),
+            vgptri->grad[0],vgptri->grad[1],vgptri->grad[2],
+           vgptri->velocity[0],vgptri->velocity[1],vgptri->velocity[2]);
+  */         
+ /*      if ( !(attr & HIT_WALL) )    */
+        { if ( sparse_constraints_flag )
+          { sp_hash_search(&LS,bi,bi,tmp); }
+          else
+            rleftside[bi][bi] += tmp;
+        }
+        for ( vgptrj = vgptri->chain ; vgptrj ; vgptrj = vgptrj->chain )
+        { tmp = SDIM_dot(vgptri->grad,vgptrj->velocity);
+          bj = vgptrj->fixnum;
+          if ( (bj < 0) || (bj >= fixcount) ) continue;
+/*        if ( !(attr & HIT_WALL) ) */
+          {
+            if ( sparse_constraints_flag )
+            { if ( bi < bj ) sp_hash_search(&LS,bi,bj,tmp);
+              else sp_hash_search(&LS,bj,bi,tmp);
+            }
+            else
+            { rleftside[bi][bj] += tmp;
+              rleftside[bj][bi] += tmp;
+            }
+          }
+        }
+      }
+    }
+}
+
 /************************************************************************
 *
 * function: calc_lagrange()
@@ -573,17 +648,86 @@ for ( i = 0  ; i < fixcount ; i++ )
 void calc_lagrange()
 { int i,k;
   body_id b_id;
-  int bi;
   struct gen_quant *gq;
-  vertex_id v_id;
 
-  if ( !fixed_constraint_flag ) return;
+  if ( fixcount == 0 ) return;
 
   calc_leftside();  /* set up rleftside inverse */
 
   rightside = (REAL*)temp_calloc(maxquants,sizeof(REAL));
   vpressures = (REAL*)temp_calloc(maxquants,sizeof(REAL));
 
+  #ifdef MPI_EVOLVER
+  mpi_calc_rightside(maxquants);
+  #else
+  local_calc_rightside();
+  #endif
+
+  /* optimizing_parameter contributions */
+  if ( optparamcount )
+  for ( i = 0 ; i < gen_quant_count ; i++ )
+  { int fixi = GEN_QUANT(i)->fixnum;
+    int voli = GEN_QUANT(i)->vol_number;
+    if ( fixi < 0 ) continue;
+    for ( k = 0 ; k < optparamcount ; k++ )
+      rightside[fixi] -= optparam_congrads[k][voli]*optparam[k].velocity;
+      /* negative since using grad, not force */ 
+  }
+
+  /* solve for coefficients */
+  if ( sparse_constraints_flag )
+  { ysmp_solve(&LS,rightside,vpressures);
+    free_system(&LS);
+  }
+  else
+/*
+    matvec_mul(rleftside,rightside,vpressures,fixcount,fixcount);
+*/
+    LD_solve(rleftside,rightside,vpressures,fixcount);
+
+  /* check for singular matrix */
+  for ( i = 0 ; i < fixcount ; i++ )
+  { if ( !is_finite(vpressures[i]) )
+    { sprintf(errmsg,"Constraint adjustment matrix singular. \nMore constraints than degrees of freedom?\nConstraints with no elements?");
+      kb_error(3005,errmsg,RECOVERABLE);
+    }
+  }
+  /* install pressures into body structures */
+  if ( !web.pressure_flag )
+    MFOR_ALL_BODIES(b_id)
+    { if ( get_battr(b_id) & FIXEDVOL )
+      { 
+        if ( everything_quantities_flag )
+        { int fixi;
+          gq = GEN_QUANT(get_body_volquant(b_id));
+          fixi = gq->fixnum;
+          set_body_pressure(b_id,(fixi<0)?0.0 : -vpressures[fixi]);
+        }
+        else
+        { int fixi = get_body_fixnum(b_id);
+          set_body_pressure(b_id,(fixi<0)?0.0:-vpressures[fixi]);
+        }
+      }
+    }
+    for ( k = 0 ; k < gen_quant_count ; k++ )
+    { gq = GEN_QUANT(k);
+      if ( gq->flags & (Q_FIXED|Q_CONSERVED) )
+        if ( !valid_id(gq->b_id) || !web.pressure_flag )
+          gq->pressure = -vpressures[gq->fixnum];
+    }
+
+} /* end calc_lagrange() */
+
+/**************************************************************************
+*
+* function: local_calc_rightside()
+*
+* purpose: local part of calculation of Lagrange multiplier right side.
+*/
+void local_calc_rightside()
+{ int bi;
+  vertex_id v_id;
+
   /* generate right side of matrix equation */
   if ( !approx_curve_flag )
     FOR_ALL_VERTICES(v_id)
@@ -625,76 +769,61 @@ void calc_lagrange()
           if (valid_id(vgptri->bb_id) &&!web.pressure_flag &&  !everything_quantities_flag )
             { if ( !(get_battr(vgptri->bb_id)&FIXEDVOL) ) continue;
             }
-          rightside[bi] += SDIM_dot(f,&vgef[bi][SDIM*ordinal(v_id)]);
+          rightside[bi] += SDIM_dot(f,&vgef[bi][SDIM*loc_ordinal(v_id)]);
         }
       }
   }  /* end approx_curve_flag */
+}
 
-  /* optimizing_parameter contributions */
-  if ( optparamcount )
-  for ( i = 0 ; i < gen_quant_count ; i++ )
-  { int fixi = GEN_QUANT(i)->fixnum;
-    int voli = GEN_QUANT(i)->vol_number;
-    if ( fixi < 0 ) continue;
-    for ( k = 0 ; k < optparamcount ; k++ )
-      rightside[fixi] -= optparam_congrads[k][voli]*optparam[k].velocity;
-      /* negative since using grad, not force */ 
-  }
-
-  /* solve for coefficients */
-  if ( sparse_constraints_flag )
-    ysmp_solve(&LS,rightside,vpressures);
-  else
-/*
-    matvec_mul(rleftside,rightside,vpressures,fixcount,fixcount);
+/*************************************************************************
+*
+* function: lagrange_adjust()
+*
+* purpose: wrapper for local_lagrange_adjust()
 */
-    LD_solve(rleftside,rightside,vpressures,fixcount);
+void lagrange_adjust()
+{ int i,k;
 
-  /* install pressures into body structures */
-  if ( !web.pressure_flag )
-    FOR_ALL_BODIES(b_id)
-    { if ( get_battr(b_id) & FIXEDVOL )
-      { 
-        if ( everything_quantities_flag )
-        { int fixi;
-          gq = GEN_QUANT(get_body_volquant(b_id));
-          fixi = gq->fixnum;
-          set_body_pressure(b_id,(fixi<0)?0.0 : -vpressures[fixi]);
-        }
-        else
-        { int fixi = get_body_fixnum(b_id);
-          set_body_pressure(b_id,(fixi<0)?0.0:-vpressures[fixi]);
-        }
+  if ( !fixed_constraint_flag ) return;
+
+  #ifdef MPI_EVOLVER
+  mpi_lagrange_adjust(maxquants);
+  #else
+  local_lagrange_adjust();
+  #endif
+
+  /* optimizing parameter adjust */
+  if ( optparamcount )
+    for ( i = 0 ; i < optparamcount ; i++ )
+      for ( k = 0 ; k < gen_quant_count ; k++ )
+      { int fixi = GEN_QUANT(k)->fixnum;
+        int volk = GEN_QUANT(k)->vol_number;
+        if ( fixi >= 0 )
+          optparam[i].velocity += vpressures[fixi]*
+              globals(optparam[i].pnum)->attr.varstuff.pscale*
+                 optparam_congrads[i][volk];
       }
-    }
-    for ( k = 0 ; k < gen_quant_count ; k++ )
-    { gq = GEN_QUANT(k);
-      if ( gq->flags & (Q_FIXED|Q_CONSERVED) )
-        if ( !valid_id(gq->b_id) || !web.pressure_flag )
-          gq->pressure = -vpressures[gq->fixnum];
-    }
 
-} /* end calc_lagrange() */
+} /* end lagrange_adjust() */
 
 /*********************************************************************
 *
-* function: lagrange_adjust()
+* function: local_lagrange_adjust()
 *
 * purpose: adjust forces by Lagrange multipliers of constraint grads.
 *
 */
 
-void lagrange_adjust()
+void local_lagrange_adjust()
 { vertex_id v_id;
-  int i,k;
+  int k;
   
-  if ( !fixed_constraint_flag ) return;
   /* subtract multiples of volume gradients from force */
 
   FOR_ALL_VERTICES(v_id)
   {
     volgrad *vgptr;
-    int ord = ordinal(v_id);
+    int ord = loc_ordinal(v_id);
     REAL *f;
     int bi; /* row number for constraint */
 
@@ -703,6 +832,7 @@ void lagrange_adjust()
     for ( vgptr = get_vertex_vgrad(v_id) ; vgptr ; vgptr = vgptr->chain )
     { bi = vgptr->fixnum;
       if ( (bi < 0) || (bi >= fixcount) ) continue;
+
       if ( approx_curve_flag )
          for ( k = 0 ; k < SDIM ; k++ )
             f[k] -= vpressures[bi]*vgev[bi][SDIM*ord +k];
@@ -712,18 +842,7 @@ void lagrange_adjust()
     }
   }
 
-  /* optimizing parameter adjust */
-  if ( optparamcount )
-    for ( i = 0 ; i < optparamcount ; i++ )
-      for ( k = 0 ; k < gen_quant_count ; k++ )
-      { int fixi = GEN_QUANT(k)->fixnum;
-        int volk = GEN_QUANT(k)->vol_number;
-        if ( fixi >= 0 )
-          optparam[i].velocity += vpressures[fixi]
-              *globals(optparam[i].pnum)->attr.varstuff.pscale*optparam_congrads[i][volk];
-      }
-
-} /* end lagrange_adjust() */
+} /* end local_lagrange_adjust() */
         
 /*******************************************************************
 *
@@ -737,11 +856,9 @@ void lagrange_adjust()
 void volume_restore(stepsize,mode)
 REAL stepsize; /* multiplier for motion; usually 1 */
 int mode;/* TEST_MOVE or ACTUAL_MOVE */
-{ int bi;
-  body_id bi_id;
+{ body_id bi_id;
   struct gen_quant *gq;
   int i,k;
-  vertex_id v_id;
   int fixi;
 
   if ( !fixed_constraint_flag ) return;
@@ -753,7 +870,6 @@ int mode;/* TEST_MOVE or ACTUAL_MOVE */
   if ( !web.pressure_flag && !everything_quantities_flag )
     FOR_ALL_BODIES(bi_id)
      {
-       bi =  bodystart + ordinal(bi_id);
        if ( !(get_battr(bi_id) & FIXEDVOL) ) continue;
        fixi = get_body_fixnum(bi_id);
        if ( fixi < 0 ) continue;
@@ -772,6 +888,7 @@ int mode;/* TEST_MOVE or ACTUAL_MOVE */
   if ( sparse_constraints_flag )
   { if ( LS.A == NULL ) calc_leftside(); 
     ysmp_solve(&LS,vol_deficit,vol_restore);
+    free_system(&LS);
   }
   else
   { if ( rleftside == NULL ) calc_leftside();
@@ -797,11 +914,37 @@ int mode;/* TEST_MOVE or ACTUAL_MOVE */
             *globals(optparam[i].pnum)->attr.varstuff.pscale*optparam_congrads[i][volk];
      }
 
+  #ifdef MPI_EVOLVER
+  mpi_volume_restore(maxquants,stepsize,mode);
+  #else
+  local_volume_restore(stepsize,mode);
+  #endif
+
+  partner_move();  /* in case doing partners */
+  
+  if ( vol_deficit ) temp_free((char *)vol_deficit);  vol_deficit = NULL;
+  if ( vol_restore ) temp_free((char *)vol_restore);  vol_restore = NULL;
+
+} /* end volume_restore() */
+
+  
+/***************************************************************************
+*
+* function: local_volume_restore()
+*
+* purpose: apply volume restoration to local part of surface
+*/
+void local_volume_restore(stepsize,mode)
+REAL stepsize;
+int mode;
+{ vertex_id v_id;
+  int bi,i,k;
+  
   /* vertices */
   FOR_ALL_VERTICES(v_id)
   { REAL *x;
     volgrad *vgptr;
-    int ord = ordinal(v_id);
+    int ord = loc_ordinal(v_id);
     int attr = get_vattr(v_id);
 
     x = get_coord(v_id);
@@ -813,11 +956,11 @@ int mode;/* TEST_MOVE or ACTUAL_MOVE */
         { bi = vgptr->fixnum;
           if ( ( bi < 0 ) || (bi >= fixcount) ) continue;
           if ( approx_curve_flag )
-          for ( k = 0 ; k < SDIM ; k++ )
-             p[k] += stepsize*vol_restore[bi]*vgev[bi][SDIM*ord+k];
+            for ( k = 0 ; k < SDIM ; k++ )
+              p[k] += stepsize*vol_restore[bi]*vgev[bi][SDIM*ord+k];
           else
-          for ( k = 0 ; k < SDIM ; k++ )
-             p[k] += stepsize*vol_restore[bi]*vgptr->velocity[k];
+            for ( k = 0 ; k < boundary->pcount ; k++ )
+              p[k] += stepsize*vol_restore[bi]*vgptr->velocity[k];
         }
       for ( i = 0 ; i < SDIM ; i++ )
         x[i] = eval(boundary->coordf[i],p,v_id,NULL);
@@ -833,19 +976,15 @@ int mode;/* TEST_MOVE or ACTUAL_MOVE */
           for ( k = 0 ; k < SDIM ; k++ )
              x[k] += stepsize*vol_restore[bi]*vgev[bi][SDIM*ord+k];
         else
-          for ( k = 0 ; k < SDIM ; k++ )
+        { for ( k = 0 ; k < SDIM ; k++ )
              x[k] += stepsize*vol_restore[bi]*vgptr->velocity[k];
+        }
+           
       }
       if ( attr & CONSTRAINT ) project_v_constr(v_id,mode);
     }
   }
-
-  if ( vol_deficit ) temp_free((char *)vol_deficit);  vol_deficit = NULL;
-  if ( vol_restore ) temp_free((char *)vol_restore);  vol_restore = NULL;
-
-} /* end volume_restore() */
-
-  
+}
 
 /*******************************************************************
 *
@@ -855,13 +994,23 @@ int mode;/* TEST_MOVE or ACTUAL_MOVE */
 *
 */
 
+char *vgrad_attr_name = "__vgrad_head";
+int vgrad_attr;
 
 void vgrad_init(qfixed)
 int qfixed; /* how many fixed quantities (aside from body volumes)*/
-{
+{ int one = 1;
+  vertex_id v_id;
+  int i;
+  
+  vgrad_end();  /* take care of any leftovers */
+  
   /* allocate chain start for each vertex */
-  vgptrbase = (volgrad **)temp_calloc(web.skel[VERTEX].maxcount,
-                                    sizeof(volgrad *));
+  vgrad_attr = find_attribute(VERTEX,vgrad_attr_name);
+  if ( vgrad_attr < 0 )
+    vgrad_attr = add_attribute(VERTEX,vgrad_attr_name,PTR_TYPE,0,&one,0,NULL);
+  MFOR_ALL_VERTICES(v_id)
+    VPTR(v_id,vgrad_attr)[0] = NULL;
 
   /* allocate initial block of structures, using info from last time
      around, plus a little margin for growth. 
@@ -870,6 +1019,12 @@ int qfixed; /* how many fixed quantities (aside from body volumes)*/
   vgradbase->max = vgradlastused ? (vgradlastused + vgradlastused/10) :
                      web.skel[VERTEX].count;
   vgradbase->base = (volgrad*)temp_calloc(vgradbase->max,sizeof(volgrad));
+  vgradbase->values = (REAL*)temp_calloc(vgradbase->max,3*SDIM*sizeof(REAL));
+  for ( i = 0 ; i < vgradbase->max ; i++ )
+  { vgradbase->base[i].grad = vgradbase->values + 3*i*SDIM;
+    vgradbase->base[i].normal = vgradbase->values + (3*i+1)*SDIM;
+    vgradbase->base[i].velocity = vgradbase->values + (3*i+2)*SDIM;
+  }
   vgradbase->top = 0;
   vgradbase->next = NULL;
   vgradtop = 0;
@@ -889,12 +1044,13 @@ void vgrad_end()
 
   vgradlastused = vgradtop;
   for ( block = vgradbase ; block ; block = next )
-  { temp_free((char *)block->base); 
+  { temp_free((char *)block->base);
+    temp_free((char *)block->values);
     next = block->next;
     temp_free((char *)block);
   }
   vgradbase = NULL; 
-  if ( vgptrbase ) { temp_free((char *)vgptrbase); vgptrbase = NULL; }
+  
   vgradmax = 0;
   if ( sparse_constraints_flag )
   { free_system(&LS);
@@ -906,6 +1062,12 @@ void vgrad_end()
   if ( vpressures ) temp_free((char *)vpressures); vpressures = NULL;
   if ( optparam_congrads ) free_matrix(optparam_congrads);
   optparam_congrads = NULL;
+
+  #ifdef MPI_EVOLVER
+  if ( this_machine == 0 )
+    mpi_vgrad_end(); 
+  #endif
+
 } /* end vgrad_end() */
 
 /********************************************************************
@@ -917,8 +1079,22 @@ void vgrad_end()
 
 volgrad *get_vertex_vgrad(v_id)
 vertex_id v_id;
+{ if ( !vgradbase ) return NULL;
+  return (volgrad*)VPTR(v_id,vgrad_attr)[0];
+}
+
+/********************************************************************
+*
+* function: set_vertex_vgrad()
+*
+* purpose: set pointer to first vgrad structure in vertex's chain
+*/
+
+void set_vertex_vgrad(v_id,vgptr)
+vertex_id v_id;
+volgrad *vgptr;
 {
-  return vgptrbase ? vgptrbase[ordinal(v_id)] : NULL;
+  VPTR(v_id,vgrad_attr)[0] = (char*)vgptr;
 }
 
 /**********************************************************************
@@ -937,10 +1113,17 @@ struct volgrad * new_vgrad()
 
 LOCK_WEB;
   if ( vgradbase->top >= vgradbase->max ) 
-  { /* need new block */
+  { int i;
+    /* need new block */
     newblock = (struct vgradblock *)temp_calloc(1,sizeof(struct vgradblock));
     newblock->max = vgradtop/3+100;
     newblock->base = (volgrad*)temp_calloc(newblock->max,sizeof(volgrad));
+    newblock->values = (REAL*)temp_calloc(newblock->max,3*SDIM*sizeof(REAL));
+    for ( i = 0 ; i < newblock->max ; i++ )
+    { newblock->base[i].grad = newblock->values + 3*i*SDIM;
+      newblock->base[i].normal = newblock->values + (3*i+1)*SDIM;
+      newblock->base[i].velocity = newblock->values + (3*i+2)*SDIM;
+    }
     newblock->top = 0;
     newblock->next = vgradbase;
     vgradbase = newblock;
@@ -989,7 +1172,8 @@ vertex_id v_id;
 {
   volgrad  **ptrptr;  /* pointer to pointer so can update if need be */
 
-  ptrptr = vgptrbase + ordinal(v_id);
+  ptrptr = (volgrad**)VPTR(v_id,vgrad_attr);
+
   while ( *ptrptr )
     if ( (*ptrptr)->fixnum == fixnum ) return *ptrptr;
     else ptrptr = &(*ptrptr)->chain;
@@ -1024,14 +1208,14 @@ int mode; /* bits for CALC_FORCE and CALC_VOLGRADS */
     /* each coordinate gives a right side */
     FOR_ALL_VERTICES(v_id)
       { 
-        int vnum = SDIM*ordinal(v_id);
+        int vnum = SDIM*loc_ordinal(v_id);
         for ( j = 0 ; j < SDIM ; j++ )
            B[vnum+j] = get_force(v_id)[j];
       }
     mobility_mult(B);
     FOR_ALL_VERTICES(v_id)
       { REAL *vel = get_velocity(v_id);
-        int vnum = SDIM*ordinal(v_id);
+        int vnum = SDIM*loc_ordinal(v_id);
         for ( j = 0 ; j < SDIM ; j++ )
           vel[j] = B[vnum+j];
       }
@@ -1051,7 +1235,7 @@ int mode; /* bits for CALC_FORCE and CALC_VOLGRADS */
      {
         volgrad *vgptri;
         ATTR attr = get_vattr(v_id);
-        int ord = ordinal(v_id);
+        int ord = loc_ordinal(v_id);
 
         if ( attr & FIXED ) continue;
 
