@@ -201,7 +201,7 @@ void sqgauss_energy()
         for ( i = 0 ; i < FACET_VERTS ; i++ )
         { c = -st[i]/sqrt(ss[i]*ss[(i+2)%3]);
           angle = acos(c);
-          gv = gverts + ordinal(v[i]);
+          gv = gverts + loc_ordinal(v[i]);
           gv->angle += angle;
           gv->area  += area/3;
           gv->star_area  += area/(3-fixcount);
@@ -210,10 +210,10 @@ void sqgauss_energy()
   
   /* calc square gauss curvature at each vertex */
   FOR_ALL_VERTICES(v_id)
-    { struct gvert *vg = gverts + ordinal(v_id);
+    { struct gvert *vg = gverts + loc_ordinal(v_id);
       if ( get_vattr(v_id) & (FIXED|BOUNDARY) ) continue;
       gc = (wedge_angle(v_id) - vg->angle)/vg->area;
-      web.total_energy += modulus*gc*gc*vg->star_area;
+      binary_tree_add(web.total_energy_addends,modulus*gc*gc*vg->star_area);
     }
 
 
@@ -248,9 +248,10 @@ vertex_id v_id;
   /* two constraints, so find angle between */
 
   if ( concount > 2 ) 
-  { sprintf(errmsg,"gauss_curvature_integral: More than two constraints on vertex %d.\n",
-          ordinal(v_id)+1);
-     kb_error(1593,errmsg,RECOVERABLE);
+  { sprintf(errmsg,
+     "gauss_curvature_integral: More than two constraints on vertex %s.\n",
+          ELNAME(v_id));
+    kb_error(1593,errmsg,RECOVERABLE);
   }
   for ( j = 1, concount = 0 ; j <= (int)conmap[0] ; j++ ) 
      { 
@@ -328,9 +329,9 @@ void sqgauss_force()
           struct gvert *gv;
           int jj1,jj2;
 
-          gv = gverts + ordinal(v[i]);
-          ge1 = gedges + ordinal(e[i]);
-          ge2 = gedges + ordinal(e[i2]);
+          gv = gverts + loc_ordinal(v[i]);
+          ge1 = gedges + loc_ordinal(e[i]);
+          ge2 = gedges + loc_ordinal(e[i2]);
           jj1 =  inverted(e[i]) ? 1 : 0;
           jj2 =  inverted(e[i2]) ? 0 : 1;
 
@@ -367,10 +368,10 @@ void sqgauss_force()
   
   /* calc square gauss curvature at each vertex */
   FOR_ALL_VERTICES(v_id)
-    { struct gvert *gv = gverts + ordinal(v_id);
+    { struct gvert *gv = gverts + loc_ordinal(v_id);
       if ( get_vattr(v_id) & (FIXED|BOUNDARY) ) continue;
       gc = (wedge_angle(v_id) - gv->angle)/gv->area;
-      web.total_energy += modulus*gc*gc*gv->star_area;
+      binary_tree_add(web.total_energy_addends,modulus*gc*gc*gv->star_area);
       gv->gc = gc;
 
       /* now self terms in derivative */
@@ -383,9 +384,9 @@ void sqgauss_force()
   
   /* now cross terms from edges */
   FOR_ALL_EDGES(e_id)
-    { struct gvert *gv1 = gverts + ordinal(get_edge_tailv(e_id));
-      struct gvert *gv2 = gverts + ordinal(get_edge_headv(e_id));
-      struct gedge *ge = gedges + ordinal(e_id);
+    { struct gvert *gv1 = gverts + loc_ordinal(get_edge_tailv(e_id));
+      struct gvert *gv2 = gverts + loc_ordinal(get_edge_headv(e_id));
+      struct gedge *ge = gedges + loc_ordinal(e_id);
 
       /* head energy as function of tail */
       if ( !(get_vattr(get_edge_headv(e_id)) & (FIXED|BOUNDARY) ) )
@@ -481,7 +482,7 @@ struct method_instance *mi;
         for ( i = 0 ; i < FACET_VERTS ; i++ )
         { c = -st[i]/sqrt(ss[i]*ss[(i+2)%3]);
           angle = acos(c);
-          gv = gverts + ordinal(v[i]);
+          gv = gverts + loc_ordinal(v[i]);
           gv->angle += angle;
           gv->area  += area/3;
           gv->star_area  += area/(3-fixcount);
@@ -513,9 +514,9 @@ struct method_instance *mi;
           struct gedge *ge1,*ge2;
           int jj1,j2;
 
-          gv = gverts + ordinal(v[i]);
-          ge1 = gedges + ordinal(e[i]);
-          ge2 = gedges + ordinal(e[i2]);
+          gv = gverts + loc_ordinal(v[i]);
+          ge1 = gedges + loc_ordinal(e[i]);
+          ge2 = gedges + loc_ordinal(e[i2]);
           jj1 =  inverted(e[i]) ? 1 : 0;
           j2 =  inverted(e[i2]) ? 0 : 1;
 
@@ -580,7 +581,7 @@ struct qinfo *v_info;
   REAL gc;  /* gaussian curvarture */
 
   /* calc square gauss curvature at each vertex */
-  struct gvert *vg = gverts + ordinal(v_info->id);
+  struct gvert *vg = gverts + loc_ordinal(v_info->id);
   if ( get_vattr(v_info->id) & (FIXED|BOUNDARY) ) return 0.0;
   gc = (wedge_angle(v_info->id) - vg->angle)/vg->area;
   return modulus*gc*gc*vg->star_area;
@@ -607,7 +608,7 @@ struct qinfo *v_info;
 
   /* calc square gauss curvature at each vertex */
   v_id = v_info->id;
-  gv = gverts + ordinal(v_id);
+  gv = gverts + loc_ordinal(v_id);
   if ( get_vattr(v_id) & (FIXED|BOUNDARY) ) return 0.0;
   gc = (wedge_angle(v_id) - gv->angle)/gv->area;
   energy += modulus*gc*gc*gv->star_area;
@@ -624,8 +625,8 @@ struct qinfo *v_info;
   if ( !(get_vattr(v_id) & (FIXED|BOUNDARY) ) )
     do
     { 
-      struct gvert *gv2 = gverts + ordinal(get_edge_headv(e_id));
-      struct gedge *ge = gedges + ordinal(e_id);
+      struct gvert *gv2 = gverts + loc_ordinal(get_edge_headv(e_id));
+      struct gedge *ge = gedges + loc_ordinal(e_id);
 
       /* head energy as function of tail */
         for ( j = 0 ; j < SDIM ; j++ )
@@ -659,12 +660,13 @@ REAL star_gauss_method_value(v_info)
 struct qinfo *v_info;
 { REAL deficit = 2*M_PI;
   int k;
+  int fudge = (v_info->flags & INCOMPLETE_STAR) ? 1 : 0;
 
   if ( v_info->vcount <= 1 ) return 0.0;
 
-  for ( k = 1 ; k < v_info->vcount ; k++ )
+  for ( k = 1 ; k < v_info->vcount-fudge ; k++ )
   { REAL ss1,ss2,s1s2;
-    int kk = (k==(v_info->vcount-1)) ? 0 : k;
+    int kk = (!fudge && k==(v_info->vcount-1)) ? 0 : k;
     ss1 = SDIM_dot(v_info->sides[0][k-1],v_info->sides[0][k-1]);
     ss2 = SDIM_dot(v_info->sides[0][kk],v_info->sides[0][kk]);
     s1s2 = SDIM_dot(v_info->sides[0][k-1],v_info->sides[0][kk]);
@@ -677,12 +679,13 @@ REAL star_gauss_method_grad(v_info)
 struct qinfo *v_info;
 { REAL deficit = 2*M_PI;
   int k,i;
+  int fudge = (v_info->flags & INCOMPLETE_STAR) ? 1 : 0;
 
   if ( v_info->vcount <= 1 ) return 0.0;
 
-  for ( k = 1 ; k < v_info->vcount ; k++ )
+  for ( k = 1 ; k < v_info->vcount-fudge ; k++ )
   { REAL ss1,ss2,s1s2;
-    int kk = (k==(v_info->vcount-1)) ? 0 : k;
+    int kk = (!fudge && k==(v_info->vcount-1)) ? 0 : k;
     ss1 = SDIM_dot(v_info->sides[0][k-1],v_info->sides[0][k-1]);
     ss2 = SDIM_dot(v_info->sides[0][kk],v_info->sides[0][kk]);
     s1s2 = SDIM_dot(v_info->sides[0][k-1],v_info->sides[0][kk]);
@@ -690,11 +693,11 @@ struct qinfo *v_info;
   }
 
   /* now individual vertex gradients */
-  for ( k = 1 ; k < v_info->vcount ; k++ )
-  {  REAL ss1,ss2,s1s2;
-     int kk = (k==(v_info->vcount-1)) ? 0 : k;
-     REAL *s1 = v_info->sides[0][k-1],*s2 = v_info->sides[0][kk];
-     REAL denom;
+  for ( k = 1 ; k < v_info->vcount-fudge ; k++ )
+  { REAL ss1,ss2,s1s2;
+    int kk = (!fudge && k==(v_info->vcount-1)) ? 0 : k;
+    REAL *s1 = v_info->sides[0][k-1],*s2 = v_info->sides[0][kk];
+    REAL denom;
 
      ss1 = SDIM_dot(v_info->sides[0][k-1],v_info->sides[0][k-1]);
      ss2 = SDIM_dot(v_info->sides[0][kk],v_info->sides[0][kk]);
@@ -885,8 +888,8 @@ int mode; /* METHOD_VALUE, METHOD_GRADIENT, or METHOD_HESSIAN */
     s1s2 = SDIM_dot(v_info->sides[0][k-1],v_info->sides[0][kk]);
     deficit -= acos(s1s2/sqrt(ss1*ss2));
     area += sqrt(ss1*ss2 - s1s2*s1s2)/6;
-    gc = deficit/area;
   }
+  gc = deficit/area;
 
   /* mean curvature */
   pairs = (v_info->vcount - 1);
@@ -954,7 +957,7 @@ int mode; /* METHOD_VALUE, METHOD_GRADIENT, or METHOD_HESSIAN */
 #endif
 
   if ( !is_finite(energy) )
-	  kb_error(2586,"Levine energy infinite.\n",WARNING);
+      kb_error(2586,"Levine energy infinite.\n",WARNING);
   if ( mode == METHOD_VALUE )
     return energy;
 
