@@ -28,28 +28,29 @@ static FILE *pfd = NULL;
 
 static char gv_version[100];  /* geomview version string */
 static  char pipename[100]; /* for named pipe to geomview */
+static  char geom_name[100]; /* for geomview name of object */
 char *base_name; /* for forming pipe name */
-static int gdim;  /* 3 or 4 dimensions */
+static int gdim; /* 3 or 4 dimensions     */
 
 /* for facets */
 static float *vvlist;  /* vertex list */
-static struct flist_t { int  num;  /* number of vertices */
-                      int  v[3]; /* vertex list */
-                      int  colors; /* number of colors following */
-                      float c[4];  /* color rgba */
-                    } *fflist;
+static struct flist_t { int  num;    /* number of vertices         */
+                        int  v[3];   /* vertex list                */
+                        int  colors; /* number of colors following */
+                        float c[4];  /* color rgba                 */
+                      } *fflist;
 static int *vindex; /* for translating vertex id to index */
 static int vercount,vmaxcount;
 static int ecount;
 static int fcount,fmaxcount;
 
 /* for edges */
-struct elist_t { float x[2][4];  /* 2 endpoints */
-               float c[4];  /* colors */
-               vertex_id v_id[2];  /* endpoints, for picking */
-             } *eelist;
+struct elist_t { float x[2][4];      /* 2 endpoints            */
+                 float c[4];         /* colors                 */
+                 vertex_id v_id[2];  /* endpoints, for picking */
+               } *eelist;
 int edgecount; /* actual edges */
-int edgemax; /* allocated */
+int edgemax;   /* allocated    */
 
 /***************************************************************
 *
@@ -70,12 +71,15 @@ char *outname; /* if want particular name for pipe, NULL else */
   base_name = strrchr(datafilename,PATHCHAR);
   if ( base_name == NULL ) base_name = datafilename;
   else base_name++;  /* skip '/' */
+
   /* using named pipe */
   if ( outname ) strcpy(pipename,outname);
   else 
   { sprintf(pipename,"/tmp/%s.XXXXXX",base_name); 
 #ifdef LINUX
-    mkstemp(pipename);
+    /* gcc complains about mktemp, so we use mkstemp */
+    mkstemp(pipename); /* actually creates file */
+    unlink(pipename);  /* so can recreate it later as pipe */
 #else
     mktemp(pipename);
 #endif
@@ -136,25 +140,27 @@ char *outname; /* if want particular name for pipe, NULL else */
   else /* just a pipe, without geomview */
     {
 #ifdef WIN32
-      kb_error(2415,"Named pipes not available in Windows.\n",RECOVERABLE);
+      outstring("Warning: Windows named pipes do not appear as regular files,\n");
+      outstring("so an ordinary file will be created instead and not deleted when pipe closed.\n");
+      prompt("Enter file name: ",pipename,sizeof(pipename));
+
 #else
-      sprintf(msg,"test -p %s || /etc/mknod %s p || /usr/sbin/mknod %s p || /bin/mknod %s p",
+      sprintf(msg,"test -p %s || (test -x /etc/mknod && /etc/mknod %s p) || (test -s /usr/sbin/mknod && /usr/sbin/mknod %s p) || /bin/mknod %s p",
           pipename,pipename,pipename,pipename);
       if ( system(msg) != 0 )
-         { sprintf(errmsg,"Pipe creation failed for %s\n",pipename);
-            kb_error(1661,errmsg,RECOVERABLE) ;
-
-         }
+      { sprintf(errmsg,"Pipe creation failed for %s\n",pipename);
+        kb_error(1661,errmsg,RECOVERABLE) ;
+      }
       sleep(1);  
       sprintf(msg,"Pipe name: %s\n",pipename);
       outstring(msg);
+#endif
       pfd = fopen(pipename,"w");
       if ( pfd == NULL )
-      {  perror(pipename);
-          kb_error(1662,"Check that /tmp exists and try again.\n",RECOVERABLE);
-
+      { perror(pipename);
+        kb_error(1662,"Check that needed directories exists and try again.\n",
+          RECOVERABLE);
       }
-#endif
     }
 
   geomview_flag = 1;     /* tell everybody we're initialized */
@@ -172,8 +178,8 @@ char *outname; /* if want particular name for pipe, NULL else */
 
 void end_geomview_object()
 {
-  if ( geomview_flag  && base_name[0] )
-     fprintf(pfd,"(delete %s)\n", base_name);
+  if ( geomview_flag  && geom_name[0] )
+     fprintf(pfd,"(delete %s)\n", geom_name);
 }
 
 /*******************************************************************
@@ -186,14 +192,17 @@ void end_geomview_object()
 
 void End_geomview()
 {
-  fprintf(pfd,"(exit)"); 
+  fprintf(pfd,"(exit)\n"); 
   fclose(pfd);
+#ifndef WIN32
   if ( geompipe_flag == GEOM_NAMED_PIPE)
   { sprintf(msg,"rm %s",pipename);
     system(msg);
   }
+#endif
   geomview_flag = 0;
   geompipe_flag = GEOM_TO_GEOMVIEW;
+  geom_name[0] = 0;
   OOGL_flag = 0;
   pfd = NULL;
 
@@ -238,7 +247,7 @@ void geomview_start()
      REAL *x = get_coord(v_id);
      for ( i = 0 ; i < gdim ; i++,vv++ )
         *vv = (float)x[i]; 
-     vindex[ordinal(v_id)] = k; /* so can reference by list order */
+     vindex[loc_ordinal(v_id)] = k; /* so can reference by list order */
      vpicklist[k] = v_id;
      k++;
   }
@@ -272,28 +281,28 @@ edge_id e_id;
      color = DEFAULT_EDGE_COLOR;
 
   if ( edgecount >= edgemax-1 )
-     { eelist = (struct elist_t*)temp_realloc((char*)eelist,
-             2*edgemax*sizeof(struct elist_t),edgemax*sizeof(struct elist_t));
-       edgemax *= 2;
-     }
+  { eelist = (struct elist_t*)temp_realloc((char*)eelist,
+          2*edgemax*sizeof(struct elist_t));
+    edgemax *= 2;
+  }
   ee = eelist + edgecount;
 
   if ( edge_rgb_color_attr > 0 )
-    { ee->c[0] = (float)(((color>>24)&0xFF)/255.);
-      ee->c[1] = (float)(((color>>16)&0xFF)/255.);
-      ee->c[2] = (float)(((color>>8)&0xFF)/255.);
-      ee->c[3] = (float)(facet_alpha_flag ? (color&0xFF)/255. : facet_alpha);
-    }
+  { ee->c[0] = (float)(((color>>24)&0xFF)/255.);
+    ee->c[1] = (float)(((color>>16)&0xFF)/255.);
+    ee->c[2] = (float)(((color>>8)&0xFF)/255.);
+    ee->c[3] = (float)(facet_alpha_flag ? (color&0xFF)/255. : facet_alpha);
+  }
   else
-  {  for ( j = 0 ; j < 3 ; j++ )
+  { for ( j = 0 ; j < 3 ; j++ )
        ee->c[j] = (float)(rgb_colors[color][j]);
-     ee->c[3] = (float)(facet_alpha);
+    ee->c[3] = (float)(facet_alpha);
   }
 
   /* record for later VECT format */
   for ( k = 0 ; k < 2 ; k++ )
-      for ( j = 0 ; j < SDIM ; j++ )
-          ee->x[k][j] = (float)gdata[k].x[j];
+    for ( j = 0 ; j < SDIM ; j++ )
+      ee->x[k][j] = (float)gdata[k].x[j];
   /* pick data */
   if ( valid_id(e_id) )
   { ee->v_id[0] = get_edge_tailv(e_id);
@@ -327,12 +336,12 @@ facet_id f_id;
      color = DEFAULT_EDGE_COLOR;
 
   if ( fcount >= fmaxcount-2 )
-     { fflist = (struct flist_t*)temp_realloc((char*)fflist,
-             2*fmaxcount*sizeof(struct flist_t),fmaxcount*sizeof(struct flist_t));
-        fpicklist = (facet_id*)kb_realloc((char*)fpicklist,
-             2*fmaxcount*sizeof(facet_id),fmaxcount*sizeof(facet_id));
-        fmaxcount *= 2;
-     }
+  { fflist = (struct flist_t*)temp_realloc((char*)fflist,
+         2*fmaxcount*sizeof(struct flist_t));
+    fpicklist = (facet_id*)kb_realloc((char*)fpicklist,
+         2*fmaxcount*sizeof(facet_id));
+    fmaxcount *= 2;
+  }
   if ( color != CLEAR )
   {
     ff = fflist + fcount;
@@ -353,15 +362,15 @@ facet_id f_id;
 
     if ( gdata[0].flags & LIST_FACET )
      for ( i = 0 ; i < FACET_VERTS ; i++ )
-      {
-         ff->v[i] = vindex[ordinal(gdata[i].v_id)];
-      }
+     {
+       ff->v[i] = vindex[loc_ordinal(gdata[i].v_id)];
+     }
     else if ( gdata[0].flags & SIMPLE_FACET ) /* can use regular vertices */
     { facetedge_id fe = get_facet_fe(f_id);
        if ( valid_id(fe) )
         for ( i = 0 ; i < FACET_VERTS ; i++ )
         {
-           ff->v[i] = vindex[ordinal(get_fe_tailv(fe))];
+           ff->v[i] = vindex[loc_ordinal(get_fe_tailv(fe))];
            fe = get_next_edge(fe);
         }
     }
@@ -369,17 +378,17 @@ facet_id f_id;
     {
        while ( vercount + FACET_VERTS > vmaxcount )
        { vvlist=(float*)temp_realloc((char*)vvlist,
-               gdim*2*vmaxcount*sizeof(float), gdim*vmaxcount*sizeof(float));
+               gdim*2*vmaxcount*sizeof(float));
          vpicklist=(vertex_id*)kb_realloc((char*)vpicklist,
-               2*vmaxcount*sizeof(vertex_id),vmaxcount*sizeof(vertex_id));
+               2*vmaxcount*sizeof(vertex_id));
          vmaxcount *= 2;
        }
        for ( k = 0 ; k < FACET_VERTS ; k++,vercount++ )
-        { for ( j = 0 ; j < gdim ; j++ )
-             vvlist[vercount*gdim+j] = (float)gdata[k].x[j];
-          vpicklist[vercount] = gdata[k].v_id;
-          ff->v[k] = vercount;
-        }
+       { for ( j = 0 ; j < gdim ; j++ )
+            vvlist[vercount*gdim+j] = (float)gdata[k].x[j];
+         vpicklist[vercount] = gdata[k].v_id;
+         ff->v[k] = vercount;
+       }
     }
   fcount++;
   }
@@ -405,9 +414,9 @@ facet_id f_id;
       fpicklist[fcount] = f_id;
       while ( vercount + 3 > vmaxcount )
       { vvlist=(float*)temp_realloc((char*)vvlist,
-           gdim*2*vmaxcount*sizeof(float), gdim*vmaxcount*sizeof(float));
+           gdim*2*vmaxcount*sizeof(float));
         vpicklist=(vertex_id*)kb_realloc((char*)vpicklist,
-           2*vmaxcount*sizeof(vertex_id),vmaxcount*sizeof(vertex_id));
+           2*vmaxcount*sizeof(vertex_id));
         vmaxcount *= 2;
       }
      for ( k = 0 ; k < 3 ; k++,vercount++ )
@@ -424,7 +433,7 @@ facet_id f_id;
 do_edges:
   gdata[FACET_VERTS] = gdata[0];  /* last is first */
   for ( k = 0 ; k < FACET_VERTS ; k++ )
-     if ( (gdata[k].etype != INVISIBLE_EDGE) )
+     if ( (gdata[k].etype & EBITS != INVISIBLE_EDGE) )
         geomview_edge(gdata+k,gdata[k].id);
 }
 
@@ -450,8 +459,9 @@ void geomview_end()
   setting_backcull = 0; /* so read_pick() knows */
   /* if ( gdim < 4 ) */
   { /* command format wrapper */
-    fprintf(pfd,"(geometry %s ",base_name); 
-    if ( transforms_flag && transform_count && !transform_colors_flag )
+    strncpy(geom_name,base_name,sizeof(geom_name));
+    fprintf(pfd,"(geometry %s ",geom_name); 
+    if ( transforms_flag && (transform_count>1) && !transform_colors_flag )
       fprintf(pfd," { INST \nunit { = \n");
     fprintf(pfd,"LIST {\n");
     if ( strcmp(gv_version,"\"1.6") > 0 )
@@ -464,37 +474,37 @@ void geomview_end()
   /* facets */
   if ( fcount > 0 )
   {
-     if ( gv_binary_flag )
+    if ( gv_binary_flag )
+    { if ( gdim == 3 )
+          ret =fprintf(pfd,"{OFF BINARY\n");
+      else 
+          ret =fprintf(pfd,"{4OFF BINARY\n");
+      if ( ret < 0 ) return; /* broken pipe */
+      fwrite(&vercount,1,sizeof(int),pfd);
+      fwrite(&fcount,1,sizeof(int),pfd);
+      fwrite(&ecount,1,sizeof(int),pfd);
+      fwrite(vvlist,vercount*gdim,sizeof(float),pfd);
+      fwrite(fflist,fcount,sizeof(struct flist_t),pfd);
+    }
+    else
+    { if ( gdim == 3 )
+         ret =fprintf(pfd,"{OFF\n");  /* COFF used for vertex colors only */
+      else 
+         ret =fprintf(pfd,"{4OFF\n"); /* COFF used for vertex colors only */
+      if ( ret < 0 ) return; /* broken pipe */
+      fprintf(pfd,"%d %d %d\n",vercount,fcount,ecount); 
+      for ( k = 0, vv = vvlist ; (k < vercount)  ; k++,vv+=gdim )
       { if ( gdim == 3 )
-            ret =fprintf(pfd,"{OFF BINARY\n");
-         else 
-            ret =fprintf(pfd,"{4OFF BINARY\n");
-         if ( ret < 0 ) return; /* broken pipe */
-         fwrite(&vercount,1,sizeof(int),pfd);
-         fwrite(&fcount,1,sizeof(int),pfd);
-         fwrite(&ecount,1,sizeof(int),pfd);
-         fwrite(vvlist,vercount*gdim,sizeof(float),pfd);
-         fwrite(fflist,fcount,sizeof(struct flist_t),pfd);
+          fprintf(pfd,"%f %f %f\n",(DOUBLE)vv[0],(DOUBLE)vv[1],(DOUBLE)vv[2]);
+        else
+          fprintf(pfd,"%f %f %f %f\n",(DOUBLE)vv[0],(DOUBLE)vv[1],(DOUBLE)vv[2],
+              (DOUBLE)vv[3]);
       }
-     else
-      { if ( gdim == 3 )
-            ret =fprintf(pfd,"{OFF\n");  /* COFF used for vertex colors only */
-         else 
-            ret =fprintf(pfd,"{4OFF\n"); /* COFF used for vertex colors only */
-         if ( ret < 0 ) return; /* broken pipe */
-         fprintf(pfd,"%d %d %d\n",vercount,fcount,ecount); 
-         for ( k = 0, vv = vvlist ; (k < vercount)  ; k++,vv+=gdim )
-          { if ( gdim == 3 )
-              fprintf(pfd,"%f %f %f\n",(DOUBLE)vv[0],(DOUBLE)vv[1],(DOUBLE)vv[2]);
-            else
-              fprintf(pfd,"%f %f %f %f\n",(DOUBLE)vv[0],(DOUBLE)vv[1],(DOUBLE)vv[2],
-                  (DOUBLE)vv[3]);
-          }
-         for ( k = 0, ff = fflist ; (k < fcount)  ; k++,ff++ )
-          fprintf(pfd,"3 %d %d %d  %f %f %f\n",ff->v[0],ff->v[1],
-              ff->v[2], (DOUBLE)ff->c[0],(DOUBLE)ff->c[1],(DOUBLE)ff->c[2]);
-     }
-     fprintf(pfd,"}\n");
+      for ( k = 0, ff = fflist ; (k < fcount)  ; k++,ff++ )
+        fprintf(pfd,"3 %d %d %d  %f %f %f\n",ff->v[0],ff->v[1],
+            ff->v[2], (DOUBLE)ff->c[0],(DOUBLE)ff->c[1],(DOUBLE)ff->c[2]);
+    }
+    fprintf(pfd,"}\n");
   }      /* end facets */
 
   /* edges */
@@ -512,7 +522,7 @@ void geomview_end()
      }
      gv_vect_start = vercount;
      vpicklist=(vertex_id*)kb_realloc((char*)vpicklist,
-                      new_vmax*sizeof(vertex_id),vmaxcount*sizeof(vertex_id));
+                      new_vmax*sizeof(vertex_id));
 
      for ( i = 0, ee = eelist ; i < edgecount ; i++,ee++,vercount += 2 )
      { vpicklist[vercount] = ee->v_id[0];
@@ -537,9 +547,9 @@ void geomview_end()
           fwrite(&tmp,1,sizeof(int),pfd);     /* colors per line */
        for ( i = 0,ee=eelist ; (i < edgecount) ; i++,ee++ ) 
             /* vertex coords */
-          { fwrite(ee->x[0],gdim,sizeof(float),pfd); /* vertices */
-             fwrite(ee->x[1],gdim,sizeof(float),pfd);
-          }
+       { fwrite(ee->x[0],gdim,sizeof(float),pfd); /* vertices */
+         fwrite(ee->x[1],gdim,sizeof(float),pfd);
+       }
        for ( i = 0,ee=eelist ; (i < edgecount)  ; i++,ee++ ) 
            /* line colors */
            fwrite(ee->c,4,sizeof(float),pfd); /* colors */
@@ -557,58 +567,60 @@ void geomview_end()
            fprintf(pfd,"1\n"); /* colors per edge */
        for ( i = 0,ee=eelist ; (i < edgecount) ; i++,ee++ ) 
           /* vertex coords */
-          if ( gdim == 4 )
+       {  if ( gdim == 4 )
            fprintf(pfd,"%f %f %f %f  %f %f %f %f\n",(DOUBLE)ee->x[0][0],
               (DOUBLE)ee->x[0][1], (DOUBLE)ee->x[0][2],(DOUBLE)ee->x[0][3],
               (DOUBLE)ee->x[1][0],(DOUBLE)ee->x[1][1],(DOUBLE)ee->x[1][2],
               (DOUBLE)ee->x[1][3]);
           else 
-           fprintf(pfd,"%f %f %f    %f %f %f\n",(DOUBLE)ee->x[0][0],(DOUBLE)ee->x[0][1],
-              (DOUBLE)ee->x[0][2],
+           fprintf(pfd,"%f %f %f    %f %f %f\n",
+              (DOUBLE)ee->x[0][0],(DOUBLE)ee->x[0][1],(DOUBLE)ee->x[0][2],
               (DOUBLE)ee->x[1][0],(DOUBLE)ee->x[1][1],(DOUBLE)ee->x[1][2]);
+       }
        for ( i = 0,ee=eelist ; (i < edgecount)  ; i++,ee++ ) 
         /* line colors */
-        fprintf(pfd,"%f %f %f %f\n",(DOUBLE)ee->c[0],(DOUBLE)ee->c[1],(DOUBLE)ee->c[2],
-          (DOUBLE)ee->c[3]);
+        fprintf(pfd,"%f %f %f %f\n",(DOUBLE)ee->c[0],(DOUBLE)ee->c[1],
+          (DOUBLE)ee->c[2],(DOUBLE)ee->c[3]);
      }
      fprintf(pfd,"}\n");
   } /* end edges */
 
-/* if ( gdim < 4 ) */
-{ /* end wrapping */
+  /* end wrapping */
   fprintf(pfd,"}\n"); /* end LIST */
 
   /* transforms, if any */
-  if ( transforms_flag && transform_count && !transform_colors_flag  )
-     { fprintf(pfd,"\n}\ntlist { =\n");
-       fprintf(pfd,"\n");
-       for ( k = 0 ; k < transform_count ; k++ )
-       { for ( i = 0 ; i < 4 ; i++ )
-          { for ( j = 0 ; j < 4 ; j++ )
-                if ( view_4D_flag ) /* pass on 4x4 corner */
-                  fprintf(pfd,"%f ",(DOUBLE)view_transforms[k][j][i]);
-                else if ( (i < SDIM) && (j < SDIM) )
-                  fprintf(pfd,"%f ",(DOUBLE)view_transforms[k][j][i]);
-                else if ( (i < SDIM) && (j == 3) )
-                  fprintf(pfd,"%f ",(DOUBLE)view_transforms[k][SDIM][i]);
-                else if ( (j < SDIM) && (i == 3) )
-                  fprintf(pfd,"%f ",(DOUBLE)view_transforms[k][j][SDIM]);
-                else if ( (j == 3) && (i == 3) )
-                  fprintf(pfd,"%f ",(DOUBLE)view_transforms[k][SDIM][SDIM]);
-                else 
-                  fprintf(pfd,"%f ",(DOUBLE)identmat[j][i]);
-             fprintf(pfd,"\n");
-          }
-          fprintf(pfd,"\n");
+  if ( transforms_flag && (transform_count>1) && !transform_colors_flag  )
+  { fprintf(pfd,"\n}\ntlist { =\n");
+    fprintf(pfd,"\n");
+    for ( k = 0 ; k < transform_count ; k++ )
+    { for ( i = 0 ; i < 4 ; i++ )
+      { for ( j = 0 ; j < 4 ; j++ )
+        { if ( view_4D_flag ) /* pass on 4x4 corner */
+            fprintf(pfd,"%f ",(DOUBLE)view_transforms[k][j][i]);
+          else if ( (i < SDIM) && (j < SDIM) )
+            fprintf(pfd,"%f ",(DOUBLE)view_transforms[k][j][i]);
+          else if ( (i < SDIM) && (j == 3) )
+            fprintf(pfd,"%f ",(DOUBLE)view_transforms[k][SDIM][i]);
+          else if ( (j < SDIM) && (i == 3) )
+            fprintf(pfd,"%f ",(DOUBLE)view_transforms[k][j][SDIM]);
+          else if ( (j == 3) && (i == 3) )
+            fprintf(pfd,"%f ",(DOUBLE)view_transforms[k][SDIM][SDIM]);
+          else 
+            fprintf(pfd,"%f ",(DOUBLE)identmat[j][i]);
         }
-        fprintf(pfd,"}\n}\n");
-     }
+        fprintf(pfd,"\n");
+      }
+      fprintf(pfd,"\n");
+    }
+    fprintf(pfd,"}\n}\n");
 
-    fprintf(pfd,"\n)"); 
   }
+  fprintf(pfd,"\n)"); /* matches (geometry */
 
-fprintf(pfd,"\n(interest (pick World))\n");  /* enable picking */ 
-fflush(pfd);
+ fprintf(pfd,"\n(event-pick on)\n");
+ fprintf(pfd,"(pickable %s)\n",geom_name);
+ fprintf(pfd,"(interest (pick World))\n");  /* enable picking */ 
+ fflush(pfd);
 
 g_exit:
   temp_free((char*)vvlist);
@@ -657,4 +669,3 @@ char *cmd;
 
 }
 
-
