@@ -13,9 +13,7 @@
 */
 
 
-#ifdef WIN32
-#include <windows.h>
-#endif
+
 #include "include.h"
 #ifndef TRUE
 #define  TRUE 1
@@ -43,14 +41,32 @@ static int showflag;
 *
 * function: update_display()
 *
+* purpose: wrapper for local_update_display()
+*/
+void update_display()
+{
+  #ifdef MPI_EVOLVER
+  if ( this_machine == 0 )
+     mpi_update_display();
+  else 
+     local_update_display();
+  #else
+  local_update_display();
+  #endif
+}
+
+/********************************************************************
+*
+* function: local_update_display()
+*
 * purpose: Check for necessity of re-displaying surface.
 *
 */
 
-void update_display()
+void local_update_display()
 {
   graph_timestamp = ++global_timestamp;  /* new surface */
-  if (go_display_flag )
+  if ( go_display_flag )
   {
     if ( OOGL_flag ) UpdateOOGL();
     else
@@ -172,12 +188,12 @@ int view_transform(string)
 char *string;
 {
   char *c;
-  int legal; /* number of legal characters at start of string */
+  size_t legal; /* number of legal characters at start of string */
 
   /* test for illegal characters */
   legal =  strspn(string,
           "0123456789.+-udrlcCRmzsABDxqtvwbeETH?h\034\035\036\037\033\133\n\r");
-  if ( legal != (int)strlen(string) )
+  if ( legal != strlen(string) )
   { sprintf(msg,"Illegal character in graphics command: %c",string[legal]);
     kb_error(1044,msg,WARNING);
     showflag = 0;  /* don't reshow */
@@ -423,8 +439,8 @@ void init_view()
 
 
   /* set up identity matrix */
-  identmat = dmatrix(0,SDIM,0,SDIM);
-  for ( i = 0 ; i <= SDIM ; i++ )
+  identmat = dmatrix(0,MAXCOORD,0,MAXCOORD);   /* SDIM may be too small */
+  for ( i = 0 ; i <= MAXCOORD ; i++ )
   {
     identmat[i][i] = 1.0;
   }
