@@ -15,9 +15,6 @@
 int hess_debug;  /* debugging flag */
 /* use ysmp */
 
-#ifdef WIN32
-#include <windows.h>
-#endif
 
 #include "include.h"
 #include "f2c.h"
@@ -39,7 +36,6 @@ REAL rhs_norm;
 
 static int ritz_done_flag;
 REAL **ritzvecs;
-static REAL *ritzev;
 static int ritzdim;
 void optparamhess ARGS((struct linsys *,REAL *));  /* right side */
 
@@ -143,7 +139,7 @@ REAL **self; /* full dim values */
 { int j,k;
   struct hess_verlist *v;
 
-  v = vhead + ordinal(v_id);
+  v = get_vertex_vhead(v_id);
 
   if ( v->proj )
   { MAT2D(temp_mat,MAXCOORD,MAXCOORD);
@@ -184,8 +180,8 @@ REAL **mixed; /* full dim values */
     return;
   }
 
-  v1 = vhead + ordinal(v_id1); 
-  v2 = vhead + ordinal(v_id2);
+  v1 = get_vertex_vhead(v_id1); 
+  v2 = get_vertex_vhead(v_id2);
   if ( v1->proj )
   { tr_mat_mul(v1->proj,mixed,temp_mat,SDIM,v1->freedom,SDIM);
     oo = temp_mat;
@@ -317,28 +313,28 @@ void hess_hash_end()
 
 void hessian_auto() /* automatic hessian */
 { REAL old_energy = web.total_energy;
-  REAL *rhs;
-  REAL *X;
+  REAL *rhs = NULL;
+  REAL *X = NULL;
   struct linsys S;
 
   hmode = hessian_normal_flag;
   hessian_cleanup(); /* cleanup from previous round */
-  hessian_init(&S,&rhs,&X);
+  hessian_init(&S,&rhs);
   hess_flag = 1; rhs_flag = 1; 
-  hessian_fill(&S,rhs); 
-  sp_Hessian_solver(&S,rhs,X);
+  hessian_fill(&S,&rhs); 
+  sp_Hessian_solver(&S,rhs,&X);
   hessian_move((REAL)1.0,ACTUAL_MOVE,X);
 
   if ( check_increase_flag && 
          (web.total_energy > (1+100*machine_eps)*old_energy) )
-     { kb_error(1596,"Hessian move would have raised energy.  Restoring coords.\n",
+  { kb_error(1596,"Hessian move would have raised energy.  Restoring coords.\n",
           WARNING);
-        restore_coords(&saved,SAVE_IN_ATTR);
-     }
+    restore_coords(&saved,SAVE_IN_ATTR);
+  }
   hessian_exit(X);
   free_system(&S);
   temp_free((char*)rhs);
-  temp_free((char*)X);
+  temp_free((char*)X); 
 }
 
 /**************************************************************
@@ -354,18 +350,18 @@ void hessian_auto() /* automatic hessian */
 REAL hessian_seek(maxscale) /* automatic hessian */
 REAL maxscale;
 { REAL best_scale;
-  REAL *rhs,*X;
+  REAL *rhs=NULL,*X=NULL;
   struct linsys S;
  
   hmode = hessian_normal_flag;
-  hessian_init(&S,&rhs,&X);
+  hessian_init(&S,&rhs);
   hess_flag = 1; rhs_flag = 1; 
-  hessian_fill(&S,rhs); 
+  hessian_fill(&S,&rhs); 
   if ( hessian_linear_metric_flag )
     linear_metric_setup(&S,&Met);
   else if ( web.area_norm_flag )
     star_metric_setup(&S,&Met);
-  sp_Hessian_solver(&S,rhs,X);
+  sp_Hessian_solver(&S,rhs,&X);
   best_scale = hessian_line_seek(&S,maxscale,X);
   hessian_exit(X);
   free_system(&S);
@@ -405,13 +401,13 @@ REAL square_grad()
   { REAL star;
      f = get_force(v_id);
      if ( hessian_linear_metric_flag )
-        star = Met.apinv[vhead[ordinal(v_id)].rownum];
+        star = Met.apinv[get_vertex_vhead(v_id)->rownum];
      else star = 1.0;
      if ( hessian_normal_flag )
      { struct hess_verlist *v;              /* current  vertex */
         REAL r[MAXCOORD];
         REAL *rr = r;
-        v = vhead + ordinal(v_id);
+        v = get_vertex_vhead(v_id);
         vec_mat_mul(f,v->proj,rr,SDIM,v->freedom);
         sum += dot(r,r,v->freedom)*star;
      }
@@ -433,13 +429,13 @@ REAL square_grad()
 */
 
 void square_grad_forces()
-{ REAL *rhs,*X;
+{ REAL *rhs=NULL,*X=NULL;
   struct linsys S;
  
   hmode = hessian_normal_flag;
-  hessian_init(&S,&rhs,&X);
+  hessian_init(&S,&rhs);
   hess_flag = 1; rhs_flag = 1; 
-  hessian_fill(&S,rhs); 
+  hessian_fill(&S,&rhs); 
 
   /* ?
   if ( hessian_linear_metric_flag )
@@ -448,6 +444,7 @@ void square_grad_forces()
     star_metric_setup(&S,&Met);
   */
 
+  X = (REAL*)temp_calloc(S.N,sizeof(REAL));
   sp_hessian_mult(&S,rhs,X);
   hessian_move(0,SET_VELOCITY,X);
 
@@ -485,15 +482,18 @@ REAL *X;
   if ( maxscale <= 0.0 ) return 0.0;
 
   typescale = maxscale/100;
-
-  /* figure which way to go */
+  
+  ENTER_GRAPH_MUTEX;
+  
+  /* base energy; may be volume adjustments, so can't just take current values */
+  hessian_move(0,ACTUAL_MOVE/*TEST_MOVE*/,X); 
   energy0 = min_square_grad_flag ? square_grad(): web.total_energy;
+  restore_coords(&saved,SAVE_IN_ATTR);
   if ( itdebug )
   { sprintf(msg,"stepsize 0   energy0 %18.15g\n",energy0);
     outstring(msg);
   }
 
-  ENTER_GRAPH_MUTEX;
   hessian_move(typescale*1e-3,ACTUAL_MOVE/*TEST_MOVE*/,X);
   energy1 = min_square_grad_flag ? square_grad(): web.total_energy;
   restore_coords(&saved,SAVE_IN_ATTR);
@@ -606,13 +606,13 @@ void hessian_saddle(maxscale)
 REAL maxscale;
 { REAL low;
   REAL stepsize;
-  REAL *rhs,*X;
+  REAL *rhs=NULL,*X=NULL;
   struct linsys S;
 
   hmode = hessian_normal_flag;
-  hessian_init(&S,&rhs,&X);
+  hessian_init(&S,&rhs);
   hess_flag = 1; rhs_flag = 1; 
-  hessian_fill(&S,rhs); 
+  hessian_fill(&S,&rhs); 
   (*sp_AIJ_setup_func)(S.A_rows,&S);
   (*sp_constraint_setup_func)
          (web.skel[BODY].max_ord+1 + gen_quant_count,&S);
@@ -626,6 +626,7 @@ REAL maxscale;
        star_metric_setup(&S,&Met);
      }
 
+  X = (REAL*)temp_calloc(S.N,sizeof(REAL));
   low =  lowest_eigenpair(&S,X);
   if ( low >= 0.0 )
      { 
@@ -694,6 +695,12 @@ void print_hessian_menu()
   outstring("M. Toggle projecting to global constraints in move. (Leave this alone!)\n");
   outstring("G. Toggle minimizing square gradient in seek.\n");
   outstring("D. Dump Hessian to text file.\n");
+  outstring("=. Execute Evolver commands in subprompt.\n");
+#ifdef MPI_EVOLVER
+  outstring("A. Build corona (MPI version only).\n");
+  outstring("J. Dissolve corona (MPI version only).\n");
+  outstring("I. Free discard lists (MPI version only).\n");
+#endif
   outstring("0. Exit hessian.\n");
 }
 
@@ -722,14 +729,21 @@ void hessian_menu()
     hmode = hessian_normal_flag;
     for(;;)
      {
-        prompt("Choice(?,1,2,3,4,7,9,B,C,E,F,L,R,Z,X,P,V,S,Y,U,M,G,D,0,q): ",
-            response,sizeof(response));
+#ifdef MPI_EVOLVER
+        prompt("Choice(?,1,2,3,4,7,9,B,C,E,F,L,R,Z,X,P,V,S,Y,U,M,G,D,A,J,I,=,0,q): ",
+		 response,sizeof(response));
+#else
+        prompt("Choice(?,1,2,3,4,7,9,B,C,E,F,L,R,Z,X,P,V,S,Y,U,M,G,D,=,0,q): ",
+		 response,sizeof(response));
+#endif
         switch ( toupper(response[0]) )
           { case '?': print_hessian_menu(); break;
              case '1': 
                  hessian_cleanup(); /* cleanup from previous round */
-                 hessian_init(&S,&rhs,&X);
-                 hess_flag = 1; rhs_flag = 1; hessian_fill(&S,rhs); 
+                 hessian_init(&S,&rhs);
+                 hess_flag = 1; rhs_flag = 1; 
+                 hessian_fill(&S,&rhs); 
+                 X = (REAL*)temp_calloc(S.N,sizeof(REAL));
                  (*sp_AIJ_setup_func)(S.A_rows,&S);
                  (*sp_constraint_setup_func) (
                            web.skel[BODY].max_ord+1 + gen_quant_count,&S);
@@ -748,7 +762,12 @@ void hessian_menu()
              case '2': 
                   if ( left_flag == 0 )
                      { outstring("Error: Must do step 1 first.\n"); break; }
-                  rhs_norm = dot(rhs,rhs,S.total_rows);
+                  if ( hess_debug )
+                  { int i;
+                    for ( i = 0 ; i < S.N ; i++ )
+                      printf("B[%d]  %20.15f\n",i,(DOUBLE)(rhs[i]));
+                  }
+                  rhs_norm = dot(rhs,rhs,S.N);
 #ifdef LONGDOUBLE
                   sprintf(msg,"RHS norm: %20.*Lg\n",DPREC, rhs_norm);
 #else
@@ -766,7 +785,7 @@ void hessian_menu()
                  if ( !(S.flags & S_FACTORED) ) (*sp_factor_func)(&S);
                  if ( !(S.flags & S_PROJECTED)) (*sp_hess_project_setup_func)(&S);
                  sp_hessian_solve(&S,rhs,X,SET_PRESSURE);
-                 outstring("Motion found.\n");
+                 outstring("Motion found.  Pressures adjusted.\n");
                  dir_flag = 1;
                  break;
 
@@ -795,6 +814,13 @@ void hessian_menu()
                          { outstring("Error: Must do step 1 first.\n"); break; }
                        write_hessian(&S); break;
 
+             case '=': 
+                   subshell_flag = 1;
+                   exec_commands(NULL,"hessian_menu subcommand: "); 
+                   subshell_flag = 0;
+                   /* command read and execute loop */
+                   break;
+
              case 'Q': case '0': case 0: 
                    hessian_exit(X); 
                    free_system(&S);
@@ -823,18 +849,6 @@ void hessian_menu()
                  ritz_done_flag = 1;
                  break;
               }
-             case 'A':  /* new lowest eigenvalues, by conjugate gradient */ 
-              { if ( left_flag == 0 )
-                     { outstring("Error: Must do step 1 first.\n"); break; }
-                 prompt("Number of eigenvalues: ",response,sizeof(response));
-                 ritzdim = atoi(response);
-                 if ( ritzvecs ) free_matrix(ritzvecs);
-                 ritzvecs = dmatrix(0,ritzdim-1,0,S.N+S.concount);
-                 ritzev = (REAL*)temp_calloc(ritzdim,sizeof(REAL));
-                 cg_ritz(&S,ritzdim,ritzvecs,ritzev); 
-                 ritz_done_flag = 1;
-                 break;
-              }
              case 'X': /* pick ritz vector for motion */
               { int ritznum;
                  int i;
@@ -964,7 +978,7 @@ void hessian_menu()
              case 'S':
                  if ( dir_flag == 0 )
                      { outstring("Error: Must get direction first.\n"); break; }
-                 stepsize = hessian_line_seek(&S,1.0,X);
+                 stepsize = hessian_line_seek(&S,10.0,X);
                  sprintf(msg,"stepsize %g\n",(DOUBLE)stepsize);
                  outstring(msg);
 #ifdef LONGDOUBLE
@@ -988,7 +1002,7 @@ void hessian_menu()
                     sp_solve_func = xmd_solve;
                     sp_solve_multi_func = xmd_solve_multi;
                     sp_ordering_func = NULL;
-                    sp_CHinvC_func = xmd_CHinvC;
+                    sp_CHinvC_func = sp_CHinvC;
                     outstring("Using alternate minimal degree factoring.\n");
                     if ( left_flag ) outstring("Must do step 1 again.\n");
                     dir_flag = left_flag = right_flag = 0;
@@ -1033,7 +1047,7 @@ void hessian_menu()
              case 'D': /* dump Hessian, 1-based indexing */
                  { FILE *fd;
                    char hname[200];
-                   int baselength;
+                   size_t baselength;
                    int i,j,k;
 gethname:
                    outstring("Hessian dump. Note indexing is 1-based.\n");
@@ -1101,7 +1115,7 @@ gethname:
                      }
                    }
                    fclose(fd);
-                   if ( !augmented_hessian_flag )
+                   if ( !augmented_hessian_mode )
                    { strcpy(hname+baselength,"_C.dat");
                      sprintf(msg,
             "Dumping constraint matrix in row col value format to %s.\n",hname);
@@ -1112,7 +1126,7 @@ gethname:
                      for ( i = 0 ; i < S.CN ; i++ )
                      { if ( S.CIA )
                        for ( j = S.CIA[i] ; j < S.CIA[i+1] ; j++ )
-                       { fprintf(fd,"%6d %6d %20.15f\n",i+1,S.CJA[k]-A_OFF+1,S.CA[j]);
+                       { fprintf(fd,"%6d %6d %20.15f\n",i+1,S.CJA[j]-A_OFF+1,S.CA[j]);
                        }
                        else /* dense */
                        for ( j = 0 ; j < S.N ; j++ )
@@ -1135,6 +1149,12 @@ gethname:
                    fclose(fd);
                  }
                break;
+#ifdef MPI_EVOLVER
+             case 'A' :
+   				 mpi_synch_corona(); break;
+             case 'J' :
+   				 mpi_dissolve_corona(); break;
+#endif
              default : outstring("Illegal choice.\n");
           }
      }
@@ -1233,10 +1253,9 @@ void hessian_legal()
 * purpose: Allocate Hessian
 */
 
-void hessian_init(S,rhs,X)
+void hessian_init(S,rhs)
 struct linsys *S;
 REAL **rhs;  /* right side, allocated by hessian_init */
-REAL **X;  /* solution, allocated by hessian_init */
 {
   vertex_id v_id;
   int j,i,gam,a,b,m,n;
@@ -1251,7 +1270,7 @@ REAL **X;  /* solution, allocated by hessian_init */
   MAT3D(second,MAXCOORD,MAXCOORD,MAXCOORD);
   REAL dummy;
   int total_proj,vproj_count,total_conhess,vproj_alloc,conhess_alloc;
-  int vproj_spot;
+  int vproj_spot,vcount;
   REAL ***v_normal = NULL;
 
   memset(S,0,sizeof(struct linsys));
@@ -1261,28 +1280,47 @@ REAL **X;  /* solution, allocated by hessian_init */
   /* in case this doesn't work */
   if ( saved.coord == NULL ) save_coords(&saved,SAVE_IN_ATTR);  
 
+  /* vertex attribute to hold vhead index */
+  vhead_attr = find_attribute(VERTEX,VHEAD_ATTR_NAME);
+  if ( vhead_attr < 0 )
+  { int one = 1;
+    vhead_attr = add_attribute(VERTEX,VHEAD_ATTR_NAME,INTEGER_TYPE,
+                    0,&one,0,NULL);
+  }
+
   vhead_count = web.skel[VERTEX].max_ord+1;
   vhead_count += optparamcount;
   vhead_count += gen_quant_count;
   if ( !everything_quantities_flag ) 
-       vhead_count += web.skel[BODY].max_ord+1;
+  { vhead_count += web.skel[BODY].max_ord+1;
+    bhead_attr = find_attribute(BODY,BHEAD_ATTR_NAME);
+    if ( bhead_attr < 0 )
+    { int one = 1;
+      bhead_attr = add_attribute(BODY,BHEAD_ATTR_NAME,INTEGER_TYPE,
+                      0,&one,0,NULL);
+    }
+  }
+
+
   vhead = (struct hess_verlist*)temp_calloc(vhead_count,
         sizeof(struct hess_verlist));
   if ( hmode == NORMAL_MOTION ) v_normal = dmatrix3(vhead_count,SDIM,SDIM);
-
-  if  ( vedge_timestamp < top_timestamp ) make_vedge_lists(); 
+      /* v_normal will use vhead_attr as index */
 
   /* populate vertex list and count degrees of freedom */
   total_proj = 0;    /* count columns needed */
   vproj_count = 0;  /* number of vertices needing projection */
   total_conhess = 0;    /* count rows needed */
-  FOR_ALL_VERTICES(v_id)
+  vcount = 0;
+  MFOR_ALL_VERTICES(v_id)
   { conmap_t * conmap;
     int oncount;
     int kk;
 
+    set_vertex_vhead(v_id,vcount); /* new way */
+    vcount++;
     if ( get_vattr(v_id) & FIXED ) continue;
-    v = vhead + ordinal(v_id);
+    v = get_vertex_vhead(v_id);   /* old way */
     v->v_id = v_id;
 
     if ( hmode == SINGLE_DEGREE )
@@ -1297,8 +1335,8 @@ REAL **X;  /* solution, allocated by hessian_init */
     else /* possible level set constraints */
     { 
       if ( hmode == NORMAL_MOTION ) 
-      {
-        kk = new_calc_vertex_normal(v_id,v_normal[ordinal(v_id)]);
+      { REAL **norm = get_vertex_v_normal(v_id);
+        kk = new_calc_vertex_normal(v_id,norm);
         if ( kk < SDIM )
         { v->freedom = kk;
           total_proj += kk;
@@ -1328,17 +1366,18 @@ REAL **X;  /* solution, allocated by hessian_init */
     }
   } /* end FOR_ALL_VERTICES first time through */
 
+
   vproj_base = (REAL**)mycalloc(vproj_count*SDIM,sizeof(REAL*));
   vproj_space = (REAL*)mycalloc(total_proj*SDIM,sizeof(REAL));;
   conhess_base = dmatrix3(total_conhess,SDIM-1,SDIM);
   vproj_spot = vproj_alloc = conhess_alloc = 0;
 
   /* populate vertex list vproj and conhess */
-  FOR_ALL_VERTICES(v_id)
+  MFOR_ALL_VERTICES(v_id)
   { conmap_t * conmap;
     int oncount;
 
-    v = vhead + ordinal(v_id);
+    v = get_vertex_vhead(v_id);
 /*    v->rownum  = currow; */
     v->slant = 1.0;
     if ( v->freedom == 0 ) continue;
@@ -1364,7 +1403,7 @@ REAL **X;  /* solution, allocated by hessian_init */
     else /* possible level set constraints */
     { 
       if (v->freedom < SDIM)  /* copy over normal basis */
-      { 
+      { REAL **vn = get_vertex_v_normal(v_id);
         v->proj = vproj_base + vproj_alloc; vproj_alloc += SDIM;
         for ( j = 0 ; j < SDIM ; j++ ) 
         { v->proj[j] = vproj_space + vproj_spot;
@@ -1372,9 +1411,8 @@ REAL **X;  /* solution, allocated by hessian_init */
         }
         for ( j = 0 ; j < SDIM ; j++ )
           for ( i = 0 ; i < v->freedom ; i++ )
-            v->proj[j][i] = v_normal[ordinal(v->v_id)][i][j];
+            v->proj[j][i] = vn[i][j];
       }
-
       conmap = get_v_constraint_map(v_id);     
       oncount = 0;
       if ( conmap[0] )
@@ -1440,7 +1478,7 @@ REAL **X;  /* solution, allocated by hessian_init */
         }
         if ( (hessian_normal_flag && (v->freedom == 1)) || hessian_double_normal_flag)
         { /* get cosine of normal and degree of freedom */
-          REAL *nor = v_normal[ordinal(v->v_id)][0];
+          REAL *nor = get_vertex_v_normal(v_id)[0];
           v->slant = 0.0;
           for ( i = 0 ; i < SDIM ; i++ )
              v->slant += v->proj[i][0]*nor[i];
@@ -1470,7 +1508,7 @@ REAL **X;  /* solution, allocated by hessian_init */
           }
        }
     } /* end level set constraints */
-  
+ 
     if ( hessian_double_normal_flag && v->proj )
     { REAL sum;
       int k;
@@ -1493,7 +1531,7 @@ REAL **X;  /* solution, allocated by hessian_init */
     } 
 
     if ( hess_debug && v->proj )
-    { printf("v_id %d proj\n",ordinal(v_id)+1);
+    { printf("v_id %s proj\n",ELNAME(v_id));
       for ( j = 0 ; j < v->freedom ; j++ )
       { for ( m = 0 ; m < SDIM ; m++ ) printf("%f ",(DOUBLE)v->proj[m][j]);
         printf("\n");
@@ -1505,7 +1543,7 @@ REAL **X;  /* solution, allocated by hessian_init */
   if ( v_normal ) free_matrix3(v_normal); 
 
   /* assign row numbers */
-  for ( i = 0, v = vhead, currow = 0 ; i <= web.skel[VERTEX].max_ord ; i++,v++ )
+  for ( i = 0, v = vhead, currow = 0 ; i < vhead_count ; i++,v++ )
   { v->rownum = currow;
     currow += v->freedom;
   }
@@ -1517,8 +1555,11 @@ REAL **X;  /* solution, allocated by hessian_init */
 
   /* rows for optimizing parameters */
   S->optparamrowstart = vertex_rows = currow;
-  for ( i = 0 ; i < optparamcount ; i++,v++)
-  { v->rownum = currow;
+  for ( i = 0 ; i < optparamcount ; i++)
+  { 
+    optparam[i].vhead_index = i + web.skel[VERTEX].max_ord+1;
+    v = vhead + optparam[i].vhead_index;
+    v->rownum = currow;
     v->freedom = 1;
     optparam[i].rownum = currow++;
   }
@@ -1561,28 +1602,30 @@ REAL **X;  /* solution, allocated by hessian_init */
     { body_id b_id;
       FOR_ALL_BODIES(b_id)
       { int a = get_battr(b_id);
-        v = vhead + web.skel[VERTEX].max_ord+1 + ordinal(b_id);
+      
+        set_body_vhead(b_id,web.skel[VERTEX].max_ord+1 + loc_ordinal(b_id));
+        v = get_body_vhead(b_id);
         if ( (a & (FIXEDVOL|PRESSURE)) && !(a & REDUNDANT_BIT)  )
         { 
           v->v_id = b_id;
           v->freedom = 1;
-          S->coninx[ordinal(b_id)] = S->CN;
-          S->coninxinv[S->CN] = ordinal(b_id);
+          S->coninx[loc_ordinal(b_id)] = S->CN;
+          S->coninxinv[S->CN] = loc_ordinal(b_id);
           S->CN++;
         }
-        else S->coninx[ordinal(b_id)] = -1;
+        else S->coninx[loc_ordinal(b_id)] = -1;
+      }
+      FOR_ALL_BODIES(b_id)
+      { v = get_body_vhead(b_id);
+        v->rownum = currow;
+        currow += v->freedom;      
       }
-      for ( n=0, v=vhead+web.skel[VERTEX].max_ord+1 ;
-                     n <= web.skel[BODY].max_ord ; n++,v++ )
-        { v->rownum = currow;
-          currow += v->freedom;      
-        }
     }
     else  S->concount = gen_quant_count;
-    for ( n = 0, v = vhead+web.skel[VERTEX].max_ord+1+optparamcount+
-               (everything_quantities_flag ? 0 : web.skel[BODY].max_ord+1)
-                 ; n < gen_quant_count ; n++,v++ )
+    for ( n = 0 ; n < gen_quant_count ; n++,v++ )
     { int nn = everything_quantities_flag ? n : web.skel[BODY].max_ord+1 + n ;
+      GEN_QUANT(n)->vhead_index = web.skel[VERTEX].max_ord+1+optparamcount+nn;
+      v = vhead + GEN_QUANT(n)->vhead_index;
       v->rownum = currow;
       v->v_id = n;
       if ( (GEN_QUANT(n)->flags & (Q_FIXED|Q_CONSERVED) ) &&
@@ -1598,14 +1641,19 @@ REAL **X;  /* solution, allocated by hessian_init */
     }
   }
 
+  /* figure whether to do constraints as augmentation or separately */
+  if ( augmented_hessian_flag < 0 )
+  { augmented_hessian_mode = (sparse_constraints_flag && (S->CN >= 50)) ? 1 : 0;
+  }
+  else augmented_hessian_mode = augmented_hessian_flag;
+
   pressures = (REAL*)temp_calloc(web.skel[BODY].max_ord+1+gen_quant_count,
                 sizeof(REAL));
   conrhs = (REAL*)temp_calloc(web.skel[BODY].max_ord+1+gen_quant_count,
                 sizeof(REAL));
 
-    /* allocate right hand side and solution of augmented matrix */
+  /* allocate right hand side and solution of augmented matrix */
   if ( rhs ) *rhs = (REAL *)temp_calloc(S->total_rows,sizeof(REAL));
-  if ( X ) *X = (REAL *)temp_calloc(S->total_rows,sizeof(REAL));
 
   S->P = (int *)temp_calloc(S->total_rows,sizeof(int));
   S->IP = (int *)temp_calloc(S->total_rows,sizeof(int));
@@ -1613,6 +1661,8 @@ REAL **X;  /* solution, allocated by hessian_init */
   /* initialize hash table */
   sp_hash_init(S,0);
 
+  /* tell solvers element data is appropriate */
+  S->flags |= S_USE_ELEMENTS;
 } /* end hessian_init */
 
 /*************************************************************************
@@ -1621,19 +1671,30 @@ REAL **X;  /* solution, allocated by hessian_init */
 *
 * purpose: call routines that calculate hessian.
 */
-void hessian_fill(S,rhs)
+void hessian_fill(S,rhsptr)
 struct linsys *S;
-REAL *rhs;
-{
+REAL **rhsptr;
+{ int k;
+  body_id b_id;
+  REAL *rhs = rhsptr ? *rhsptr : NULL;
+
+  #ifdef MPI_EVOLVER
+  if ( this_machine == 0 )
+  { mpi_hessian_fill(S,rhsptr);
+    temp_free((char*)S->hashtable); /* since not calling hash_end() */
+    rhs = rhsptr ? *rhsptr : NULL;  /* mpi_hessian_fill reallocates it */
+    goto rhs_constraints;
+  }
+  #endif
+ 
   /* clear entries */
   if ( rhs_flag ) memset((char*)rhs,0,S->total_rows*sizeof(REAL));
 
-
   /* fill in hessian */
   if ( hessian_by_diff_flag )
   { /* hessian by crude numeric difference formulas */
-     difference_hessian(S,vhead,rhs);
-     if ( optparamcount ) optparamhess(S,rhs);
+    difference_hessian(S,vhead,rhs);
+    if ( optparamcount ) optparamhess(S,rhs);
   }
   else /* hessian by explicit formulas */
   {
@@ -1675,6 +1736,27 @@ quantplace:
       sp_hash_end(S,S->A_rows,S->total_rows,A_OFF);
 
   }
+
+#ifdef MPI_EVOLVER
+rhs_constraints:
+#endif
+
+  /* rhs for body constraint rows */
+  if ( rhs && !everything_quantities_flag )
+  FOR_ALL_BODIES(b_id)
+  { REAL *current = rhs + S->bodyrowstart + loc_ordinal(b_id);
+    if ( get_battr(b_id) & FIXEDVOL )
+         *current = -(get_body_fixvol(b_id) - get_body_volume(b_id));
+  }
+
+  /* rhs for quantity constraint rows */
+  if ( rhs )
+  for ( k = 0 ; k < gen_quant_count ; k++ )
+  { struct gen_quant *q = GEN_QUANT(k);
+    if ( q->flags & Q_FIXED )
+      rhs[S->quanrowstart + k] = -(q->target - q->value);
+  }
+
 }  /* end hessian_fill() */
 
 
@@ -1690,8 +1772,9 @@ REAL stepsize;
 int mode; /* ACTUAL_MOVE or TEST_MOVE or SET_VELOCITY */
 REAL *X;  /* direction */
 {
-  int i,j,n;
+  int i,j;
   struct hess_verlist *vc;              /* current  vertex */
+  vertex_id v_id;
 
   last_hessian_scale = stepsize;
 
@@ -1702,18 +1785,24 @@ REAL *X;  /* direction */
       globals(optparam[i].pnum)->value.real += stepsize*X[optparam[i].rownum];
   }
   /* move vertices */
-  for ( n = 0, vc = vhead ; n <= web.skel[VERTEX].max_ord ; n++, vc++ )
+  #ifdef MPI_EVOLVER
+  if ( this_machine == 0 ) 
+    mpi_hessian_move(stepsize,mode,X);
+  else
+  #endif
+  FOR_ALL_VERTICES(v_id)
   {
      REAL *coord;
      REAL *velocity;
 
+     vc = get_vertex_vhead(v_id);
      if ( vc->freedom <= 0 ) continue;
-     coord = get_coord(vc->v_id);
-     velocity = get_velocity(vc->v_id);
-     if ( get_vattr(vc->v_id) & BOUNDARY )
-     { struct boundary *boundary = get_boundary(vc->v_id);
+     coord = get_coord(v_id);
+     velocity = get_velocity(v_id);
+     if ( get_vattr(v_id) & BOUNDARY )
+     { struct boundary *boundary = get_boundary(v_id);
        int pcount = boundary->pcount;
-       REAL *param = get_param(vc->v_id);
+       REAL *param = get_param(v_id);
 
        for ( i = 0 ; i < pcount ; i++ )
        { velocity[i] = X[vc->rownum + i];
@@ -1722,7 +1811,7 @@ REAL *X;  /* direction */
        }
        if ( mode != SET_VELOCITY )
         for ( i = 0 ; i < SDIM ; i++ )
-         coord[i] = eval(boundary->coordf[i],param,vc->v_id,NULL);
+         coord[i] = eval(boundary->coordf[i],param,v_id,NULL);
      }
      else
      { for ( j = 0 ; j < SDIM ; j++ )
@@ -1738,6 +1827,11 @@ REAL *X;  /* direction */
          }
      }
   }
+
+  #ifdef MPI_EVOLVER
+  if ( this_machine != 0 ) return;
+  #endif
+
   if ( mode == SET_VELOCITY ) return;
   if ( hess_move_con_flag )
   {
@@ -1762,35 +1856,35 @@ REAL *X;  /* direction */
 
 void hessian_exit(X)
 REAL *X; /* direction, if any */
-{ int j,n;
+{ int j;
   struct hess_verlist *vc;              /* current  vertex */
+  vertex_id v_id;
 
   hess_debug = 0;
-
   
   /* store move in vertex force */
   if ( X && vhead )
-     for ( n = 0, vc = vhead ; n < web.skel[VERTEX].max_ord ; n++, vc++ )
-     {
-        REAL *f;
+    FOR_ALL_VERTICES(v_id)
+    {
+      REAL *f;
+      vc = get_vertex_vhead(v_id);
+      if ( vc->freedom <= 0 ) continue;
+      f = get_force(v_id);
+      if ( get_vattr(v_id) & BOUNDARY )
+      { struct boundary *boundary = get_boundary(v_id);
+        int pcount = boundary->pcount;
 
-        if ( vc->freedom <= 0 ) continue;
-        f = get_force(vc->v_id);
-        if ( get_vattr(vc->v_id) & BOUNDARY )
-        { struct boundary *boundary = get_boundary(vc->v_id);
-          int pcount = boundary->pcount;
-
-          for ( j = 0 ; j < pcount ; j++ )
-                f[j] = X[vc->rownum + j];
-        }
-        else
-        { for ( j = 0 ; j < SDIM ; j++ )
-            if ( vc->proj )
-              f[j] = dot(vc->proj[j],X+vc->rownum,vc->freedom);
-            else
-              f[j] = X[vc->rownum + j];
-        }
-     }
+        for ( j = 0 ; j < pcount ; j++ )
+          f[j] = X[vc->rownum + j];
+      }
+      else
+      { for ( j = 0 ; j < SDIM ; j++ )
+          if ( vc->proj )
+             f[j] = dot(vc->proj[j],X+vc->rownum,vc->freedom);
+          else
+        f[j] = X[vc->rownum + j];
+      }
+    }
   hessian_cleanup();
 }
 
@@ -1821,7 +1915,7 @@ REAL *C; /* product */
         }
       }
     }
-    else if ( augmented_hessian_flag )
+    else if ( augmented_hessian_mode )
     { for ( i = 0 ; i < S->A_rows ; i++ )
         C[i] = -S->lambda*B[i];
       for (  ; i < S->N ; i++ )
@@ -1858,12 +1952,9 @@ void hessian_cleanup()
   if ( vproj_space ) myfree((char*)vproj_space); vproj_space = NULL;
   if ( conhess_base ) free_matrix3(conhess_base); conhess_base = NULL;
   unsave_coords(&saved,SAVE_IN_ATTR);
-  if ( pressures ) temp_free((char*)pressures);
-  if ( conrhs ) temp_free((char*)conrhs);
+  if ( pressures ) temp_free((char*)pressures); pressures = NULL;
+  if ( conrhs ) temp_free((char*)conrhs); conrhs = NULL;
   if ( ritzvecs ) free_matrix(ritzvecs); ritz_done_flag = 0; ritzvecs = NULL;
-  vhead = NULL;
-  pressures = NULL;
-  conrhs = NULL;
   return;
 
 }
@@ -1883,6 +1974,7 @@ struct linsys *S;
   char name[100];
   int i,j,n,row;
   struct hess_verlist *v;
+  vertex_id v_id;
 
   prompt("Enter hessian file name: ",name,sizeof(name));
   fd = fopen(name,"w");
@@ -1905,36 +1997,40 @@ struct linsys *S;
 
   /* per row info:
       point coords, basis vectors for each degree of freedom,
-    */
+   */
   
-  for ( i = 0, v = vhead ; i <= web.skel[VERTEX].max_ord ; i++,v++ )
-  { REAL *x = get_coord(v->v_id);
-     for ( j = 0 ; j < v->freedom ; j++ )
-     { fprintf(fd,"%d ",v->rownum+j);
-        for ( n = 0 ; n < SDIM ; n++ ) fprintf(fd," %21.15e",(DOUBLE)x[n]);
-        if ( v->proj )
-         for ( n = 0 ; n < SDIM ; n++ ) fprintf(fd," %21.15e",(DOUBLE)v->proj[n][j]);
-        else
-         for ( n = 0 ; n < SDIM ; n++ ) fprintf(fd," %d",n==j);
-        fprintf(fd,"\n");
-      }
+  FOR_ALL_VERTICES(v_id)
+  { REAL *x = get_coord(v_id);
+    v = get_vertex_vhead(v_id);
+    for ( j = 0 ; j < v->freedom ; j++ )
+    { fprintf(fd,"%d ",v->rownum+j);
+      for ( n = 0 ; n < SDIM ; n++ ) fprintf(fd," %21.15e",(DOUBLE)x[n]);
+      if ( v->proj )
+        for ( n = 0 ; n < SDIM ; n++ ) 
+          fprintf(fd," %21.15e",(DOUBLE)v->proj[n][j]);
+      else
+        for ( n = 0 ; n < SDIM ; n++ ) fprintf(fd," %d",n==j);
+      fprintf(fd,"\n");
+    }
   }
 
   /* the Hessian */
-  for ( i = 0, v = vhead ; i <= web.skel[VERTEX].max_ord ; i++,v++ )
+  FOR_ALL_VERTICES(v_id)
   { 
-     if ( v->freedom == 0 ) continue;
-     for ( row = v->rownum ; row < v->rownum + v->freedom ; row++ )
-     { int start = S->IA[row]-A_OFF;
-        int end = S->IA[row+1] - A_OFF;
-        for ( j = start ; j < end ; j++ )
-          fprintf(fd,"%d %d %21.15e\n",row,S->JA[j]-A_OFF,(DOUBLE)S->A[j]);
-     }
+    v = get_vertex_vhead(v_id);
+    if ( v->freedom == 0 ) continue;
+    for ( row = v->rownum ; row < v->rownum + v->freedom ; row++ )
+    { int start = S->IA[row]-A_OFF;
+      int end = S->IA[row+1] - A_OFF;
+      for ( j = start ; j < end ; j++ )
+        fprintf(fd,"%d %d %21.15e\n",row,S->JA[j]-A_OFF,(DOUBLE)S->A[j]);
+    }
   }
   /* the metric */
   if (hessian_linear_metric_flag)
-  for ( i = 0, v = vhead ; i <= web.skel[VERTEX].max_ord ; i++,v++ )
-  { 
+   FOR_ALL_VERTICES(v_id)
+   { 
+     v = get_vertex_vhead(v_id);
      if ( v->freedom == 0 ) continue;
      for ( row = v->rownum ; row < v->rownum + v->freedom ; row++ )
      { int start = Met.IA[row]-A_OFF;
@@ -1942,7 +2038,7 @@ struct linsys *S;
         for ( j = start ; j < end ; j++ )
           fprintf(fd,"%d %d %21.15e\n",row,Met.JA[j]-A_OFF,(DOUBLE)Met.A[j]);
      }
-  }
+   }
 
   /* constraints */
   for ( i = 0 ; i < S->CN ; i++ )
