@@ -70,7 +70,7 @@ REAL *rhs;
   /* move vertices 1 by 1 */
   FOR_ALL_VERTICES(v_id)
   {
-     int ord = ordinal(v_id);
+     int ord = loc_ordinal(v_id);
      struct hess_verlist *v = verhead+ord;
      REAL *x = get_coord(v_id);
 
@@ -101,7 +101,7 @@ REAL *rhs;
   if ( hess_flag )
   FOR_ALL_VERTICES(v_id)
   {
-     int ord = ordinal(v_id);
+     int ord = loc_ordinal(v_id);
      struct hess_verlist *v = verhead+ord;
      REAL *x = get_coord(v_id);
 
@@ -109,7 +109,7 @@ REAL *rhs;
      
      FOR_ALL_VERTICES(vv_id)
      {
-        int ordd = ordinal(vv_id);
+        int ordd = loc_ordinal(vv_id);
         struct hess_verlist *vv = verhead+ordd;
         REAL *xx = get_coord(vv_id);
 
@@ -170,7 +170,7 @@ int seekmode; /* 1 if want to search in direction of min */
   int r,s;
   REAL area;
   REAL cota;
-  int i,j,n;
+  int i,j;
   facet_id f_id;
   edge_id e_id;
   vertex_id v[3];
@@ -184,11 +184,12 @@ int seekmode; /* 1 if want to search in direction of min */
   REAL scale;
   REAL best_scale;
   struct linsys S;
+  vertex_id v_id;
 
   if ( fixed_constraint_flag ) 
      kb_error(2081,"Cannot do dirichlet with constraints.\n",RECOVERABLE);
   hmode = SINGLE_DEGREE; /* same hessian for each coordinate */
-  hessian_init(&S,NULL,NULL);
+  hessian_init(&S,NULL);
   rs = dmatrix(0,SDIM-1,0,S.A_rows); /* right side for each coord */
   xx = dmatrix(0,SDIM-1,0,S.A_rows); /* solution vector for each coord */
                       /*  differences from current */
@@ -213,8 +214,8 @@ int seekmode; /* 1 if want to search in direction of min */
         { ii = (i+1)%FACET_VERTS; iii = (i+2)%FACET_VERTS;
           x = get_coord(v[ii]);
           y = get_coord(v[iii]);
-          rv = vhead+ordinal(v[ii]);
-          sv = vhead+ordinal(v[iii]);
+          rv = get_vertex_vhead(v[ii]);
+          sv = get_vertex_vhead(v[iii]);
           r = rv->rownum;
           s = sv->rownum;
           cota = -sd[i]/area;
@@ -271,15 +272,18 @@ int seekmode; /* 1 if want to search in direction of min */
   { REAL energies[3]; 
     REAL denom;
     int k;
-
+    vertex_id v_id;
+    
      save_coords(&saved,SAVE_IN_ATTR);  /* in case this doesn't work */
      energies[0] = web.total_energy;
      for ( scale = 1.0, k = 1 ; k < 3 ; scale += 1.0, k++ ) 
-     { for ( n = 0, vc = vhead ; n <= web.skel[VERTEX].max_ord ; n++, vc++ )
+     { 
+       FOR_ALL_VERTICES(v_id)
         { REAL *coord;
 
+          vc = get_vertex_vhead(v_id);
           if ( vc->freedom == 0 ) continue;
-          coord = get_coord(vc->v_id);
+          coord = get_coord(v_id);
           for ( j = 0 ; j < SDIM ; j++ )
              coord[j] = coord[j] - scale*xx[j][vc->rownum];
         }
@@ -301,14 +305,14 @@ int seekmode; /* 1 if want to search in direction of min */
             
   }
   scale = best_scale;
-  for ( n = 0, vc = vhead ; n <= web.skel[VERTEX].max_ord ; n++, vc++ )
-     { REAL *coord;
-
-        if ( vc->freedom == 0 ) continue;
-        coord = get_coord(vc->v_id);
-        for ( j = 0 ; j < SDIM ; j++ )
-          coord[j] = coord[j] - scale*xx[j][vc->rownum];
-     }
+  FOR_ALL_VERTICES(v_id)
+  { REAL *coord;
+    vc = get_vertex_vhead(v_id);
+    if ( vc->freedom == 0 ) continue;
+    coord = get_coord(vc->v_id);
+    for ( j = 0 ; j < SDIM ; j++ )
+      coord[j] = coord[j] - scale*xx[j][vc->rownum];
+  }
 
   free_matrix(rs);
   free_matrix(xx);
@@ -349,10 +353,11 @@ int seekmode;  /* 1 if seek for best stepsize */
   facet_id f_id;
   edge_id e_id;
   vertex_id v[3];
+  vertex_id v_id;
   facetedge_id fe;
   REAL ss[FACET_VERTS],sd[FACET_VERTS];
   int ii,iii;
-  REAL *rs;  /* one right side for each coordinate */
+  REAL *rs,*rstmp;  /* one right side for each coordinate */
   REAL *rs_save;
   struct hess_verlist *vc;              /* current  vertex */
   struct hess_verlist *rv,*sv;
@@ -366,9 +371,10 @@ int seekmode;  /* 1 if seek for best stepsize */
 kb_error(2083,"Sobolev not working properly.\n",WARNING);
 
   hmode = UNRESTRICTED; /* full hessian needed */
-  hessian_init(&S,NULL,NULL);
+  hessian_init(&S,NULL);
   rs = (REAL*)temp_calloc(S.A_rows,sizeof(REAL)); /* right side for each coord */
   rs_save = (REAL*)temp_calloc(S.A_rows,sizeof(REAL)); /* right side for each coord */
+  rstmp = (REAL*)temp_calloc(S.A_rows,sizeof(REAL)); /* right side for each coord */
 
   /* fill in */
   FOR_ALL_FACETS(f_id)
@@ -388,8 +394,8 @@ kb_error(2083,"Sobolev not working properly.\n",WARNING);
     area = sqrt(ss[0]*ss[2] - sd[0]*sd[0]);
     for ( i = 0 ; i < FACET_VERTS ; i++ )
     { ii = (i+1)%FACET_VERTS; iii = (i+2)%FACET_VERTS;
-      rv = vhead+ordinal(v[ii]);
-      sv = vhead+ordinal(v[iii]);
+      rv = get_vertex_vhead(v[ii]);
+      sv = get_vertex_vhead(v[iii]);
       r = rv->rownum;
       /* vertex self-term */
       if ( !(get_vattr(v[iii]) & FIXED) )
@@ -401,38 +407,39 @@ kb_error(2083,"Sobolev not working properly.\n",WARNING);
              sp_hash_search(&S,sv->rownum+j,sv->rownum+k,
                 -side[i][j]*side[i][k]/area);
          }
-             }
-          /* edge cross-term */
-          if ( !(get_vattr(v[iii]) & FIXED) && !(get_vattr(v[ii]) & FIXED) )
-             for ( j = 0 ; j < SDIM ; j++ )
-             { 
-               if ( rv->rownum > sv->rownum )
-               { sp_hash_search(&S,sv->rownum+j,rv->rownum+j,sd[i]/area);
-                 for ( k = 0 ; k < SDIM ; k++ )
-                    sp_hash_search(&S,sv->rownum+k,rv->rownum+j,
-                       -side[i][j]*side[iii][k]/area);
-               }
-               else 
-               {
-                 for ( k = 0 ; k < SDIM ; k++ )
-                   sp_hash_search(&S,rv->rownum+k,sv->rownum+j,
-                      -side[i][j]*side[iii][k]/area);
-               }
-             }
-          /* right side */ 
-          if ( !(get_vattr(v[ii]) & FIXED) )
-             for ( j = 0 ; j < SDIM ; j++ )
-              rs[r+j] -= (sd[i]*side[iii][j] - ss[iii]*side[i][j])/area;
-         }
-     }
+      }
+      /* edge cross-term */
+      if ( !(get_vattr(v[iii]) & FIXED) && !(get_vattr(v[ii]) & FIXED) )
+        for ( j = 0 ; j < SDIM ; j++ )
+        { 
+          if ( rv->rownum > sv->rownum )
+          { sp_hash_search(&S,sv->rownum+j,rv->rownum+j,sd[i]/area);
+            for ( k = 0 ; k < SDIM ; k++ )
+               sp_hash_search(&S,sv->rownum+k,rv->rownum+j,
+                   -side[i][j]*side[iii][k]/area);
+          }
+          else 
+          {
+            for ( k = 0 ; k < SDIM ; k++ )
+              sp_hash_search(&S,rv->rownum+k,sv->rownum+j,
+                  -side[i][j]*side[iii][k]/area);
+          }
+        }
+      /* right side */ 
+      if ( !(get_vattr(v[ii]) & FIXED) )
+        for ( j = 0 ; j < SDIM ; j++ )
+          rs[r+j] -= (sd[i]*side[iii][j] - ss[iii]*side[i][j])/area;
+    }
+  }
      
 if ( hess_debug )
 { printf("Gradient: \n");
-  for ( n = 0, vc = vhead ; n < vhead_count ; n++, vc++ )
-  { if ( vc->freedom == 0 ) continue;
-     printf("n: %d  ",n);
-     for ( j = 0 ; j < SDIM ; j++ ) printf ("%f ",(DOUBLE)rs[vc->rownum+j]);
-     printf("\n");
+  FOR_ALL_VERTICES(v_id)
+  { vc = get_vertex_vhead(v_id);
+    if ( vc->freedom == 0 ) continue;
+    printf("n: %s  ",ELNAME(v_id));
+    for ( j = 0 ; j < SDIM ; j++ ) printf ("%f ",(DOUBLE)rs[vc->rownum+j]);
+    printf("\n");
   }
 }
 
@@ -442,12 +449,13 @@ if ( hess_debug )
   /* solve system */
   memcpy((char*)rs_save,(char*)rs,sizeof(REAL)*S.A_rows);
   (*sp_AIJ_setup_func)(S.A_rows,&S);
-  if ( sp_ordering_func ) (*sp_ordering_func)(&S);
   (*sp_constraint_setup_func)
             (web.skel[BODY].max_ord+1 + gen_quant_count,&S);
+  if ( sp_ordering_func ) (*sp_ordering_func)(&S);
   (*sp_factor_func)(&S);
   (*sp_hess_project_setup_func)(&S);
-  sp_hessian_solve(&S,rs,rs,NO_SET_PRESSURE);
+  sp_hessian_solve(&S,rs,rstmp,NO_SET_PRESSURE);
+  for ( j = 0 ; j < S.N ; j++ ) rs[j] = rstmp[j];
 
 if ( hess_debug )
 { printf("Motion: \n");
@@ -469,17 +477,17 @@ if ( hess_debug )
      save_coords(&saved,SAVE_IN_ATTR);  /* in case this doesn't work */
      energies[0] = web.total_energy;
      for ( scale = 1.0, k = 1 ; k < 3 ; scale += 1.0, k++ ) 
-     { for ( n = 0, vc = vhead ; n <= web.skel[VERTEX].max_ord ; n++, vc++ )
-        { REAL *coord;
-
-          if ( vc->freedom == 0 ) continue;
-          coord = get_coord(vc->v_id);
-          for ( j = 0 ; j < SDIM ; j++ )
+     { FOR_ALL_VERTICES(v_id)
+       { REAL *coord;
+         vc = get_vertex_vhead(v_id);
+         if ( vc->freedom == 0 ) continue;
+         coord = get_coord(v_id);
+         for ( j = 0 ; j < SDIM ; j++ )
              coord[j] = coord[j] - scale*rs[vc->rownum+j];
-        }
-        global_timestamp++;
-        calc_energy();  /* energy after motion */
-        if ( hess_debug || itdebug )
+       }
+       global_timestamp++;
+       calc_energy();  /* energy after motion */
+       if ( hess_debug || itdebug )
 #ifdef LONGDOUBLE
           printf("scale %f energy %*.*Lf\n",(DOUBLE)scale,DWIDTH,DPREC,web.total_energy);
 #else
@@ -494,17 +502,18 @@ if ( hess_debug )
      else best_scale = 0.5*(3*energies[0]-4*energies[1]+energies[2])/denom;
   }
   scale = best_scale;
-  for ( n = 0, vc = vhead ; n <= web.skel[VERTEX].max_ord ; n++, vc++ )
-     { REAL *coord;
-
-        if ( vc->freedom == 0 ) continue;
-        coord = get_coord(vc->v_id);
-        for ( j = 0 ; j < SDIM ; j++ )
-          coord[j] = coord[j] - scale*rs[vc->rownum+j];
-     }
+  FOR_ALL_VERTICES(v_id)
+  { REAL *coord;
+    vc = get_vertex_vhead(v_id);
+    if ( vc->freedom == 0 ) continue;
+    coord = get_coord(v_id);
+    for ( j = 0 ; j < SDIM ; j++ )
+      coord[j] = coord[j] - scale*rs[vc->rownum+j];
+  }
 
   temp_free((char*)rs);
   temp_free((char*)rs_save);
+  temp_free((char*)rstmp);
   free_system(&S);
   hessian_exit(NULL);
 
@@ -727,7 +736,7 @@ struct linsys *S;
   REAL minev; /* eigenvalue lower bound */
   REAL nextev;
   REAL coeffA,coeffc;
-  REAL rayleigh;  /* Rayleigh quotient */
+  REAL rayleigh=0.0;  /* Rayleigh quotient */
   REAL *X;
 
   v = (REAL *)temp_calloc(S->N+1,sizeof(REAL));
@@ -803,16 +812,18 @@ int iters;  /* iteration bound */
   int i,k;
   REAL t,oldt;
   REAL *W = NULL; /* for use when metric is needed */
-  REAL *X; /* for eigenvector */
+  REAL *X = NULL; /* for eigenvector */
   hmode = hessian_normal_flag;
-  hessian_init(&S,NULL,&X);
-   hess_flag = 1; rhs_flag = 0; 
+  hessian_init(&S,NULL);
+  hess_flag = 1; rhs_flag = 0; 
   hessian_fill(&S,NULL); 
   S.lambda = lambda;
+   
+
   (*sp_AIJ_setup_func)(S.A_rows,&S);
-   if ( sp_ordering_func ) (*sp_ordering_func)(&S);
   (*sp_constraint_setup_func)
        (web.skel[BODY].max_ord+1 + gen_quant_count,&S);
+  if ( sp_ordering_func ) (*sp_ordering_func)(&S);
   if ( hessian_linear_metric_flag )
   { 
     linear_metric_setup(&S,&Met);
@@ -821,8 +832,11 @@ int iters;  /* iteration bound */
   { 
     star_metric_setup(&S,&Met);
   }
+
  (*sp_factor_func)(&S);
+
  (*sp_hess_project_setup_func)(&S);
+
   sprintf(msg,"Eigencounts:    %d <,  %d ==,  %d > \n",S.neg,S.zero,S.pos);
   outstring(msg);
   
@@ -896,14 +910,14 @@ int krydim;  /* dimension of Krylov subspace */
   int i,nprint;
   
   hmode = hessian_normal_flag;
-  hessian_init(&S,NULL,NULL);
+  hessian_init(&S,NULL);
   hess_flag = 1; rhs_flag = 0; 
   hessian_fill(&S,NULL); 
   S.lambda = lambda;
   (*sp_AIJ_setup_func)(S.A_rows,&S);
-  if ( sp_ordering_func ) (*sp_ordering_func)(&S);
   (*sp_constraint_setup_func)
       (web.skel[BODY].max_ord+1 + gen_quant_count,&S);
+  if ( sp_ordering_func ) (*sp_ordering_func)(&S);
   if ( hessian_linear_metric_flag )
   {
     linear_metric_setup(&S,&Met);
@@ -1325,16 +1339,16 @@ int ritzdim;  /* dimension of Rayleigh-Ritz subspace */
   memset((char*)&S,0,sizeof(struct linsys)); 
   
   hmode = hessian_normal_flag;
-  hessian_init(&S,&rhs,NULL);
+  hessian_init(&S,&rhs);
   hess_flag = 1; rhs_flag = 0; 
-  hessian_fill(&S,rhs); 
+  hessian_fill(&S,&rhs); 
   if ( sparse_constraints_flag )
     (*sp_AIJ_setup_func)(S.total_rows,&S);
   else
     (*sp_AIJ_setup_func)(S.A_rows,&S);
-  if ( sp_ordering_func ) (*sp_ordering_func)(&S);
   (*sp_constraint_setup_func)
        (web.skel[BODY].max_ord+1 + gen_quant_count,&S);
+  if ( sp_ordering_func ) (*sp_ordering_func)(&S);
   if ( hessian_linear_metric_flag )
   {
     linear_metric_setup(&S,&Met);
@@ -1386,11 +1400,6 @@ REAL **ritzvec; /* non-null if want eigenvectors returned */
   int ritzwanted;
   int maxritz;
 
-#ifdef PROFIL
- static u_short probuf[0x1000];  /* for do-it-myself profiling */
-#endif
-
-
   /* see if user wants extra ritz vectors */
   ritzwanted = ritzdim % 1000;
   ritzdim    = (ritzdim/1000) +  ritzwanted;
@@ -1405,14 +1414,7 @@ REAL **ritzvec; /* non-null if want eigenvectors returned */
 
   if ( ritzdim == 0 ) return;
 
-#ifdef PROFIL
-/* a little do-it-yourself profiling */
-if ( hessian_quiet_flag == 0 )
-{ memset(probuf,0,sizeof(probuf));
-  profil(probuf,0x1000,(int)xmd_solve,0x20000); /* one byte resolution */
-}
-#endif
-  maxritz = S->N - (augmented_hessian_flag ? 2 : 1)* S->CN; 
+  maxritz = S->N - (augmented_hessian_mode ? 2 : 1)* S->CN; 
   if ( ritzdim > maxritz )
     ritzdim = maxritz;
   if ( ritzwanted > ritzdim ) ritzwanted = ritzdim;
@@ -1449,7 +1451,7 @@ if ( hessian_quiet_flag == 0 )
     }
     else sp_hessian_solve_multi(S,rbasis+converged,Arbasis+converged,
           ritzdim-converged);
-    if  ( augmented_hessian_flag )
+    if  ( augmented_hessian_mode )
     { /* set augmented part to 0 */
       for ( i = converged ; i < ritzdim ; i++ )
         for ( j = S->A_rows ; j < S->N ; j++ )
@@ -1550,15 +1552,6 @@ if ( hessian_quiet_flag == 0 )
   else
   last_eigenvalue = evalues[0];
 
-#ifdef PROFIL
-/* end profiling and display results */
-if ( hessian_quiet_flag == 0 )
-{ profil(NULL,0,0,0);
-  for ( i = 0 ; i < 0x1000 ; i++ )
-    if ( probuf[i] > 0 )
-       printf("xmd_solve+%3d: %5d\n",i,probuf[i]);
-}
-#endif
   free_matrix(Lmat);
   free_matrix(A);
   if ( ritzvec == NULL ) free_matrix(rbasis);
@@ -1569,3 +1562,4 @@ if ( hessian_quiet_flag == 0 )
   temp_free((char*)work);
   free_matrix(mw);
 } /* end do_ritz() */
+
