@@ -51,7 +51,7 @@ REAL *rhs;
     fe_id = get_facet_fe(f_id);
     for ( i = 0 ; i < FACET_EDGES ; i++ )
     { v_id[i] = get_fe_tailv(fe_id);
-      v[i] = vhead + ordinal(v_id[i]);
+      v[i] = get_vertex_vhead(v_id[i]);
       get_fe_side(fe_id,side[i]);
       fe_id = get_next_edge(fe_id);
     }
@@ -136,7 +136,7 @@ REAL *rhs;
         if ( ! get_e_constraint_map(e_id)[0] ) continue;
 
         for ( i = 0 ; i < 2 ; i++ )
-          v[i] = vhead + ordinal(v_id[i]);
+          v[i] = get_vertex_vhead(v_id[i]);
 
         memset((char*)first[0],0,sizeof(REAL)*2*MAXCOORD);
         memset((char*)second[0][0][0],0,sizeof(REAL)*2*2*MAXCOORD*MAXCOORD);
@@ -152,7 +152,7 @@ REAL *rhs;
           for ( j = i ; j < 2 ; j++ )
           {
              if ( (v[i]->freedom==0) || (v[j]->freedom==0) ) continue;
-             if ( ordinal(v_id[i]) > ordinal(v_id[j]) )
+             if ( loc_ordinal(v_id[i]) > loc_ordinal(v_id[j]) )
                  { tail = i; head = j; }
              else { tail = j; head = i; }
 
@@ -263,7 +263,7 @@ REAL *rhs;
   Z = (REAL *)temp_calloc(web.skel[BODY].max_ord+1,sizeof(REAL));
   FOR_ALL_BODIES(b_id) 
      if ( get_battr(b_id) & FIXEDVOL )
-        Z[ordinal(b_id)] = get_body_pressure(b_id);
+        Z[loc_ordinal(b_id)] = get_body_pressure(b_id);
   /* Z is vector of coefficients for adding constraint hessians */
 
   if ( hess_debug ) 
@@ -275,7 +275,7 @@ REAL *rhs;
   /* rhs for body constraint rows */
   if ( rhs_flag && !everything_quantities_flag )
   FOR_ALL_BODIES(b_id)
-  { REAL *current = rhs + S->bodyrowstart + ordinal(b_id);
+  { REAL *current = rhs + S->bodyrowstart + loc_ordinal(b_id);
     if ( get_battr(b_id) & FIXEDVOL )
          *current = -(get_body_fixvol(b_id) - get_body_volume(b_id));
   }
@@ -309,7 +309,7 @@ REAL *rhs;
 
     fe_id = get_facet_fe(f_id);
     for ( i = 0 ; i < FACET_EDGES ; i++ )
-    { v[i] = vhead + ordinal(get_fe_tailv(fe_id));
+    { v[i] = get_vertex_vhead(get_fe_tailv(fe_id));
       x[i] = get_coord(get_fe_tailv(fe_id));
       get_fe_side(fe_id,side[i]);
       fe_id = get_next_edge(fe_id);
@@ -317,9 +317,9 @@ REAL *rhs;
 
     coe = 0.0;
     if ( valid_id(b_id) )
-       coe += Z[ordinal(b_id)];
+       coe += Z[loc_ordinal(b_id)];
     if ( valid_id(bb_id) )
-       coe -= Z[ordinal(bb_id)];
+       coe -= Z[loc_ordinal(bb_id)];
 
     zsum = (x[0][2]+x[1][2]+x[2][2]);
     ssum = side[0][0]*side[1][1] - side[0][1]*side[1][0];
@@ -354,7 +354,7 @@ REAL *rhs;
 
       if ( do_b )
       {
-        currentrow = S->bodyrowstart + ordinal(b_id);
+        currentrow = S->bodyrowstart + loc_ordinal(b_id);
         for ( j = 0 ; j < v[i]->freedom ; j++ )
         { sp_hash_search(S,v[i]->rownum+j,currentrow,ggg[j]/6);
         }
@@ -362,7 +362,7 @@ REAL *rhs;
          
       if ( do_bb )
       {
-        currentrow = S->bodyrowstart + ordinal(bb_id);
+        currentrow = S->bodyrowstart + loc_ordinal(bb_id);
         for ( j = 0 ; j < v[i]->freedom ; j++ )
         { sp_hash_search(S,v[i]->rownum+j,currentrow,-ggg[j]/6);
         }
@@ -389,17 +389,17 @@ REAL *rhs;
         coe = 0.0;
         b_id = get_facet_body(f_id);
         if ( valid_id(b_id) )
-          coe += Z[ordinal(b_id)];
+          coe += Z[loc_ordinal(b_id)];
         b_id = get_facet_body(inverse_id(f_id));
         if ( valid_id(b_id) )
-          coe -= Z[ordinal(b_id)];
+          coe -= Z[loc_ordinal(b_id)];
         coe /= 6; /* tetrahedron factor */
         if ( coe == 0.0 ) continue;
 
         fe_id = get_facet_fe(f_id);
         for ( i = 0 ; i < FACET_EDGES ; i++ )
         { v_id[i] = get_fe_tailv(fe_id);
-          v[i] = vhead + ordinal(v_id[i]);
+          v[i] = get_vertex_vhead(v_id[i]);
           x[i] = get_coord(v_id[i]);
           fe_id = get_next_edge(fe_id);
         }
@@ -482,6 +482,8 @@ struct linsys *S;
   int PATH,FLAG=0,ESP=0;
   REAL *RSP,EMAX;
 
+  PROF_START(hessian_factor);
+
   /* need just enough at first to do symbolic factorization */
   if ( S->NSP == 0 )
   { S->NSP = (4 + (int)(log(S->N+1)))*(S->N + S->IA[S->N]);
@@ -498,23 +500,25 @@ struct linsys *S;
       if ( S->JA[j]-A_OFF == i )
          { S->A[j] -= S->lambda; break; }
   }
-
   /*  Call ODRV to perform the reordering on A, if not done yet */
   if ( !(S->flags & S_ODRV_REORDERED) )
   { PATH = (S->flags & S_ORDERFOUND) ? 3 : 2; 
     if ( S->P == NULL ) S->P = (int *)temp_calloc(S->N,sizeof(int));
-	if ( S->IP == NULL ) S->IP = (int *)temp_calloc(S->N,sizeof(int));
+    if ( S->IP == NULL ) S->IP = (int *)temp_calloc(S->N,sizeof(int));
     odrv_( &S->N, S->IA,S->JA,S->A, S->P,S->IP, &S->NSP,S->ISP,&PATH,&FLAG );
     odrv_flag_check(FLAG,S->N);
     S->flags |= S_ODRV_REORDERED;
   }
+  
+
 
   /*  Call SDRVMD to factor symbolically and numerically */
   PATH = 4; /* symbolic factor */
   RSP = (REAL *) S->ISP;
   sdrvmd_(&S->N,S->P,S->IP,S->IA,S->JA,S->A,NULL,NULL,&S->NSP,
             S->ISP,RSP,&ESP, &PATH,&FLAG,&EMAX);
-  sdrv_flag_check(0,FLAG,S->N);
+ 
+ sdrv_flag_check(0,FLAG,S->N);
   PATH = 6; /* numeric factor */
   if ( !hessian_quiet_flag )
   { if ( ESP < 0 )
@@ -526,7 +530,7 @@ struct linsys *S;
     outstring(msg);
   }
   S->ISP = (int*)temp_realloc((char*)(S->ISP),
-          (S->NSP-ESP+10)*sizeof(REAL), S->NSP*sizeof(REAL));
+          (S->NSP-ESP+10)*sizeof(REAL));
   if ( !hessian_quiet_flag )
   { sprintf(msg,"New allocation at %p.\n",S->ISP);  outstring(msg); }
   S->NSP += -ESP + 10;
@@ -534,9 +538,9 @@ struct linsys *S;
   sdrvmd_(&S->N,S->P,S->IP,S->IA,S->JA,S->A,NULL,NULL,&S->NSP,
             S->ISP,RSP,&ESP, &PATH,&FLAG,&EMAX);
   sdrv_flag_check(ESP,FLAG,S->N);
-  eigen_neg = S->neg = mat_index;
-  eigen_zero = S->zero = mat_null;
-  eigen_pos = S->pos = S->N - S->neg - S->zero;
+  S->neg = mat_index;
+  S->zero = mat_null;
+  S->pos = S->N - mat_index - mat_null;
 
   /* add lambda back to diagonal */
   if ( S->lambda != 0.0 )
@@ -545,6 +549,8 @@ struct linsys *S;
             if ( S->JA[j]-A_OFF == i )
             { S->A[j] += S->lambda; break; }
       }
+
+  PROF_FINISH(hessian_factor);
 }  /* end ysmp_factor() */
 
 /*************************************************************************
@@ -561,12 +567,16 @@ REAL *x; /* solution, may be B */
 { int PATH,FLAG=0,ESP;
   REAL *RSP,EMAX;
 
+  PROF_START(hessian_factor);
+
   if ( S->N == 0 ) return;
   RSP = (REAL *) S->ISP;
   PATH = 3;  /* solution only */
   sdrvmd_(&S->N,S->P,S->IP,S->IA,S->JA,S->A,b,x,&S->NSP,S->ISP,RSP,&ESP, 
         &PATH,&FLAG,&EMAX);
   sdrv_flag_check(ESP,FLAG,S->N);
+
+  PROF_FINISH(hessian_factor);
 }
 
 /**********************************************************************
@@ -677,14 +687,14 @@ struct linsys *M;  /* pointer to empty structure */
   if ( ysmp_flag != MINDEG_FACTORING )
   { ysmp_flag = MINDEG_FACTORING;
     sp_mul_func = bk_mul;
-    sp_AIJ_setup_func= bk_AIJ_setup;
+    sp_AIJ_setup_func = bk_AIJ_setup;
     sp_constraint_setup_func = bk_constraint_setup;
     sp_hess_project_setup_func= BK_hess_project_setup;
     sp_factor_func = xmd_factor;
     sp_solve_func = xmd_solve;
     sp_solve_multi_func = xmd_solve_multi;
     sp_ordering_func = NULL;
-    sp_CHinvC_func = xmd_CHinvC;
+    sp_CHinvC_func = sp_CHinvC;
     outstring("Using alternate minimal degree with area normalization.\n");
   }
   M->N = S->N;
@@ -700,7 +710,7 @@ struct linsys *M;  /* pointer to empty structure */
   if ( M->IP == NULL ) M->IP = (int *)temp_calloc(M->N,sizeof(int));
 
   /* go through, vertex by vertex */
-  for ( i = 0, k = 0 ; i <= web.skel[VERTEX].max_ord ; i++ )
+  for ( i = 0, k = 0 ; i <= vhead_count ; i++ )
   { vertex_id v_id;
     REAL area;
 
@@ -759,11 +769,12 @@ void linear_metric_setup(S,M)
 struct linsys *S;  /* hessian to build metric for */
 struct linsys *M;  /* pointer to empty structure */
 {
-  int i,j,jj,k,kk;
+  int i,jj,k,kk,m;
+  size_t j;
   int sum;
   edge_id e_id;
   vertex_id v_id;
-  int ti,hi;
+  int ti=0,hi=0;
   REAL len;
   MAT2D(weights,MAXCOORD+1,MAXCOORD+1);
   MAT2D(temp_mat,MAXCOORD,MAXCOORD);
@@ -783,14 +794,14 @@ if ( optparamcount > 0 )
   if ( ysmp_flag != MINDEG_FACTORING )
   { ysmp_flag = MINDEG_FACTORING;
     sp_mul_func = bk_mul;
-    sp_AIJ_setup_func= bk_AIJ_setup;
+    sp_AIJ_setup_func = bk_AIJ_setup;
     sp_constraint_setup_func = bk_constraint_setup;
-    sp_hess_project_setup_func= BK_hess_project_setup;
+    sp_hess_project_setup_func = BK_hess_project_setup;
     sp_factor_func = xmd_factor;
     sp_solve_func = xmd_solve;
     sp_solve_multi_func = xmd_solve_multi;
     sp_ordering_func = NULL;
-    sp_CHinvC_func = xmd_CHinvC;
+    sp_CHinvC_func = sp_CHinvC;
     outstring(
         "Using alternate minimal degree with linear interpolation metric.\n");
   }
@@ -814,14 +825,15 @@ if ( optparamcount > 0 )
     if ( vfacet_timestamp < top_timestamp ) make_vfacet_lists();
     nbrlist = (vertex_id *)temp_calloc(nbralloc,sizeof(vertex_id));
     FOR_ALL_VERTICES(v_id)
-    { i = ordinal(v_id);
+    { struct hess_verlist *vh = get_vertex_vhead(v_id);
+      i = loc_ordinal(v_id);
       nbrcount = 1;
       nbrlist[0] = v_id;
       f_id = start_f = get_vertex_facet(v_id);
       do
       { v = get_facet_vertices(f_id);
         for ( k = 0 ; k <= web.dimension ; k++ )
-          if ( ordinal(v[k]) > i )
+          if ( loc_ordinal(v[k]) > i )
           { for ( m = 0 ; m < nbrcount ; m++ )
               if ( nbrlist[m] == v[k] ) break;
             if ( m == nbrcount )
@@ -834,29 +846,36 @@ if ( optparamcount > 0 )
           f_id = get_next_vertex_facet(v_id,f_id);
       } while ( !equal_element(f_id,start_f));
       /* now count total degrees of freedom */
-      for ( j = vhead[i].rownum, jj = 0 ; jj < vhead[i].freedom ; jj++,j++ )
-        M->IA[j] += vhead[i].freedom-jj;
+      for ( j = vh->rownum, jj = 0 ; jj < vh->freedom ; jj++,j++ )
+        M->IA[j] += vh->freedom-jj;
       for ( m = 1 ; m < nbrcount ; m++ )
-      { hi = ordinal(nbrlist[m]);
-        for ( j = vhead[i].rownum, jj = 0 ; jj < vhead[i].freedom ; j++,jj++ )
-           M->IA[j] += vhead[hi].freedom;
+      { struct hess_verlist *vhi = get_vertex_vhead(nbrlist[m]);
+        for ( j = vh->rownum, jj = 0 ; jj < vh->freedom ; j++,jj++ )
+           M->IA[j] += vhi->freedom;
       }
     }
   }
   else 
   { /* fe model */
     FOR_ALL_VERTICES(v_id)
-    { i = ordinal(v_id);
-      for ( j = vhead[i].rownum, jj = 0 ; jj < vhead[i].freedom ; jj++,j++ )
-        M->IA[j] += vhead[i].freedom-jj;      
+    { struct hess_verlist *vh = get_vertex_vhead(v_id);
+      for ( j = vh->rownum, jj = 0 ; jj < vh->freedom ; jj++,j++ )
+        M->IA[j] += vh->freedom-jj;      
     }
     FOR_ALL_EDGES(e_id)
-    { ti = ordinal(get_edge_tailv(e_id));
-      hi = ordinal(get_edge_headv(e_id));
-      if ( ti > hi ) { int tt = ti; ti = hi ; hi = tt; }
-      if ( vhead[hi].freedom == 0 ) continue;
-      for ( j = vhead[ti].rownum, jj = 0 ; jj < vhead[ti].freedom ; j++,jj++ )
-         M->IA[j] += vhead[hi].freedom;
+    { vertex_id tailv = get_edge_tailv(e_id);
+      vertex_id headv = get_edge_headv(e_id);
+      struct hess_verlist *vt = get_vertex_vhead(tailv),*vhtmp;
+      struct hess_verlist *vh = get_vertex_vhead(headv);
+      ti = loc_ordinal(tailv);
+      hi = loc_ordinal(headv);
+      if ( ti > hi ) 
+      { int tt = ti; ti = hi ; hi = tt; 
+        vhtmp = vh; vh = vt; vt = vhtmp;
+      }
+      if ( vh->freedom == 0 ) continue;
+      for ( j = vt->rownum, jj = 0 ; jj < vt->freedom ; j++,jj++ )
+         M->IA[j] += vh->freedom;
     }
   }
   for ( i = S->optparamrowstart ; i < S->N ; i++ )
@@ -879,6 +898,7 @@ if ( optparamcount > 0 )
     M->IP[i] = M->P[i] = i;    /* unpermuted */
   }
 
+
   /* set up interpolation weight matrix */
 
   for ( i = 0 ; i < optparamcount ; i++ )
@@ -887,23 +907,30 @@ if ( optparamcount > 0 )
 
   dim = web.dimension;
   for ( i = 0 ; i <= dim ; i++ )  /* index for vertices of element */
-    for ( j = 0 ; j <= dim ; j++ )
-    { if ( i == j )
-         weights[i][j] = linear_metric_mix*2.0/(dim+1.0)/(dim+2.0)
+    for ( m = 0 ; m <= dim ; m++ )
+    { if ( i == m )
+         weights[i][m] = linear_metric_mix*2.0/(dim+1.0)/(dim+2.0)
               + (1.0 - linear_metric_mix)/(dim+1.0);
-      else weights[i][j] = linear_metric_mix/(dim+1.0)/(dim+2.0);
+      else weights[i][m] = linear_metric_mix/(dim+1.0)/(dim+2.0);
     }
 
   if ( web.representation == STRING )
   { /* edge by edge */
     FOR_ALL_EDGES(e_id)
-    { ti = ordinal(get_edge_tailv(e_id));
-      hi = ordinal(get_edge_headv(e_id));
-      if ( ti > hi ) { ii[0] = hi; ii[1] = ti; }
-      else { ii[0] = ti; ii[1] = hi; }
+    { struct hess_verlist *vh[2];
+      vertex_id tailv = get_edge_tailv(e_id);
+      vertex_id headv = get_edge_headv(e_id);
+      if ( loc_ordinal(ti) > loc_ordinal(hi) )
+      { vh[0] = get_vertex_vhead(headv);
+        vh[1] = get_vertex_vhead(tailv);
+      }
+      else
+      { vh[0] = get_vertex_vhead(tailv);
+        vh[1] = get_vertex_vhead(headv);
+      }
       len = get_edge_length(e_id);
       for ( i = 0 ; i < 2 ; i++ )
-      { struct hess_verlist *v = vhead + ii[i];
+      { struct hess_verlist *v = vh[i];
         REAL fudge = v->slant*v->slant;
          
         /* self */
@@ -917,19 +944,19 @@ if ( optparamcount > 0 )
           for ( k = jj ; k < v->freedom ; k++ )
           { col = v->rownum + k + A_OFF;
             end = M->IA[v->rownum+jj+1]-A_OFF;
-            for ( j = M->IA[v->rownum+jj]-A_OFF ; j < end  ; j++ )
-              if ( (M->JA[j] < A_OFF) || (M->JA[j] == col) )
-              { M->JA[j] = col;
-                M->A[j] += fudge*(3-linear_metric_mix)*len/6*mat[jj][k];
+            for ( m = M->IA[v->rownum+jj]-A_OFF ; m < end  ; m++ )
+              if ( (M->JA[m] < A_OFF) || (M->JA[m] == col) )
+              { M->JA[m] = col;
+                M->A[m] += fudge*(3-linear_metric_mix)*len/6*mat[jj][k];
                 break;
               }
-            if ( j == end )
+            if ( m == end )
                kb_error(1851,"Internal error in linear_metric_setup.\n",RECOVERABLE);
           }
 
           /* cross terms */
           for ( k = i+1 ; k < 2 ; k++ )
-          { struct hess_verlist *vv = vhead + ii[k];
+          { struct hess_verlist *vv = vh[k];
             REAL cfudge = v->slant*vv->slant;
             if ( v->proj && vv->proj )
             { tr_mat_mul(v->proj,vv->proj,temp_mat,SDIM,v->freedom,
@@ -973,6 +1000,7 @@ if ( optparamcount > 0 )
     { vertex_id *ix;
       REAL density = get_facet_density(f_id); 
       REAL normal[MAXCOORD];
+      struct hess_verlist *vh[MAXCOORD+1];
 
       area = get_facet_area(f_id);
       if ( density != 0.0 ) area *= density;
@@ -984,15 +1012,19 @@ if ( optparamcount > 0 )
       }
 
       ix = get_facet_vertices(f_id);
-      for ( i = 0 ; i <= dim ; i++ )
-      { j = ordinal(ix[i]);
+      for ( i = 0 ; i <= dim ; i++ )  /* bubble sort vertices */
+      { j = loc_ordinal(ix[i]);
         for ( k = i ; k > 0 ; k-- )
-          if ( ii[k-1] > j ) ii[k] = ii[k-1];
+          if ( ii[k-1] > (int)j ) 
+          { ii[k] = ii[k-1];
+            vh[k] = vh[k-1];
+          }
           else break;
-        ii[k] = j;
+        ii[k] = (int)j;
+        vh[k] = get_vertex_vhead(ix[i]);
       }
       for ( i = 0 ; i <= dim ; i++ )
-      { struct hess_verlist *v = vhead + ii[i];
+      { struct hess_verlist *v = vh[i];
         REAL fudge = v->slant*v->slant;
         /* self */
         if ( v->proj )
@@ -1015,7 +1047,7 @@ if ( optparamcount > 0 )
           for ( k = jj ; k < v->freedom ; k++ )
           { col = v->rownum + k + A_OFF;
             end = M->IA[v->rownum+jj+1]-A_OFF;
-            for ( j = M->IA[v->rownum+jj]-A_OFF ; j < end  ; j++ )
+            for ( j = M->IA[v->rownum+jj]-A_OFF ; (int)j < end  ; j++ )
               if ( (M->JA[j] < A_OFF) || (M->JA[j] == col) )
               { M->JA[j] = col;
                 M->A[j] += fudge*weights[i][i]*area*mat[jj][k];
@@ -1027,7 +1059,7 @@ if ( optparamcount > 0 )
 
         /* cross terms */
         for ( k = i+1 ; k <= dim ; k++ )
-        { struct hess_verlist *vv = vhead + ii[k];
+        { struct hess_verlist *vv = vh[k];
           REAL cfudge = v->slant*vv->slant;
           if ( v->proj && vv->proj )
           { 
@@ -1081,7 +1113,7 @@ if ( optparamcount > 0 )
            for ( kk = 0 ; kk < vv->freedom ; kk++ )  
            { col = vv->rownum + kk + A_OFF ;
              end = M->IA[v->rownum+jj+1]-A_OFF;
-             for ( j = M->IA[v->rownum+jj]+1-A_OFF ; j < end ; j++ )
+             for ( j = M->IA[v->rownum+jj]+1-A_OFF ; (int)j < end ; j++ )
                if ( (M->JA[j] < A_OFF) || (M->JA[j] == col)  )
                { M->JA[j] = col; 
                  M->A[j] += cfudge*mat[jj][kk]*weights[i][k]*area;
@@ -1095,6 +1127,8 @@ if ( optparamcount > 0 )
      }
   }
 
+
+
   /* compact matrix by looking for -1 in M->JA */
   for ( i = 0 ; i < M->IA[M->N]-A_OFF ; i++ )
      if ( M->JA[i] < A_OFF ) break;
@@ -1103,14 +1137,14 @@ if ( optparamcount > 0 )
     REAL *a_to_spot= M->A,*a_from_spot= M->A;
     for ( i = 0 ; i < M->N ; i++ )
     { end = M->IA[i+1]-A_OFF;
-      for ( j = ja_from_spot - M->JA ; j < end ; j++,ja_from_spot++,a_from_spot++ )
+      for ( j = ja_from_spot - M->JA ; (int)j < end ; j++,ja_from_spot++,a_from_spot++ )
       { if ( *ja_from_spot >= A_OFF )
         { *ja_to_spot = *ja_from_spot;
           *a_to_spot  = *a_from_spot;
           a_to_spot++; ja_to_spot++;
         }
       }
-      M->IA[i+1] = ja_to_spot - M->JA + A_OFF;
+      M->IA[i+1] = (int)(ja_to_spot - M->JA + A_OFF);
     }
   }
 
@@ -1122,7 +1156,7 @@ apinv_setup:
   { int start = M->IA[i] - A_OFF;
     end = M->IA[i+1] - A_OFF;
     M->apinv[i] += M->A[start];
-    for ( j = start+1 ; j < end ; j++ )
+    for ( j = start+1 ; (int)j < end ; j++ )
     { M->apinv[i] += M->A[j];
       M->apinv[M->JA[j]-A_OFF] += M->A[j];
     }
@@ -1175,19 +1209,20 @@ struct linsys *M;  /* pointer to empty structure */
   REAL **mat;
   int col,end;
   int ii[FACET_CTRL];
+  vertex_id vv[FACET_CTRL];
   int lo;
 
   if ( ysmp_flag != MINDEG_FACTORING )
   { ysmp_flag = MINDEG_FACTORING;
     sp_mul_func = bk_mul;
-    sp_AIJ_setup_func= bk_AIJ_setup;
+    sp_AIJ_setup_func = bk_AIJ_setup;
     sp_constraint_setup_func = bk_constraint_setup;
     sp_hess_project_setup_func= BK_hess_project_setup;
     sp_factor_func = xmd_factor;
     sp_solve_func = xmd_solve;
     sp_solve_multi_func = xmd_solve_multi;
     sp_ordering_func = NULL;
-    sp_CHinvC_func = xmd_CHinvC;
+    sp_CHinvC_func = sp_CHinvC;
     outstring(
        "Using alternate minimal degree with linear interpolation metric.\n");
   }
@@ -1206,25 +1241,28 @@ struct linsys *M;  /* pointer to empty structure */
 
   /* have to count number of higher-ordinal neighbors of each vertex */
   FOR_ALL_VERTICES(v_id)
-  { i = ordinal(v_id);
-    for ( j = vhead[i].rownum, jj = 0 ; jj < vhead[i].freedom ; jj++,j++ )
-       M->IA[j] += vhead[i].freedom-jj;
+  { struct hess_verlist *vh = get_vertex_vhead(v_id);
+    for ( j = vh->rownum, jj = 0 ; jj < vh->freedom ; jj++,j++ )
+       M->IA[j] += vh->freedom-jj;
   }
   FOR_ALL_EDGES(e_id)
   { vertex_id v[3];
-    v[0] = ordinal(get_edge_tailv(e_id));
-    v[1] = ordinal(get_edge_headv(e_id));
-    v[2] = ordinal(get_edge_midv(e_id));
+    struct hess_verlist *vh[3];
+    v[0] = get_edge_tailv(e_id);
+    v[1] = get_edge_headv(e_id);
+    v[2] = get_edge_midv(e_id);
+    for ( i = 0 ; i < 3 ; i ++ )
+      vh[i] = get_vertex_vhead(v[i]);
     for ( i = 0 ; i < 3 ; i ++ )
       for ( k = 0 ; k < 3 ; k++ )
-        if ( v[i] < v[k] )
-           for ( j = vhead[v[i]].rownum, jj = 0 ; jj < vhead[v[i]].freedom ; 
-                j++,jj++ )
-              M->IA[j] += vhead[v[k]].freedom;
+        if ( loc_ordinal(v[i]) < loc_ordinal(v[k]) )
+           for ( j = vh[i]->rownum, jj = 0 ; jj < vh[i]->freedom ; j++,jj++ )
+              M->IA[j] += vh[k]->freedom;
   }
   if ( web.representation == SOAPFILM )
   FOR_ALL_FACETS(f_id)
   { vertex_id vv[FACET_CTRL];
+    struct hess_verlist *vh[FACET_CTRL];
     facetedge_id  fe_id = get_facet_fe(f_id);
     int v[FACET_CTRL];
 
@@ -1235,11 +1273,16 @@ struct linsys *M;  /* pointer to empty structure */
        fe_id = get_next_edge(fe_id);
     }
 
-    for ( i = 0 ; i < FACET_CTRL ; i++ ) v[i] = ordinal(vv[i]);
+    for ( i = 0 ; i < FACET_CTRL ; i++ ) 
+    { v[i] = loc_ordinal(vv[i]);
+      vh[i] = get_vertex_vhead(v[i]);
+    }
+
 #define ADDON(a,b) \
-    if ( v[a] > v[b] ) {lo = v[b]; hi = v[a]; } else { lo= v[a]; hi=v[b];}\
-    for ( j = vhead[lo].rownum, jj = 0 ; jj < vhead[lo].freedom ; j++,jj++ )\
-        M->IA[j] += vhead[hi].freedom;
+    if ( v[a] > v[b] ) {lo = b; hi = a; } else { lo= a; hi=b;}\
+    for ( j = vh[lo]->rownum, jj = 0 ; jj < vh[lo]->freedom ; j++,jj++ )\
+        M->IA[j] += vh[hi]->freedom;
+
     ADDON(0,4); ADDON(1,5); ADDON(2,3); ADDON(4,5); ADDON(3,4); ADDON(3,5);
   }
   for ( i = 0, sum = 0 ; i < M->N ; i++ ) /* add to running totals */
@@ -1272,9 +1315,12 @@ struct linsys *M;  /* pointer to empty structure */
     /* edge by edge */
     FOR_ALL_EDGES(e_id)
     { 
-      ii[0] = ordinal(get_edge_tailv(e_id));
-      ii[1] = ordinal(get_edge_midv(e_id));
-      ii[2] = ordinal(get_edge_headv(e_id));
+      vv[0] = get_edge_tailv(e_id);
+      vv[1] = get_edge_midv(e_id);
+      vv[2] = get_edge_headv(e_id);
+      ii[0] = loc_ordinal(vv[0]);
+      ii[1] = loc_ordinal(vv[1]);
+      ii[2] = loc_ordinal(vv[2]);
       get_edge_verts(e_id,x,NULL);
       len = get_edge_length(e_id);
 
@@ -1302,23 +1348,23 @@ struct linsys *M;  /* pointer to empty structure */
 
       /* fill in metric matrix */
       for ( i = 0 ; i < 3 ; i++ )
-      { struct hess_verlist *v = vhead + ii[i];
+      { struct hess_verlist *vh = get_vertex_vhead(vv[i]);
          
         /* self */
-        if ( v->proj )
+        if ( vh->proj )
         { 
-          tr_mat_mul(v->proj,v->proj,temp_mat,SDIM,v->freedom,v->freedom);
+          tr_mat_mul(vh->proj,vh->proj,temp_mat,SDIM,vh->freedom,vh->freedom);
           mat = temp_mat;
         }
         else mat = identmat;
-        for ( jj = 0 ; jj < v->freedom ; jj++ )
-          for ( k = jj ; k < v->freedom ; k++ )
-          { col = v->rownum + k + A_OFF;
-            end = M->IA[v->rownum+jj+1]-A_OFF;
-            for ( j = M->IA[v->rownum+jj]-A_OFF ; j < end  ; j++ )
+        for ( jj = 0 ; jj < vh->freedom ; jj++ )
+          for ( k = jj ; k < vh->freedom ; k++ )
+          { col = vh->rownum + k + A_OFF;
+            end = M->IA[vh->rownum+jj+1]-A_OFF;
+            for ( j = M->IA[vh->rownum+jj]-A_OFF ; j < end  ; j++ )
               if ( (M->JA[j] < A_OFF) || (M->JA[j] == col) )
               { M->JA[j] = col;
-                M->A[j] += v->slant*v->slant*weights[i][i]*mat[jj][k];
+                M->A[j] += vh->slant*vh->slant*weights[i][i]*mat[jj][k];
                 break;
               }
               if ( j == end )
@@ -1328,28 +1374,28 @@ struct linsys *M;  /* pointer to empty structure */
 
         /* cross terms */
         for ( k = 0 ; k < 3 ; k++ )
-        { struct hess_verlist *vv = vhead + ii[k];
+        { struct hess_verlist *vvh = get_vertex_vhead(vv[k]);
           if ( ii[k] <= ii[i] ) continue;
-          if ( v->proj && vv->proj )
-          { tr_mat_mul(v->proj,vv->proj,temp_mat,SDIM,v->freedom,
-                                                                                vv->freedom);
+          if ( vh->proj && vvh->proj )
+          { tr_mat_mul(vh->proj,vvh->proj,temp_mat,SDIM,vh->freedom,
+               vvh->freedom);
             mat = temp_mat;
           }
-          else if ( v->proj )
-          { tr_mat_mul(v->proj,identmat,temp_mat,SDIM,v->freedom,vv->freedom);
+          else if ( vh->proj )
+          { tr_mat_mul(vh->proj,identmat,temp_mat,SDIM,vh->freedom,vvh->freedom);
             mat = temp_mat;
           }
-          else if ( vv->proj ) mat = vv->proj;
+          else if ( vvh->proj ) mat = vvh->proj;
           else mat = identmat;
 
-          for ( jj = 0 ; jj < v->freedom ; jj++ )  
-            for ( kk = 0 ; kk < vv->freedom ; kk++ )  
-            { col = vv->rownum + kk + A_OFF ;
-              end = M->IA[v->rownum+jj+1]-A_OFF;
-              for ( j = M->IA[v->rownum+jj]+1-A_OFF ; ; j++ )
+          for ( jj = 0 ; jj < vh->freedom ; jj++ )  
+            for ( kk = 0 ; kk < vvh->freedom ; kk++ )  
+            { col = vvh->rownum + kk + A_OFF ;
+              end = M->IA[vh->rownum+jj+1]-A_OFF;
+              for ( j = M->IA[vh->rownum+jj]+1-A_OFF ; ; j++ )
                 if ( (M->JA[j] < A_OFF) || (M->JA[j] == col)  )
                 { M->JA[j] = col; 
-                  M->A[j] += v->slant*vv->slant*mat[jj][kk]*weights[i][k];
+                  M->A[j] += vh->slant*vvh->slant*mat[jj][kk]*weights[i][k];
                   break;
                 }
               if ( j == end )
@@ -1384,7 +1430,8 @@ struct linsys *M;  /* pointer to empty structure */
          fe_id = get_next_edge(fe_id);
        }
        area = get_facet_area(f_id);
-       for ( i = 0 ; i < FACET_CTRL ; i++ ) ii[i] = ordinal(ix[i]);
+       for ( i = 0 ; i < FACET_CTRL ; i++ ) 
+          ii[i] = loc_ordinal(ix[i]);
 
        /* weight matrix */
        get_facet_verts(f_id,x,NULL);
@@ -1410,10 +1457,11 @@ struct linsys *M;  /* pointer to empty structure */
          }
        }
        for ( i = 0 ; i < FACET_CTRL ; i++ )
-         for ( j = 0 ; j < FACET_CTRL ; j++ ) weights[i][j] *= density;
+         for ( j = 0 ; j < FACET_CTRL ; j++ ) 
+           weights[i][j] *= density;
 
        for ( i = 0 ; i < FACET_CTRL ; i++ )
-       { struct hess_verlist *v = vhead + ii[i];
+       { struct hess_verlist *v = get_vertex_vhead(ix[i]);
 
          /* self */
          if ( v->proj )
@@ -1440,7 +1488,7 @@ struct linsys *M;  /* pointer to empty structure */
 
          /* cross terms */
          for ( k = 0 ; k < FACET_CTRL ; k++ )
-         { struct hess_verlist *vv = vhead + ii[k];
+         { struct hess_verlist *vv = get_vertex_vhead(ix[k]);
             
            if ( ii[k] <= ii[i] ) continue;
 
@@ -1534,7 +1582,7 @@ struct linsys *M;  /* pointer to empty structure */
      sp_solve_func = xmd_solve;
      sp_solve_multi_func = xmd_solve_multi;
      sp_ordering_func = NULL;
-     sp_CHinvC_func = xmd_CHinvC;
+     sp_CHinvC_func = sp_CHinvC;
      outstring(
         "Using alternate minimal degree with Lagrange linear interpolation metric.\n");
   }
@@ -1565,8 +1613,8 @@ struct linsys *M;  /* pointer to empty structure */
     /* edge by edge */
     FOR_ALL_EDGES(e_id)
     { vertex_id *v = get_edge_vertices(e_id);
-      for ( i = 0 ; i < ctrl ; i++ ) ii[i] = ordinal(v[i]);
-        get_edge_verts(e_id,x,NULL);
+      for ( i = 0 ; i < ctrl ; i++ ) ii[i] = loc_ordinal(v[i]);
+      get_edge_verts(e_id,x,NULL);
 
       /* weight matrix */
       for ( i = 0 ; i < ctrl ; i++ )
@@ -1594,7 +1642,7 @@ struct linsys *M;  /* pointer to empty structure */
 
       /* fill in metric matrix */
       for ( i = 0 ; i < ctrl ; i++ )
-      { struct hess_verlist *vh = vhead + ii[i];
+      { struct hess_verlist *vh = get_vertex_vhead(v[i]);
         
         /* self */
         if ( vh->proj )
@@ -1614,7 +1662,7 @@ struct linsys *M;  /* pointer to empty structure */
 
           /* cross terms */
           for ( k = 0 ; k < ctrl ; k++ )
-          { struct hess_verlist *vv = vhead + ii[k];
+          { struct hess_verlist *vv = get_vertex_vhead(v[k]);
             if ( ii[k] <= ii[i] ) continue;
             if ( vh->proj && vv->proj )
             { tr_mat_mul(vh->proj,vv->proj,temp_mat,SDIM,vh->freedom,vv->freedom);
@@ -1659,7 +1707,7 @@ struct linsys *M;  /* pointer to empty structure */
     { vertex_id *ix = get_facet_vertices(f_id);
       REAL density = get_facet_density(f_id);
 
-      for ( i = 0 ; i < ctrl ; i++ ) ii[i] = ordinal(ix[i]);
+      for ( i = 0 ; i < ctrl ; i++ ) ii[i] = loc_ordinal(ix[i]);
       /* weight matrix */
       get_facet_verts(f_id,x,NULL);
       for ( i = 0 ; i < ctrl ; i++ )
@@ -1694,7 +1742,7 @@ struct linsys *M;  /* pointer to empty structure */
         for ( j = 0 ; j < ctrl ; j++ ) weights[i][j] *= density;
 
       for ( i = 0 ; i < ctrl ; i++ )
-      { struct hess_verlist *v = vhead + ii[i];
+      { struct hess_verlist *v = get_vertex_vhead(ix[i]);
 
         /* self */
         if ( v->proj )
@@ -1715,7 +1763,7 @@ struct linsys *M;  /* pointer to empty structure */
 
         /* cross terms */
         for ( k = 0 ; k < ctrl ; k++ )
-        { struct hess_verlist *vv = vhead + ii[k];
+        { struct hess_verlist *vv = get_vertex_vhead(ix[k]);
             
           if ( ii[k] <= ii[i] ) continue;
 
