@@ -50,7 +50,7 @@ int mode; /* energy or gradient */
 struct method_instance *mi;
 {
   charge_attr = find_attribute(VERTEX,CHARGE_ATTR_NAME);
-		  /* see if charge attribute */
+          /* see if charge attribute */
 
   exponent_param = lookup_global(KNOTPOWER_NAME);
   if ( exponent_param < 0 ) /* missing, so add */
@@ -224,14 +224,16 @@ struct qinfo *v_info;
   }
   e_id = get_vertex_edge(v_info->id);
   b = get_coord(v1_id=get_edge_headv(e_id));
-  e_id = get_next_tail_edge(e_id);  /* assuming exactly two edges per vertex */
+  e_id = get_next_tail_edge(e_id);  /* assuming <= two edges per vertex */
   c = get_coord(v2_id=get_edge_headv(e_id)); /* b and c are now two nbrs of v */
-  dradp(a,b,c, v_info->grad[0], 2.); /* case of triples (v,v1,v2),(v,v2,v1) */
+  if (v1_id != v2_id) /* skip this if v had valence 1 */
+    dradp(a,b,c, v_info->grad[0], 2.); /* case of triples (v,v1,v2),(v,v2,v1) */
   FOR_ALL_VERTICES(v_id)
   { if (v_id == v_info->id || v_id == v1_id || v_id == v2_id) continue;
     d = get_coord(v_id);
     dradp(a,b,d, v_info->grad[0], 1.);
-    dradp(a,c,d, v_info->grad[0], 1.);
+    if (v1_id != v2_id)
+      dradp(a,c,d, v_info->grad[0], 1.);
   }
   return thick;
 }
@@ -264,7 +266,7 @@ struct qinfo *v_info;
     r = rad(a,b,c);
     if (thick < 0 || r < thick) thick = r;
   }
-  return thick;
+  return thick; /* returns -1 if component only has 2 edges */
 }
 
 
@@ -336,6 +338,7 @@ struct qinfo *v_info;
 *  purpose: calculates global radius of curvature at one vertex v,
 *      as Lp integral of  r(v,w1,w2).  Includes factors of length at v and w.
 *      Here w1 and w2 are the two neighbors of vertex w.
+*      This has not been extended to allow open arcs (valence 1 verts).
 *
 *  input: info about vertex is in qinfo structure.
 *
@@ -400,7 +403,7 @@ struct qinfo *v_info;
   v2_id=get_edge_headv(e2_id); lv2 = get_vertex_star(v2_id);
    /* v1 and v2 are two nbrs of v  and lv1,lv2 are their star lengths */
   get_edge_side(e2_id,e2); l2 = get_edge_length(e2_id);
-	/* e1, e2 are edge vectors out of v; l1, l2 are their lengths */
+    /* e1, e2 are edge vectors out of v; l1, l2 are their lengths */
   for ( i = 0 ; i < SDIM ; i++ )
   {
     e1[i] /= 2*l1; e2[i] /= 2*l2; /* now they're half unit vectors */
@@ -416,21 +419,21 @@ struct qinfo *v_info;
     e_id = get_next_tail_edge(e_id);  /* assuming two edges per vertex */
     if ((w2_id = get_edge_headv(e_id))==v_id) continue;
     thick += dradp(a,get_coord(w1_id),get_coord(w2_id),
-	             temp, get_vertex_star(w_id));
+                 temp, get_vertex_star(w_id));
   }     /* now temp is sum_w l(w)*d_v(rp(v,w1,w2)) */
   for ( i = 0 ; i < SDIM ; i++ )
     v_info->grad[0][i] += (temp[i]*lv - thick*(e1[i]+e2[i]));
-	/* so far we should have the gradient of knot_thickness_p2(v_info) */
-	/* which is: d_v(l(v))*thick + l(v)*temp */
+    /* so far we should have the gradient of knot_thickness_p2(v_info) */
+    /* which is: d_v(l(v))*thick + l(v)*temp */
   thick *= lv;  /* this is the energy at v, to be returned */
 
     /* Now we must compute derivative due to v of all other terms,
-	first just its effect on length for energy of neighbors, then
-	when v is playing the role of w (affects only length) or w1 or w2 */
+    first just its effect on length for energy of neighbors, then
+    when v is playing the role of w (affects only length) or w1 or w2 */
   for ( i = 0 ; i < SDIM ; i++ )
     v_info->grad[0][i] -= (  knot_thickness_id2(v1_id)*e1[i]/lv1
-			   + knot_thickness_id2(v2_id)*e2[i]/lv2 );
-	/* this is effect of v on l(v1) and l(v2) in energy at v1 and v2 */
+               + knot_thickness_id2(v2_id)*e2[i]/lv2 );
+    /* this is effect of v on l(v1) and l(v2) in energy at v1 and v2 */
 
   b = get_coord(v1_id); c = get_coord(v2_id);  /* will need these nbrs */
   e_id = get_next_head_edge(e1_id); /* move out beyond v1 */
@@ -526,9 +529,10 @@ struct qinfo *v_info;
   e_id = get_vertex_edge(v_info->id);
   v1_id=get_edge_headv(e_id);
   get_edge_side(e_id,e1); l1 = get_edge_length(e_id);
-  e_id = get_next_tail_edge(e_id);  /* assuming exactly two edges per vertex */
+  e_id = get_next_tail_edge(e_id);  /* assuming <= two edges per vertex */
   v2_id=get_edge_headv(e_id); /* v1 and v2 are two nbrs of v */
   get_edge_side(e_id,e2); l2 = get_edge_length(e_id);
+  if (v1_id == v2_id) for (i=0; i<SDIM; i++) e2[i] = 0.; /* v val. 1, no e2 */
 
   for ( i = 0 ; i < SDIM ; i++ ) temp[i] = 0.;
   FOR_ALL_EDGES(e_id)
@@ -541,32 +545,35 @@ struct qinfo *v_info;
   for ( i = 0 ; i < SDIM ; i++ )
     v_info->grad[0][i] += (temp[i] * get_vertex_star(v_info->id) -
                              thick * (e1[i]/l1 + e2[i]/l2)/2);
-	/* so far we should have the gradient of knot_thickness_p(v_info) */
-	/* which is: d_v(l(v))*thick + l(v)*temp */
+    /* so far we should have the gradient of knot_thickness_p(v_info) */
+    /* which is: d_v(l(v))*thick + l(v)*temp */
   thick *= get_vertex_star(v_info->id);
 
-	/* Next we consider the triple v1,v,v2,
-	   which enters into thickness at v1 and v2 */
+    /* Next we consider the triple v1,v,v2,
+       which enters into thickness at v1 and v2 */
   b = get_coord(v1_id); c = get_coord(v2_id);
-  dradp(a,b,c, v_info->grad[0],       /* triples (v,v1,v2), (v,v2,v1) */
+  if (v1_id != v2_id) /* skip if v valence 1 */
+  {
+    dradp(a,b,c, v_info->grad[0],       /* triples (v,v1,v2), (v,v2,v1) */
         l1*get_vertex_star(v2_id) + l2*get_vertex_star(v1_id));
 	/* this term gets d_v rp(a,b,c) times appropriate lengths */
-  for ( i = 0 ; i < SDIM ; i++ )
-  { v_info->grad[0][i] -= radp(a,b,c) *  /* - is from orientation of ei */
+    for ( i = 0 ; i < SDIM ; i++ )
+    { v_info->grad[0][i] -= radp(a,b,c) *  /* - is from orientation of ei */
 			  ( get_vertex_star(v2_id)*e1[i]/l1 +
 			    get_vertex_star(v1_id)*e2[i]/l2  );
 	/* term above accounts for changing lengths at v as l(e) factor */
-    v_info->grad[0][i] -=
+      v_info->grad[0][i] -=
        (  knot_thickness_id(v1_id)*e1[i]/l1/get_vertex_star(v1_id)/2
         + knot_thickness_id(v2_id)*e2[i]/l2/get_vertex_star(v2_id)/2 );
 	/* this is effect of v on l(v1) and l(v2) in energy at v1 and v2 */
+    }
   }
 
   FOR_ALL_VERTICES(w_id)
   { if (w_id == v_info->id || w_id == v1_id || w_id == v2_id) continue;
     d = get_coord(w_id);
     dradp(a,b,d, v_info->grad[0], l1*get_vertex_star(w_id));
-    dradp(a,c,d, v_info->grad[0], l2*get_vertex_star(w_id));
+    if (v1_id != v2_id) dradp(a,c,d, v_info->grad[0], l2*get_vertex_star(w_id));
 	/* these are changes d_v (rp(w,ei)) times lengths, i=1,2 */
     for ( i = 0 ; i < SDIM ; i++ )
       v_info->grad[0][i] -= (  radp(a,b,d) * get_vertex_star(w_id) * e1[i]/l1
@@ -704,9 +711,9 @@ struct qinfo *v_info;
         hessvw[i][j] = - hessij;
         hessvv[i][j] += 2*hessij;
       }
-    fill_mixed_entry(v_info->S,v_info->id, w_id, hessvw);
+    fill_mixed_entry(v_info->S, v_info->id, w_id, hessvw);
   }
-  fill_self_entry(v_info->S,v_info->id, hessvv);
+  fill_self_entry(v_info->S, v_info->id, hessvv);
   return modulus*energy;
 }
 
@@ -736,9 +743,9 @@ struct method_instance *mi;
     if ((nv = web.skel[VERTEX].max_ord+1) != cg_nverts)
     {
          if (charge_grads) list_free((char*)charge_grads,ETERNAL_BLOCK);
-         charge_grads = (REAL*)list_calloc(nv, MAXCOORD*sizeof(REAL),ETERNAL_BLOCK);
+         charge_grads = (REAL*)my_list_calloc(nv, MAXCOORD*sizeof(REAL),ETERNAL_BLOCK);
          cg_nverts = nv;
-	 }
+     }
 
     FOR_ALL_VERTICES(u_id)
     {
@@ -852,7 +859,7 @@ struct method_instance *mi;
      calc_edge(e_id);
      add_vertex_star(get_edge_tailv(e_id),get_edge_length(e_id)/2);
      add_vertex_star(get_edge_headv(e_id),get_edge_length(e_id)/2);
-  }
+  } /* each vertex_star is now half the total length of incident edges */
 }
 
 /**************************************************************
@@ -1066,14 +1073,6 @@ struct method_instance *mi;
   }
     
 
-#if defined(PVM) || defined(MPI)
-  if ( self_ord == HOST_ORD )
-    { /* basic sums */
-      if ( f_sums ) myfree((char*)f_sums);
-      f_sums = (REAL *)mycalloc(web.skel[VERTEX].max_ord+1,sizeof(REAL));
-      pvm_facet_knot_init(f_sums);
-    }
-#else
   {
   REAL *x;
   vertex_id v_id,vv_id; 
@@ -1127,7 +1126,7 @@ struct method_instance *mi;
      }
   }
   }
-#endif
+
 }
 
 /**************************************************************
@@ -1424,7 +1423,7 @@ struct qinfo *v_info;
 *
 *  purpose: initialization for knot_energy() and 
 *              knot_energy_gradient().
-*  Does not do fsums in attempt to be more efficient with PVM or MPI
+*  Does not do fsums in attempt to be more efficient with MPI
 */
 
 
