@@ -62,7 +62,8 @@ void kusner_energy()
 
       cos_th = (s1s2*s1t2 - s2t2*s1s1)/a1/a2;
 
-      web.total_energy += modulus*s1s1*(1 - cos_th)/(a1 + a2);
+      binary_tree_add(web.total_energy_addends,
+                     modulus*s1s1*(1 - cos_th)/(a1 + a2));
     }
 }
 
@@ -230,8 +231,8 @@ void approx_curvature()
   /* count entries needed for each row */
   FOR_ALL_EDGES(e_id)
     {
-      int tail = ordinal(get_edge_tailv(e_id));
-      int head = ordinal(get_edge_headv(e_id));
+      int tail = loc_ordinal(get_edge_tailv(e_id));
+      int head = loc_ordinal(get_edge_headv(e_id));
       if ( tail > head ) IA[head]++;
       else IA[tail]++;
     }
@@ -249,8 +250,8 @@ void approx_curvature()
   /* and fill in facet star areas */
   FOR_ALL_EDGES(e_id)
     {
-      int tail = ordinal(get_edge_tailv(e_id));
-      int head = ordinal(get_edge_headv(e_id));
+      int tail = loc_ordinal(get_edge_tailv(e_id));
+      int head = loc_ordinal(get_edge_headv(e_id));
       facetedge_id fe = get_edge_fe(e_id);
       REAL area1 = get_facet_area(get_fe_facet(fe));
       REAL area2;
@@ -313,7 +314,7 @@ for ( i = 0 ; i < N ; i++ )
      { vertex_id v_id;
         FOR_ALL_VERTICES(v_id)
           { 
-             int vnum = ordinal(v_id);
+             int vnum = loc_ordinal(v_id);
              B[vnum] = get_force(v_id)[j];
           }
         RSP = (REAL *) ISP;
@@ -322,7 +323,7 @@ for ( i = 0 ; i < N ; i++ )
         sdrv_flag_check(ESP,FLAG,N);
         FOR_ALL_VERTICES(v_id)
           { 
-             int vnum = ordinal(v_id);
+             int vnum = loc_ordinal(v_id);
              get_force(v_id)[j] = B[vnum];
           }
      }
@@ -421,7 +422,7 @@ void mobility_setup()
   cb_count = 0;
   mobN = 0;
   FOR_ALL_VERTICES(v_id)
-     { int ord = ordinal(v_id);
+     { int ord = loc_ordinal(v_id);
         if ( get_vattr(v_id) & FIXED ) continue;  /* leave dimf as 0 */
         if ( get_vattr(v_id) & BOUNDARY ) 
           dimf[ord] = bdry_basis(v_id,conbasis[cb_count].vec);
@@ -430,7 +431,7 @@ void mobility_setup()
         else { mobN += SDIM; dimf[ord] = SDIM;  continue; }
         conbasis[cb_count].v_id = v_id;
         mobN += dimf[ord];
-        cb_list[ordinal(v_id)] = conbasis + cb_count;
+        cb_list[loc_ordinal(v_id)] = conbasis + cb_count;
         cb_count++;
      }
 
@@ -449,7 +450,7 @@ void mobility_setup()
   /* count entries needed for each row, row size accum in IA */
   k = 0;
   FOR_ALL_VERTICES(v_id) /* diagonal elements */
-     { int ord = ordinal(v_id);
+     { int ord = loc_ordinal(v_id);
        cb = cb_list[ord];
        IA_INV[ord] = k;
        dim_a = dimf[ord];
@@ -457,8 +458,8 @@ void mobility_setup()
          IA[k++] = dim_a - i;
      }
   FOR_ALL_EDGES(e_id)
-    { int tord = ordinal(get_edge_tailv(e_id));
-      int hord = ordinal(get_edge_headv(e_id));
+    { int tord = loc_ordinal(get_edge_tailv(e_id));
+      int hord = loc_ordinal(get_edge_headv(e_id));
       int tail = IA_INV[tord];
       int head = IA_INV[hord];
       int dim_h = dimf[hord];
@@ -469,7 +470,7 @@ void mobility_setup()
   /* set up IA pointers */
   count = 0;
   FOR_ALL_VERTICES(v_id)
-    { int ord = ordinal(v_id);
+    { int ord = loc_ordinal(v_id);
       int dim = dimf[ord];
       i = IA_INV[ord];
       for ( k = 0 ; k < dim ; k++ )
@@ -488,8 +489,8 @@ void mobility_setup()
 
   if ( web.representation == STRING )
   FOR_ALL_EDGES(e_id)
-    { int tord = ordinal(get_edge_tailv(e_id));
-      int hord = ordinal(get_edge_headv(e_id));
+    { int tord = loc_ordinal(get_edge_tailv(e_id));
+      int hord = loc_ordinal(get_edge_headv(e_id));
       int tail = IA_INV[tord];
       int head = IA_INV[hord];
       int base,addend;
@@ -598,8 +599,8 @@ void mobility_setup()
          proj[i][i] += area;
       }
       for ( m = 0 ; m < 3 ; m++, fe = get_next_edge(fe) )
-        { int tord = ordinal(get_fe_tailv(fe));
-          int hord = ordinal(get_fe_headv(fe));
+        { int tord = loc_ordinal(get_fe_tailv(fe));
+          int hord = loc_ordinal(get_fe_headv(fe));
           int tail = IA_INV[tord];
           int head = IA_INV[hord];
           int base,addend;
@@ -723,7 +724,7 @@ REAL *Y;  /* outgoing, degrees of freedom per vertex */
   struct basis *cb;
 
   FOR_ALL_VERTICES(v_id)
-     { int ord = ordinal(v_id);
+     { int ord = loc_ordinal(v_id);
         int dim = dimf[ord];
         k = ord*SDIM;
         i = IA_INV[ord];
@@ -757,7 +758,7 @@ REAL *Y;  /* outgoing, SDIM coords per vertex */
   struct basis *cb;
 
   FOR_ALL_VERTICES(v_id)
-     { int ord = ordinal(v_id);
+     { int ord = loc_ordinal(v_id);
         int dim = dimf[ord];
         k = ord*SDIM;
         i = IA_INV[ord];
@@ -924,7 +925,7 @@ void conf_edge_curv_energy()
          center[i] /= 2*det;
 
       r_square = SDIM_dot(center,center);
-      web.total_energy += modulus/r_square*(a1+a2)/3;
+      binary_tree_add(web.total_energy_addends,modulus/r_square*(a1+a2)/3);
     }
 
  free_matrix(mat);
