@@ -166,7 +166,7 @@ REAL ***a;
 int maxi,maxj,maxk; /* dimensions */
 { int i,j,k;
   REAL ***newa;
-  newa = kb_dmatrix3(maxi,maxj,maxk);
+  newa = dmatrix3(maxi,maxj,maxk);
   for ( i = 0 ; i < maxi; i++ )
     for ( j = 0 ; j < maxj ; j++ )
       for ( k = 0 ; k < maxk ; k++ )
@@ -1327,7 +1327,7 @@ REAL *work;  /* space for 2*n values */
   } /* end i */
 
   kb_error(2548,"50 Jacobi iterations should never happen.\n",WARNING);
-  { temp_free((char*)z); temp_free((char*)b); return; }
+  return;
 
 jacobi_exit:
   /* sort eigenpairs in descending order, insertion sort */
@@ -1607,9 +1607,11 @@ void sp_hash_init(S,size_estimate)
 struct linsys *S;
 int size_estimate; /* from sp_hash_end of previous invocation */
 { int i;
- 
-  S->table_size = size_estimate > 0 ? size_estimate : 
-         6*SDIM*web.skel[VERTEX].max_ord;
+  int estimate = SDIM*(web.skel[VERTEX].max_ord+10);
+   
+  S->table_size = size_estimate > 0 ? 
+       ( size_estimate > estimate ? estimate : size_estimate) : 
+         estimate;
   S->max_fill = 4*S->table_size/5;  /* leave enough empty for decent hash */
   if ( !hessian_quiet_flag )
   { sprintf(msg,"Sparse init alloc: %d\n",S->table_size);
@@ -1682,6 +1684,7 @@ REAL value;  /* value to add */
   struct sp_entry *e;
   int spot;
 
+
   if ( value == 0.0 ) return;   
 
   if ( S->hashcount >= S->max_fill ) 
@@ -1692,16 +1695,18 @@ REAL value;  /* value to add */
   e = S->hashtable + spot;
   while ( e->row != HASHEMPTY )
   { if ( (e->row == row) && (e->col == col) )
-     { e->value += value; return; 
-     }
-     spot++;
-     if ( spot >= S->table_size ) spot -= S->table_size;
-     e = S->hashtable + spot;
-     S->hash_extraprobes++;
+    { e->value += value;
+      return;
+    }
+    spot++;
+    if ( spot >= S->table_size ) spot -= S->table_size;
+    e = S->hashtable + spot;
+    S->hash_extraprobes++;
   }
   /* if here, then have empty slot and need to insert */
   e->col = col; e->row = row;  S->hashcount++; 
   e->value = value;
+
 }
 
 
@@ -2399,4 +2404,468 @@ int N;  /* size of system */
 #endif
 /**********************************************************************/
 
+/* Now some Bezier and Lagrange stuff for refining and graphics
+   subdivision
+*/
  
+/**********************************************************************
+*
+* function: bezier_eval()
+*
+* purpose: Evaluation of multi-dimensional simplicial Bezier basis
+*          function at given point.
+*/
+REAL bezier_eval(order,dim,inx,x)
+int order; /* Lagrange order */
+int dim;  /* simplicial dimension */
+int *inx; /* which basis function, barycentric, sums to order  */
+REAL *x;   /* barycentric coords of target, sums to 1 */
+{ REAL prod, prodpart[MAXCOORD];
+  int i,j,k,mm;
+  for ( k = 0 ; k <= dim ; k++ ) prodpart[k] = 0.0;
+    for ( i = 0, prod = 1.0, mm = 1 ; i <= dim ; i++ )
+      for ( j = 0 ; j < inx[i] ; j++, mm++ )
+      { for ( k = 0 ; k <= dim ; k++ )
+        { prodpart[k] *= mm*x[i]/(j+1);
+          if ( i == k ) prodpart[k] += prod*mm/(j+1.);
+        }
+        prod *= mm*x[i]/(j+1.);
+      }
+  return prod;
+}
+
+/***************************************************************************
+*
+* function: bezier_eval_1d()
+*
+* purpose: Get point on curve at given parameter.
+*/
+void bezier_eval_1d(order,sdim,p,ctrl,dest)
+int order;
+int sdim; /* space dimension*/
+REAL p;   /* parameter, 0 to 1 */
+REAL **ctrl;  /* control point coordinates */
+REAL *dest;  /* output point */
+{ int inx[2];
+  REAL t[2];
+  int i,k;
+
+  t[0] = p; t[1] = 1 - p;
+  for ( k = 0 ; k < sdim ; k++ ) dest[k] = 0.0;
+  for ( i = 0 ; i <= order ; i++ )
+  { REAL bezval;
+    inx[0] = order-i;
+    inx[1] = i;
+    bezval = bezier_eval(order,1,inx,t);
+    for ( k = 0 ; k < sdim ; k++ )
+      dest[k] += bezval*ctrl[i][k];
+  }
+
+}
+
+/***************************************************************************
+*
+* function: bezier_convert_1D_init()
+*
+* purpose: initialize matrix for converting from points on curve to
+*          control points.
+*
+*/
+
+void bezier_convert_1D_init(order)
+int order;
+{ int i,j;
+  REAL x[2];  /* barycentric coords of target point */
+  int inx[2]; /* barycentric index of basis function */
+  REAL **a = dmatrix(0,order,0,order); 
+  REAL **b = dmatrix(0,order,0,order); 
+
+  if ( bezier1invert[order] ) return;
+
+  /* fill in forward matrix */
+  for ( i = 0 ; i <= order ; i++ )  /* point on curve */
+  { x[0] = (order-i)/(REAL)order;
+    x[1] = i/(REAL)order;
+    for ( j = 0 ; j <= order ; j++ ) /* control point */
+    { inx[0] = order-j;
+      inx[1] = j;
+      a[i][j] = b[i][j] = bezier_eval(order,1,inx,x);
+    }
+  } 
+  mat_inv(a,order+1);
+  bezier1invert[order] = a;
+  bezier1revert[order] = b;
+}
+
+/***************************************************************************
+*
+* function: bezier_convert_2D_init()
+*
+* purpose: initialize matrix for converting from points on surface to
+*          control points.
+*
+*/
+
+void bezier_convert_2D_init(order)
+int order;
+{ int i,j,k,m,kk,mm;
+  REAL x[3];  /* barycentric coords of target point */
+  int inx[3]; /* barycentric index of basis function */
+  int numpts = ((order+1)*(order+2))/2;
+  REAL **a = dmatrix(0,numpts-1,0,numpts-1); 
+  REAL **b = dmatrix(0,numpts-1,0,numpts-1); 
+
+  if ( bezier2invert[order] ) return;
+
+  /* fill in forward matrix */
+  for ( i = 0, kk=0 ; i <= order ; i++ )  /* point on curve */
+    for ( k = 0 ; i+k <= order ; k++,kk++ )  /* point on curve */
+    { x[0] = (order-i-k)/(REAL)order;
+      x[1] = i/(REAL)order;
+      x[2] = k/(REAL)order;
+      for ( j = 0, mm=0 ; j <= order ; j++ ) /* control point */
+        for ( m = 0 ; m+j <= order ; m++,mm++ ) /* control point */
+        { inx[0] = order-j-m;
+          inx[1] = j;
+          inx[2] = m;
+          a[kk][mm] = b[kk][mm] = bezier_eval(order,2,inx,x);
+        }
+  } 
+  mat_inv(a,numpts);
+  bezier2invert[order] = a;
+  bezier2revert[order] = b;
+
+}
+
+/*************************************************************************
+*
+* function: bezier_trans_1d()
+*
+* purpose: Create transition matrix from old order bezier control points
+*          to new.  Goes through intermediary of curve points.
+*/
+
+void bezier_trans_1d(old_order,new_order,trans)
+int old_order;
+int new_order;
+REAL **trans; /* trans[new][old] */
+{ int i,k;
+  int inx[2];
+  REAL t[2];
+  MAT2D(x,MAXLAGRANGE+1,MAXLAGRANGE+1);  /* old control to curve points */
+
+  if ( !bezier1invert[new_order] )
+    bezier_convert_1D_init(new_order);
+
+  /* old control to curve point matrix */
+  for ( i = 0 ; i <= new_order ; i++ )
+  { 
+    t[0] = (new_order - i)/(REAL)new_order;
+    t[1] = i/(REAL)new_order;
+    for ( k = 0 ; k <= old_order ; k++ )
+    { inx[0] = old_order-k; inx[1] = k;
+      x[i][k] = bezier_eval(old_order,1,inx,t);
+    }
+  }
+  
+  /* now get final transition matrix */
+  mat_mult(bezier1invert[new_order],x,trans,new_order+1,new_order+1,old_order+1);
+}
+
+/*************************************************************************
+*
+* function: bezier_trans_2d()
+*
+* purpose: Create transition matrix from old order bezier control points
+*          to new.  Goes through intermediary of surface points.
+*/
+
+void bezier_trans_2d(old_order,new_order,trans)
+int old_order;
+int new_order;
+REAL **trans; /* trans[new][old] */
+{ int i,j,k,m,jj,kk;
+  int inx[3];
+  REAL t[3];
+  int old_numpts = ((old_order+1)*(old_order+2))/2;
+  int new_numpts = ((new_order+1)*(new_order+2))/2;
+  REAL **x;
+
+  /* old control to surface points */
+  x = dmatrix(0,new_numpts-1,0,old_numpts-1);
+
+  if ( !bezier2invert[new_order] )
+    bezier_convert_2D_init(new_order);
+
+  /* old control to surface point matrix */
+  for ( j = 0, jj=0 ; j <= new_order ; j++ )
+    for ( i = 0 ; i+j <= new_order ; i++,jj++ )
+    { 
+      t[0] = (new_order - i - j)/(REAL)new_order;
+      t[1] = j/(REAL)new_order;
+      t[2] = i/(REAL)new_order;
+      for ( k = 0, kk = 0 ; k <= old_order ; k++ )
+        for ( m = 0 ; m+k <= old_order ; m++, kk++ )
+        { inx[0] = old_order-k-m; inx[1] = k; inx[2] = m;
+          x[jj][kk] = bezier_eval(old_order,2,inx,t);
+        }
+    }
+  
+  /* now get final transition matrix */
+  mat_mult(bezier2invert[new_order],x,trans,new_numpts,new_numpts,old_numpts);
+}
+
+/***************************************************************************
+*
+* function:  bezier_refine_1d_init()
+*
+* purpose: set up matrices for calculating new control points for 
+*          refined edge.
+*/
+
+void bezier_refine_1d_init(order)
+int order;
+{ int i,k;
+  int inx[2];
+  REAL t[2];
+  MAT2D(x,2*MAXLAGRANGE+1,MAXLAGRANGE+1);  /* old control to curve points */
+
+  if ( bezier_refine_1d[order] ) return;
+  bezier_refine_1d[order] = dmatrix(0,2*order,0,order);
+
+  if ( !bezier1invert[order] )
+    bezier_convert_1D_init(order);
+
+  /* old control to curve point matrix */
+  for ( i = 0 ; i <= 2*order ; i++ )
+  { 
+    t[0] = (2*order - i)/(REAL)(2*order);
+    t[1] = i/(REAL)(2*order);
+    for ( k = 0 ; k <= order ; k++ )
+    { inx[0] = order-k; inx[1] = k;
+      x[i][k] = bezier_eval(order,1,inx,t);
+    }
+  }
+  
+  /* now get final transition matrix */
+  mat_mult(bezier1invert[order],x,bezier_refine_1d[order],
+	  order+1,order+1,order+1);
+  mat_mult(bezier1invert[order],x+order,bezier_refine_1d[order]+order,
+	  order+1,order+1,order+1);
+}
+
+/***************************************************************************
+*
+* function:  bezier_refine_2d_init()
+*
+* purpose: set up matrices for calculating new control points for 
+*          refined facet.
+*/
+
+void bezier_refine_2d_init(order)
+int order;
+{ int i,j,k,m,kk,mm,jj;
+  int inx[3];
+  REAL t[3];
+  int old_numpts = ((order+1)*(order+2))/2;
+  int new_numpts = (2*order+1)*(order+1);
+  REAL ***x = dmatrix3(2*order+1,2*order+1,old_numpts);
+              /* old control to curve points */
+  REAL **z,**w; /* for pieces of whole matrix */
+
+  if ( bezier_refine_2d[order] ) return;
+  bezier_refine_2d[order] = dmatrix(0,new_numpts,0,old_numpts);
+
+  if ( !bezier2invert[order] )
+    bezier_convert_2D_init(order);
+
+  /* old control to curve point matrix */
+  for ( j = 0 ; j <= 2*order ; j++ )
+    for ( i = 0 ; i+j <= 2*order ; i++ )
+    { 
+      t[0] = (2*order - i - j)/(REAL)(2*order);
+      t[1] = j/(REAL)(2*order);
+      t[2] = i/(REAL)(2*order);
+      for ( k = 0, kk = 0 ; k <= order ; k++ )
+        for ( m = 0 ; m+k <= order ; m++,kk++ )
+        { inx[0] = order-k-m; inx[1] = k; inx[2] = m;
+          x[i][j][kk] = bezier_eval(order,2,inx,t);
+        }
+    }
+  
+  /* now get final transition matrix in four sections */
+  z =  dmatrix(0,old_numpts-1,0,old_numpts-1);
+  w =  dmatrix(0,old_numpts-1,0,old_numpts-1);
+  /* lower left */
+  for ( j = 0, jj = 0 ; j <= order ; j++ )
+    for ( i = 0 ; i+j <= order ; i++, jj++ )
+      for ( kk = 0 ; kk < old_numpts ; kk++ )
+        z[jj][kk] = x[i][j][kk];
+  mat_mult(bezier2invert[order],z,w,old_numpts,old_numpts,old_numpts);
+  for ( j = 0, jj = 0, mm = 0 ; j <= order ; j++, mm += order )
+    for ( i = 0 ; i+j <= order ; i++, jj++, mm++ )
+      for ( kk = 0 ; kk < old_numpts ; kk++ )
+        bezier_refine_2d[order][mm][kk] = w[jj][kk];
+
+  /* lower right */
+  for ( j = 0, jj = 0 ; j <= order ; j++ )
+    for ( i = 0 ; i+j <= order ; i++, jj++ )
+      for ( kk = 0 ; kk < old_numpts ; kk++ )
+        z[jj][kk] = x[i+order][j][kk];
+  mat_mult(bezier2invert[order],z,w,old_numpts,old_numpts,old_numpts);
+  for ( j = 0, jj = 0, mm = order ; j <= order ; j++, mm += order )
+    for ( i = 0 ; i+j <= order ; i++, jj++, mm++ )
+      for ( kk = 0 ; kk < old_numpts ; kk++ )
+        bezier_refine_2d[order][mm][kk] = w[jj][kk];
+
+  /* upper left */
+  for ( j = 0, jj = 0 ; j <= order ; j++ )
+    for ( i = 0 ; i+j <= order ; i++, jj++ )
+      for ( kk = 0 ; kk < old_numpts ; kk++ )
+        z[jj][kk] = x[i][j+order][kk];
+  mat_mult(bezier2invert[order],z,w,old_numpts,old_numpts,old_numpts);
+  for ( j = 0, jj = 0, mm = 3*((order+1)*order)/2 ; j <= order ; j++ )
+    for ( i = 0 ; i+j <= order ; i++, jj++, mm++ )
+      for ( kk = 0 ; kk < old_numpts ; kk++ )
+        bezier_refine_2d[order][mm][kk] = w[jj][kk];
+
+  /* middle */
+  for ( j = order, jj = 0 ; j >= 0 ; j-- )
+    for ( i = order ; i+j >= order ; i--, jj++ )
+      for ( kk = 0 ; kk < old_numpts ; kk++ )
+        z[jj][kk] = x[i][j][kk];
+  mat_mult(bezier2invert[order],z,w,old_numpts,old_numpts,old_numpts);
+  for ( j=order, jj=0, mm = ((3*order+2)*(order+1))/2-1 ; j >= 0 ;
+                   j--,mm -= 2*(order-j)-1 )
+    for ( i = order ; i+j >= order ; i--, jj++, mm-- )
+      for ( kk = 0 ; kk < old_numpts ; kk++ )
+         bezier_refine_2d[order][mm][kk] = w[jj][kk];
+
+
+
+  free_matrix3(x);
+  free_matrix(z);
+  free_matrix(w);
+}
+
+/***********************************************************************
+*
+* function: lagrange_eval_1d()
+*
+* purpose: Evaluate one curve point using Lagrange interpolation.
+*/
+
+void lagrange_eval_1d(order,sdim,t,ctrl,dest)
+int order;
+int sdim; /* space dimension*/
+REAL t;   /* parameter, 0 to 1 */
+REAL **ctrl;  /* control point coordinates */
+REAL *dest;  /* output point */
+{
+  int i,k,m;
+
+  for ( k = 0 ; k < sdim ; k++ )
+    dest[k] = 0.0;
+  for ( i = 0 ; i <= order ;  i++ )
+  {
+    REAL p=1.0;
+    for ( m = 0 ; m <= order ; m++ )
+    { if ( m == i ) continue;
+      p *= (t*order - m)/(i - m);
+    }
+    for ( k = 0 ; k < sdim ; k++ )
+      dest[k] += p*ctrl[i][k];
+  }
+}
+
+
+/***********************************************************************
+*
+* function: lagrange_eval_2d()
+*
+* purpose: Evaluate one surface point using Lagrange 2D interpolation.
+*/
+
+void lagrange_eval_2d(order,sdim,t,ctrl,dest)
+int order;
+int sdim; /* space dimension*/
+REAL *t;   /* parameters, 0 to 1 */
+REAL **ctrl;  /* control point coordinates, in linear order */
+REAL *dest;  /* output point */
+{
+  int i,j,k,m,jj;
+  
+  for ( m = 0 ; m < sdim ; m++ )
+    dest[m] = 0.0;
+  for ( j = 0, jj = 0 ; j <= order ;  j++ )
+  {
+    for ( i = 0 ; i+j <= order ; i++,jj++ )
+    { REAL p=1.0;
+
+      k = order-i-j;
+      for ( m = 0 ; m < i ; m++ )
+        p *= (t[0]*order - m)/(i - m);
+      for ( m = 0 ; m < j ; m++ )
+        p *= (t[1]*order - m)/(j - m);
+      for ( m = 0 ; m < k ; m++ )
+        p *= ((1-t[0]-t[1])*order - m)/(k - m);
+
+      for ( m = 0 ; m < sdim ; m++ )
+        dest[m] += p*ctrl[jj][m];
+    }
+  }
+}
+
+
+/***************************************************************************
+*
+* function: bezier_eval_2d()
+*
+* purpose: Get point on surface at given parameters.
+*/
+void bezier_eval_2d(order,sdim,p,ctrl,dest)
+int order;
+int sdim; /* space dimension*/
+REAL *p;   /* two parameters, 0 to 1 barycentric */
+REAL **ctrl;  /* control point coordinates, in linear order */
+REAL *dest;  /* output point */
+{ int inx[3];
+  REAL t[3];
+  int i,j,k,jj;
+
+  t[0] = p[0]; t[1] = p[1]; t[2] = 1.0 - t[0] - t[1];
+  for ( k = 0 ; k < sdim ; k++ ) dest[k] = 0.0;
+  for ( j = 0, jj = 0  ; j <= order ; j++ )
+    for ( i = 0 ; i+j <= order ; i++,jj++ )
+    { REAL bezval;
+      inx[0] = order-i-j;
+      inx[1] = j;
+      inx[2] = i;
+      bezval = bezier_eval(order,2,inx,t);
+      for ( k = 0 ; k < sdim ; k++ )
+        dest[k] += bezval*ctrl[jj][k];
+    }
+
+}
+
+/**************************************************************************
+*
+* function: tetra_vol()
+*
+* purpose: calculate volume of a tetrahedron in 3D from its four vertices.
+*/
+
+REAL tetra_vol(a,b,c,d)
+REAL *a,*b,*c,*d;
+{ REAL bb[3],cc[3],dd[3];
+  REAL vol;
+  int i;
+  for ( i = 0 ; i < 3 ; i++ )
+  { bb[i] = b[i] - a[i];
+    cc[i] = c[i] - a[i];
+    dd[i] = d[i] - a[i];
+  }
+  vol = (bb[0]*(cc[1]*dd[2] - cc[2]*dd[1]) + bb[1]*(cc[2]*dd[0] - cc[0]*dd[2])
+      + bb[2]*(cc[0]*dd[1] - cc[1]*dd[0]))/6;
+  return vol;
+}
