@@ -77,17 +77,16 @@ struct qinfo *f_info;
   if ( area > 0.0 )
      area = sqrt(area)/web.simplex_factorial;
   else area = 0.0;
-  if ( quantities_only_flag )
+  if ( METH_INSTANCE(f_info->method)->flags & DEFAULT_INSTANCE )
   { set_facet_area(f_info->id,area);
 #ifdef SHARED_MEMORY
      if ( nprocs > 1 ) 
       proc_total_area[GET_THREAD_ID] += area;
      else
 #endif
-     web.total_area += area;
+     binary_tree_add(web.total_area_addends,area);
   }
-  if ( basic_gen_methods[METH_INSTANCE(f_info->method)->gen_method].spec_flags 
-                & SPEC_USE_DENSITY )
+  if ( METH_INSTANCE(f_info->method)->flags & USE_DENSITY )
       area *= get_facet_density(f_info->id);
   return area;
 }
@@ -119,8 +118,7 @@ struct qinfo *f_info;
     area /= web.simplex_factorial;
   }
   else fudge = area = 0.0;
-  if ( basic_gen_methods[METH_INSTANCE(f_info->method)->gen_method].spec_flags 
-                & SPEC_USE_DENSITY )
+  if ( METH_INSTANCE(f_info->method)->flags & USE_DENSITY )
   { REAL density = get_facet_density(f_info->id);
     area *= density; fudge *= density;
   }
@@ -164,8 +162,7 @@ struct qinfo *f_info;
      det *= diag[i];
   }
   area = sqrt(det)/web.simplex_factorial; 
-  if ( basic_gen_methods[METH_INSTANCE(f_info->method)->gen_method].spec_flags 
-                & SPEC_USE_DENSITY )
+  if ( METH_INSTANCE(f_info->method)->flags & USE_DENSITY )
       area *= get_facet_density(f_info->id);
   for ( i = 0 ; i < web.dimension ; i++ )
      for ( j = 0 ; j < SDIM ; j++ )
@@ -227,8 +224,7 @@ struct qinfo *f_info;
   if ( ssdet <= 0.0 ) {  return 0.0; }
   energy = area = sqrt(ssdet)/web.simplex_factorial;
   fudge = 1/ssdet;
-  if ( basic_gen_methods[METH_INSTANCE(f_info->method)->gen_method].spec_flags 
-                & SPEC_USE_DENSITY )
+  if ( METH_INSTANCE(f_info->method)->flags & USE_DENSITY )
   { REAL density = get_facet_density(f_info->id);
      energy *= density; area *= density;
   }
@@ -295,8 +291,7 @@ struct qinfo *f_info;
      det *= diag[i];
   }
   area = sqrt(det)/web.simplex_factorial; 
-  if ( basic_gen_methods[METH_INSTANCE(f_info->method)->gen_method].spec_flags 
-                & SPEC_USE_DENSITY )
+  if ( METH_INSTANCE(f_info->method)->flags & USE_DENSITY )
       area *= get_facet_density(f_info->id);
   /* form LDL, LDU, UDU */
   for ( i = 0 ; i < web.dimension ; i++ )
