@@ -35,7 +35,7 @@ edge_id e_id;
   REAL *xt=get_coord(tv);
   REAL *xh=get_coord(hv);
   REAL midx[MAXCOORD];
-  REAL gg,gg_partial[MAXCOORD];
+  REAL gg=0.0,gg_partial[MAXCOORD];
 
   /* force due to linear tension, metric evaluated at midpoint */
   for ( i = 0 ; i < SDIM ; i++ )
@@ -113,10 +113,11 @@ edge_id e_id;
   energy = sqrt(energy);
 
   if ( web.representation == STRING )
-      web.total_area    += energy;    /* don't count triple junction as area */
+      /* don't count triple junction as area */
+      binary_tree_add(web.total_area_addends,energy);
 
   energy *= get_edge_density(e_id);
-  web.total_energy += energy;
+  binary_tree_add(web.total_energy_addends,energy);
 
 }
 
@@ -167,7 +168,8 @@ edge_id e_id;
           len += tang[k]*tang[j]*eval(&web.metric[k][j],gpt,e_id,NULL);
     len = gauss2wt[i]*sqrt(len);
     if ( web.representation == STRING )
-    { web.total_area += len;    /* don't count triple junction as area */
+    { /* don't count triple junction as area */
+      binary_tree_add(web.total_area_addends,len);
       /* accumulate  area around each vertex to scale motion */
       add_vertex_star(v[0],len);
       add_vertex_star(v[1],len);
@@ -175,7 +177,7 @@ edge_id e_id;
     }
 
     len *= get_edge_density(e_id);
-    web.total_energy += len;
+    binary_tree_add(web.total_energy_addends,len);
   }
      
   return;
@@ -713,7 +715,7 @@ int mode; /* gradient or hessian */
       proc_total_area[GET_THREAD_ID] += area;
      else
 #endif
-     web.total_area += area;
+     binary_tree_add(web.total_area_addends,area);
   }
 
   free_matrix4(adjAdA);
