@@ -11,6 +11,10 @@
 *     Purpose:    My own minimum degree sparse matrix factoring algorithm
 */
 
+/* This file is basically the old version with compaction to take up
+   less memory.
+*/
+
 /* Algorithm: at each stage, eliminated vertices fall into disjoint
    clusters, representing factored hessian of region with fixed boundary.
    Vertex of minimal degree is eliminated, sometimes resulting in 
@@ -30,7 +34,7 @@
 static struct md_vertex { 
                      int supernode; /* which it belongs to */
                      int timestamp;  
-                     int newspot; /* for merging region matrices */
+                     size_t newspot; /* for merging region matrices */
                   } *vlist;
 
 struct region { int vercount;  /* number of boundary vertices */
@@ -73,9 +77,10 @@ static int superstart; /* start of active list */
 static int margin;  /* how high to go above minimum degree, command line -m */
 static int total_fill;  /* total number of fill in factored matrix */
 static REAL total_flops; /* total operation count, counting mul+add as 2 */
+static int passes;  /* number of times through main loop */
 
 void dsolve();
-static int debug_level; /* command line arg for printing diagnostics.
+int mindeg_debug_level;  /*
                           0  none
                           1  print lowest degree
                           2  print supernode sizes
@@ -89,9 +94,19 @@ static int stimestamp;  /* for marking supernodes */
 static int rtimestamp; /* for marking regions for independence */ 
 static int old_rtimestamp; /* for saving previous value */ 
 
+int mindeg_margin=5;  /* how high to go above minimum degree, user var */
+int mindeg_min_region_size = 0; /* merge smaller regions with parent */
+
+static int total_fill;  /* total number of fill in factored matrix */
 static int L_total;  /* allocated spaces for final trianuglar matrix */
 static int elim_count; /* number of variables eliminated so far */
 
+/* macros for conveniently accessing working storage */
+#define REG(spot) ((struct region *)(S->ISP+(spot)))
+#define SNODE(spot) ((struct supernode *)(S->ISP+(spot)))
+#define INT(spot)  (S->ISP+(spot))
+#define CHAR(spot)  ((char*)(S->ISP+(spot)))
+
 /* trial stuff */
 int K;  /* current row number */
 int *xIL;  /* next uneliminated vertex */
@@ -122,6 +137,57 @@ void clean_supernodes ARGS((struct linsys *));
 void merge_supernodes ARGS((struct linsys *));
 void traverse_region_tree ARGS((struct linsys *));
 void mass_eliminate ARGS((struct linsys *,struct supernode*));
+void show_mindeg_state ARGS((struct linsys *));
+
+struct supernode **superheap;  /* for heap ordering */
+static int heapcount;  /* how many in heap */
+static struct supernode sentinel;
+
+/**************************************************************************
+ *
+ * function: show_mindeg_state()
+ * 
+ * purpose: Do graphical display of current factorization state.
+ *          Sets all edges between non-eliminated vertices to red,
+ *          others to black.  Pauses for user response.
+ */
+void show_mindeg_state(S)
+struct linsys *S;
+{ char *marks;  /* unelim node numbers */
+  edge_id e_id;
+  int j;
+  int rcount;
+  struct region *r;
+  int rnum;
+  int snum;
+  struct supernode *s;
+  
+  /* count regions */
+  for ( rnum = regionstart, rcount=0 ; rnum >= 0 ; rnum = r->next )
+  { r = REG(rnum);
+    rcount++;
+  }
+
+  /* figure out which vertices are not eliminated */
+  marks = temp_calloc(S->N,1);
+  for ( snum = superstart ; snum >= 0 ; snum = s->next )
+  { s = slist + snum; 
+    for ( j = 0 ; j < s->vercount ; j++ )
+      marks[INT(s->verlist)[j]] = 1;
+  }
+  FOR_ALL_EDGES(e_id)
+  { struct hess_verlist *vh = get_vertex_vhead(get_edge_headv(e_id));
+    struct hess_verlist *vt = get_vertex_vhead(get_edge_tailv(e_id));
+    if ( marks[vh->rownum] && marks[vt->rownum] )
+      set_edge_color(e_id,RED);
+    else set_edge_color(e_id,WHITE);
+  }
+  update_display();
+  sprintf(msg,"Elimcount %d of %d;  regions %d\n",elim_count,S->N,rcount);
+  prompt(msg,errmsg,100);
+  temp_free(marks);
+}
+
 
 /*****************************************************************************
 *
@@ -141,19 +207,13 @@ int bytes; /* bytes wanted */
   size = (bytes+3)/sizeof(int);
   if ( S->ISP[0] + size + 1 > S->NSP )
   { int newsize = (int)(1.5*S->NSP)+bytes;
-    S->ISP = (int*)temp_realloc((char*)S->ISP,newsize*sizeof(int),
-                                        S->NSP*sizeof(int));
+    S->ISP = (int*)temp_realloc((char*)S->ISP,newsize*sizeof(int));
     S->NSP = newsize;
   }
   spot = S->ISP[0] + 1;
   S->ISP[0] += size;
   return spot;
 }
-/* macros for conveniently accessing working storage */
-#define REG(spot) ((struct region *)(S->ISP+(spot)))
-#define SNODE(spot) ((struct supernode *)(S->ISP+(spot)))
-#define INT(spot)  (S->ISP+(spot))
-#define CHAR(spot)  ((char*)(S->ISP+(spot)))
 /***************************************************************************
 *
 * function: region_alloc()
@@ -186,30 +246,39 @@ struct linsys *S;
 *          and establish mapping between surface elements and rows.
 *             
 */
-
+ 
 void md_vertex_setup(S)
 struct linsys *S;
-{ int n,i,k;
+{ int i,k;
+  struct hess_verlist *vh;
+  vertex_id v_id;
+  body_id b_id;
+  
   vlist = (struct md_vertex *)temp_calloc(S->N,sizeof(struct md_vertex));
 
   /* set up reverse indexing from vlist to vheads */
-  for ( n = 0 ; n <= web.skel[VERTEX].max_ord ; n++ )
-     for ( k = 0 ; k < vhead[n].freedom ; k++ )
-        vlist[vhead[n].rownum+k].newspot = ordinal(vhead[n].v_id);
+  FOR_ALL_VERTICES(v_id)
+  { vh = get_vertex_vhead(v_id);
+     for ( k = 0 ; k < vh->freedom ; k++ )
+       vlist[vh->rownum+k].newspot = vh - vhead;
+  }
   for ( i = 0 ; i < optparamcount ; i++ )
-    vlist[vhead[n+i].rownum].newspot = n + i;
-  n += i;
-  if ( augmented_hessian_flag )
+  { vh = vhead + optparam[i].vhead_index;
+    vlist[vh->rownum].newspot = vh - vhead;
+  }
+  if ( augmented_hessian_mode )
   { 
     if ( !everything_quantities_flag ) /* bodies */
-    { for ( i = 0 ; i <= web.skel[BODY].max_ord ; i++ )
-        if ( vhead[n+i].freedom )
-          vlist[vhead[n+i].rownum].newspot = n + i;
-      n += i;
+    { FOR_ALL_BODIES(b_id)
+      { vh = get_body_vhead(b_id);
+        if ( vh->freedom )
+          vlist[vh->rownum].newspot = vh - vhead;
+      }
     }
     for ( i=0 ; i < gen_quant_count ; i++ )
-    { if ( vhead[n+i].freedom )
-        vlist[vhead[n+i].rownum].newspot =  n + i;
+    { vh = vhead + GEN_QUANT(i)->vhead_index;
+      if ( vh->freedom )
+        vlist[vh->rownum].newspot =  vh - vhead;
     }
   }
 
@@ -224,13 +293,14 @@ struct linsys *S;
 
 void md_region_string(S)
 struct linsys *S;
-{ int j,k,n1,n2,p,q;
+{ int j,k,p,q;
+  size_t n1,n2;
   int rcount;
   struct region *r;
   char *entrymark;  /* for tagging done entries of A */
   int size;
   int newspot,spot;  /* index into workspace */
-  int n1_end;
+  size_t n1_end;
 
   entrymark = temp_calloc(S->IA[S->N],sizeof(char));
 
@@ -243,24 +313,30 @@ struct linsys *S;
   rcount = 0;
   for ( p = 0 ; p < S->N ; p++ )
   { for ( q =  S->IA[p]-A_OFF ; q < S->IA[p+1]-A_OFF ; q++ )
-    { 
+    { int s1,s2;
+
       if ( entrymark[q] ) continue;
       n1 = vlist[p].newspot;
       n2 = vlist[S->JA[q]-A_OFF].newspot;
       if ( n1 == n2 ) continue;  /* don't do just one vertex here */
 
-      if ( n1 > n2 ) { int tmp = n1; n1 = n2; n2 = tmp; }
+      if ( n1 > n2 ) { size_t tmp = n1; n1 = n2; n2 = tmp; }
 
       /* need new region */
       spot = region_alloc(S);
       r = REG(spot);
 
-      r->supercount = 2;
+      /* find its supernodes */
+      s1 = vlist[vhead[n1].rownum].supernode;
+      s2 = vlist[vhead[n2].rownum].supernode;
+      r->supercount = (s1==s2) ? 1 : 2;
       newspot = md_alloc(S,r->supercount*sizeof(int));
       r = REG(spot);  /* in case of reallocation of workspace */
       r->superlist = newspot;
-      INT(r->superlist)[0] = vlist[vhead[n1].rownum].supernode;
-      INT(r->superlist)[1] = vlist[vhead[n2].rownum].supernode;
+      INT(r->superlist)[0] = s1;
+      if ( s1 != s2 )
+        INT(r->superlist)[1] = s2;
+
       size = vhead[n1].freedom + vhead[n2].freedom;
       r->size = r->vercount = size;
       newspot = md_alloc(S,r->vercount*sizeof(int));
@@ -320,7 +396,7 @@ struct linsys *S;
   }
 
   /* now see if any vertices have been missed */
-  n1_end = augmented_hessian_flag ? vhead_count : 
+  n1_end = augmented_hessian_mode ? vhead_count : 
         web.skel[VERTEX].max_ord+1 + optparamcount;
   for ( n1 = 0 ; n1 < n1_end ; n1++ )
   { if ( vhead[n1].flags & HV_DONE ) continue;
@@ -423,19 +499,89 @@ int supercount;
 
 void md_supernode_setup(S)
 struct linsys *S;
-{ int n,i;
+{ int n,i,j;
   struct supernode *s;
   int vhead_hi;
-  
-  vhead_hi = augmented_hessian_flag ? vhead_count : 
+  struct hess_verlist *vh;
+ 
+  vhead_hi = augmented_hessian_mode ? vhead_count : 
         web.skel[VERTEX].max_ord+1 + optparamcount;
 
   /* allocate supernode list */
   for ( n = 0, supercount = 0; n < vhead_hi ; n++ )
      if ( vhead[n].freedom > 0 ) supercount++;
   slist = (struct supernode *)temp_calloc(supercount,sizeof(struct supernode));
-  for ( n = 0, s = slist, supercount = 0; n < vhead_hi ; n++ )
-    if ( vhead[n].freedom > 0 )
+  s = slist;
+  supercount = 0;
+
+  /* Higher-order Lagrange edge and facet innards */
+  if ( web.lagrange_order >= 3 )  /* do edge innards */
+  { edge_id e_id;
+    vertex_id *v;
+    FOR_ALL_EDGES(e_id)
+    { int df; /* total degrees of freedom on edge */
+      v = get_edge_vertices(e_id);
+      for ( j = 1, df = 0 ; j < web.lagrange_order ; j++ )
+      { vh = get_vertex_vhead(v[j]); 
+        df += vh->freedom;
+        vh->flags |= SUPERNODE_DONE;
+      }
+      if ( df == 0 ) continue;
+      s->verlist = md_alloc(S,df*sizeof(int));
+      for ( j = 1 ; j < web.lagrange_order ; j++ )
+      { vh = get_vertex_vhead(v[j]);
+        if ( vh->freedom > 0 )
+        { 
+          for ( i = 0 ; i < vh->freedom ; i++ )
+          { INT(s->verlist)[s->vercount+i] = vh->rownum + i;
+            vlist[INT(s->verlist)[s->vercount+i]].supernode = supercount;
+          }
+          s->vercount += vh->freedom;
+        }
+      }
+      s->next = supercount+1;
+      s->prev = supercount-1;
+      supercount++;
+      s++; 
+    }
+  } /* end edge innards */
+
+  /* do facet innards */
+  if ( (web.lagrange_order >= 4) && (web.representation == SOAPFILM) )  
+  { facet_id f_id;
+    vertex_id *v;
+    int ctrl = (web.lagrange_order+1)*(web.lagrange_order+2)/2;
+
+    FOR_ALL_FACETS(f_id)
+    { int df; /* total degrees of freedom on facet interior */
+      v = get_facet_vertices(f_id);
+      for ( j = 1, df = 0 ; j < ctrl ; j++ )
+      { if ( !(get_vattr(v[j]) & Q_MIDFACET) ) continue;
+        vh = get_vertex_vhead(v[j]); 
+        df += vh->freedom;
+        vh->flags |= SUPERNODE_DONE;
+      }
+      if ( df == 0 ) continue;
+      s->verlist = md_alloc(S,df*sizeof(int));
+      for ( j = 1 ; j < ctrl ; j++ )
+      { if ( !(get_vattr(v[j]) & Q_MIDFACET) ) continue;
+        vh = get_vertex_vhead(v[j]);
+        for ( i = 0 ; i < vh->freedom ; i++ )
+        { INT(s->verlist)[s->vercount+i] = vh->rownum + i;
+          vlist[INT(s->verlist)[s->vercount+i]].supernode = supercount;
+        }
+        s->vercount += vh->freedom;
+      }
+      s->next = supercount+1;
+      s->prev = supercount-1;
+      supercount++;
+      s++; 
+    }
+  } /* end facet innards */
+
+  /* now all single vertices and miscellaneous */
+  for ( n = 0 ; n < vhead_hi ; n++ )
+    if ( vhead[n].freedom > 0 && !(vhead[n].flags & SUPERNODE_DONE) )
     { s->vercount = vhead[n].freedom;
       s->verlist = md_alloc(S,s->vercount*sizeof(int));
       for ( i = 0 ; i < vhead[n].freedom ; i++ )
@@ -544,7 +690,7 @@ struct linsys *S;
     if ( supernode_compare(sslist+n ,&s) == 0 )
     { /* merge */
       ss = sslist[n];
-      if ( debug_level > 7 )
+      if ( mindeg_debug_level > 7 )
         printf("Merging supernode %d into %d.\n",ss-slist,s-slist);
       newverlist = md_alloc(S,(s->vercount+ss->vercount)*sizeof(int));
       memcpy(CHAR(newverlist),CHAR(s->verlist),s->vercount*sizeof(int));
@@ -645,14 +791,14 @@ struct linsys *S;
       {
         if ( r->supercount > maxab ) maxab = r->supercount;
         do_region_absorb(S,rnum,i);
-        if ( debug_level >= 5 )
+        if ( mindeg_debug_level >= 5 )
            printf("Absorbing region %d with %d.\n",i,rnum);
         absorbcount++;
         break;
       }
     }
   }
-  if ( debug_level > 1 ) 
+  if ( mindeg_debug_level > 1 ) 
       printf("Absorbed %d; max size %d\n",absorbcount,maxab);
 }
 
@@ -746,17 +892,12 @@ struct supernode *s;
 * of nodes bordering the region left after removing the supernode.
 */
 
-struct supernode **superheap;  /* for heap ordering */
-int heapcount;  /* how many in heap */
-
 void degree_sort(S)
 struct linsys *S;
 { int n;
   struct supernode *s;
   int spot;
 
-  superheap = (struct supernode**)temp_calloc(supercount+1,
-                  sizeof(struct supernode*));
   heapcount = 0; 
   for ( n = superstart ; n >= 0 ; n = s->next )
   { s = slist + n;
@@ -798,13 +939,17 @@ struct supernode *s;
   int size,ssize;
   int scount;
   int rkeep;
+  int sontotal = 0; /* of elim nodes of sons */
 
-  /* check independence */
+  /* check independence and gather son elim totals */
   for ( i = 0 ; i < s->rcount ; i++ )
-  { r = REG(INT(s->rlist)[i]);
+  { int rnum = INT(s->rlist)[i];
+    r = REG(rnum);
     if ( r->timestamp == rtimestamp ) return; /* not independent */
+    if ( r->son < 0 )
+      sontotal += r->size - r->vercount;
   }
-  if ( debug_level >= 2 )
+  if ( mindeg_debug_level >= 2 )
      printf("Eliminating supernode %d, size %d.\n",s-slist,s->vercount);
   /* mark timestamps */
   for ( i = 0 ; i < s->rcount ; i++ )
@@ -813,8 +958,10 @@ struct supernode *s;
   }
 
   /* create merged supernode and vertex lists */
-  /* with elim supernode at end */
-  size = s->degree+s->vercount;
+  /* with eliminated supernode vertices at end */
+  if ( sontotal > mindeg_min_region_size )
+    sontotal = 0; /* don't merge with sons */
+  size = s->degree + s->vercount + sontotal;
   newvlist = md_alloc(S,size*sizeof(int));
   ssize = exact_sdegree(S,s);
   newslist = md_alloc(S,ssize*sizeof(int));
@@ -831,6 +978,16 @@ struct supernode *s;
     S->P[elim_count+k] = INT(s->verlist)[k];
     S->IP[INT(s->verlist)[k]] = elim_count+k;
   }
+  /* and sons, if merging */
+  if ( sontotal > 0 )
+    for ( i = 0 ; i < s->rcount ; i++ )
+    { int j;
+      r = REG(INT(s->rlist)[i]);
+      if ( r->son >= 0 ) continue;
+      for ( j = r->vercount ; j < r->size ; j++ )
+        INT(newvlist)[vercount++] = INT(r->verlist)[j];
+    }
+  
   vercount = 0;
   scount = 0;
   for ( i = 0 ; i < s->rcount ; i++ )
@@ -851,10 +1008,6 @@ struct supernode *s;
     }
   }
 
-  /* record fills per vertex for later factoring */
-  for ( k = vercount ; k < size ; k++ )
-     S->LIA[INT(newvlist)[k]] += size - (k-vercount);
-
   /* set up new region */
   rkeep = region_alloc(S);
   rk = REG(rkeep);
@@ -862,20 +1015,31 @@ struct supernode *s;
   rk->vercount = vercount;
   rk->superlist = newslist;
   rk->supercount = scount;
+  rk->son = -1;
   if ( vercount != s->degree ) 
      printf("vercount %d != degree %d, supernode %d\n",vercount,s->degree,s-slist);
      
   /* mark all old regions as merged and remove from active list */
   for ( i = 0 ; i < s->rcount ; i++ )
-  { 
-     r = REG(INT(s->rlist)[i]);
+  {  int rnum = INT(s->rlist)[i]; 
+     r = REG(rnum);
      r->merged = rkeep;
-     r->BROTHER = rk->son; rk->son = INT(s->rlist)[i];
+     if ( (r->size > r->vercount) || ( r->son >= 0 ) )
+     { /* keep nonleaf regions */
+       r->BROTHER = rk->son; 
+       rk->son = rnum;
+     }
+     else if ( mindeg_debug_level >= 5 )
+     { if (r->size == r->vercount)
+         printf("Dropping nodeless region %d\n",rnum);
+       else
+         printf("Incorporating small region %d into region %d\n",rnum,rkeep);
+     }
      if ( r->next >= 0 ) REG(r->next)->prev = r->prev; 
      else last_region = r->prev;
      if ( r->prev >= 0 ) REG(r->prev)->next = r->next;
      else regionstart = r->next;
-     if ( debug_level >= 5 )
+     if ( mindeg_debug_level >= 5 )
         printf("Merging region %d with %d.\n",INT(s->rlist)[i],rkeep);
   }
 
@@ -883,6 +1047,17 @@ struct supernode *s;
   rk = REG(rkeep);
   rk->size = size;
 
+  if ( mindeg_debug_level >= 5 )
+     printf("New region %d size %d vercount %d son %d.\n",
+          rkeep,rk->size,rk->vercount,rk->son);
+  if ( mindeg_debug_level >= 7 )
+  { printf("Border: ");
+    for ( i = 0 ; i < rk->vercount ; i++ )
+       printf("%d ",INT(rk->verlist)[i]);
+    printf("   elim: ");
+    for ( ; i < rk->size ; i++ )
+       printf("%d ",INT(rk->verlist)[i]);
+  }
   elim_count += s->vercount;
 
   /* delete this supernode */
@@ -892,12 +1067,118 @@ struct supernode *s;
   s->verlist = -1;
   supercount--;
 
+  s->rcount = 0; /* inactivate */
+}
+
+
+
+/******************************************************************************
+*
+*  function: groom_regions_recur()
+*
+*  purpose: Traverse newly created region tree, gathering stats
+*           (and maybe compacting)
+*/
+
+static int final_regions;  /* number making it to factoring */
+static int final_versizes;  /* number making it to factoring */
+
+void groom_regions_recur(S,r)
+struct linsys *S;
+struct region *r;
+{ struct region *rr;
+  int son,k;
+  
+  for ( son = r->son ; son >= 0 ; son = rr->BROTHER )
+  { rr = REG(son);
+    groom_regions_recur(S,rr);
+  }
+
+  /* record fills per vertex for later factoring */
+  for ( k = r->vercount ; k < r->size ; k++ )
+  { int v = INT(r->verlist)[k];
+    S->LIA[v] += r->size - (k - r->vercount);
+  }
+
   /* gather stats */
-  { int vn = vercount,vd=vercount+s->vercount;
+  { int vn = r->vercount,vd = r->size;
     total_fill += (vd*(vd+1) - vn*(vn+1))/2;
     total_flops += ((vd+1.)*vd*(vd+2.) - (vn+1.)*vn*(vn+2.))/3;
   }
-  s->rcount = 0; /* inactivate */
+  final_regions++;
+  final_versizes += r->size;
+}
+
+/******************************************************************************
+ *
+ *  function: compact_recur()
+ *  
+ *  purpose: Compact one region information to free up large amounts
+ *           of storage before numerical factoring. Recursive.
+ *
+ *  return:  Offset (in ints) of new location in new_ISP
+ */
+static int *new_ISP;  /* compacted storage */
+static int new_ISP_spot;
+
+int compact_recur(S,r)
+struct linsys *S;
+struct region *r;
+{ struct region *rr;
+  int son;
+  int *v;
+  int retval;
+  int *rnumptr;
+  int i;
+
+  rnumptr = &(r->son);
+  for ( son = r->son ; son >= 0 ; son = rr->BROTHER )
+  { rr = REG(son);
+    *rnumptr = compact_recur(S,rr);
+    rnumptr = &(((struct region*)(new_ISP + *rnumptr))->BROTHER);
+  }
+
+  /* now do this region */
+  retval = new_ISP_spot;
+  rr = (struct region *)(new_ISP + new_ISP_spot);
+  *rr = *r;
+  new_ISP_spot += sizeof(struct region)/sizeof(int);
+  v = INT(r->verlist);
+  rr->verlist = new_ISP_spot;
+  for ( i = 0 ; i < r->size ; i++ )
+    new_ISP[new_ISP_spot++] = v[i];
+
+  return retval;
+
+}
+
+/******************************************************************************
+ *
+ *  function: compact_regions()
+ *  
+ *  purpose: Compact region information to free up large amounts
+ *           of storage before numerical factoring.
+ */
+void compact_regions(S)
+struct linsys *S;
+{ int rnum;
+  int *new_rnum;
+  int new_NSP;
+  struct region *r;
+
+  new_NSP = final_regions*sizeof(struct region) + final_versizes*sizeof(int);
+  new_ISP = (int*)temp_calloc(new_NSP,1);
+  new_ISP_spot = 0;
+
+  new_rnum = &regionstart;
+  for ( rnum = regionstart ; rnum >= 0 ; rnum = r->next )
+  { r = REG(rnum);
+    *new_rnum = compact_recur(S,r);
+    new_rnum = &(((struct region*)(new_ISP+*new_rnum))->next);
+  }
+  temp_free((char*)S->ISP);
+  S->ISP = new_ISP;
+  S->NSP = new_NSP;
 }
 
 /******************************************************************************
@@ -913,15 +1194,20 @@ struct linsys *S;
   int lowdegree;
   int bounddegree; /* cutoff for multiple elimination */
   int spot;
-  struct supernode sentinel;
 
   old_rtimestamp = rtimestamp; /* save, so know who changed */
   rtimestamp++;    /* new round of independence */
   lowdegree = superheap[1]->degree;  /* so we know where we started */
-  if ( debug_level >= 1 ) printf("Low degree %d.\n",lowdegree);
+  if ( mindeg_debug_level >= 1 ) printf("Low degree %d.\n",lowdegree);
+
+  /* using higher bounddegree leads to about 1/3 as many passes through
+     main loop, but not much reduction in total time */
   bounddegree = lowdegree+margin;        /* conservative to start with */
+  bounddegree = mindeg_margin < lowdegree ? lowdegree+mindeg_margin : 
+    2*lowdegree;
+
   sentinel.degree = 2+bounddegree;    /* big degree */
-  sentinel.verlist = 1 << (sizeof(int)-2); /* big for degree_compare */
+  sentinel.verlist = 1 << (8*sizeof(int)-2); /* big for degree_compare */
   while ( superheap[1]->degree <= bounddegree )
   { 
     mass_eliminate(S,superheap[1]);
@@ -1001,7 +1287,7 @@ struct region *r;
   struct region *rr;
   int son;
 
-  if ( debug_level > 5 ) 
+  if ( mindeg_debug_level > 5 ) 
      printf("permute_recur region %d, vercount %d rsize %d\n",
          (int*)r-S->ISP,r->vercount,r->size);
   for ( son = r->son ; son >= 0 ; son = rr->BROTHER )
@@ -1035,9 +1321,13 @@ struct linsys *S;
 
   /* allocate */
 
-  S->pIA = (int*)temp_calloc(S->N+1,sizeof(int));
-  S->pJA = (int*)temp_calloc(S->IA[S->N]-A_OFF,sizeof(int));
-  S->pA = (REAL*)temp_calloc(S->IA[S->N]-A_OFF,sizeof(REAL));
+  S->pIA = (int*)temp_realloc((char*)S->pIA,(S->N+1)*sizeof(int));
+  S->pJA = (int*)temp_realloc((char*)S->pJA,(S->IA[S->N]-A_OFF)*sizeof(int));
+  S->pA = (REAL*)temp_realloc((char*)S->pA,(S->IA[S->N]-A_OFF)*sizeof(REAL));
+  memset((char*)S->pIA,0,(S->N+1)*sizeof(int));
+  memset((char*)S->pJA,0,(S->IA[S->N]-A_OFF)*sizeof(int));
+  memset((char*)S->pA,0,(S->IA[S->N]-A_OFF)*sizeof(REAL));
+  
   cIA = (int*)temp_calloc(S->N+1,sizeof(int));
   cJA = (int*)temp_calloc(S->IA[S->N]-A_OFF,sizeof(int));
   cA = (REAL*)temp_calloc(S->IA[S->N]-A_OFF,sizeof(REAL));
@@ -1152,7 +1442,7 @@ struct region *r;
       }
       else if ( web.area_norm_flag )
          base[0] -= S->lambda*Met.A[S->P[K+i]]; /* special metric matrix */
-      else if ( augmented_hessian_flag )
+      else if ( augmented_hessian_mode )
       { if ( S->P[K+i] < S->A_rows )   /* do not shift constraints! */
           base[0] -= S->lambda;
       }
@@ -1160,6 +1450,7 @@ struct region *r;
     }
 
     /* next, add in previous rows that have entry in col K */
+    ii_next = 0;
     for ( ii = xJL[K+i] ; ii >= 0 ; ii = ii_next )
     { REAL *subbase = base - (K+i);
       REAL *la;
@@ -1369,6 +1660,15 @@ struct region *r;
   K += to_elim;
 }
 
+/**************************************************************************
+*
+* function: traverse_region_tree()
+*
+* purpose: do numerical factoring.  
+*          Permutes nodes to factoring order according to region tree.
+*          Calls factor_recur().
+*/
+
 void traverse_region_tree(S)
 struct linsys *S;
 { struct region *r;
@@ -1403,10 +1703,13 @@ struct linsys *S;
 
   /* allocate L space */
   S->Lsize = total_fill; 
-  S->LJA = (int*)temp_calloc(IJA_base,sizeof(int));
-  S->LIJA = (int*)temp_calloc(S->N+1,sizeof(int));
-  S->LA = (REAL*)temp_calloc(S->Lsize,sizeof(REAL));
-
+  S->LJA = (int*)temp_realloc((char*)S->LJA,IJA_base*sizeof(int));
+  S->LIJA = (int*)temp_realloc((char*)S->LIJA,(S->N+1)*sizeof(int));
+  S->LA = (REAL*)temp_realloc((char*)S->LA,S->Lsize*sizeof(REAL));
+  memset(S->LJA,0,IJA_base*sizeof(int));
+  memset(S->LIJA,0,(S->N+1)*sizeof(int));
+  memset(S->LA,0,S->Lsize*sizeof(REAL));
+  
   K = 0;  /* current row */
   IJA_base = 0 ;
   for ( rnum = regionstart ; rnum >= 0 ; rnum = r->next )
@@ -1448,7 +1751,7 @@ REAL *x;    /* solution, may be rhs */
 
   /* Use existing working storage if possible */
   if ( S->NSP*sizeof(int) < 2*S->N*sizeof(REAL))
-  { S->ISP = (int*)temp_realloc((char*)S->ISP,2*S->N*sizeof(REAL),S->NSP*sizeof(int));
+  { S->ISP = (int*)temp_realloc((char*)S->ISP,2*S->N*sizeof(REAL));
     S->NSP = (2*S->N*sizeof(REAL))/sizeof(int);
   }
   BB = (REAL*)S->ISP;           /* intermediate solutions */
@@ -1700,7 +2003,9 @@ struct linsys *S;
 void xmd_factor(S)
 struct linsys *S; 
 { int i,j;
-
+  int rnum;
+  struct region *r;
+  
   if ( S->N <= 0 )
   { kb_error(1839,"Internal error: Empty linear system.\n",WARNING); 
     return;
@@ -1715,10 +2020,12 @@ struct linsys *S;
 
   /* working storage */
   S->NSP = 4*S->IA[S->N];
-  S->ISP = (int*)temp_calloc(S->NSP,sizeof(int));
+  S->ISP = (int*)temp_realloc((char*)S->ISP,S->NSP*sizeof(int));
+  memset(S->ISP,0,S->NSP*sizeof(int));
 
   /* row magnitudes for nullity detection */
-  S->rowmag = (REAL *)temp_calloc(S->N,sizeof(REAL));
+  S->rowmag = (REAL *)temp_realloc((char*)S->rowmag,S->N*sizeof(REAL));
+  memset(S->rowmag,0,S->N*sizeof(REAL));
   for ( i = 0 ; i < S->N ; i++ )
   { int end = S->IA[i+1] - A_OFF;
     for ( j = S->IA[i]-A_OFF ; j < end ; j++ )
@@ -1734,32 +2041,51 @@ struct linsys *S;
   md_supernode_setup(S);
   md_region_string(S);
   md_supernode_regions(S);
-
+  /* space for lists */
+  superheap = (struct supernode**)temp_calloc(supercount+1,
+                  sizeof(struct supernode*));
+                  
   /* final lower triangular matrix storage */
-  S->psize = (int*)temp_calloc(S->N,sizeof(int));
-  S->LIA = (int*)temp_calloc(S->N+1,sizeof(int));
+  S->psize = (int*)temp_realloc((char*)S->psize,S->N*sizeof(int));
+  S->LIA = (int*)temp_realloc((char*)S->LIA,(S->N+1)*sizeof(int));
+  memset((char*)S->psize,0,S->N*sizeof(int));
+  memset((char*)S->LIA,0,(S->N+1)*sizeof(int));
+  
   L_total = 0;
   elim_count = 0;
 
-  for(;;)
+  for(passes=0;;passes++)
   {
      if ( supercount == 0 ) break;
      degree_sort(S);
      multiple_eliminate(S);
-     region_absorb(S);
+     region_absorb(S);  /* needed to have few passes through main loop */
      clean_supernodes(S);  /* supernodes stamped */
      merge_supernodes(S);
-     temp_free((char*)superheap); superheap = NULL;
+
+  if ( mindeg_debug_level == -1 )
+    show_mindeg_state(S);
+
   }
   S->LIA[elim_count] = L_total;  /* final sentinel */
 
-  /* free temp storage */
+  /* free temp storage */ 
   temp_free((char*)slist);
   temp_free((char*)vlist); vlist = NULL;
+  temp_free((char*)superheap); superheap = NULL;
+
+  /* gather stats */
+  final_regions = 0;
+  final_versizes   = 0;
+  for ( rnum = regionstart ; rnum >= 0 ; rnum = r->next )
+  { r = REG(rnum);
+    groom_regions_recur(S,r);
+  }
+  compact_regions(S);
 
   traverse_region_tree(S); 
 
-  eigen_pos = S->pos;
+  eigen_pos = S->pos; 
   eigen_neg = S->neg;
   eigen_zero = S->zero;
 
@@ -1767,61 +2093,11 @@ struct linsys *S;
   {
     printf("Variables: %d  Original fill: %d\n",S->N,S->IA[S->N]);
     printf("Workspace: %d bytes\n",S->ISP[0]*sizeof(int));
-
+    printf("Passes through main loop: %d\n",passes);
     printf("Total_fill:  %d\n",total_fill);
     printf("Total_flops: %g\n",total_flops);
   }
 }
 
 
-/*************************************************************************
-*
-* function: xmd_CHinvC()
-* 
-* purpose: Find inner product of matrix using hessian inverse as metric.
-*          C stored sparsely in S.
-*
-*          Want to use sparsity in case of large CN.  So solve H X = C
-*          for one constraint at a time, and sparse-multiply the result
-*          by the rest of the constraint rows.  Avoids forming full
-*          CN x N matrix.
-*/
-
-void xmd_CHinvC(S)
-struct linsys *S; /* factored system */
-{
-  int n; /* row index */
-  int i,j,k;
-  REAL *BB,*Y;
-
-
-  if ( S->psize == NULL )
-    kb_error(2530,"Internal error: Must call xmd_factor before xmd_CHinvC.\n",
-     RECOVERABLE);
-
-  BB = (REAL*)temp_calloc(S->N,sizeof(REAL));  /* expanded rhs */
-  Y = (REAL*)temp_calloc(S->N,sizeof(REAL));  /* solution */
-
-  for ( i = 0 ; i < S->CN ; i++ )
-  {
-    /* solve H Y = C */
-    memset(BB,0,S->N*sizeof(REAL));
-    for ( n = S->CIA[i] ; n < S->CIA[i+1] ; n++ ) 
-      BB[S->CJA[n]] = S->CA[n]; 
-    xmd_solve(S,BB,Y);
-
-    /* multiply by rows of C */
-    for ( j = 0 ; j <= i ; j++ )
-    { REAL sum;
-      for ( k = S->CIA[j] , sum = 0.0 ; k < S->CIA[j+1] ; k++ )
-        sum += S->CA[k]*Y[S->CJA[k]];
-      S->CHinvCinv[i][j] = sum;
-      if ( ! blas_flag )
-        S->CHinvCinv[j][i] = sum; /* symmetric */
-    }
-  }
-
-  temp_free((char*)BB);
-  temp_free((char*)Y);
-}
 
