@@ -186,7 +186,13 @@ int l_order;
   if ( web.lagrange_order == l_order ) return; /* already OK */
   if ( l_order < 1 ) 
      kb_error(1799,"Lagrange order must be at least 1.\n",RECOVERABLE);
+  if ( l_order > MAXLAGRANGE ) 
+  { sprintf(errmsg,"Lagrange order must be at most %d.\n",MAXLAGRANGE);
+    kb_error(3505,errmsg, RECOVERABLE);
+  }
 
+  bezier_refine_1d_init(l_order);
+  bezier_refine_2d_init(l_order);
 
   if ( web.representation == SIMPLEX )
   { simplex_lagrange_to_lagrange(l_order); return; }
@@ -208,27 +214,34 @@ int l_order;
   trans = dmatrix(0,new_fctrl,0,old_fctrl);
   newx = dmatrix(0,new_fctrl,0,SDIM);
   oldx = dmatrix(0,old_fctrl,0,SDIM);
+
   /* calculate transition matrix */
   dim = web.dimension;
   adjust_integration_orders(l_order);
   gauss_lagrange_setup(dim,l_order,web.gauss2D_order);
-  for ( m = 0 ; m <= dim ; m++ ) inx[m] = 0;
-  inx[0] = l_order;
-  new = 0;
-  do
-  { old = 0;
-    for ( m = 0 ; m <= dim ; m++ ) oldinx[m] = 0;
-    oldinx[0] = web.lagrange_order;
+  if ( bezier_flag )
+    bezier_trans_2d(web.lagrange_order,l_order,trans);
+  else
+  {
+    for ( m = 0 ; m <= dim ; m++ ) inx[m] = 0;
+    inx[0] = l_order;
+    new = 0;
     do
-     { REAL prod;
-       for ( k = 0, prod = 1.0 ; k <= dim ; k++ )
-         for ( m = 0 ; m < oldinx[k] ; m++ )
-          prod *= ((REAL)(inx[k])/l_order*web.lagrange_order - m)/(oldinx[k]-m); 
-       trans[new][old] = prod;
-       old++;
-     } while ( increment_lagrange_index(dim,oldinx) );
-    new++;
-  } while ( increment_lagrange_index(dim,inx) );
+    { old = 0;
+      for ( m = 0 ; m <= dim ; m++ ) oldinx[m] = 0;
+      oldinx[0] = web.lagrange_order;
+      do
+       { REAL prod;
+         for ( k = 0, prod = 1.0 ; k <= dim ; k++ )
+          for ( m = 0 ; m < oldinx[k] ; m++ )
+           prod *= ((REAL)(inx[k])/l_order*web.lagrange_order-m)/(oldinx[k]-m);
+         trans[new][old] = prod;
+         old++;
+       } while ( increment_lagrange_index(dim,oldinx) );
+      new++;
+    } while ( increment_lagrange_index(dim,inx) );
+  }
+
   /* calc indices of extreme vertices; handy elsewhere */
   for ( m = 0 ; m <= dim ; m++ ) inx[m] = 0;
   for ( k = 0 ; k <= dim ; k++ )
@@ -322,8 +335,9 @@ int l_order;
                   defaultp[0] = defaultp[1] = defaultp[2] = 0.1;
                   for ( i = 0 ; i < SDIM ; i++ )
                      s[i] = eval(bdry->coordf[i],defaultp,NULLID,NULL);
-                  sprintf(msg,"Could not find vertex on same boundary as facet %d.\n",
-                 get_original(f_id));
+                  sprintf(msg,
+                    "Could not find vertex on same boundary as facet %s.\n",
+                     ELNAME(f_id));
                 }
 
              parammid = get_param(v[k]);
@@ -347,23 +361,31 @@ int l_order;
   dim = (web.representation==STRING)? 1 : web.dimension-1;
   gauss_lagrange_setup(dim,l_order,web.gauss1D_order);
 
-  for ( m = 0 ; m <= dim ; m++ ) inx[m] = 0;
-  inx[0] = l_order;
-  new = 0;
-  do
-  { old = 0;
-    for ( m = 0 ; m <= dim ; m++ ) oldinx[m] = 0;
-    oldinx[0] = web.lagrange_order;
+  if ( bezier_flag )
+  { bezier_trans_1d(web.lagrange_order,l_order,trans);
+  }
+  else
+  { /* regular lagrange stuff */
+  
+    for ( m = 0 ; m <= dim ; m++ ) inx[m] = 0;
+    inx[0] = l_order;
+    new = 0;
     do
-    { REAL prod;
-      for ( k = 0, prod = 1.0 ; k <= dim ; k++ )
-        for ( m = 0 ; m < oldinx[k] ; m++ )
-         prod *= ((REAL)(inx[k])/l_order*web.lagrange_order - m)/(oldinx[k]-m); 
-      trans[new][old] = prod;
-      old++;
-    } while ( increment_lagrange_index(dim,oldinx) );
-    new++;
-  } while ( increment_lagrange_index(dim,inx) );
+    { old = 0;
+      for ( m = 0 ; m <= dim ; m++ ) oldinx[m] = 0;
+      oldinx[0] = web.lagrange_order;
+      do
+      { REAL prod;
+        for ( k = 0, prod = 1.0 ; k <= dim ; k++ )
+          for ( m = 0 ; m < oldinx[k] ; m++ )
+           prod *= ((REAL)(inx[k])/l_order*web.lagrange_order - m)/(oldinx[k]-m); 
+        trans[new][old] = prod;
+        old++;
+      } while ( increment_lagrange_index(dim,oldinx) );
+      new++;
+    } while ( increment_lagrange_index(dim,inx) );
+  }
+
   /* calc indices of extreme vertices; handy elsewhere */
   for ( m = 0 ; m <= dim ; m++ ) inx[m] = 0;
   for ( k = 0 ; k <= dim ; k++ )
@@ -445,11 +467,11 @@ int l_order;
                  base_v = tailv;
              else
                 { sprintf(errmsg,
-         "Vertices %d and %d of edge %d are on different boundaries, %d, %d, %d.\n",
-                     ordinal(headv)+1,ordinal(tailv)+1,ordinal(e_id)+1,
-                     get_boundary(headv) ? get_boundary(headv)->num : 0,
-                     get_boundary(tailv) ? get_boundary(tailv)->num : 0,
-                     bdry ? bdry->num : 0);
+         "Vertices %s and %s of edge %s are on different boundaries, %s, %s, %s.\n",
+                    ELNAME(headv),ELNAME1(tailv),ELNAME2(e_id),
+                     get_boundary(headv) ? get_boundary(headv)->name : "none",
+                     get_boundary(tailv) ? get_boundary(tailv)->name : "none",
+                     bdry ? bdry->name : "none");
                   kb_error(1803,errmsg,RECOVERABLE);
 
                 }
@@ -679,8 +701,9 @@ void linear_to_quad()
       else if ( get_boundary(tailv) == bdry )
           base_v = tailv;
       else
-         { sprintf(errmsg,"Vertices %d and %d of edge %d are on different boundaries.\n",
-              ordinal(headv)+1,ordinal(tailv)+1,ordinal(e_id)+1);
+         { sprintf(errmsg,
+  "Vertices %s and %s of edge %s are on different boundaries than the edge.\n",
+              ELNAME(headv),ELNAME1(tailv),ELNAME2(e_id)+1);
            kb_error(1808,errmsg,RECOVERABLE);
          }
 
@@ -1795,8 +1818,7 @@ int gauss_order;    /* order of polynomial to integrate exactly */
   if ( gauss_order > maxgaussorder[dim] )
   { gauss_lagrange[dim] = (struct gauss_lag *)kb_realloc(
        (char*)(gauss_lagrange[dim]),
-       (gauss_order+1)*sizeof(struct gauss_lag),
-       maxgaussorder[dim]*sizeof(struct gauss_lag));
+       (gauss_order+1)*sizeof(struct gauss_lag));
     maxgaussorder[dim] = gauss_order;
   }
          
@@ -1958,13 +1980,16 @@ do_lagrange:
          int ix;
          for ( k = 0 ; k <= dim ; k++ ) prodpart[k] = 0.0;
          for ( i = 0, prod = 1.0 ; i <= dim ; i++ )
-            for ( j = 0 ; j < inx[i] ; j++ )
-            { for ( k = 0 ; k <= dim ; k++ )
-              { prodpart[k] *= ((REAL)linx[i]-j)/(inx[i]-j);
-                 if ( i == k ) prodpart[k] += prod/(inx[i]-j);
-              }
-              prod *= ((REAL)linx[i]-j)/(inx[i]-j);
-            }
+         { 
+           for ( j = 0 ; j < inx[i] ; j++ )
+           { for ( k = 0 ; k <= dim ; k++ )
+             { prodpart[k] *= ((REAL)linx[i]-j)/(inx[i]-j);
+               if ( i == k ) 
+                  prodpart[k] += prod/(inx[i]-j);
+             }
+             prod *= ((REAL)linx[i]-j)/(inx[i]-j);
+           }
+         }
          ix = lagrange_index(dim,lagrange_order,inx);
          for ( k = 0 ; k < dim ; k++ )
             gl->lpolypart[n][k][ix] = -(prodpart[k] - prodpart[dim]);
@@ -2011,11 +2036,11 @@ int baryhash_func(n,v,x)
 int n;
 vertex_id *v;
 int  *x;
-{ int hash;
+{ unsigned int hash;
   int k;
   for ( k = 0, hash = 0 ; k < n ; k ++ ) 
-  { hash += v[k]*1843723 + x[k]*17977; }
-  return abs(hash % baryhash_size);
+  { hash += (unsigned int)(v[k]*1843723 + x[k]*17977); }
+  return  hash % baryhash_size;
 }
 
 /* find and/or insert */
@@ -2313,3 +2338,105 @@ int l_order;
   web_timestamp = top_timestamp = ++global_timestamp;
   web.headvnum = l_order;
 }
+
+/***********************************************************************
+*
+* function: lagrange_to_bezier()
+*
+* purpose: convert lagrange control points to bezier control points.
+*/
+
+void lagrange_to_bezier()
+{
+  int i,j;
+  REAL *oldx[(MAXLAGRANGE+1)*(MAXLAGRANGE+2)/2];
+  REAL **newx;
+  edge_id e_id;
+  facet_id f_id;
+  vertex_id *v;
+
+  if ( web.modeltype != LAGRANGE ) return;
+
+  newx = dmatrix(0,web.skel[FACET].ctrlpts,0,SDIM-1);
+
+  /* do facet interiors first, so can use unchanged edge vertices */ 
+  if ( web.representation == SOAPFILM )
+  FOR_ALL_FACETS(f_id)
+  { 
+    v = get_facet_vertices(f_id);
+    for ( i = 0 ; i <= web.skel[FACET].ctrlpts ; i++ )
+      oldx[i] = get_coord(v[i]);
+    mat_mult(bezier2invert[web.lagrange_order],oldx,newx,
+		web.skel[FACET].ctrlpts,web.skel[FACET].ctrlpts,SDIM);
+    for ( i = 1 ; i < web.skel[FACET].ctrlpts ; i++ ) 
+    { if ( get_vattr(v[i]) & Q_MIDFACET )
+      for ( j = 0 ; j < SDIM ; j++ )
+        oldx[i][j] = newx[i][j];
+    }
+  }
+ 
+  FOR_ALL_EDGES(e_id)
+  { 
+    v = get_edge_vertices(e_id);
+    for ( i = 0 ; i <= web.lagrange_order ; i++ )
+      oldx[i] = get_coord(v[i]);
+    mat_mult(bezier1invert[web.lagrange_order],oldx,newx,
+		web.lagrange_order+1,web.lagrange_order+1,SDIM);
+    for ( i = 1 ; i <  web.lagrange_order ; i++ ) /* no changing ends */
+      for ( j = 0 ; j < SDIM ; j++ )
+        oldx[i][j] = newx[i][j];
+  }
+
+  free_matrix(newx);
+}
+
+/***********************************************************************
+*
+* function: bezier_to_lagrange()
+*
+* purpose: convert bezier control points to lagrange control points.
+*/
+
+void bezier_to_lagrange()
+{
+  int i,j;
+  REAL *oldx[(MAXLAGRANGE+1)*(MAXLAGRANGE+2)/2];
+  REAL **newx;
+  edge_id e_id;
+  facet_id f_id;
+  vertex_id *v;
+
+  if ( web.modeltype != LAGRANGE ) return;
+
+  newx = dmatrix(0,web.skel[FACET].ctrlpts,0,SDIM-1);
+ 
+  /* do facet interior vertices first, so can use unchanged edge verts */
+  if ( web.representation == SOAPFILM )
+  FOR_ALL_FACETS(f_id)
+  { 
+    v = get_facet_vertices(f_id);
+    for ( i = 0 ; i <= web.skel[FACET].ctrlpts ; i++ )
+      oldx[i] = get_coord(v[i]);
+    mat_mult(bezier2revert[web.lagrange_order],oldx,newx,
+		web.skel[FACET].ctrlpts,web.skel[FACET].ctrlpts,SDIM);
+    for ( i = 1 ; i < web.skel[FACET].ctrlpts ; i++ ) 
+    { if ( get_vattr(v[i]) & Q_MIDFACET )
+      for ( j = 0 ; j < SDIM ; j++ )
+        oldx[i][j] = newx[i][j];
+    }
+  }
+
+  FOR_ALL_EDGES(e_id)
+  { 
+    v = get_edge_vertices(e_id);
+    for ( i = 0 ; i <= web.lagrange_order ; i++ )
+      oldx[i] = get_coord(v[i]);
+    mat_mult(bezier1revert[web.lagrange_order],oldx,newx,
+		web.lagrange_order+1,web.lagrange_order+1,SDIM);
+    for ( i = 1 ; i <  web.lagrange_order ; i++ ) /* no changing ends */
+      for ( j = 0 ; j < SDIM ; j++ )
+        oldx[i][j] = newx[i][j];
+  }
+ 
+  free_matrix(newx);
+}
