@@ -8,6 +8,7 @@
 #include "include.h"
 
 /* callable null functions of the right types */
+void null_start ARGS((void)) {}
 void null_edge(struct graphdata * g,edge_id id) {}
 void null_facet(struct graphdata * g,facet_id id) {}
 void display_null(struct tsort * t) {}
@@ -23,8 +24,8 @@ void display()
   init_graphics = null_function;
   finish_graphics = null_function;
   graph_edge = null_edge;
-  graph_start = null_facet;
-  graph_facet = null_function;
+  graph_start = null_start;
+  graph_facet = null_facet;
   graph_end = null_function;
   display_facet = display_null;
   
