@@ -128,8 +128,7 @@ void enlarge_edge_array(void)
 { int more = emax + 10;
 
   edgearray = (struct vercol*)kb_realloc((char*)edgearray,
-                    (emax+more)*sizeof(struct vercol),
-                  emax*sizeof(struct vercol));
+                    (emax+more)*sizeof(struct vercol));
   emax += more;
 }
 
@@ -144,8 +143,7 @@ void enlarge_facet_array(void)
 { int more = web.skel[FACET].count + 10;
 
   facetarray = (struct vercol*)kb_realloc((char*)facetarray,
-                     (fmax+more)*sizeof(struct vercol),
-               fmax*sizeof(struct vercol));
+                     (fmax+more)*sizeof(struct vercol));
   fmax += more;
 }
 
@@ -423,18 +421,18 @@ void CALLBACK pick_func(AUX_EVENTREC *rec)
       }
     }
     if ( valid_id(v_id) )
-    { pickvnum = ordinal(v_id) + 1;
+    { pickvnum = loc_ordinal(v_id) + 1;
       sprintf(msg,"Picked vertex %d\n",pickvnum);
       outstring(msg); 
     }
-    pickenum = ordinal(e_id) + 1;
+    pickenum = loc_ordinal(e_id) + 1;
     sprintf(msg,"Picked edge %d\n",pickenum);
     outstring(msg);
   }
   else if ( e_id == NULLEDGE )
      outstring("Picked facet subdivision edge.\n");
   if ( valid_id(f_id) ) 
-  { pickfnum = ordinal(f_id) + 1;
+  { pickfnum = loc_ordinal(f_id) + 1;
     sprintf(msg,"Picked facet %d\n",pickfnum);
     outstring(msg);
   }
@@ -1056,7 +1054,7 @@ facet_id f_id)
     { 
       if ( g[k].ecolor == CLEAR ) continue;
       if ( !edgeshow_flag || (g[0].color == UNSHOWN) )
-        { if ( g[k].etype == INVISIBLE_EDGE ) continue;      
+        { if ( (g[k].etype&EBITS) == INVISIBLE_EDGE ) continue;      
         }
       glLoadName(g[k].id); /* for picking */
       e_glColor(g[k].ecolor);
@@ -1087,7 +1085,7 @@ void Ogl_close(void)
   //InvalidateRect(hwnd,NULL,FALSE);
     kb_error(2175,"Sorry, OpenGL version can't restart show after close, so won't close.\n",
                 WARNING);
-graph_timestamp++;
+graph_timestamp = ++global_timestamp;
 draw_screen();
 }
 
@@ -1211,8 +1209,7 @@ void CALLBACK draw_screen(void)
 
     /* unify lists */
     fullarray = (struct vercol *)kb_realloc((char*)edgearray,
-               (edgecount+facetcount)*sizeof(struct vercol), 
-                edgecount*sizeof(struct vercol));       
+               (edgecount+facetcount)*sizeof(struct vercol)); 
 
     memcpy((char*)(fullarray+edgecount),(char*)facetarray,facetcount*sizeof(struct vercol));
     myfree((char*)facetarray);  facetarray = NULL; edgearray = NULL;
@@ -1716,11 +1713,9 @@ void make_strips()
   stripcount = stripnum;
   fstripcount = stripcount - estripcount;
   /* cut down arrays to needed size */
-  stripdata = (int*)kb_realloc((char*)stripdata,dataspot*sizeof(int),
-                      (edgecount+facetcount)*sizeof(int));
+  stripdata = (int*)kb_realloc((char*)stripdata,dataspot*sizeof(int));
   striparray = (struct stripstruct *)kb_realloc((char*)striparray,
-                     stripcount*sizeof(struct stripstruct),
-                (edgecount/2+facetcount/3)*sizeof(struct stripstruct));
+                     stripcount*sizeof(struct stripstruct));
       if ( q_flag )
       { sprintf(msg,"After stripping: %d edgestrips, %d facetstrips\n",
               estripcount,fstripcount);
