@@ -89,7 +89,7 @@ struct qtree_t { struct tsort *depthhead;
 int maxquaddepth = 8; /* maximum depth of quadtree */
 int get_quadindex ARGS((unsigned int));
 void qdepth_insert ARGS((struct tsort *));
-struct tsort *search_subtree ARGS((int,struct tsort *));
+struct tsort *search_subtree ARGS((int,struct tsort *,int *));
 
 /* visibility stuff */
 void visibility_stage ARGS((struct tsort *));
@@ -103,7 +103,7 @@ struct tsort *vis_list;  /* storage */
 *
 *  Function: find_bbox()
 *
-*  Purpose: find 3D bounding box for facet.
+*  Purpose: find 3D bounding box for edge or facet.
 *
 */
 
@@ -114,6 +114,54 @@ struct tsort *t;
 
   if ( (t->flag & 0xF) == EDGE )
   {
+    if ( t->flag & EDGE_ARC )
+    { REAL w1[MAXCOORD],w2[MAXCOORD],mag1,mag2,w1w2,center[2],radius;
+      REAL det,angle1,angle2;
+      int i;
+      for (i = 0 ; i < SDIM ; i++ )
+      { w1[i] = t->x[1][i] - t->x[0][i];
+        w2[i] = t->x[2][i] - t->x[0][i];
+      }
+      det = w1[0]*w2[1] - w1[1]*w2[0];
+      mag1 = SDIM_dot(w1,w1); mag2 = SDIM_dot(w2,w2);
+      w1w2 = w1[0]*w2[0] + w1[1]*w2[1];
+      for ( n = 0 ; n < gdim ; n++ )
+      { /* endpoints first */
+        if ( t->x[0][n] < t->x[2][n] )
+         { t->maxs[n] = t->x[2][n];
+           t->mins[n] = t->x[0][n];
+         }
+         else
+         { t->maxs[n] = t->x[0][n];
+           t->mins[n] = t->x[2][n];
+         }
+       }
+      if ( 4000*det*det > mag1*mag1*mag2 + mag1*mag2*mag2 - 2*mag1*w1w2*mag2 )
+      { /* circle rather that straight line */
+        center[0] = t->x[0][0] + 0.5*(w2[1]*mag1 - w1[1]*mag2)/det;
+        center[1] = t->x[0][1] + 0.5*(-w2[0]*mag1 + w1[0]*mag2)/det;
+        radius =  sqrt((mag1*mag1*mag2+mag1*mag2*mag2-2*mag1*w1w2*mag2)
+                         /4/det/det);
+        angle1 = atan2(t->x[0][1]-center[1],t->x[0][0]-center[0]);
+        angle2 = atan2(t->x[2][1]-center[1],t->x[2][0]-center[0]);
+        if ( det < 0 )
+        { REAL temp = angle1; angle1 = angle2; angle2 = temp; }
+        if ( angle2 < angle1 ) angle2 += 2*M_PI;
+        if ( (angle1 < 0.0 && angle2 > 0.0 ) || 
+             (angle1 < 2*M_PI && angle2 > 2*M_PI) )
+          t->maxs[0] = (float)(center[0] + radius);
+        if ( (angle1 < M_PI && angle2 > M_PI ) || 
+             (angle1 < 3*M_PI && angle2 > 3*M_PI) )
+          t->mins[0] = (float)(center[0] - radius);
+        if ( (angle1 < M_PI/2 && angle2 > M_PI/2 ) || 
+             (angle1 < 5*M_PI/2 && angle2 > 5*M_PI/2) )
+          t->maxs[1] = (float)(center[1] + radius);
+        if ( (angle1 < -M_PI/2 && angle2 > -M_PI/2 ) || 
+             (angle1 < 3*M_PI/2 && angle2 > 3*M_PI/2) )
+          t->mins[1] = (float)(center[1] - radius);
+        return;
+      } 
+    }
     for ( n = 0 ; n < gdim ; n++ )
     { if ( t->x[0][n] < t->x[1][n] )
       { t->maxs[n] = t->x[1][n];
@@ -251,19 +299,20 @@ edge_id e_id;
   int i,j;
   REAL a[MAXCOORD+1],b[MAXCOORD+1];
   REAL dx,dy,dz,mag,width;
+  int ctrl_pts = gdata[0].flags & EDGE_ARC ? 3 : 2;
 
   if ( gdata->color == CLEAR ) return;
   if ( count >= maxcount-2 )
   { trilist = (struct tsort *)temp_realloc((char*)trilist,
-         (maxcount+200)*sizeof(struct tsort),maxcount*sizeof(struct tsort));
+         (maxcount+200)*sizeof(struct tsort));
     maxcount += 200;
   }
 
   t = trilist + count;
   t->flag = EDGE;
-  t->flag |= (gdata->flags & (LABEL_EDGE|LABEL_HEAD|LABEL_TAIL));
+  t->flag |= (gdata->flags & (EDGE_ARC|LABEL_EDGE|LABEL_HEAD|LABEL_TAIL));
   for ( j =  SDIM ; j < HOMDIM-1 ; j++ ) a[j] = 0.0; /* filler */
-  for ( i = 0 ; i < 2 ; i++ )
+  for ( i = 0 ; i < ctrl_pts ; i++ )
   {
     for ( j = 0 ; (j < SDIM) && (j < HOMDIM-1) ; j++ ) 
         a[j] = gdata[i].x[j];
@@ -321,19 +370,26 @@ facet_id f_id;
   if ( gdata[0].color == UNSHOWN )
   { /* just do edges */
     struct graphdata ggdata[2];
+    facetedge_id fe=NULLID;
+    if ( valid_id(f_id) )
+      fe = get_facet_fe(f_id);
     for ( i = 0 ; i < FACET_EDGES ; i++ )
-    { if ( gdata[i].etype == INVISIBLE_EDGE ) continue;
+    { if ( (gdata[i].etype&EBITS) == INVISIBLE_EDGE ) continue;
       ggdata[0] = gdata[i]; 
       ggdata[0].color = gdata[i].ecolor;
       ggdata[1] = gdata[i==2 ? 0 : i+1];
-      painter_edge(ggdata,NULLID);
+      if ( valid_id(fe) )
+      { painter_edge(ggdata,get_fe_edge(fe));
+        fe = get_next_edge(fe);
+      }
+      else painter_edge(ggdata,NULLID);
     }
     return;
   }
 
   if ( count >= maxcount )
   { trilist = (struct tsort *)temp_realloc((char*)trilist,
-         (maxcount+200)*sizeof(struct tsort),maxcount*sizeof(struct tsort));
+         (maxcount+200)*sizeof(struct tsort));
     maxcount += 200;
   }
   t = trilist + count;
@@ -355,21 +411,33 @@ facet_id f_id;
       a[j] = gdata[i].x[j];
     a[HOMDIM-1] = 1.0;
     matvec_mul(view,a,b[i],HOMDIM,HOMDIM);  /* transform */
-    t->x[i][0] = (float)b[i][1];
-    t->x[i][1] = (float)b[i][2];
-    t->x[i][2] = (float)b[i][0];
+    if ( SDIM <= 2 )
+    { t->x[i][0] = (float)b[i][0];
+      t->x[i][1] = (float)b[i][1];
+    }
+    else
+    { t->x[i][0] = (float)b[i][1];
+      t->x[i][1] = (float)b[i][2];
+      t->x[i][2] = (float)b[i][0];
+    }
   } 
-  vnormal(b[0],b[1],b[2],normal); 
-  mag = sqrt(SDIM_dot(normal,normal));
-  if ( mag > 0.0 )
-  { t->normal[0] = (float)(normal[1]/mag); 
-    t->normal[1] = (float)(normal[2]/mag); 
-    t->normal[2] = (float)(normal[0]/mag);
-    if ( fabs(t->normal[2]) < 1e-6 ) t->normal[2] = 0.0; 
-  } else
-  { t->normal[0] = 0.0f; 
-    t->normal[1] = 0.0f; 
-    t->normal[2] = 1.0f;
+  if ( SDIM <= 2 )
+  { t->normal[0] = t->normal[1] = 0.0;
+    t->normal[2] = 1.0;
+  }
+  else
+  { vnormal(b[0],b[1],b[2],normal); 
+    mag = sqrt(SDIM_dot(normal,normal));
+    if ( mag > 0.0 )
+    { t->normal[0] = (float)(normal[1]/mag); 
+      t->normal[1] = (float)(normal[2]/mag); 
+      t->normal[2] = (float)(normal[0]/mag);
+      if ( fabs(t->normal[2]) < 1e-6 ) t->normal[2] = 0.0; 
+    } else
+    { t->normal[0] = 0.0f; 
+      t->normal[1] = 0.0f; 
+      t->normal[2] = 1.0f;
+    }
   }
   if ( t->normal[2] > (float)0.0 ) /* frontward normal */
   { int c;
@@ -383,9 +451,10 @@ facet_id f_id;
     c = t->ecolor[0];
     t->ecolor[0] = t->ecolor[2];
     t->ecolor[2] = c;
-    c = t->etype[0];
-    t->etype[0] = t->etype[2];
-    t->etype[2] = c;
+    c = t->etype[0] ^ LABEL_REVERSED;
+    t->etype[0] = t->etype[2] ^ LABEL_REVERSED;
+    t->etype[2] = (short)c;
+    t->etype[1] ^= LABEL_REVERSED;
     t->color = gdata[0].color;
     tv = t->v_id[1];
     t->v_id[1] = t->v_id[2];
@@ -426,34 +495,37 @@ facet_id f_id;
 * purpose: search node and subtree of quadtree depth lists for an
 *          element obscured by given element.
 * return: pointer to obscured element, or NULL if none found.
+*         also retval to indicate type of relationship
 */
-struct tsort *search_subtree(qinx,tk)
+struct tsort *search_subtree(qinx,tk,retval)
 int qinx; /* index of node in quadtree */
 struct tsort *tk;  /* given element */
+int *retval;
 { struct tsort *tj;
  
+  *retval = 0;
+
   /* check overall subtree max depth */
   if ( tk->maxs[2] <= qtree[qinx].maxdepth )
     return NULL;
 
   /* the node itself */
   for ( tj = qtree[qinx].depthhead ; tj != NULL ; tj = tj->next )
-  { int retval;
-
+  { 
     if ( tj == tk ) continue;
     if ( tk->maxs[2] <= tj->mins[2] ) 
       break; 
-    retval = in_back(tk,tj);  
-    if ( retval & (FIRST_BACK|COPLANAR|DISJOINT) ) continue;
+    *retval = in_back(tk,tj);  
+    if ( *retval & (FIRST_BACK|COPLANAR|DISJOINT) ) continue;
     return tj;
   }
 
   /* one child */
-  tj = search_subtree(2*qinx,tk);
+  tj = search_subtree(2*qinx,tk,retval);
   if ( tj ) return tj;
   
   /* other child */
-  return search_subtree(2*qinx+1,tk);
+  return search_subtree(2*qinx+1,tk,retval);
 }
 
 #ifdef XXXXX
@@ -499,6 +571,9 @@ void loopchecker()
 * purpose: sort and display facets and edges from trilist.
 */
 
+/* for debugging; just displays list as is after given number of facets */
+int debug_k = 0x7FFFFFFF;
+
 void painter_end()
 {
   int k;
@@ -506,8 +581,6 @@ void painter_end()
   struct tsort **ll,*tt;
   int quadalloc;
 
-
-
   struct tsort textra[TEXTRA]; /* for triangle fragments */
 
   in_back_calls = box_overlaps = facetfacet = facetedge = edgeedge = 
@@ -561,12 +634,16 @@ void painter_end()
   for ( k = TEXTRA ; k < count+TEXTRA ;  )
   { struct tsort *tk = tlist[k];
     struct tsort *tj;
-    int sinx,qinx;
+    int sinx,qinx=0;
     int retval;
 
     if ( breakflag ) break;
     if ( !tk->flag ) { k++; continue; }
 
+    /* for debugging and testing */
+    if ( k > debug_k )
+       goto draw_it;
+
     /* tk is current candidate back facet */
 
     /* search quadtree list for any z overlap */
@@ -585,7 +662,7 @@ void painter_end()
     }
 
     /* now search subtree for conflicts */ 
-    tj = search_subtree(qinx,tk);
+    tj = search_subtree(qinx,tk,&retval);
     if ( tj==NULL ) goto draw_it;
 
 have_conflict:
@@ -1408,9 +1485,14 @@ struct tsort *ta,*tb;
   else retval |= SECOND_BACK;
   
   /* might still not have properly detected order, so try this */
+  /*
   if ( !(retval & (FIRST_BACK|SECOND_BACK)) )
     retval = separating_plane(ta,tb,0);
+    */
+    
   return retval;
+  
+ 
 
 } /* end plane_test() */
 
@@ -1569,7 +1651,7 @@ struct brute_cut { REAL v; /* height */
  } *brute_cuts;
 int brute_cut_count;
 
-#ifdef MSC
+#ifdef PROFILING_ENABLED
 /* Profiling cycle counters */
 __int32 visibility_stage_elapsed_time[2];
 __int32 visibility_end_elapsed_time[2];
@@ -1622,7 +1704,7 @@ PROF_START(visibility_stage)
   }
   else if ( vis_count >= vis_max - 1 )
   { vis_list = (struct tsort *)temp_realloc((char*)vis_list,
-        2*vis_max*sizeof(struct tsort), vis_max*sizeof(struct tsort));
+        2*vis_max*sizeof(struct tsort));
     vis_max *= 2;
   }
 
@@ -1897,7 +1979,7 @@ PROF_START(visibility_end1)
   }
   i++;
   vis_conedges = (struct vis_conedge*)temp_realloc((char*)vis_conedges,
-     i*sizeof(struct vis_conedge),vis_conedge_max*sizeof(struct vis_conedge));
+     i*sizeof(struct vis_conedge));
   vis_conedge_max = vis_conedge_count = i;
   PROF_FINISH(visibility_end3)
 
@@ -2029,7 +2111,7 @@ PROF_START(visibility_end)
       if ( visdebuglevel >= VIS_EVENTDUMP )
         printf("crossing %d %d  at %20.15f\n",e1-vis_conedges,e2-vis_conedges,(double)next_u);
       sweep_u = next_u;   
-	  vis_crossing(e1,e2);
+      vis_crossing(e1,e2);
 PROF_START(visibility_end)
 
       continue;
@@ -2137,8 +2219,7 @@ PROF_START(vis_insert_heap)
 
   if ( vis_heap_count >= vis_heap_max-1 )
   { vis_heap = (struct vis_event *)kb_realloc((char*)vis_heap,
-                 2*vis_heap_max*sizeof(struct vis_event),
-                   vis_heap_max*sizeof(struct vis_event));
+                 2*vis_heap_max*sizeof(struct vis_event));
     vis_heap_max *= 2;
   }
 
@@ -2268,8 +2349,7 @@ PROF_START(add_layer)
   if ( i == ee->maxlayers )
   { int newcount = (ee->maxlayers > 10) ? 2*ee->maxlayers : ee->maxlayers + 10;
     ee->layer = (struct tsort **)temp_realloc((char*)(ee->layer),
-        newcount*sizeof(struct tsort*),
-          ee->maxlayers*sizeof(struct tsort*));
+        newcount*sizeof(struct tsort*));
     ee->maxlayers = newcount;
   }
   if ( i == ee->layers )
@@ -2282,8 +2362,7 @@ PROF_START(add_layer)
     { 
       if ( check_list_count >= check_list_max )
       { check_list = (struct vis_conedge **)temp_realloc((char*)check_list,
-          2*check_list_max*sizeof(struct vis_conedge *),
-            check_list_max*sizeof(struct vis_conedge *));
+          2*check_list_max*sizeof(struct vis_conedge *));
         check_list_max *= 2;
       }
       check_list[check_list_count++] = ee;
@@ -2324,8 +2403,7 @@ PROF_START(delete_layer)
       if ( !(ee->flags & V_LAYER_CHECK) )
       { if ( check_list_count >= check_list_max )
         { check_list = (struct vis_conedge **)temp_realloc((char*)check_list,
-            2*check_list_max*sizeof(struct vis_conedge *),
-              check_list_max*sizeof(struct vis_conedge *));
+            2*check_list_max*sizeof(struct vis_conedge *));
           check_list_max *= 2;
         }
         check_list[check_list_count++] = ee;
@@ -2455,7 +2533,7 @@ void dump_vislist()
   int i;
 
   fprintf(stderr,"Visibility edge list debug dump at u = %18.15f; debug_seq %d\n",
-	  sweep_u,debug_seq);
+      sweep_u,debug_seq);
   for ( e = active_edge_first ; e != &sentinel ; e = e->next_active )
   { if ( e == NULL ) { fprintf(stderr,"NULL next_active.\n"); break; }
     v = e->m*(sweep_u-e->v[0]->x[0]) + e->v[0]->x[1]; 
@@ -2509,7 +2587,7 @@ struct vis_conedge *e;
   for ( spot = active_edge_first ; spot != NULL ; spot = spot->next_active )
   { REAL spotv = spot->m*(sweep_u-spot->v[0]->x[0]) + spot->v[0]->x[1];
     if ( spotv < vprev-1e-5 )
-       kb_error(2506,"Internal error: out of order visibility list.\n",
+       kb_error(3363,"Internal error: out of order visibility list.\n",
           RECOVERABLE);
     vprev = spotv; /* debugging */
     spot->seqno = seq++;
