@@ -18,6 +18,118 @@
  extern double drand48 ARGS((void)); /* may not be in header file */
 #endif
 
+#ifdef MPI_EVOLVER
+void mpi_vertex_average ARGS((int));
+int mpi_node_checks ARGS((void));
+void mpi_remove_remote ARGS((element_id));
+void mpi_unfree_element ARGS((element_id,struct element*));
+void mpi_synch_mpi_debug ARGS((void));
+void mpi_vgrad_end ARGS((void));
+void mpi_remove_remote ARGS((element_id));
+int mpi_remote_present ARGS((element_id));
+void mpi_my_exit ARGS((void));
+void mpi_count_report ARGS((void));
+void mpi_temp_free_all ARGS((void));
+void mpi_calc_volgrads ARGS((int));
+void mpi_calc_content ARGS((int));
+void mpi_refine_edge_synch ARGS((void));
+void mpi_node_refine ARGS((void));
+struct element *mpi_remote_elptr ARGS((element_id));
+void mpi_synch_elements ARGS((int));
+void mpi_node_synch_elements ARGS((void));
+void mpi_update_display ARGS((void));
+void mpi_parallel_exec ARGS((char*));
+void mpi_calc_energy ARGS((void));
+void mpi_refine ARGS((void));
+void mpi_node_calc_energy ARGS((void));
+void mpi_calc_force ARGS((void));
+void mpi_node_calc_force ARGS((void));
+void mpi_unsave_coords ARGS((int));
+void mpi_restore_coords ARGS((int));
+void mpi_save_coords ARGS((int));
+void mpi_node_unsave_coords ARGS((int));
+void mpi_node_restore_coords ARGS((int));
+void mpi_node_save_coords ARGS((int));
+void mpi_node_convert_forms_to_vectors ARGS((int));
+void mpi_convert_forms_to_vectors ARGS((int));
+void mpi_move_vertices ARGS((int,REAL));
+void mpi_node_move_vertices ARGS((int,REAL));
+void mpi_refine_edge ARGS((element_id,element_id,element_id,int));
+void mpi_refine_init ARGS((void));
+void mpi_node_exec ARGS((int,char*));
+void mpi_calc_leftside ARGS((int));
+void mpi_calc_rightside ARGS((int));
+void mpi_lagrange_adjust ARGS((int));
+void mpi_volume_restore ARGS((int,REAL,int));
+void mpi_hessian_fill ARGS((struct linsys*,REAL**));
+void mpi_hessian_move ARGS((REAL,int,REAL*));
+void mpi_add_remote_element ARGS((struct element *));
+void mpi_synch_corona ARGS((void));
+void mpi_dissolve_corona ARGS((void));
+#endif
+
+void tree_analyze ARGS((struct linsys *));
+void fixup_edge_content_meths ARGS((edge_id));
+void fixup_vertex_content_meths ARGS((vertex_id));
+REAL tetra_vol ARGS((REAL*,REAL*,REAL*,REAL*));
+int read_single_value ARGS((int,char*));
+void set_ctypes ARGS((void));
+void kb_unput ARGS((char));
+void list_quantity ARGS((int));
+void list_method_instance ARGS((int));
+void list_boundary ARGS((int));
+void list_constraint ARGS((int));
+void torus_unwrap_edge ARGS(( edge_id ));
+int pop_vertex_to_tri ARGS(( vertex_id, edge_id *));
+int pop_vertex_to_quad ARGS(( vertex_id, edge_id *));
+int cubecone_pop ARGS(( vertex_id, int ));
+void wrap_vertex ARGS((vertex_id,int));
+void find_cpu_speed ARGS((void));
+void reset_counts ARGS((void));
+int pop_constrained_vertex ARGS((vertex_id));
+int one_con_pop_2 ARGS((vertex_id,edge_id *,edge_id *,int));
+int one_con_pop_3 ARGS((vertex_id,int ,edge_id *,edge_id *,int));
+int one_con_pop_4 ARGS((vertex_id,edge_id *,int));
+int double_con_pop ARGS((vertex_id,edge_id,edge_id *));
+int triple_con_pop ARGS((vertex_id));
+void flush_counts ARGS((void));
+void merge_vertex ARGS((vertex_id,vertex_id));
+void merge_edge ARGS((edge_id,edge_id));
+void merge_facet ARGS((facet_id,facet_id));
+void binary_tree_add ARGS((REAL*,REAL));
+void  torus_display_period_init ARGS((void));
+REAL bezier_eval ARGS((int, int, int*, REAL *));
+void bezier_convert_1D_init ARGS(( int ));
+void bezier_convert_2D_init ARGS(( int ));
+int t1_edgeswap ARGS((edge_id));
+void get_edge_tail_tangent ARGS((element_id,REAL*));
+int valid_element ARGS((element_id));
+void bezier_trans_1d ARGS((int,int,REAL**));
+void bezier_eval_1d ARGS((int,int,REAL,REAL**,REAL*));
+void lagrange_eval_1d ARGS((int,int,REAL,REAL**,REAL*));
+void bezier_refine_1d_init ARGS((int));
+void lagrange_to_bezier ARGS((void));
+void bezier_to_lagrange ARGS((void));
+
+void bezier_trans_2d ARGS((int,int,REAL**));
+void lagrange_eval_2d ARGS((int,int,REAL*,REAL**,REAL*));
+void bezier_eval_2d ARGS((int,int,REAL*,REAL**,REAL*));
+void bezier_refine_2d_init ARGS((int));
+
+void OFF_start ARGS((void));
+void OFF_edge ARGS((struct graphdata *, edge_id));
+void OFF_facet ARGS((struct graphdata *, facet_id));
+void OFF_end ARGS((void));
+unsigned int int32hash ARGS((unsigned int));
+#ifndef NOLONGLONG
+unsigned long long int64hash ARGS((unsigned long long));
+#endif
+void partner_move ARGS((void));
+void partner_shift_grads ARGS((int));
+void partner_hit_velocity_fix ARGS((void));
+void partner_hit_volgrad_fix ARGS((void));
+void detect_bdry_hits ARGS((void));
+void thread_move_vertices ARGS((void));
 void torus_period_init ARGS((void));
 void set_view_transforms_global ARGS((void));
 void set_torus_periods_global ARGS((void));
@@ -27,13 +139,14 @@ void initialize_perm_globals ARGS((void));
 void allocate_transform_colors ARGS((int));
 int pop_tri_to_edge ARGS((facet_id));
 int pop_edge_to_tri ARGS((edge_id));
+int pop_edge_to_tri_con ARGS((edge_id));
 int pop_quad_to_quad ARGS((facet_id));
 void stack_usage ARGS((struct expnode *));
-void list_top_procedures ARGS((void));
-void list_procedure ARGS((ident_t));
-void list_procedure_proto ARGS((ident_t));
-void list_function_proto ARGS((ident_t));
-void list_function ARGS((ident_t));
+void list_top_procedures ARGS((int));
+void list_procedure ARGS((struct global *));
+void list_procedure_proto ARGS((struct global *));
+void list_function_proto ARGS((struct global *));
+void list_function ARGS((struct global *));
 void init_local_scope ARGS((ident_t));
 void exit_local_scope ARGS((void));
 void begin_local_scope ARGS((void));
@@ -42,7 +155,7 @@ ident_t lookup_local_var ARGS((char*));
 ident_t add_local_var ARGS((char*));
 void read_attribute_value ARGS((struct extra *, void*));
 int equiangulate_edge ARGS((edge_id));
-void calc_view_transform_gens ARGS((void));
+int calc_view_transform_gens ARGS((void));
 void force_project ARGS(( REAL*, vertex_id, REAL *));
 int global_meth_needs ARGS((int));
 int pop_given_vertex ARGS((vertex_id));
@@ -61,8 +174,13 @@ void dy_check ARGS((void));
 void report_quantities ARGS((void));
 void list_free ARGS((char *,int));
 void list_free_all ARGS((int));
-char * list_calloc ARGS((int,int,int));
-char * list_realloc ARGS((char *,int,int)); 
+#ifdef MEMSTRINGS
+char * list_calloc ARGS((size_t,size_t,int,char*,size_t));
+char * list_realloc ARGS((char *,size_t,int,char*,size_t)); 
+#else
+char * list_calloc ARGS((size_t,size_t,int));
+char * list_realloc ARGS((char *,size_t,int)); 
+#endif
 void mem_list_summary ARGS((void));
 void mem_check_sanity ARGS((void));
 void change_hessian_functions ARGS((int,int));
@@ -77,7 +195,7 @@ void my_pvm_exit ARGS((void));
 void pvm_calc_quants ARGS((int));
 void pvm_calc_quant_grads ARGS((int));
 void pvm_facet_knot_init ARGS((REAL*));
-int two_split ARGS((facetedge_id));
+int two_split ARGS((facetedge_id,int));
 int vertex_degfree ARGS((vertex_id));
 int edge_degfree ARGS((edge_id));
 int facet_degfree ARGS((facet_id));
@@ -120,10 +238,14 @@ void break_check ARGS((void));
 int lagrange_facet_normal  ARGS((vertex_id,facet_id,REAL**));
 void test_axial_points ARGS(( facet_id ));
 int compare_edge_attr ARGS((edge_id, edge_id));
-void set_body_volume ARGS((body_id,REAL));
+int compare_vertex_edge_attr ARGS((vertex_id, edge_id));
+int compare_vertex_attr ARGS((vertex_id, vertex_id));
+void set_body_volume ARGS((body_id,REAL,int));
 void add_body_volume ARGS((body_id,REAL));
+void add_body_volume_plain ARGS((body_id,REAL));
+void add_body_abstotal ARGS((body_id,REAL));
 void set_body ARGS((element_id,element_id));
-int kb_yylex ARGS((void));
+int kb_yylex ARGS((YYSTYPE*));
 void reset_inputbuffer ARGS((void));
 char *my_fgets ARGS((char *,int,FILE *stream));
 int method_check ARGS((void));
@@ -137,6 +259,7 @@ void move_to_free_front ARGS((int,int));
 void calc_all_grads ARGS((int));
 void check_pinning ARGS((void));
 void convert_forms_to_vectors ARGS((int));
+void local_convert_forms_to_vectors ARGS((int));
 void clear_v_conmap ARGS((element_id ));
 void list_attributes ARGS((void));
 void reset_rot_order ARGS((void));
@@ -146,6 +269,8 @@ void start_keylogfile ARGS((char*));
 void stop_keylogfile ARGS((void));
 void one_sided_adjust ARGS((int));
 void calc_leftside ARGS((void));
+void local_calc_leftside ARGS((void));
+void local_calc_rightside ARGS((void));
 void load_library ARGS((char *));
 void unload_libraries ARGS((void));
 dll_func_type search_libraries ARGS((char*));
@@ -157,15 +282,15 @@ int oid ARGS((element_id));
 void flip_toggle ARGS((int*,int,char*));
 int gram_schmidt ARGS((REAL**,int,int));
 int new_vertex_average ARGS((vertex_id,int));
+int lagrange_vertex_average ARGS((vertex_id,REAL*));
 void keyword_help ARGS((char *));
 void error_help ARGS((char *));
 void write_to_console ARGS((char *));
 void read_line_from_console ARGS((char *));
 int do_edgeswap ARGS((edge_id));
 void apply_h_inverse_metric ARGS((void));
-void list_proc ARGS((int,int));
-void list_perm_proc ARGS((int,int));
-void list_procedures ARGS((void));
+void list_proc ARGS((struct global *,int));
+void list_procedures ARGS((int));
 void linear_to_lagrange ARGS((int));
 void quad_to_lagrange ARGS((int));
 void lagrange_to_linear ARGS((void));
@@ -181,6 +306,7 @@ int string_rebody ARGS((void));
 void  lagrange_to_lagrange ARGS((int));
 void  simplex_lagrange_to_lagrange ARGS((int));
 void metis_partition ARGS((int,int));
+void metis_partition_body ARGS((int,int));
 void metis_partition_dual ARGS((int,int));
 void metis_order ARGS((struct linsys *));
 void metis_vertex_order ARGS((int));
@@ -205,7 +331,7 @@ REAL square_grad ARGS((void));
 void square_grad_forces ARGS((void));
 void cg_ritz ARGS((struct linsys *,int,REAL**,REAL*));
 int check_pick ARGS((void));
-void read_extra ARGS((element_id));
+void read_extra ARGS((element_id,int));
 int simplex_quadrature ARGS((int,int,int,REAL**,REAL*));
 int eartest ARGS((vertex_id,vertex_id));
 void convert_string ARGS((char*,char*));
@@ -275,13 +401,13 @@ void bk_eigenprobe ARGS((struct linsys *));
 void bk_inverse_it ARGS((struct linsys *, REAL *));
 void sp_hessian_solve ARGS((struct linsys *,REAL *, REAL *,int));
 void sp_hessian_solve_multi ARGS((struct linsys *,REAL **, REAL **,int));
-void sp_Hessian_solver ARGS((struct linsys *, REAL *, REAL *));
+void sp_Hessian_solver ARGS((struct linsys *, REAL *, REAL **));
 void bk_constraint_setup ARGS((int, struct linsys *));
 void BK_hess_project_setup ARGS((struct linsys *));
 void xmd_factor ARGS((struct linsys *));
 void xmd_solve ARGS((struct linsys *,REAL *,REAL *));
 void xmd_solve_multi ARGS((struct linsys *,REAL **,REAL **,int));
-void xmd_CHinvC ARGS((struct linsys *));
+void sp_CHinvC ARGS((struct linsys *));
 void tree_factor ARGS((struct linsys *));
 void tree_solve ARGS((struct linsys *,REAL *,REAL *));
 void tree_solve_multi ARGS((struct linsys *,REAL **,REAL **,int));
@@ -304,7 +430,8 @@ int add_attribute ARGS((int,char*,int,int,int*,int,struct expnode *));
 int find_attribute ARGS((int,char*));
 int find_extra ARGS((char *, int *));
 int matrix_index ARGS((REAL**,int));
-void exec_commands ARGS((FILE *));
+void exec_commands ARGS((FILE *,char *));
+void *mac_exec_commands ARGS((FILE *));
 void push_fd ARGS((FILE *,char *));
 void pop_fd ARGS((void));
 facet_id get_vertex_facet ARGS((vertex_id));
@@ -324,20 +451,21 @@ void remove_vertex_facet ARGS((vertex_id,facet_id));
 void catfulltext ARGS((char*));
 int identcase ARGS((void));
 int eliminate_facet ARGS((facet_id));
+int string_eliminate_facet ARGS((facet_id));
 REAL quantity_attribute ARGS(( element_id, int ));
 REAL instance_attribute ARGS(( element_id, int ));
 int kb_checksum ARGS((char *,int));
 void apply_quantity ARGS((element_id,int));
 void unapply_quantity ARGS((element_id,int));
 void read_instance_attr ARGS(( int ));
-void read_named_quantity ARGS((void));
+int read_named_quantity ARGS((void));
 DY_OFFSET dy_calloc ARGS((int,int));
 DY_OFFSET dy_realloc ARGS((DY_OFFSET,int,int));
 void dy_free ARGS((DY_OFFSET));
 void dump_method_specs ARGS(( int ));
 void read_method_attr ARGS(( int ));
 void calc_periods ARGS((int));
-void read_method_instance ARGS((void));
+int read_method_instance ARGS((void));
 int add_standard_quantity ARGS((char *,REAL));
 int new_method_instance ARGS((char*,char*));
 int attach_method ARGS(( int, char *));
@@ -346,6 +474,7 @@ void apply_method ARGS(( element_id , char *));
 void apply_method_num ARGS(( element_id , int));
 void unapply_method ARGS(( element_id , int));
 int rebody ARGS((void));
+int merge_bodies ARGS((void));
 int get_body_valence ARGS((body_id ));
 int get_vertex_evalence ARGS((vertex_id ));
 int get_vertex_fvalence ARGS((vertex_id ));
@@ -364,6 +493,7 @@ int dissolve_edge ARGS((edge_id));
 int dissolve_facet ARGS((facet_id));
 int dissolve_body ARGS((body_id));
 REAL vertex_sq_mean_curvature ARGS((vertex_id));
+REAL vertex_mean_curvature ARGS((vertex_id));
 void burchard ARGS((int));
 void do_gfile ARGS((int,char*));
 int mat_simplify ARGS((REAL***,int));
@@ -421,8 +551,8 @@ struct sym *symbol_lookup ARGS((char*));
 vertex_id void_test ARGS((vertex_id *,int));
 char *tokname ARGS((int));
 void update_aggr ARGS((int ,REAL *,REAL ));
-void hessian_init ARGS((struct linsys *,REAL **,REAL **));
-void hessian_fill ARGS((struct linsys *,REAL *));
+void hessian_init ARGS((struct linsys *,REAL **));
+void hessian_fill ARGS((struct linsys *,REAL **));
 void find_hess_entry ARGS((struct linsys *, struct hess_verlist*,
                  struct hess_entry *[MAXCOORD][MAXCOORD]));
 void fill_hess_entry ARGS((struct linsys *, vertex_id,REAL**));
@@ -506,6 +636,7 @@ void sqgauss_energy ARGS((void));
 void sqgauss_force ARGS((void));
 int prompt ARGS((char*,char*,int));
 void update_display ARGS((void));
+void local_update_display ARGS((void));
 void set_spinr ARGS((REAL));
 void set_spinl ARGS((REAL));
 void set_tipup ARGS((REAL));
@@ -565,22 +696,23 @@ REAL splinepoly ARGS((int,REAL));
 REAL splinederiv ARGS((int,REAL));
 void spline_partial ARGS((int *,REAL,int,REAL *));
 void spline_partial_t ARGS((int *,REAL,REAL *));
-void dump_buff ARGS((char*,int));
-int  command ARGS((char *));
+void dump_buff ARGS((char*,size_t));
+int  command ARGS((char *,int));
 void newcommand ARGS((char *));
 void new_history ARGS((char *));
-void old_history ARGS((char *));
+int old_history ARGS((char *));
 int  qqparse ARGS((void));
 #ifdef MEMSTRINGS
-char *kb_calloc ARGS((int,int,char*,int));
-char *KB_realloc ARGS((char*,int,int,char*,int));
-char *kb_temp_calloc ARGS((int,int,char*,int));
+char *kb_calloc ARGS((size_t,size_t,char*,size_t));
+char *KB_realloc ARGS((char*,size_t,char*,size_t));
+char *kb_temp_calloc ARGS((size_t,size_t,char*,size_t));
+char *kb_temp_realloc ARGS((char*,size_t,char*,size_t));
 #else
-char *kb_calloc ARGS((int,int));
-char *KB_realloc ARGS((char*,int,int));
-char *kb_temp_calloc ARGS((int,int));
+char *kb_calloc ARGS((size_t,size_t));
+char *KB_realloc ARGS((char*,size_t));
+char *kb_temp_calloc ARGS((size_t,size_t));
+char *kb_temp_realloc ARGS((char*,size_t));
 #endif
-char * temp_realloc ARGS((char*,int,int));
 void temp_free ARGS((char *));
 void temp_free_all ARGS((void));
 void cg_direction ARGS((void));
@@ -595,7 +727,7 @@ void read_faces ARGS((void));
 void read_edges ARGS((void));
 void read_vertices ARGS((void));
 void read_bodies ARGS((void));
-void read_quantity ARGS((void));
+int read_quantity ARGS((void));
 int read_const ARGS((REAL *));
 void lowbound ARGS((void));
 void ridge_histogram ARGS((void));
@@ -684,30 +816,35 @@ void constr_edge_energy_q ARGS((edge_id));
 void tordup ARGS((int));
 void add_outside ARGS((void));
 REAL calc_content ARGS((int));
+void local_calc_content ARGS((int));
 int equal_constr ARGS((element_id ,element_id));
 void bdry_spring_energy ARGS((edge_id));
 void constr_spring_energy ARGS((edge_id));
 void torus_cells ARGS((void));
 void torus_edge_clip ARGS((struct graphdata *,int ));                          
+void torus_arc_clip ARGS((struct graphdata *,int ));                          
 void null_function ARGS((void));
 void diffuse ARGS((void));
 int cone_analyze ARGS((struct verfacet *,int));
 vertex_id dup_vertex ARGS((vertex_id));
 edge_id dup_edge ARGS((edge_id));
-void versplit ARGS((facetedge_id ,facetedge_id));
-int try_prop ARGS((facetedge_id *, facetedge_id));
+void versplit ARGS((facetedge_id ,facetedge_id,int));
+int try_prop ARGS((facetedge_id *, facetedge_id,int));
 int edgepop_film ARGS((void));
 char * print_express ARGS((struct expnode *,int));
 void exprint_recur ARGS((struct treenode *,int));
 char *kb_strstr ARGS((char *,char *));
 int kb_stricmp ARGS((char *,char *));
 int kb_strnicmp ARGS((char *,char *, int));
-void kb_memmove ARGS((char *,char *,int));
+void kb_memmove ARGS((char *,char *,size_t));
 int pixdump ARGS((void));
 void constr_springs ARGS((edge_id));
 void save_coords ARGS((struct oldcoord *,int));
+void local_save_coords ARGS((struct oldcoord *,int));
 void restore_coords ARGS((struct oldcoord *,int));
+void local_restore_coords ARGS((struct oldcoord *,int));
 void unsave_coords ARGS((struct oldcoord *,int));
+void local_unsave_coords ARGS((struct oldcoord *,int));
 void enforce_constraints ARGS((void));
 void kb_strupr ARGS((char *));
 void b_proj ARGS((struct boundary *,REAL *,REAL * *,int, vertex_id ));
@@ -721,11 +858,14 @@ void calc_bdry_energy_e ARGS((element_id ));
 void calc_bdry_content_v ARGS((element_id ));
 void calc_bdry_content_e ARGS((element_id ));
 void calc_force ARGS((void ));
+void local_calc_force ARGS((void ));
 void calc_energy ARGS((void));
+void local_calc_energy ARGS((void));
 void calc_pressure ARGS((void ));
 void display ARGS((void ));
 int constr_proj ARGS((int ,int ,struct constraint * *,REAL *,REAL *,
    REAL *,int *,int,vertex_id));
+int constr_proj_matrix ARGS(( vertex_id, REAL ** ));
 int project_v_constr ARGS((element_id,int ));
 void calc_constr_force_v ARGS((element_id ));
 void calc_constr_force_e ARGS((element_id ));
@@ -764,10 +904,13 @@ REAL intpoly6part ARGS((int ,int ,REAL ,REAL ));
 REAL vintzf ARGS((REAL ,REAL ));
 void vcoeff_init ARGS((void ));
 void calc_volgrads ARGS((int));
+void local_calc_volgrads ARGS((int));
 void pressure_forces ARGS((void));
 void calc_lagrange ARGS((void));
 void lagrange_adjust ARGS((void));
+void local_lagrange_adjust ARGS((void));
 void volume_restore ARGS((REAL,int));
+void local_volume_restore ARGS((REAL,int));
 int graphgen ARGS((void ));
 void plain_facets ARGS((void ));
 void plain_edges ARGS((void ));
@@ -779,13 +922,15 @@ void torus_bodies ARGS((void ));
 void reset_web ARGS((void ));
 void initialize ARGS((void));
 void read_periods ARGS((void ));
+void read_display_periods ARGS((void ));
 void wulff_initialize ARGS((char *));
-void read_boundary ARGS((void));
-void read_constraint ARGS((void));
+int  read_boundary ARGS((void));
+int  read_constraint ARGS((void));
 void read_surface_energy ARGS((void));
 void iterate ARGS((void ));
 void fix_vertices ARGS((void ));
 void move_vertices ARGS((int,REAL ));
+void local_move_vertices ARGS((int,REAL ));
 void jiggle ARGS((void ));
 void element_id_jiggle ARGS((void ));
 REAL gaussian ARGS((void ));
@@ -799,8 +944,8 @@ REAL * * * * kb_dmatrix4 ARGS((int ,int ,int, int,char*,int ));
 REAL * *kb_dmatrix ARGS((int ,int ,int ,int ));
 REAL * * * kb_dmatrix3 ARGS((int ,int ,int ));
 REAL * * * * kb_dmatrix4 ARGS((int ,int ,int, int ));
-REAL *** matrix3_reorder ARGS((REAL***,int,int,int));
 #endif
+REAL *** matrix3_reorder ARGS((REAL***,int,int,int));
 int *ivector ARGS((int ,int ));
 REAL *vector ARGS((int ,int ));
 void free_ivector ARGS((int *,int ,int ));
@@ -865,6 +1010,7 @@ void reset_skeleton ARGS((void ));
 void vgrad_init ARGS((int ));
 void vgrad_end ARGS((void ));
 struct volgrad *get_vertex_vgrad ARGS((vertex_id));
+void set_vertex_vgrad ARGS((vertex_id,struct volgrad*));
 struct volgrad *get_next_vgrad ARGS((struct volgrad *));
 struct volgrad *new_vgrad ARGS((void ));
 struct volgrad *get_bv_vgrad ARGS((int ,element_id ));
@@ -892,13 +1038,14 @@ void do_show ARGS((void ));
 int  view_transform ARGS((char *));
 void init_view ARGS((void ));
 void resize ARGS((void ));
-void old_menu ARGS((char * ));
+int old_menu ARGS((char * ));
 void extrapolate ARGS((void ));
 void save ARGS((void ));
 void restore ARGS((char *));
 void torvol ARGS((void));
 void torvol_project ARGS((int  ));
 void refine ARGS((void ));
+void local_refine ARGS((void ));
 int areaweed ARGS((REAL ));
 void area_histogram ARGS((void ));
 int edgeweed ARGS((REAL ));
@@ -921,6 +1068,7 @@ void get_edge_adjust ARGS((element_id ,REAL *));
 REAL calc_vertex_normal ARGS((element_id,element_id ,REAL *));
 void calc_vertex_smooth_normal ARGS((element_id,element_id ,REAL *));
 int new_calc_vertex_normal ARGS((element_id,REAL **));
+int project_vertex_normals ARGS(( vertex_id, REAL **, int ));
 int simplex_vertex_normal ARGS((element_id,REAL **));
 void get_edge_verts ARGS((element_id ,REAL * *,WRAPTYPE*));
 void get_facet_verts ARGS((element_id ,REAL * *,WRAPTYPE*));
@@ -937,15 +1085,15 @@ void set_facet_body ARGS((element_id ,element_id ));
 
 int makenode ARGS((NTYPE,NTYPE,NTYPE));
 
-#ifdef PVM
-void my_pvm_exit ARGS((void));
-#endif
 facet_id xget_next_vertex_facet ARGS((vertex_id,facet_id));
 facet_id get_next_vertex_facet ARGS((vertex_id,facet_id));
 void set_body_fixvol ARGS((body_id,REAL));
 facet_id get_next_body_facet ARGS((facet_id));
+facet_id get_prev_body_facet ARGS((facet_id));
 void set_next_vertex_facet ARGS((vertex_id,facet_id,facet_id));
 void set_next_body_facet ARGS((facet_id,facet_id));
+void set_prev_body_facet ARGS((facet_id,facet_id));
+void set_fe_facet ARGS((element_id ,element_id ));
 
 #if !defined (INLINE)
 /* some functions replaced by macros in serial version */
@@ -954,7 +1102,6 @@ char *get_extra ARGS((element_id,int));
 char *get_extra_ptr ARGS((element_id,struct extra *));
 void set_body_volconst ARGS((body_id,REAL));
 void set_fe_edge ARGS((element_id ,element_id ));
-void set_fe_facet ARGS((element_id ,element_id ));
 element_id get_fe_facet ARGS((element_id ));
 element_id get_prev_facet ARGS((element_id ));
 element_id get_next_facet ARGS((element_id ));
@@ -996,3 +1143,7 @@ facetedge_id get_vertex_fe ARGS((vertex_id));
   
 #endif
 
+#ifdef MAC_OS_X
+FILE *popen();
+FILE *fdopen();
+#endif
