@@ -56,6 +56,7 @@ void ps_init()
   time_t t;
   REAL xsize,ysize,scale;  /* for clipping box */
   int dummy;
+  int n;
 
   if ( (web.representation != STRING) && (gridflag < 0) )
   {
@@ -108,14 +109,23 @@ void ps_init()
   xsize = maxclipx - minclipx; ysize = maxclipy - minclipy;
   if ( xsize/ysize > 8/10.5 ) scale = 8*72/xsize;
   else scale = 10.5*72/ysize;
-  xlo = (int)floor((bbox_minx - extra - minclipx)*scale); 
-  if ( xlo < 0 ) xlo = 0;
-  xhi = (int)ceil((bbox_maxx + extra - minclipx)*scale); 
-  if ( xhi > xsize*scale ) xhi = (int)(xsize*scale);
-  ylo = (int)floor((bbox_miny - extra - minclipy)*scale); 
-  if ( ylo < 0 ) ylo = 0;
-  yhi = (int)ceil((bbox_maxy + extra - minclipy )*scale); 
-  if ( yhi > ysize*scale ) yhi = (int)(ysize*scale);
+
+  if ( full_bounding_box_flag )
+  { xlo = 0;
+    ylo = 0;
+    xhi = (int)(xsize*scale);
+    yhi = (int)(ysize*scale);
+  }
+  else /* as defined by surface */
+  { xlo = (int)floor((bbox_minx - extra - minclipx)*scale); 
+    if ( xlo < 0 ) xlo = 0;
+    xhi = (int)ceil((bbox_maxx + extra - minclipx)*scale); 
+    if ( xhi > xsize*scale ) xhi = (int)(xsize*scale);
+    ylo = (int)floor((bbox_miny - extra - minclipy)*scale); 
+    if ( ylo < 0 ) ylo = 0;
+    yhi = (int)ceil((bbox_maxy + extra - minclipy )*scale); 
+    if ( yhi > ysize*scale ) yhi = (int)(ysize*scale);
+  }
   fprintf(fd,"%%%%BoundingBox: %d %d %d %d\n",xlo,ylo,xhi,yhi);
   fprintf(fd,"%%%%Title: (%s)\n",ps_file_name);
   fprintf(fd,"%%%%Creator: Surface Evolver\n");
@@ -125,7 +135,7 @@ void ps_init()
   fprintf(fd,"%% Image is in %g\" x %g\" box aligned lower left on paper.\n",
       (DOUBLE)(xhi-xlo)/72.,(DOUBLE)(yhi-ylo)/72.);
   fputs("% Change relsize to alter relative size of labels and linewidths.\n",fd);
-  fputs("/relsize 3 def\n",fd);
+  fprintf(fd,"/relsize %f def\n",ps_labelsize);
   fprintf(fd,"%f %f scale\n",(DOUBLE)scale,(DOUBLE)scale);
   fprintf(fd,"%f %f translate\n",(DOUBLE)(-minclipx),(DOUBLE)(-minclipy));
   fprintf(fd,"newpath %f %f moveto %f %f lineto %f %f \n",
@@ -141,6 +151,7 @@ void ps_init()
       fputs("            lineto closepath fill 6 2 roll 6 copy} def % filled facet with edges\n\n",fd);
       fputs("/fc {setrgbcolor .001 relsize mul setlinewidth 6 2 roll 6 copy} def % outline only\n\n",fd);
       fputs("/edge {relsize mul setlinewidth setrgbcolor newpath moveto lineto stroke} def\n",fd);
+      fputs("/arcedge {relsize mul setlinewidth setrgbcolor newpath moveto arc stroke} def\n",fd);
     }
   else
     { fputs("/fa { newpath setgray  moveto lineto\n",fd);
@@ -149,14 +160,18 @@ void ps_init()
       fputs("            lineto closepath fill 6 2 roll 6 copy} def % filled facet with edges\n\n",fd);
       fputs("/fc {setgray 0 setgray .001 relsize mul setlinewidth 6 2 roll 6 copy} def % outline only\n\n",fd);
       fputs("/edge {relsize mul setlinewidth 0 setgray newpath moveto lineto stroke} def\n",fd);
+      fputs("/arcedge {relsize mul setlinewidth 0 setgray newpath moveto arc stroke} def\n",fd);
     }
   if ( labelflag>0 ) 
   { 
     fputs("/vorad .022 relsize mul def\n",fd);
     fputs("/arrowrad vorad def\n",fd); 
 
-    fputs("/edge { relsize mul setlinewidth 0 setgray 4 copy  newpath",fd);
-    fputs(" moveto lineto stroke\n",fd);
+    if ( ps_colorflag )
+      fputs("/edge { relsize mul setlinewidth setrgbcolor 4 copy  newpath",fd);
+    else
+      fputs("/edge { relsize mul setlinewidth 0 setgray 4 copy  newpath",fd);
+     fputs(" moveto lineto stroke\n",fd);
     fputs("        /y2 exch def /x2 exch def /y1 exch def /x1 exch def\n",fd);
     fputs("        /dy y2 y1 sub def  /dx x2 x1 sub def\n",fd);
     fputs("        /mag dx dx mul dy dy mul add sqrt def\n",fd);
@@ -175,31 +190,31 @@ void ps_init()
     fputs(" } def\n",fd);
 
     fputs("/Helvetica findfont .03 scalefont setfont\n",fd);
-     fputs("/vertex1 { gsave translate relsize relsize scale\n",fd);
-     fputs("              newpath 0 setgray 0 0 .022 0 360 arc fill\n",fd);
-     fputs("              newpath 1 setgray 0 0 .02 0 360 arc fill\n",fd);
-     fputs("              -.006 -.01 moveto 0 setgray show grestore } def\n",fd);
-     fputs("/vertex2 { gsave translate relsize relsize scale\n",fd);
-     fputs("              newpath 0 setgray 0 0 .022 0 360 arc fill\n",fd);
-     fputs("              newpath 1 setgray 0 0 .02 0 360 arc fill\n",fd);
-     fputs("              -.016 -.01 moveto 0 setgray show grestore } def\n",fd);
-     fputs("/edgenum1 { gsave translate relsize relsize scale\n",fd);
-     fputs("     newpath  -.016 -.016 moveto\n",fd);
-     fputs("    0 setgray  .032 0 rlineto 0 .032 rlineto -.032 0\n",fd);
-     fputs("    rlineto fill newpath -.015 -.015 moveto\n",fd);
-     fputs("     1 setgray  .03 0 rlineto 0 .03 rlineto -.03 0\n",fd);
-     fputs("    rlineto fill 0 setgray -.005 -.01 moveto show grestore } def\n",fd);
-     fputs("/edgenum2 { gsave translate relsize relsize scale\n",fd);
-     fputs("     newpath -.016 -.016 moveto\n",fd);
-     fputs("    0 setgray  .040 0 rlineto 0 .032 rlineto -.040 0\n",fd);
-     fputs("    rlineto fill newpath -.015 -.015 moveto\n",fd);
-     fputs("     1 setgray  .038 0 rlineto 0 .03 rlineto -.038 0\n",fd);
-     fputs("    rlineto fill 0 setgray -.011 -.01 moveto show grestore } def\n",fd);
+    for ( n = 1 ; n <= 5 ; n++ )
+    { fprintf(fd,"/vertex%d { gsave translate relsize relsize scale\n",n);
+      fprintf(fd,"              newpath 0 setgray 0 0 .022 0 360 arc fill\n");
+      fprintf(fd,"              newpath 1 setgray 0 0 .020 0 360 arc fill\n");
+      if ( n > 2 )
+        fprintf(fd,"              2 %d div 2 %d div scale\n",n,n);
+      fprintf(fd,"               %6.4f -.01 moveto 0 setgray show grestore } def\n",
+         .004-.01*n);
 
-     fputs("/facenum { gsave translate relsize relsize scale\n",fd);
-     fputs("    newpath 1 setgray -.02 -.015 moveto\n",fd);
-     fputs("    .05 0 rlineto 0 .03 rlineto -.05 0 rlineto fill 0 setgray \n",fd);
-     fputs("    newpath -.019 -.01 moveto show grestore } def \n",fd);
+      fprintf(fd,"/edgenum%d { gsave translate relsize relsize scale\n",n);
+      fprintf(fd,"     newpath  %5.3f -.016 moveto\n",-(.008+.004*n));
+      fprintf(fd,"    0 setgray  %5.3f 0 rlineto 0 .032 rlineto %6.3f 0\n",
+        0.022+.013*n,-(0.022+0.013*n));
+      fprintf(fd,"    rlineto fill newpath %5.3f -.014 moveto\n",-(.0078+.004*n));
+      fprintf(fd,"     1 setgray  %5.3f 0 rlineto 0 .03 rlineto %6.3f 0\n",
+         0.020+.013*n,-(0.020+.013*n));
+      fprintf(fd,
+        "    rlineto fill 0 setgray %6.3f -.01 moveto show grestore } def\n",
+          0.001-0.006*n);
+
+      fprintf(fd,"/facenum%d { gsave translate relsize relsize scale\n",n);
+      fputs("    newpath 1 setgray -.02 -.015 moveto\n",fd);
+      fprintf(fd,"   %5.3f 0 rlineto 0 .03 rlineto %5.3f 0 rlineto fill 0 setgray \n",(n+1)*0.025,-(n+1)*0.025);
+      fputs("    newpath -.019 -.01 moveto show grestore } def \n",fd);
+    }
   }
   fprintf(fd,"/ew {%7.5f edge} def  %% normal string edge width\n",
     ps_stringwidth);
@@ -237,13 +252,68 @@ struct tsort *t;
 { int i;
 
   if ( t->color == CLEAR ) return;
-  fprintf(fd,"  %7.4f %7.4f ",(DOUBLE)t->x[0][0],(DOUBLE)t->x[0][1]);
-  fprintf(fd,"  %7.4f %7.4f ",(DOUBLE)t->x[1][0],(DOUBLE)t->x[1][1]);
+  if ( t->flag & EDGE_ARC )
+  { REAL w1[MAXCOORD],w2[MAXCOORD],mag1,mag2,w1w2,center[2],radius;
+    REAL angle1,angle2,det;
+
+     for (i = 0 ; i < SDIM ; i++ )
+     { w1[i] = t->x[1][i] - t->x[0][i];
+       w2[i] = t->x[2][i] - t->x[0][i];
+     }
+     det = w1[0]*w2[1] - w1[1]*w2[0];
+     mag1 = SDIM_dot(w1,w1); mag2 = SDIM_dot(w2,w2);
+     w1w2 = w1[0]*w2[0] + w1[1]*w2[1];
+     if ( 4000*det*det <= mag1*mag1*mag2 + mag1*mag2*mag2 - 2*mag1*w1w2*mag2 )
+     { /* practically straight line */
+       for ( i = 0 ; i < SDIM ; i++ )
+          t->x[1][i] = t->x[2][i];
+       fprintf(fd,"  %7.4f %7.4f ",(DOUBLE)t->x[0][0],(DOUBLE)t->x[0][1]);
+       fprintf(fd,"  %7.4f %7.4f ",(DOUBLE)t->x[1][0],(DOUBLE)t->x[1][1]);
+       t->flag &= ~EDGE_ARC;
+     }
+     else
+     { /* circle */
+       center[0] = t->x[0][0] + 0.5*(w2[1]*mag1 - w1[1]*mag2)/det;
+       center[1] = t->x[0][1] + 0.5*(-w2[0]*mag1 + w1[0]*mag2)/det;
+       radius =  sqrt((mag1*mag1*mag2+mag1*mag2*mag2-2*mag1*w1w2*mag2)/4/det/det);
+       angle1 = 180/M_PI*atan2(t->x[0][1]-center[1],t->x[0][0]-center[0]);
+       angle2 = 180/M_PI*atan2(t->x[2][1]-center[1],t->x[2][0]-center[0]);
+       if ( det > 0 )
+       { fprintf(fd," %7.4f %7.4f %7.4f %7.4f %7.4f %7.4f %7.4f ",
+         center[0],center[1],
+         radius,angle1,angle2,(DOUBLE)t->x[0][0],(DOUBLE)t->x[0][1]);
+       } else
+       { fprintf(fd," %7.4f %7.4f %7.4f %7.4f %7.4f %7.4f %7.4f ",
+           center[0],center[1],
+         radius,angle2,angle1,(DOUBLE)t->x[2][0],(DOUBLE)t->x[2][1]);
+       }
+     }
+  }
+/*
+  else if ( t->flag & SPHERE_ARC )
+  {
+  }
+*/
+  else
+  {
+    fprintf(fd,"  %7.4f %7.4f ",(DOUBLE)t->x[0][0],(DOUBLE)t->x[0][1]);
+    fprintf(fd,"  %7.4f %7.4f ",(DOUBLE)t->x[1][0],(DOUBLE)t->x[1][1]);
+  }
+  /* orientation for arrow label */
+  if ( labelflag && (t->etype[0] & LABEL_REVERSED) )
+     fprintf(fd," 4 2 roll ");
   /* color, if wanted */
   if ( ps_colorflag > 0 )
      for ( i = 0 ; i < 3 ; i++ )
         fprintf(fd," %5.3f",(DOUBLE)rgb_colors[t->color][i]);
   /* width of edge, in descending order of thickness */
+  if ( t->flag & EDGE_ARC )
+  { if ( ps_widthattr >= 0 ) 
+      fprintf(fd," %5.3f arcedge ",*EREAL(t->f_id,ps_widthattr));
+    else fprintf(fd," %5.3f arcedge\n",ps_gridedgewidth);
+  } 
+  else
+  {
   if ( ps_widthattr >= 0 ) 
     fprintf(fd," %5.3f edge ",*EREAL(t->f_id,ps_widthattr));
   else if ( t->etype[0] & BARE_EDGE ) fprintf(fd," bw\n");
@@ -253,31 +323,45 @@ struct tsort *t;
   else if ( t->etype[0] & SINGLE_EDGE ) fprintf(fd," ww\n");
   else if ( t->etype[0] & TRIPLE_EDGE ) fprintf(fd," tw\n");
   else fprintf(fd," gw\n"); /* regular grid interior edge */
+  }
 
   if ( labelflag>0 )
-  { int hv=0,tv=0,en=0;
+  { char *hv=NULL,*tv=NULL,*en=NULL;
      switch ( labelflag )
      { case LABEL_ID:
-          hv = ordinal(get_edge_headv(t->f_id))+1;
-          tv = ordinal(get_edge_tailv(t->f_id))+1;
-          en = ordinal(t->f_id)+1;
+       {  hv = ELNAME(get_edge_headv(t->f_id));
+          tv = ELNAME1(get_edge_tailv(t->f_id));
+          en = ELNAME2(t->f_id);
           break;
+       }
         case LABEL_ORIG:
-          hv = get_original(get_edge_headv(t->f_id));
-          tv = get_original(get_edge_tailv(t->f_id));
-          en = get_original(t->f_id);
+        {
+          hv = ELNAME(get_original(get_edge_headv(t->f_id)));
+          tv = ELNAME1(get_original(get_edge_tailv(t->f_id)));
+          en = ELNAME2(get_original(t->f_id));
           break;
+        }
      }
-     if ( t->flag & LABEL_TAIL )
-       fprintf(fd," (%d) %f %f vertex%1d\n",tv,
-        (DOUBLE)t->x[0][0],(DOUBLE)t->x[0][1],tv<10?1:2);
-     if ( t->flag & LABEL_HEAD )
-       fprintf(fd," (%d) %f %f vertex%1d\n",hv,
-        (DOUBLE)t->x[1][0],(DOUBLE)t->x[1][1],hv<10?1:2);
-     if ( t->flag & LABEL_EDGE )
-       fprintf(fd," (%d) %f %f edgenum%1d\n",en,
+     if ( strlen(tv) > 5 ) tv[5] = 0;
+     if ( strlen(hv) > 5 ) hv[5] = 0;
+     if ( strlen(en) > 5 ) en[5] = 0; /* max 5 chars in labels */
+
+     if ( (t->flag & LABEL_TAIL) && (strlen(tv) > 0) )
+       fprintf(fd," (%s) %f %f vertex%1d\n",tv,
+        (DOUBLE)t->x[0][0],(DOUBLE)t->x[0][1],strlen(tv));
+     if ( (t->flag & LABEL_HEAD) && (strlen(hv) > 0) )
+       fprintf(fd," (%s) %f %f vertex%1d\n",hv,
+        (DOUBLE)t->x[1][0],(DOUBLE)t->x[1][1],strlen(hv));
+     if ( (t->flag & LABEL_EDGE) && (strlen(en) > 0) )
+     { if ( inverted(t->f_id) )
+       fprintf(fd," (%s) %f %f edgenum%1d\n",en,
+        (DOUBLE)((3*t->x[0][0]+2*t->x[1][0])/5),
+        (DOUBLE)(3*t->x[0][1]+2*t->x[1][1])/5,strlen(en));
+       else 
+       fprintf(fd," (%s) %f %f edgenum%1d\n",en,
         (DOUBLE)((2*t->x[0][0]+3*t->x[1][0])/5),
-        (DOUBLE)(2*t->x[0][1]+3*t->x[1][1])/5,en<10?1:2);
+        (DOUBLE)(2*t->x[0][1]+3*t->x[1][1])/5,strlen(en));
+     }
   }
 }
 
@@ -335,11 +419,13 @@ struct tsort *t;
 
     for ( i = 0 ; i < FACET_VERTS ; i++ )
       if ( (t->ecolor[i] == CLEAR) || 
-      (((gridflag<=0) || (t->color==UNSHOWN)) && (t->etype[i] == INVISIBLE_EDGE)  ) 
-      || (t->etype[i] & SPLITTING_EDGE) )
+      (((gridflag<=0) || (t->color==UNSHOWN)) && 
+           ((t->etype[i]&EBITS) == INVISIBLE_EDGE)  ) 
+      || ((t->etype[i]&EBITS) & SPLITTING_EDGE) )
           strcat(ptr," no"); 
       else 
-      { do_flag = 1;
+      { int ii;
+        do_flag = 1;
         if ( ps_colorflag > 0 )
         { int c = t->ecolor[i]>=0 ? t->ecolor[i] : 0;
           strcat(ptr,"\n "); ptr += strlen(ptr);
@@ -351,8 +437,15 @@ struct tsort *t;
              ptr += strlen(ptr);
            }
          }
+         /* see if need to reverse edge for arrow labelling */
+         if ( labelflag && (t->etype[i] & LABEL_REVERSED) )
+            strcat(line," 4 2 roll ");
          /* width of edge, in descending order of thickness */
-         if ( (ps_widthattr >= 0) && valid_id(t->f_id) ) 
+         ii = (i==2)?0:(i+1);
+         if ( (t->x[ii][0]-t->x[i][0])*(t->x[ii][0]-t->x[i][0]) + 
+              (t->x[ii][1]-t->x[i][1])*(t->x[ii][1]-t->x[i][1])  < 1e-10 )
+           strcat(line," no");
+         else if ( (ps_widthattr >= 0) && valid_id(t->f_id) ) 
          { facetedge_id fe = get_facet_fe(t->f_id);
            for ( j = 0 ; j < i ; j++ ) fe = get_next_edge(fe);
            sprintf(line+strlen(line)," %7.5f edge ",
@@ -376,12 +469,15 @@ if ( edgeredrawflag )
     for ( i = 0 ; i < FACET_VERTS ; i++ )
     { vertex_id v_id = t->v_id[i]; 
       if ( valid_id(v_id) )
-      { int erlp = edgeredrawhead[ordinal(v_id)];
+      { int erlp = edgeredrawhead[loc_ordinal(v_id)];
          for ( ; erlp != 0 ; erlp = edgeredrawlist[erlp].next )
          { struct erl *ep = edgeredrawlist + erlp;
 
          if ( fabs(t->x[i][0]-ep->v[0])+fabs(t->x[i][1]-ep->v[1]) > .00001 )
            continue;
+         if ( ep->x[0]*ep->x[0] + ep->x[1]*ep->x[1] < (labelflag?0.01:1e-10) )
+           continue;
+
          if ( ps_colorflag > 0 )
          { int c = ep->color; 
            if ( edge_rgb_color_attr > 0 )
@@ -395,8 +491,13 @@ if ( edgeredrawflag )
               (DOUBLE)rgb_colors[c][2]);
          }
          else
-          sprintf(line,"%10.6f %10.6f %10.6f %10.6f ",
-      t->x[i][0],t->x[i][1],t->x[i][0]+ep->x[0],t->x[i][1]+ep->x[1]);
+          sprintf(line,"%10.6f %10.6f %10.6f %10.6f ", t->x[i][0],
+             t->x[i][1],t->x[i][0]+ep->x[0],t->x[i][1]+ep->x[1]);
+
+         /* see if need to reverse edge for arrow labelling */
+         if ( labelflag && (ep->etype & LABEL_REVERSED) )
+            strcat(line," 4 2 roll ");
+
          if ( ep->etype & BARE_EDGE ) strcat(line," bw");
          else if ( ep->etype & FIXED_EDGE ) strcat(line," fw");
          else if ( ep->etype & CONSTRAINT_EDGE ) strcat(line," ww");
@@ -421,8 +522,9 @@ if ( edgeredrawflag )
       int ii = i >= FACET_VERTS-1 ? 0 : i+1;
 
       if ( (t->ecolor[i] == CLEAR) || 
-      (((gridflag<=0) || (t->color==UNSHOWN)) && (t->etype[i] == INVISIBLE_EDGE)  ) 
-      || (t->etype[i] & SPLITTING_EDGE) )
+      (((gridflag<=0) || (t->color==UNSHOWN)) && 
+           ((t->etype[i]&EBITS) == INVISIBLE_EDGE)  ) 
+      || ((t->etype[i]&EBITS) & SPLITTING_EDGE) )
           continue;
 
       len = sqrt((t->x[ii][0]-t->x[i][0])*(t->x[ii][0]-t->x[i][0]) +
@@ -432,13 +534,13 @@ if ( edgeredrawflag )
 
       if ( edgeredrawcount > edgeredrawmax-2 )
       { edgeredrawlist = (struct erl *)temp_realloc((char*)edgeredrawlist,
-            2*edgeredrawmax*sizeof(struct erl),edgeredrawmax*sizeof(struct erl));
+            2*edgeredrawmax*sizeof(struct erl));
          edgeredrawmax *= 2;
       }
       v_id = t->v_id[i];
       if ( valid_id(v_id) )
       { int didflag = 0,erlp;
-         for ( erlp = edgeredrawhead[ordinal(v_id)] ; erlp ; 
+         for ( erlp = edgeredrawhead[loc_ordinal(v_id)] ; erlp ; 
                     erlp = edgeredrawlist[erlp].next )
          { struct erl *ep = edgeredrawlist + erlp;
             if ( fabs(t->x[i][0]-ep->v[0])+fabs(t->x[i][1]-ep->v[1])
@@ -452,23 +554,23 @@ if ( edgeredrawflag )
             edgeredrawlist[edgeredrawcount].x[1] = fudge*(t->x[ii][1]-t->x[i][1]);
             edgeredrawlist[edgeredrawcount].v[0] = t->x[i][0];
             edgeredrawlist[edgeredrawcount].v[1] = t->x[i][1];
-            edgeredrawlist[edgeredrawcount].next = edgeredrawhead[ordinal(v_id)];
+            edgeredrawlist[edgeredrawcount].next = edgeredrawhead[loc_ordinal(v_id)];
             edgeredrawlist[edgeredrawcount].etype = t->etype[i];
             edgeredrawlist[edgeredrawcount].color = t->ecolor[i];
-            edgeredrawhead[ordinal(v_id)] = edgeredrawcount;
+            edgeredrawhead[loc_ordinal(v_id)] = edgeredrawcount;
             edgeredrawcount++;
          }
       }
       v_id = t->v_id[ii];
       if ( valid_id(v_id) )
-      { int didflag = 0,erlp;
-         for ( erlp = edgeredrawhead[ordinal(v_id)] ; erlp ; 
+      {  int didflag = 0,erlp;
+         for ( erlp = edgeredrawhead[loc_ordinal(v_id)] ; erlp ; 
                     erlp = edgeredrawlist[erlp].next )
          { struct erl *ep = edgeredrawlist + erlp;
-            if ( fabs(t->x[ii][0]-ep->v[0])+fabs(t->x[ii][1]-ep->v[1])
-         + fabs(ep->x[0]-fudge*(t->x[i][0]-t->x[ii][0]))
-         + fabs(ep->x[1]-fudge*(t->x[i][1]-t->x[ii][1])) < .0001 )
-     {  didflag = 1; break; }
+           if ( fabs(t->x[ii][0]-ep->v[0])+fabs(t->x[ii][1]-ep->v[1])
+                + fabs(ep->x[0]-fudge*(t->x[i][0]-t->x[ii][0]))
+                + fabs(ep->x[1]-fudge*(t->x[i][1]-t->x[ii][1])) < .0001 )
+           {  didflag = 1; break; }
          }
          if ( didflag == 0 )
          {
@@ -476,10 +578,10 @@ if ( edgeredrawflag )
             edgeredrawlist[edgeredrawcount].x[1] = fudge*(t->x[i][1]-t->x[ii][1]);
             edgeredrawlist[edgeredrawcount].v[0] = t->x[ii][0];
             edgeredrawlist[edgeredrawcount].v[1] = t->x[ii][1];
-            edgeredrawlist[edgeredrawcount].next = edgeredrawhead[ordinal(v_id)];
+            edgeredrawlist[edgeredrawcount].next = edgeredrawhead[loc_ordinal(v_id)];
             edgeredrawlist[edgeredrawcount].etype = t->etype[i];
             edgeredrawlist[edgeredrawcount].color = t->ecolor[i];
-            edgeredrawhead[ordinal(v_id)] = edgeredrawcount;
+            edgeredrawhead[loc_ordinal(v_id)] = edgeredrawcount;
             edgeredrawcount++;
          }
       }
@@ -489,26 +591,28 @@ if ( edgeredrawflag )
 
   if ( labelflag>0 )
   { facetedge_id fe;
-     int fn=0;
+     char * fn=NULL;
      switch ( labelflag )
      { case LABEL_ID:
-          fn = ordinal(t->f_id)+1;
+          fn = ELNAME(t->f_id);
           break;
         case LABEL_ORIG:
-          fn = get_original(t->f_id);
+          fn = ELNAME(get_original(t->f_id));
           break;
      }
-     
+     if ( fn ) fn[5] = 0; 
      if ( (t->color != CLEAR) && (t->color != UNSHOWN) && (t->flag&LABEL_FACET) )
-        fprintf(fd," (%c%d) %f %f facenum\n",(t->flag&FLIPPED_FACET)?'+':'-',fn,
+     if ( strlen(fn) > 0 )
+       fprintf(fd," (%c%s) %f %f facenum%d\n",
+          (t->flag&FLIPPED_FACET)?'+':'-',fn,
          (DOUBLE)(t->x[0][0]+t->x[1][0]+t->x[2][0])/3,
-         (DOUBLE)(t->x[0][1]+t->x[1][1]+t->x[2][1])/3);
+         (DOUBLE)(t->x[0][1]+t->x[1][1]+t->x[2][1])/3,strlen(fn));
      fe = get_facet_fe(t->f_id);
      if ( valid_id(fe) )
      for ( i = 0 ; i < FACET_EDGES ; i++ , fe = get_next_edge(fe) )
      { edge_id e_id = get_fe_edge(fe);
-        int tv=0,hv=0;
-        int en=0;
+        char *tv=NULL,*hv=NULL;
+        char *en=NULL;
         REAL a;
         int ii,jj,kk;
 
@@ -516,32 +620,47 @@ if ( edgeredrawflag )
               { kk = 2 - i; ii = (3-i)%3 ; jj = (5-i)%3; }
         else { kk = i; ii = i; jj = (i+1)%3; }
         if (t->ecolor[kk] == CLEAR) continue;
-        if ( (gridflag<0) && ((t->etype[kk] == INVISIBLE_EDGE) || (t->etype[kk] & SPLITTING_EDGE)))
+        if ( (gridflag<0) &&  (((t->etype[kk]&EBITS) == INVISIBLE_EDGE)
+                                          || (t->etype[kk] & SPLITTING_EDGE)))
              continue;
         switch ( labelflag )
         { case LABEL_ID:
-             tv = ordinal(get_edge_tailv(e_id))+1;
-             hv = ordinal(get_edge_headv(e_id))+1;
-             en = ordinal(e_id)+1;
+          {  tv = ELNAME(get_edge_tailv(e_id));
+             hv = ELNAME1(get_edge_headv(e_id));
+             en = ELNAME2(e_id);
+             break; 
+          }
+          case LABEL_ORIG: 
+          {  
+             tv = ELNAME(get_original(get_edge_tailv(e_id)));
+             hv = ELNAME1(get_original(get_edge_headv(e_id)));
+             en = ELNAME2(get_original(e_id));
              break;
-          case LABEL_ORIG:
-             tv = get_original(get_edge_tailv(e_id));
-             hv = get_original(get_edge_headv(e_id));
-             en = get_original(e_id);
-             break;
-        }
-        if ( tv > 0 )
-        fprintf(fd,"(%d) %f %f vertex%1d\n",tv,(DOUBLE)t->x[ii][0],
-                (DOUBLE)t->x[ii][1],tv<10?1:2);
-        if ( hv > 0 )
-        fprintf(fd,"(%d) %f %f vertex%1d\n",hv,(DOUBLE)t->x[jj][0],
-                (DOUBLE)t->x[jj][1],hv<10?1:2);
+          }
+        }     
+        if ( strlen(tv) > 5 ) tv[5] = 0;
+        if ( strlen(hv) > 5 ) hv[5] = 0;
+        if ( strlen(en) > 5 ) en[5] = 0; /* max 5 chars in labels */
 
-        a = inverted(e_id) ? .65 : .35;
-        if ( en > 0 )
-        fprintf(fd," (%d) %f %f edgenum%1d\n",en,
-        (DOUBLE)(a*t->x[ii][0]+(1-a)*t->x[jj][0]),
-        (DOUBLE)(a*t->x[ii][1]+(1-a)*t->x[jj][1]),en<10?1:2);
+        if ( (tv > 0) && (strlen(tv) > 0) )
+        fprintf(fd,"(%s) %f %f vertex%1d\n",tv,(DOUBLE)t->x[ii][0],
+                (DOUBLE)t->x[ii][1],strlen(tv));
+        if ( (hv > 0) && (strlen(hv) > 0) )
+        fprintf(fd,"(%s) %f %f vertex%1d\n",hv,(DOUBLE)t->x[jj][0],
+                (DOUBLE)t->x[jj][1],strlen(hv));
+
+        a = .35;
+        if ( (en > 0) && (strlen(en) > 0) )
+        { 
+          if ( inverted(e_id) )
+           fprintf(fd," (%s) %f %f edgenum%1d\n",en,
+            (DOUBLE)((1-a)*t->x[ii][0]+a*t->x[jj][0]),
+            (DOUBLE)((1-a)*t->x[ii][1]+a*t->x[jj][1]),strlen(en));
+          else
+           fprintf(fd," (%s) %f %f edgenum%1d\n",en,
+            (DOUBLE)(a*t->x[ii][0]+(1-a)*t->x[jj][0]),
+            (DOUBLE)(a*t->x[ii][1]+(1-a)*t->x[jj][1]),strlen(en));
+        }
 
      }
   }
