@@ -13,7 +13,7 @@
 struct qgrad { REAL *g;};
 struct qhess { REAL **gg;};
 #ifndef MAXMETH
-#define MAXMETH 10
+#define MAXMETH 4 
 #endif
 
 #ifdef NOPROTO
@@ -29,6 +29,7 @@ typedef struct method_instance *MIPTR;
 /* structure for passing info to element methods */
 struct qinfo {
     element_id id;
+    int flags;  /* see below */
     int method;    /* instance number; unreliable to use pointer here */
     int vcount;  /* number of vertices involved */
     vertex_id v[MAXVCOUNT];  /* vertex list */
@@ -52,6 +53,9 @@ struct qinfo {
     struct linsys *S;  /* linear system associated with hessian */
     };
 
+/* flags values */
+#define INCOMPLETE_STAR  1
+
 /* for type of initialization */
 #define METHOD_VALUE      1767
 #define METHOD_GRADIENT  4321
@@ -78,14 +82,17 @@ struct gen_quant {
     REAL modulus;           /* overall multiplier */
     REAL tolerance;         /* for fixed quantity */
     REAL pressure;          /* Lagrange multiplier */
+    REAL oldpressure;       /* for restore_coords() */
     REAL volconst;          /* additive constant, for torus volume */
     body_id b_id;           /* body this is volume of, if any */
     int  vol_number;        /* equivalent body number for fixvol */
     int  fixnum;            /* order in fixvol computations */
     struct qgrad *grad;     /* gradients at vertices */
     struct qhess *hess;     /* hessian on vertices and edges */
+    int vhead_index;        /* index into vhead list for hessian */
     int method_count;       /* number of methods */
-    int meth_inst[MAXMETH]; /* method instances contributing */
+    int *meth_inst;         /* method instances contributing */
+    int meth_inst_space[MAXMETH];  /* in case of only a few instances */
     struct expnode  expr;   /* for compound formula */
     int next_compound;      /* -1 for end of list */
     long timestamp;         /* when quantity last calculated */
@@ -99,9 +106,10 @@ struct gen_quant {
 #define Q_COMPOUND 0x0100    /* quantity defined by compound formula */
 #define STANDARD_QUANTITY 0x1000
 #define DEFAULT_QUANTITY  0x2000
-#define TORUS_MODULO_MUNGE 0x4000
 #define Q_PRESSURE_SET  0x8000
 #define Q_REDUNDANT     0x10000  /* skip this quantity due to torus_filled */
+#define TORUS_MODULO_MUNGE 0x40000
+#define Q_DELETED  0x80000
 
 /* structure for one quantity calculating method */
 /* split so can use same method for many quantities, */
@@ -127,10 +135,13 @@ struct gen_quant_method {
 #define NEED_STAR  0x100
 #define NEED_STRING_STAR  0x200
 #define NEED_MARKED_WINGS 0x400
-#define ALL_NEEDS 0xFF0
+#define NEED_PART_STAR 0x800
+#define NEED_FULL_STAR 0x1000
+#define ALL_NEEDS 0xFFF0
 /* flags bit for orientability */
 #define ORIENTABLE_METHOD 0x10000
-/*#define TORUS_MODULO_MUNGE 0x4000 also used in flags */
+/*#define TORUS_MODULO_MUNGE 0x40000 also used in flags */
+
 /* bits for spec_flags */
 #define NOSPEC          0
 #define SPEC_SCALAR    0x0001
@@ -158,6 +169,7 @@ struct method_instance {
   REAL oldvalue;    /* value for restore_coords() */
   REAL newvalue;     /* accumulating value of this instance */
   REAL abstotal;     /* total of abs values of elements */
+  REAL value_addends[MAXADDENDS];  /* for binary tree addition */
 #ifdef SHARED_MEMORY
   REAL procvalue[MAXPROCS]; /* separate multiprocessor values */
   REAL procabstotal[MAXPROCS]; /* separate multiprocessor values */
@@ -168,24 +180,34 @@ struct method_instance {
   REAL parameter_1;  /* parameter particular to this instance */
   vertex_id *vlist; /* for use in compound Hessian */
   long timestamp;  /* when value last calculated */
+#ifdef PROF_START
+  /* for profiling execution time */
+  int    value_call_count;
+  int    grad_call_count;
+  int    hess_call_count;
+  double value_elapsed_time;  /* in seconds */
+  double grad_elapsed_time;
+  double hess_elapsed_time;
+#endif
+  
 /*  struct method_instance *next; */  /* linked list */
   };
 /* flags, (avoid quantity flag bits) */
-#define GLOBAL_INST  0x10000
-#define IMPLICIT_INSTANCE 0x20000
-#define DEFAULT_INSTANCE 0x40000
-#define METH_PARAMETER_1 0x80000
-#define BODY_INSTANCE    0x100000
-#define IGNORE_CONSTR    0x200000
-#define FAKE_IMPLICIT    0x400000
-#define IGNORE_FIXED    0x800000
-#define ELEMENT_MODULUS_FLAG 0x1000000
-#define USE_DENSITY     0x2000000
+#define GLOBAL_INST  0x100000
+#define IMPLICIT_INSTANCE 0x200000
+#define DEFAULT_INSTANCE 0x400000
+#define METH_PARAMETER_1 0x800000
+#define BODY_INSTANCE    0x1000000
+#define IGNORE_CONSTR    0x2000000
+#define FAKE_IMPLICIT    0x4000000
+#define IGNORE_FIXED    0x8000000
+#define ELEMENT_MODULUS_FLAG 0x10000000
+#define USE_DENSITY     0x20000000
 /* start for self_id, to distinguish from bodies and quantities */
 #define METHBASE 0x10000
 
 #define METH_INST ((struct method_instance *)(dymem + dy_meth_inst))
-#define METH_INSTANCE(n)  (meth_inst_list[n])
+#define METH_INSTANCE(n)  (meth_inst_list[abs(n)])
 #define dy_meth_inst web.dy_meth_inst_w
 extern struct method_instance *Meth_inst;
 #define meth_inst_alloc web.meth_inst_alloc_w  /* number allocated */
@@ -193,7 +215,6 @@ extern struct method_instance *Meth_inst;
 #define LOW_INST 1
 
 #define dy_gen_quants web.dy_gen_quants_w
-#define GEN_QUANTS ((struct gen_quant *)(dymem + dy_gen_quants))
 #define GEN_QUANT(n)  (gen_quant_list[n])
 extern struct gen_quant *Gen_quants;
 #define gen_quant_count web.gen_quant_count_w /* used */
@@ -271,6 +292,19 @@ extern REAL circular_arc_area_value(QINFO);
 extern REAL circular_arc_area_grad(QINFO);
 extern REAL circular_arc_area_hess(QINFO);
 
+extern void spherical_arc_length_init ARGS((QINT,struct method_instance*));
+extern REAL spherical_arc_length_value(QINFO);
+extern REAL spherical_arc_length_grad(QINFO);
+extern REAL spherical_arc_length_hess(QINFO);
+
+extern void spherical_arc_area_init ARGS((QINT,struct method_instance*));
+extern REAL spherical_arc_area_n_value(QINFO);
+extern REAL spherical_arc_area_n_grad(QINFO);
+extern REAL spherical_arc_area_n_hess(QINFO);
+extern REAL spherical_arc_area_s_value(QINFO);
+extern REAL spherical_arc_area_s_grad(QINFO);
+extern REAL spherical_arc_area_s_hess(QINFO);
+
 
 extern REAL q_edge_area(QINFO);
 extern REAL q_edge_area_grad(QINFO);
@@ -524,6 +558,12 @@ extern REAL star_sqcurve_method_value(QINFO);
 extern REAL star_sqcurve_method_grad(QINFO);
 extern REAL star_sqcurve_method_hess(QINFO);
 
+extern void stokes2d_init ARGS((QINT,struct method_instance*));
+extern REAL stokes2d_value(QINFO);
+extern REAL stokes2d_grad(QINFO);
+extern REAL stokes2d_hess(QINFO);
+extern REAL stokes2d_laplacian(QINFO);
+
 extern void hooke_energy_init ARGS((QINT,struct method_instance*));
 extern REAL hooke_energy(QINFO);
 extern REAL hooke_energy_gradient(QINFO);
@@ -577,6 +617,16 @@ extern REAL relaxed_elastic2_A_energy(QINFO);
 extern REAL relaxed_elastic2_A_gradient(QINFO);
 extern REAL relaxed_elastic2_A_hessian(QINFO);
 
+extern void SVK_init ARGS((QINT,struct method_instance*));
+extern REAL SVK_energy(QINFO);
+extern REAL SVK_gradient(QINFO);
+extern REAL SVK_hessian(QINFO);
+
+extern void Neo_Hookean_init ARGS((QINT,struct method_instance*));
+extern REAL Neo_Hookean_energy(QINFO);
+extern REAL Neo_Hookean_gradient(QINFO);
+extern REAL Neo_Hookean_hessian(QINFO);
+
 extern void knot_energy_init ARGS((QINT,struct method_instance*));
 
 extern void knot_power_init ARGS((QINT,struct method_instance*));
