@@ -18,8 +18,19 @@
  *           another composes two elements, another takes inverses,
  *           and the last pulls back a one-form under the group action.
  *
+ *           NOTE: The compose function needs special care in the
+ *           order of composition for nonabelian groups.  The
+ *           compose function is called to find the net wrap along
+ *           consecutive edges, with the first edge wrap being the
+ *           first argument.  If the first edge goes from U to V
+ *           with wrap A, and the second goes from V to W with wrap B,
+ *           then the wrapped edges are (U,AV) and (V,BW), so
+ *           the wrapped triplet is (U,AV,ABW).  So compose(A,B)
+ *           should return AB.
+ *
  *************************************************************/
 
+
  /*************************************************************
  *     Symmetry group of rotations in the xy-plane.
  *       Uses parameter "rotation_order" for the order of the group.
@@ -92,7 +103,7 @@ int genpower_var = -1;  /* reset at start of surface */
    genpower = (int)globals(genpower_var)->value.real;
 
    rotmat[0][0] = rotmat[1][1] = cos(2*M_PI*genpower/rotorder);
-   rotmat[0][1] = -(rotmat[1][0] = sin(2*M_PI*genpower/rotorder));
+   rotmat[1][0] = -(rotmat[0][1] = sin(2*M_PI*genpower/rotorder));
  }
 
  /* called by recalc() even if not using symmetry, hence don't set user var */
@@ -354,19 +365,22 @@ int perm, signs;
  {
    int flip1,flip2,rot;
    flip1=perm1&4; flip2=perm2&4;
-   rot = (perm2&3) + (flip1?-1:1)*(perm1&3);
+   rot = (perm2&3) + (flip2?-1:1)*(perm1&3);
    rot = (rot<0)? rot+3 : ( (rot>2)? rot-3:rot );
    return (flip1^flip2) + rot;
  }
 
- WRAPTYPE pgcube_compose(wrap2,wrap1) /* note reverse order */
+ WRAPTYPE pgcube_compose(wrap1,wrap2) 
  WRAPTYPE wrap1,wrap2;  /* the elements to compose */
  {
-   int sgns1,sgns2,perm1,perm2;
+   int sgns1,sgns2,perm1,perm2,comp,inv,xxx,yyy;
    sgns1 = pgcube_sgns(wrap1); perm1 = pgcube_perm(wrap1);
    sgns2 = pgcube_sgns(wrap2); perm2 = pgcube_perm(wrap2);
-   return pgcube_wr(sgns1 ^ pgcube_permutesigns(sgns2,pgcube_invperm(perm1)),
-   pgcube_compperm(perm1,perm2));
+   comp = pgcube_compperm(perm1,perm2); 
+   inv = pgcube_invperm(perm1);
+   xxx =  pgcube_permutesigns(sgns2,inv);
+   yyy = pgcube_wr(sgns1 ^ xxx,comp);
+   return yyy;
  }
 
  /********************************************************************
@@ -381,9 +395,12 @@ int perm, signs;
  WRAPTYPE wrap;  /* the element to invert */
  {
    int sgns, perm;
+   WRAPTYPE result;
+   
    sgns = pgcube_sgns(wrap); perm = pgcube_perm(wrap);
-   return pgcube_wr(pgcube_permutesigns(sgns,perm),
+   result = pgcube_wr(pgcube_permutesigns(sgns,perm),
             pgcube_invperm(perm));
+   return result;
  }
 
 
@@ -473,14 +490,14 @@ int perm, signs;
    return wrap;
  }
 
- WRAPTYPE cubel_compose(wrap2,wrap1)  /* note reverse order: wrap2 after wrap1 */
+ WRAPTYPE cubel_compose(wrap1,wrap2) 
  WRAPTYPE wrap1,wrap2;  /* the elements to compose */
  {
    int pg1, pg2, tr1, tr2;
    pg1 = cubel_pg(wrap1); tr1 = cubel_tr(wrap1);
    pg2 = cubel_pg(wrap2); tr2 = cubel_tr(wrap2);
-   return cubel_wr( pg_tr_compose(pg2,tr1)+tr2,
-           pgcube_compose(pg2,pg1) ) ;
+   return cubel_wr( pg_tr_compose(pg1,tr2)+tr1,
+           pgcube_compose(pg1,pg2) ) ;
  }
 
  WRAPTYPE cubel_inverse(wrap)
