@@ -18,11 +18,24 @@
 #include "inline.h"
 #endif
 
-/* extra attribute type sizes */
-int attr_type_size[NUMATTRTYPES] = 
+/* extra attribute type sizes, matching order in skeleton.h */
+
+int datatype_size[NUMDATATYPES] = 
   {0,sizeof(REAL),sizeof(int),sizeof(unsigned long), sizeof(unsigned char),
-    sizeof(unsigned int)};
-char *attr_type_name[NUMATTRTYPES] = { NULL,"real","integer","ulong","uchar","uint"};
+   sizeof(unsigned short), sizeof(unsigned int),
+   sizeof(long int),sizeof(char),sizeof(short),0,sizeof(char*),sizeof(char*),
+   sizeof(vertex_id),sizeof(edge_id),sizeof(facet_id),sizeof(body_id),
+   sizeof(facetedge_id),sizeof(element_id),sizeof(int),sizeof(int),sizeof(int),sizeof(int),
+   sizeof(int)
+  };
+
+char *datatype_name[NUMDATATYPES] =
+  { " ","real","integer","ulong_int","uchar","ushort_int","uint",
+    "long_int","char","short_int"," ","string","pointer", "vertex_type",
+    "edge_type","facet_type","body_type","facetedge_type", "element_id",
+    "boundary_type","constraint_type","quantity_type","method_instance_type",
+    "procedure"
+  };
 
 /* quadratic interpolation coefficients */
 /* partials of interpolation polynomials at midpoints of patch edges */
@@ -41,7 +54,7 @@ REAL dip[FACET_EDGES][FACET_CTRL][2] = {
 
 int Ord(id)    /* useful in debugger */
 element_id id;
-{ return ordinal(id); }
+{ return loc_ordinal(id); }
 
 void vloop ARGS((vertex_id));
 
@@ -64,42 +77,128 @@ element_id v_id;
 
 void set_facet_body(f_id,b_id)
 facet_id f_id;
-body_id  b_id;
-{
+body_id  b_id;  /* may be invalid for unsetting */
+{ body_id bb_id;
+  
   if ( web.skel[BODY].count == 0 ) return;
   if ( !valid_id(f_id) ) return;
 
-  if ( !valid_id(get_body_fe(b_id)) )
-    set_body_fe(b_id,get_facet_fe(f_id));
+  bb_id = get_facet_body(f_id);
+  if ( equal_id(bb_id,b_id) ) return;
+
+  /* check if old body of this facet links to this facet */
+  if ( valid_id(bb_id) )
+  { facetedge_id fe = get_body_fe(bb_id),ffe;
+    facet_id ff_id;
+    facet_id next_f,prev_f;
+    
+    if ( valid_id(fe) ) 
+    { ff_id = get_fe_facet(fe);
+      if ( equal_id(f_id,ff_id) )  /* need to give body new link */
+      { 
+        prev_f = get_prev_body_facet(f_id);
+        if ( equal_id(prev_f,f_id) || !valid_id(prev_f) ||
+         !equal_id(bb_id,get_facet_body(prev_f))  )
+            set_body_fe(bb_id,NULLID);
+        else
+        { ffe = get_facet_fe(prev_f);
+          set_body_fe(bb_id,ffe);
+        }
+      }
+     
+      /* remove from old body facet list */
+      next_f = get_next_body_facet(f_id);
+      prev_f = get_prev_body_facet(f_id);
+      set_next_body_facet(prev_f,next_f);
+      set_prev_body_facet(next_f,prev_f);
+    }
+  }
+ 
+
+  /* insert in new body list */
+  if ( valid_id(b_id) )
+  { facetedge_id fe = get_body_fe(b_id);
+    if ( valid_id(fe) )
+    { facet_id nextf = get_fe_facet(fe);
+      facet_id prevf = get_prev_body_facet(nextf);
+      set_next_body_facet(f_id,nextf);
+      set_prev_body_facet(nextf,f_id);
+      set_next_body_facet(prevf,f_id);
+      set_prev_body_facet(f_id,prevf);
+    }
+    else /* first facet for body */
+    { 
+      set_next_body_facet(f_id,f_id);
+      set_prev_body_facet(f_id,f_id);
+      set_body_fe(b_id,get_facet_fe(f_id));
+    }
+  }
 
   if ( everything_quantities_flag )
-  { 
+  {
      if ( inverted(f_id) )  /* back facet */
      { 
-        body_id bb_id = FULONG(f_id,F_BODY_LIST_ATTR)[1];
+        body_id bb_id = F_ELID(f_id,F_BODY_LIST_ATTR)[1];
         if ( equal_id(bb_id,b_id) ) return;
-        if ( valid_id(bb_id) )
-          unapply_method(f_id,get_body_volmeth(bb_id)); 
-        FULONG(f_id,F_BODY_LIST_ATTR)[1] = b_id;
+        if ( valid_id(bb_id) ) /* cancel out if already there */
+          apply_method_num(inverse_id(f_id),get_body_volmeth(bb_id)); 
+        F_ELID(f_id,F_BODY_LIST_ATTR)[1] = b_id;
         if ( valid_id(b_id) )
           apply_method_num(f_id,get_body_volmeth(b_id)); 
      }
      else /* front facet */
      { 
-        body_id bb_id = FULONG(f_id,F_BODY_LIST_ATTR)[0];
+        body_id bb_id = F_ELID(f_id,F_BODY_LIST_ATTR)[0];
         if ( equal_id(bb_id,b_id) ) return;
         if ( valid_id(bb_id) )
-          unapply_method(f_id,get_body_volmeth(bb_id)); 
-        FULONG(f_id,F_BODY_LIST_ATTR)[0] = b_id;
+          apply_method_num(inverse_id(f_id),get_body_volmeth(bb_id)); 
+        F_ELID(f_id,F_BODY_LIST_ATTR)[0] = b_id;
         if ( valid_id(b_id) )
           apply_method_num(f_id,get_body_volmeth(b_id)); 
      }
+     /* and content integrands on edges */
+     if ( web.representation == SOAPFILM )
+     { facetedge_id fe = get_facet_fe(f_id);
+       edge_id e_id;
+       int i;
+       unsigned int k;
+       conmap_t *map;
+       char name[100];
+
+       for ( i = 0 ; i < FACET_EDGES ; i++, fe = get_next_edge(fe))
+       { e_id = get_fe_edge(fe);
+         if ( !get_eattr(e_id) & BDRY_CONTENT ) continue;
+         map = get_e_constraint_map(e_id);
+         for ( k = 1 ; k <= map[0] ; k++ )
+         { struct constraint *con = get_constraint(map[k]);
+           if ( con->attr & CON_CONTENT )
+           { if ( valid_id(bb_id) )
+             { if ( con->attr & NAMED_THING )
+                 sprintf(name,"body_%d_%s_meth",ordinal(bb_id)+1,con->name);
+               else
+                 sprintf(name,"body_%d_con_%d_meth",ordinal(bb_id)+1,map[k]);
+               apply_method(inverse_id(e_id),name); /* cancel */
+             }
+             if ( valid_id(b_id) )
+             { if ( con->attr & NAMED_THING ) 
+                 sprintf(name,"body_%d_%s_meth",ordinal(b_id)+1,con->name);
+               else 
+                 sprintf(name,"body_%d_con_%d_meth",ordinal(b_id)+1,map[k]);
+               apply_method(e_id,name);
+             }
+           }
+         }
+       }
+     }
   }
   else
   {
-     if ( inverted(f_id) )  FULONG(f_id,F_BODY_LIST_ATTR)[1] = b_id;
-     else  FULONG(f_id,F_BODY_LIST_ATTR)[0] = b_id;
+     if ( inverted(f_id) )  
+       F_ELID(f_id,F_BODY_LIST_ATTR)[1] = b_id;
+     else  
+       F_ELID(f_id,F_BODY_LIST_ATTR)[0] = b_id;
   }
+  top_timestamp = ++global_timestamp;
 
 }
 /**************************************************************************
@@ -123,6 +222,7 @@ facetedge_id fe;
     if ( valid_id(b_id) )
        set_body_fe(b_id,inverse_id(fe));
   }
+  top_timestamp = ++global_timestamp;
 }
 
 
@@ -193,13 +293,13 @@ int get_vertex_fvalence(v_id) vertex_id v_id;
 facet_id get_next_vertex_facet(v_id,f_id) vertex_id v_id; facet_id f_id;
 { int i; 
   vertex_id *v = get_facet_vertices(f_id);
-  facet_id *f = FULONG(f_id,F_NEXT_VFACET_ATTR);
+  facet_id *f = F_ELID(f_id,F_NEXT_VFACET_ATTR);
   int vmax = web.skel[FACET].ctrlpts; 
 
   for ( i = 0 ; i < vmax ; i++ )
      if (equal_id(v_id,v[i]))  return f[i] ; 
-  sprintf(errmsg,"Internal error: get_next_vertex_facet failure v %d f %d\n",ordinal(v_id)+1,
-     ordinal(f_id)+1);
+  sprintf(errmsg,"Internal error: get_next_vertex_facet failure v %s f %s\n",
+      ELNAME(v_id), ELNAME1(f_id));
   kb_error(1306,errmsg,RECOVERABLE);
 
   return NULLID;
@@ -210,7 +310,7 @@ void set_next_vertex_facet(v_id,f_id,ff_id)
       vertex_id v_id; facet_id f_id,ff_id;
 { 
    vertex_id *v = get_facet_vertices(f_id);
-   facet_id *f = FULONG(f_id,F_NEXT_VFACET_ATTR);
+   facet_id *f = F_ELID(f_id,F_NEXT_VFACET_ATTR);
    int vmax = web.skel[FACET].ctrlpts; 
    int i;
    for ( i = 0 ; i < vmax ; i++ )
@@ -222,43 +322,75 @@ void set_next_vertex_facet(v_id,f_id,ff_id)
 }
 
 void set_next_body_facet(f_id,ff_id)  facet_id f_id,ff_id;
-  { FULONG(f_id,F_NEXT_BFACET_ATTR)[inverted(f_id)?1:0] = (ff_id) ; }
+  { F_ELID(f_id,F_NEXT_BFACET_ATTR)[inverted(f_id)?1:0] = (ff_id) ; }
+
+void set_prev_body_facet(f_id,ff_id)  facet_id f_id,ff_id;
+  { F_ELID(f_id,F_NEXT_BFACET_ATTR)[inverted(f_id)?3:2] = (ff_id) ; }
 
 facet_id get_next_body_facet(f_id) facet_id f_id;
-{ return    FULONG(f_id,F_NEXT_BFACET_ATTR)[inverted(f_id)?1:0]; }
+{ return    F_ELID(f_id,F_NEXT_BFACET_ATTR)[inverted(f_id)?1:0]; }
 
-void set_body_volume(b_id,v)  body_id b_id; REAL v;
-{ if ( !valid_id(b_id) ) return;
-  bptr(b_id)->volume = v; 
-  bptr(b_id)->abstotal = fabs(v); 
-  bptr(b_id)->voltimestamp = global_timestamp;
+facet_id get_prev_body_facet(f_id) facet_id f_id;
+{ return    F_ELID(f_id,F_NEXT_BFACET_ATTR)[inverted(f_id)?3:2]; }
+
+void set_body_volume(b_id,v,mode)  
+body_id b_id; 
+REAL v;
+int mode; /* NOSETSTAMP or SETSTAMP */
+{ struct body *b = bptr(b_id);
+  if ( !valid_id(b_id) ) return;
+  b->volume = v; 
+  b->abstotal = fabs(v); 
+  if ( mode == SETSTAMP )
+    b->voltimestamp = global_timestamp;
+/*
   if ( web.representation == STRING)
   { facet_id f_id = get_fe_facet(get_body_fe(b_id));
     if ( valid_id(f_id) ) set_facet_area(f_id,v);
   }
+*/
   if ( everything_quantities_flag )
-  { struct gen_quant *q = GEN_QUANT(get_body_volquant(b_id));
+  { struct gen_quant *q = GEN_QUANT(b->volquant);
     q->value = v;
   }
 } 
 
-void add_body_volume(b_id,v)  body_id b_id; REAL v;
+void add_body_volume(b_id,v)  /* uses binary tree add */ 
+body_id b_id; REAL v;
 { struct body *b;
   if ( !valid_id(b_id) ) return;
   b = bptr(b_id);
-  b->volume += v; 
+  binary_tree_add(b->volume_addends,v);
+  b->abstotal += fabs(v); 
+}
+
+void add_body_volume_plain(b_id,v)  /* with updates of related quantities */
+body_id b_id; REAL v;
+{ struct body *b;
+  if ( !valid_id(b_id) ) return;
+  b = bptr(b_id);
+  b->volume += v;
   b->abstotal += fabs(v); 
   b->voltimestamp = global_timestamp;
+/*
   if ( web.representation == STRING)
   { facet_id f_id = get_fe_facet(b->fe_id);
     if ( valid_id(f_id) ) set_facet_area(f_id,b->volume);
   }
+*/
   if ( everything_quantities_flag )
   { struct gen_quant *q = GEN_QUANT(b->volquant);
     q->value = b->volume;
   }
 } 
 
+void add_body_abstotal(b_id,v)
+body_id b_id; REAL v;
+{ struct body *b;
+  if ( !valid_id(b_id) ) return;
+  b = bptr(b_id);
+  b->abstotal += fabs(v); 
+}
 
 void set_body_fixvol(b_id,v) 
 body_id b_id; 
@@ -280,7 +412,7 @@ REAL v;
     }
   }
   else
-  { sprintf(errmsg,"fix body volume: illegal body %d.\n",ordinal(b_id)+1);
+  { sprintf(errmsg,"fix body volume: illegal body %s.\n",ELNAME(b_id));
     kb_error(1307,errmsg,RECOVERABLE);
   }
 }
@@ -313,12 +445,14 @@ vertex_id dup_vertex(old_v)
 vertex_id old_v;
 { 
   vertex_id v_id;
+  struct vertex *v_id_p;
 
   v_id = new_element(VERTEX,NULLID);
-  memcpy((char *)&(elptr(v_id)->attr),(char *)&(elptr(old_v)->attr),
-              web.sizes[VERTEX] - 2*sizeof(element_id));
-  elptr(v_id)->self_id = v_id;  /* restore new id */
-  vptr(v_id)->e_id = NULLID;
+  v_id_p = vptr(v_id);
+  memcpy((char *)&(v_id_p->attr),(char *)&(elptr(old_v)->attr),
+              web.sizes[VERTEX] - ((char*)&(v_id_p->attr)-(char*)v_id_p));
+  v_id_p->self_id = v_id;  /* restore new id */
+  v_id_p->e_id = NULLID;
   return v_id;
 }
 
@@ -369,24 +503,28 @@ edge_id dup_edge(old_e)
 edge_id old_e;
 { 
   edge_id e_id;
-  vertex_id newmid;
+  struct edge *e_id_p,*old_e_p;
+  vertex_id newmid; 
 
   e_id = new_element(EDGE,NULLID);
-  memcpy((char *)&(elptr(e_id)->attr),(char *)&(elptr(old_e)->attr),
-              web.sizes[EDGE] - 2*sizeof(element_id));
-  elptr(e_id)->self_id = e_id; /* restore new id */
-  eptr(e_id)->next_vedge[0] = NULLID;
-  eptr(e_id)->next_vedge[1] = NULLID;
+  e_id_p = eptr(e_id);
+  old_e_p = eptr(old_e);
+  memcpy((char *)&(e_id_p->attr),(char *)&(old_e_p->attr),
+              web.sizes[EDGE] - ((char*)&(e_id_p->attr)-(char*)e_id_p));
+  e_id_p->self_id = e_id; /* restore new id */
+  e_id_p->next_vedge[0] = NULLID;
+  e_id_p->next_vedge[1] = NULLID;
   if ( web.modeltype == QUADRATIC )
-  { newmid = dup_vertex(get_edge_midv(old_e));
+  { newmid = new_vertex(NULL,old_e);
     set_edge_midv(e_id,newmid);
+	set_attr(newmid,Q_MIDPOINT);
+	set_vertex_edge(newmid,e_id);
   } 
   else if ( web.modeltype == LAGRANGE )
   { int i;
     vertex_id *v = get_edge_vertices(e_id);
-    vertex_id *oldv = get_edge_vertices(old_e);
     for ( i = 1 ; i < web.lagrange_order ; i++ )
-       v[i] = dup_vertex(oldv[i]);
+      v[i] = new_vertex(NULL,e_id);
   } 
   if ( inverted(old_e) ) return inverse_id(e_id);
   return e_id;
@@ -406,9 +544,6 @@ facet_id  new_facet ()
   facet_id f_id;
 
   f_id = new_element(FACET,NULLID);
-  set_facet_body(f_id,NULLBODY);
-  set_facet_body(inverse_id(f_id),NULLBODY);
-  set_facet_fe(f_id,NULLFACETEDGE);
   set_facet_color(f_id,DEFAULT_FACET_COLOR);
   set_facet_density(f_id,1.0);
   return f_id;
@@ -418,20 +553,43 @@ facet_id dup_facet(old_f)
 facet_id old_f;
 { 
   facet_id f_id;
+  struct facet *f_id_p;
+  body_id b_id,*fb;
+  int sign = inverted(old_f);
 
+  old_f = positive_id(old_f);  /* since new facet id will be positive */
+  
   f_id = new_element(FACET,NULLID);
-  memcpy((char *)&(elptr(f_id)->attr),(char *)&(elptr(old_f)->attr),
-              web.sizes[FACET] - 2*sizeof(element_id));
+  f_id_p = fptr(f_id);
+  memcpy((char *)&(f_id_p->attr),(char *)&(elptr(old_f)->attr),
+              web.sizes[FACET] - ((char*)&(f_id_p->attr)-(char*)f_id_p));
   set_attr(f_id,NEWELEMENT);
-  elptr(f_id)->self_id = f_id; /* restore new id */
-  return f_id;
+  f_id_p->self_id = f_id; /* restore new id */
+
+  /* update body facet lists */
+  if ( web.skel[BODY].count )
+  { fb = F_ELID(f_id,F_BODY_LIST_ATTR);
+    fb[0] = fb[1] = NULLID; /* so set_facet_body works */
+    fb = F_ELID(f_id,F_NEXT_BFACET_ATTR);
+    fb[0] = fb[1] = fb[2] = fb[3] = NULLID;
+    b_id = get_facet_body(old_f);
+    if ( valid_id(b_id) )
+      set_facet_body(f_id,b_id);
+    b_id = get_facet_body(inverse_id(old_f));
+    if ( valid_id(b_id) )
+      set_facet_body(inverse_id(f_id),b_id);
+  }
+
+  return sign ? inverse_id(f_id) : f_id;
 }
 
 body_id new_body()
 { int two = 2;
+  int four = 4;
   body_id b_id;
 
   expand_attribute(FACET,F_BODY_LIST_ATTR,&two);
+  expand_attribute(FACET,F_NEXT_BFACET_ATTR,&four);
   b_id = new_element(BODY,NULLID);
   set_body_fe(b_id,NULLFACETEDGE);
   if ( everything_quantities_flag ) 
@@ -444,11 +602,13 @@ body_id dup_body(old_b)
 body_id old_b;
 { 
   body_id b_id;
+  struct body *b_id_p;
 
   b_id = new_element(BODY,NULLID);
-  memcpy((char *)&(elptr(b_id)->attr),(char *)&(elptr(old_b)->attr),
-              web.sizes[BODY] - 2*sizeof(element_id));
-  elptr(b_id)->self_id = b_id; /* restore new id */
+  b_id_p = bptr(b_id);
+  memcpy((char *)&(b_id_p->attr),(char *)&(elptr(old_b)->attr),
+              web.sizes[BODY] - ((char*)&(b_id_p->attr)-(char*)b_id_p));
+  b_id_p->self_id = b_id; /* restore new id */
   web.bodycount++;
   return b_id;
 }
@@ -467,12 +627,14 @@ edge_id e_id;
   set_next_edge(fe_id,NULLFACETEDGE);
   set_prev_facet(fe_id,NULLFACETEDGE);      
   set_prev_facet(fe_id,NULLFACETEDGE);
-  tailv = get_edge_tailv(e_id);
+  #ifndef MPI_EVOLVER
+  tailv = get_edge_tailv(e_id);  
+  headv = get_edge_headv(e_id);
   if ( !valid_id(get_vertex_edge(tailv)) )
     set_vertex_edge(tailv,e_id);
-  headv = get_edge_headv(e_id);
   if ( !valid_id(get_vertex_edge(headv)) )
     set_vertex_edge(headv,e_id);
+  #endif
 
   if ( web.representation==STRING && everything_quantities_flag )
   { /* attach volume quantities */
@@ -495,6 +657,45 @@ edge_id e_id;
   return fe_id;
 }
 
+
+void set_fe_facet(fe_id,f_id)
+facetedge_id fe_id;
+facet_id f_id;
+{ facet_id oldf = get_fe_facet(fe_id);
+
+  feptr(fe_id)->fe_facet_id = inverted(fe_id) ? inverse_id(f_id) : f_id;
+
+  if ( web.representation==STRING && everything_quantities_flag &&
+        !equal_id(oldf,f_id) )
+  { /* detach old volume quantity and attach new volume quantity */
+     body_id b_id, old_b_id;
+     edge_id e_id = get_fe_edge(fe_id);
+     b_id = get_facet_body(f_id);
+
+     old_b_id = valid_id(oldf) ? get_facet_body(oldf) : NULLID;
+     if ( valid_id(old_b_id) )
+        unapply_method(e_id,get_body_volmeth(old_b_id));
+     old_b_id = valid_id(oldf) ? get_facet_body(inverse_id(oldf)) : NULLID;
+     if ( valid_id(old_b_id) )
+        unapply_method(e_id,get_body_volmeth(old_b_id));
+              
+              
+     if ( valid_id(b_id) )
+     { if ( same_sign(f_id,e_id) )
+          apply_method_num(e_id,get_body_volmeth(b_id));
+        else
+          apply_method_num(inverse_id(e_id),get_body_volmeth(b_id));
+     }
+     b_id = get_facet_body(inverse_id(f_id));
+     if ( valid_id(b_id) )
+     { if ( same_sign(f_id,e_id) )
+          apply_method_num(inverse_id(e_id),get_body_volmeth(b_id));
+        else
+          apply_method_num(e_id,get_body_volmeth(b_id));
+     }
+  }
+}
+
 REAL get_edge_length(e_id)
 edge_id e_id;
 {
@@ -510,73 +711,123 @@ facet_id f_id;
 }
 
 
+/********************************************************************
+*
+* Function: compare_vertex_attr(va,vb)
+* 
+* Purpose: Compare constraints of vertex and vertex.
+*
+* Return value: 0 INCOMPARABLE  Incomparable
+*               1 A_SUB_B Vertex proper subset of vertex
+*               2 A_EQ_B Vertex equal vertex
+*               3 A_SUPER_B Vertex superset of vertex
+*/
+
+int compare_vertex_attr(va,vb)
+vertex_id va,vb;
+{ int i,j;
+  conmap_t * con1,*con2;
+  int same = 0;
+
+  /* check constraints */
+  con1 = get_v_constraint_map(va);
+  con2 = get_v_constraint_map(vb);
+
+  for ( i = 1 ; i <= (int)con2[0] ; i++ )
+  { for ( j = 1 ; j <= (int)con1[0] ; j++ )
+      if ( (con1[j]&CONMASK) == (con2[i]&CONMASK) )
+      { same++;
+        break;
+      }
+  }
+
+  if ( (same==(int)con1[0]) && (same==(int)con2[0]) )
+    return A_EQ_B;
+  if ( (same==(int)con1[0]) && (same < (int)con2[0]) )
+    return A_SUB_B;
+  if ( (same < (int)con1[0]) && (same==(int)con2[0]) )
+    return A_SUPER_B;
+  return INCOMPARABLE;
+}
+
+/********************************************************************
+*
+* Function: compare_vertex_edge_attr(va,eb)
+* 
+* Purpose: Compare constraints of vertex and edge.
+*
+* Return value: 0 INCOMPARABLE  Incomparable
+*               1 A_SUB_B Vertex proper subset of edge
+*               2 A_EQ_B Vertex equal edge
+*               3 A_SUPER_B Vertex superset of edge
+*/
+
+int compare_vertex_edge_attr(va,eb)
+vertex_id va;
+edge_id eb;
+{ int i,j;
+  conmap_t * con1,*con2;
+  int same = 0;
+
+  /* check constraints */
+  con1 = get_v_constraint_map(va);
+  con2 = get_e_constraint_map(eb);
+
+  for ( i = 1 ; i <= (int)con2[0] ; i++ )
+  { for ( j = 1 ; j <= (int)con1[0] ; j++ )
+      if ( (con1[j]&CONMASK) == (con2[i]&CONMASK) )
+      { same++;
+        break;
+      }
+  }
+
+  if ( (same==(int)con1[0]) && (same==(int)con2[0]) )
+    return A_EQ_B;
+  if ( (same==(int)con1[0]) && (same < (int)con2[0]) )
+    return A_SUB_B;
+  if ( (same < (int)con1[0]) && (same==(int)con2[0]) )
+    return A_SUPER_B;
+  return INCOMPARABLE;
+
+}
+
 /********************************************************************
 *
 * Function: compare_edge_attr(ea,eb)
 * 
-* Purpose: See if edge eb can be merged with edge ea.
-*          Used in eliminate_edge().  Checks constraints
-*          and named methods. Those of edge eb must be
-*          a subset of those of edge ea.
+* Purpose: Compare constraints of edge and edge.
 *
-* Return value: 0  Cannot
-*               1  Can
+* Return value: 0 INCOMPARABLE  Incomparable
+*               1 A_SUB_B edge proper subset of edge
+*               2 A_EQ_B edge equal edge
+*               3 A_SUPER_B edge superset of edge
 */
 
 int compare_edge_attr(ea,eb)
 edge_id ea,eb;
 { int i,j;
   conmap_t * con1,*con2;
-  int meth_offset = get_meth_offset(EDGE); 
-  struct edge *ea_ptr,*eb_ptr;
-
+  int same = 0;
+  
   /* check constraints */
   con1 = get_e_constraint_map(ea);
   con2 = get_e_constraint_map(eb);
 
-  if ( con2[0] > con1[0] )
-  { if ( verbose_flag ) 
-    { sprintf(msg,"Edge %d has more constraints than edge %d.\n",
-      ordinal(eb)+1,ordinal(ea)+1);
-     outstring(msg);
-    }
-    return 0;
-  }
   for ( i = 1 ; i <= (int)con2[0] ; i++ )
-  { for ( j = 1 ; j <= (int)con1[0] ; j++ )
-      if ( (con1[j]&CONMASK) == (con2[i]&CONMASK) ) break;
-    if ( j > (int)con1[0] )
-    { if ( verbose_flag )
-      { sprintf(msg,"Edge %d is on constraint %s, but edge %d isn't.\n",
-        ordinal(eb)+1,get_constraint((int)con2[i])->name,ordinal(ea)+1);
-        outstring(msg);
-      }          
-      return 0;
-    }
+  { for ( j = 1 ; j <= (int)con1[0] ; j++ ) 
+      if ( (con1[j]&CONMASK) == (con2[i]&CONMASK) )
+      { same++;
+        break;
+      }
   }
 
-  /* check methods */
-  ea_ptr = eptr(ea);
-  eb_ptr = eptr(eb);
-  for ( i = 0 ; i < (int)eb_ptr->method_count ; i++ )
-  { int m;
-    m = abs(((int*)((char*)eb_ptr+meth_offset))[i]);
-    for ( j = 0 ; j < (int)ea_ptr->method_count ; j++ )
-    { int n;
-      n = abs(((int*)((char*)ea_ptr+meth_offset))[j]);
-    if ( n == m ) break;
-    }
-    if ( j >= (int)ea_ptr->method_count )
-    {  if ( verbose_flag )
-       { sprintf(msg,"Edge %d is on method %s but edge %d isn't.\n",
-           ordinal(eb)+1,METH_INSTANCE(m)->name,ordinal(ea)+1);
-         outstring(msg);
-       }       
-       return 0;
-    }
-  }
-
-  return 1;
+  if ( (same==(int)con1[0]) && (same==(int)con2[0]) )
+    return A_EQ_B;
+  if ( (same==(int)con1[0]) && (same < (int)con2[0]) )
+    return A_SUB_B;
+  if ( (same < (int)con1[0]) && (same==(int)con2[0]) )
+    return A_SUPER_B;
+  return INCOMPARABLE;
 }
 
 /**************************************************************
@@ -643,13 +894,13 @@ element_id id1,id2;
 int add_attribute(e_type,name,attr_type,dim,dims,dumpflag,code)
 int e_type; /* VERTEX ,... */
 char *name;
-int attr_type; /* REAL_ATTR or INTEGER_ATTR or ULONG_ATTR */
+int attr_type; /* REAL_TYPE or INTEGER_TYPE or ULONG_TYPE etc */
 int dim; /* number of dimensions, 0 for scalar */
 int *dims; /* sizes of dimensions, NULL for all sizes 0 */
             /* Note: scalar still needs size of 0 or 1 */ 
 int dumpflag; /* whether appears in dump file */
 struct expnode *code; /* nonnull for function attribute */
-{ int newsize,newcount;
+{ int newsize,newcount=0;
   struct extra *ex;
   int oldsize;
 
@@ -664,11 +915,11 @@ struct expnode *code; /* nonnull for function attribute */
   /* get old used space, minus any padding, and pad to proper size */
   if ( web.skel[e_type].extra_count > 0 )
   { ex = EXTRAS(e_type) + web.skel[e_type].extra_count-1;
-    oldsize = ex->offset + ex->datacount*attr_type_size[ex->type];
+    oldsize = ex->offset + ex->datacount*datatype_size[ex->type];
   }
   else oldsize = web.sizes[e_type];
-  if  (oldsize % attr_type_size[attr_type])
-    oldsize += attr_type_size[attr_type] - (oldsize % attr_type_size[attr_type]);
+  if  (oldsize % datatype_size[attr_type])
+    oldsize += datatype_size[attr_type] - (oldsize % datatype_size[attr_type]);
 
   ex = EXTRAS(e_type) + web.skel[e_type].extra_count;
 
@@ -686,21 +937,12 @@ struct expnode *code; /* nonnull for function attribute */
     }
   }
   if ( (newcount > 0) || (oldsize > web.sizes[e_type]) )
-  { newsize =  newcount*attr_type_size[attr_type];
+  { newsize =  newcount*datatype_size[attr_type];
     expand(e_type,oldsize + newsize); 
   }
   strncpy(ex->name,name,ATTR_NAME_SIZE);
   ex->type = attr_type;
-  switch ( ex->type )
-  { case INTEGER_ATTR: ex->itemsize = sizeof(int); break;
-    case REAL_ATTR: ex->itemsize = sizeof(REAL); break;
-    case ULONG_ATTR: ex->itemsize = sizeof(unsigned long); break;
-    case UINT_ATTR: ex->itemsize = sizeof(unsigned int); break;
-    default:
-     sprintf(errmsg,"Internal: Unexpected attribute type: %d\n",ex->type);
-     kb_error(2831,errmsg,WARNING);
-     ex->itemsize = sizeof (REAL);
-  }
+  ex->itemsize = datatype_size[attr_type];
  
   ex->offset = oldsize;
   ex->datacount = newcount;
@@ -711,7 +953,7 @@ struct expnode *code; /* nonnull for function attribute */
 
   if ( stricmp(name,EXTRA_BDRY_NAME) == 0 )
   { extra_bdry_attr = web.skel[e_type].extra_count;
-    if ( ex->type != INTEGER_ATTR )
+    if ( ex->type != INTEGER_TYPE )
       kb_error(2842,"Attribute extra_boundary must be of type integer.\n",
          RECOVERABLE);
     if ( e_type != VERTEX )
@@ -720,8 +962,8 @@ struct expnode *code; /* nonnull for function attribute */
   }
   else if ( stricmp(name,EXTRA_BDRY_PARAM_NAME) == 0 )
   { extra_bdry_param_attr = web.skel[e_type].extra_count;
-    if ( ex->type != REAL_ATTR )
-      kb_error(2844,"Attribute extra_boundary_param must be of type real.\n",
+    if ( ex->type != REAL_TYPE )
+      kb_error(3369,"Attribute extra_boundary_param must be of type real.\n",
          RECOVERABLE);
     if ( e_type != VERTEX )
       kb_error(2845,
@@ -770,7 +1012,7 @@ int *newsizes; /* new numbers of components, even for scalar */
   if ( attr_num < web.skel[e_type].extra_count-1 )
          available = ex[1].offset - ex[0].offset;
   else available = web.sizes[e_type] - ex->offset;
-  needed = newsize*attr_type_size[ex->type];
+  needed = newsize*datatype_size[ex->type];
   if ( ex->adim >= 2 ) 
      temp = (char*)temp_calloc(needed,1);
   if ( needed > available )
@@ -778,7 +1020,7 @@ int *newsizes; /* new numbers of components, even for scalar */
     /* check alignment of following fields */
     diff = needed - available;
     for ( k = attr_num+1 ; k < web.skel[e_type].extra_count ; k++ )
-      while ( diff % attr_type_size[EXTRAS(e_type)[k].type] )
+      while ( diff % datatype_size[EXTRAS(e_type)[k].type] )
         diff++;
     expand(e_type,web.sizes[e_type] + diff); 
     /* move stuff above */
@@ -824,7 +1066,7 @@ skipentry:    ;
      /* check alignment of following fields */
      diff = available - needed;
      for ( k = attr_num+1 ; k < web.skel[e_type].extra_count ; k++ )
-        while ( diff % attr_type_size[EXTRAS(e_type)[k].type] )
+        while ( diff % datatype_size[EXTRAS(e_type)[k].type] )
           diff--;
      /* move stuff above */
      if ( attr_num < web.skel[e_type].extra_count-1 )
@@ -866,8 +1108,17 @@ skipentry2:    ;
   if ( ex->adim >= 2 ) temp_free(temp);
   ex->datacount = newsize;
   for ( n = 0 ; n < ex->adim ; n++ )
-	ex->sizes[n] = newsizes[n];
+    ex->sizes[n] = newsizes[n];
   parallel_update_flag[e_type] = 1;
+
+#ifdef MPI_EVOLVER
+  mpi_export_voffset = EXTRAS(VERTEX)[web.mpi_export_attr[VERTEX]].offset;
+  mpi_export_eoffset = EXTRAS(EDGE)[web.mpi_export_attr[EDGE]].offset;
+  mpi_export_foffset = EXTRAS(FACET)[web.mpi_export_attr[FACET]].offset;
+  mpi_export_boffset = EXTRAS(BODY)[web.mpi_export_attr[BODY]].offset;
+  mpi_export_feoffset =
+          EXTRAS(FACETEDGE)[web.mpi_export_attr[FACETEDGE]].offset; 
+#endif
 }
 
 /*************************************************************************
@@ -904,11 +1155,11 @@ int *etype; /* for returning element type */
   struct extra *ex;
 
   for ( el_type = VERTEX, qnum = -1 ; el_type <= FACETEDGE ; el_type++ )
-     { ex = EXTRAS(el_type);
-        for ( n = 0 ; n < web.skel[el_type].extra_count ; n++,ex++ )
-          if ( stricmp(ex->name,name) == 0 )
-             {*etype = el_type;qnum = n;break;}
-     }
+  { ex = EXTRAS(el_type);
+    for ( n = 0 ; n < web.skel[el_type].extra_count ; n++,ex++ )
+      if ( stricmp(ex->name,name) == 0 )
+      {*etype = el_type;qnum = n;break;}
+  }
   return qnum;
 }
 
@@ -1011,13 +1262,19 @@ int n;
          fe = get_next_facet(fe);
          if ( !valid_id(f_id) ) continue;
          b_id = get_facet_body(f_id);
-         sprintf(name,"body_%d_con_%d_meth",ordinal(b_id)+1,n);
+         if ( constr->attr & NAMED_THING )
+           sprintf(name,"body_%d_%s_meth",ordinal(b_id)+1,constr->name);
+         else
+           sprintf(name,"body_%d_con_%d_meth",ordinal(b_id)+1,n);
          if ( valid_id(b_id) )
          { attach_method(get_body_volquant(b_id),name);
            apply_method(inverse_id(v_id),name);
          }
          b_id = get_facet_body(inverse_id(f_id));
-         sprintf(name,"body_%d_con_%d_meth",ordinal(b_id)+1,n);
+         if ( constr->attr & NAMED_THING )
+           sprintf(name,"body_%d_%s_meth",ordinal(b_id)+1,constr->name);
+         else
+           sprintf(name,"body_%d_con_%d_meth",ordinal(b_id)+1,n);
          if ( valid_id(b_id) )
          { attach_method(get_body_volquant(b_id),name);
            apply_method(v_id,name);
@@ -1213,6 +1470,7 @@ int n;
     if ( everything_quantities_flag )
     { facetedge_id fe,first_fe = get_edge_fe(e_id);
       fe = first_fe;
+      if ( valid_id(first_fe) )
       do
       { char name[100];
         body_id b_id;
@@ -1222,13 +1480,19 @@ int n;
         if ( !valid_id(f_id) ) continue;
         if ( get_fattr(f_id) & NONCONTENT ) continue;
         b_id = get_facet_body(f_id);
-        sprintf(name,"body_%d_con_%d_meth",ordinal(b_id)+1,n);
+        if ( constr->attr & NAMED_THING )
+          sprintf(name,"body_%d_%s_meth",ordinal(b_id)+1,constr->name);
+        else
+          sprintf(name,"body_%d_con_%d_meth",ordinal(b_id)+1,n);
         if ( valid_id(b_id) )
         { attach_method(get_body_volquant(b_id),name);
           apply_method(e_id,name);
         }
         b_id = get_facet_body(inverse_id(f_id));
-        sprintf(name,"body_%d_con_%d_meth",ordinal(b_id)+1,n);
+        if ( constr->attr & NAMED_THING )
+          sprintf(name,"body_%d_%s_meth",ordinal(b_id)+1,constr->name);
+        else
+          sprintf(name,"body_%d_con_%d_meth",ordinal(b_id)+1,n);
         if ( valid_id(b_id) )
         { attach_method(get_body_volquant(b_id),name);
           apply_method(inverse_id(e_id),name);
@@ -1359,7 +1623,8 @@ int n;
 
 int get_tag(f_id)
 facet_id f_id;
-{ if ( EXTRAS(FACET)[F_TAG_ATTR].datacount > 0 )
+{ if ( F_TAG_ATTR == 0 ) return 0;
+  if ( EXTRAS(FACET)[F_TAG_ATTR].datacount > 0 )
      return (*FINT(f_id,F_TAG_ATTR));
   else return 0;
 }
