@@ -11,7 +11,15 @@
 *
 */
 
-/* structure for expression */
+/* depth of binary tree addition */
+#define MAXADDENDS 30
+
+
+/**********************************************************************
+*
+* structure for expression; goes here instead of express.h since needed
+*/
+ 
 #define EXPNAMESIZE 64
 struct expnode {
                  struct treenode *start;  /* start of node list */
@@ -22,6 +30,11 @@ struct expnode {
                  char name[EXPNAMESIZE];  /* for error messages */
                };
 
+/* flags, also used for treenode? */
+#define USERCOPY 1
+#define NOUSERCOPY 0
+#define USING_PARAM_FLAG 0x200 
+
 /************************************************************************
 * structure for defining extra attributes
 */
@@ -37,23 +50,50 @@ struct extra { char name[ATTR_NAME_SIZE+1];
                struct expnode code;  /* for function attribute */
                int flags;   /* see below */
             };
-/* attribute types */
-#define NULL_ATTR     0
-#define REAL_ATTR     1
-#define INTEGER_ATTR 2
-#define ULONG_ATTR    3
-#define ELEMENTID_ATTR ULONG_ATTR
-#define UCHAR_ATTR    4
-#define UINT_ATTR     5
-#define NUMATTRTYPES 6
-extern char *attr_type_name[NUMATTRTYPES];
-extern int attr_type_size[NUMATTRTYPES];
 
 /* flag bits */
 #define DUMP_ATTR     1
 #define FUNCTION_ATTR 2
 #define DIMENSIONED_ATTR 4
 
+/* useful for debugging */
+extern struct extra *Extras[NUMELEMENTS];
+
+/* datatype types */
+#define NULL_TYPE     0
+/* these types will be stored on the eval stack as reals */
+#define REAL_TYPE     1
+#define INTEGER_TYPE  2
+#define ULONG_TYPE    3
+#define UCHAR_TYPE    4
+#define USHORT_TYPE   5
+#define UINT_TYPE     6
+#define LONG_TYPE    7
+#define CHAR_TYPE    8
+#define SHORT_TYPE   9 
+#define MAX_NUMERIC_TYPE 10
+/* end of numeric types */
+#define STRING_TYPE  11
+#define PTR_TYPE   12
+#define VERTEX_TYPE 13
+#define EDGE_TYPE   14
+#define FACET_TYPE  15
+#define BODY_TYPE   16
+#define FACETEDGE_TYPE 17
+#define ELEMENTID_TYPE 18 
+#define BOUNDARY_TYPE  19
+#define CONSTRAINT_TYPE 20
+#define QUANTITY_TYPE 21
+#define INSTANCE_TYPE 22
+#define PROCEDURE_TYPE 23
+
+#define NUMDATATYPES 24
+/* the following are set in skeleton.c to match above defines */
+extern char *datatype_name[NUMDATATYPES];
+extern int datatype_size[NUMDATATYPES];
+
+
+
 /************************************************************************
 * base structure for a skeleton of a particular dimension 
 */
@@ -92,15 +132,24 @@ typedef long int WRAPTYPE; /* symmetry group element */
 typedef int NTYPE; /* for node indexes, types, etc. */
 
      /* common fields; added to each element struct type */
+#ifdef MPI_EVOLVER
+#define BASIC_STUFF \
+     element_id    forechain;       /* for element and free list forechain */\
+     element_id    backchain;       /* for element and free list backchain */\
+     element_id    local_id;        /* for freelisting; uses ibase index */\
+     ATTR          attr;            /* attribute bits */\
+     element_id    self_id;         /* for identifying self */ 
+#else
 #define BASIC_STUFF \
      element_id    forechain;       /* for element and free list forechain */\
      element_id    backchain;       /* for element and free list backchain */\
      ATTR          attr;            /* attribute bits */\
-     element_id    self_id;         /* for identifying self */\
-     int           lock;            /* for multithreading */
+     element_id    self_id;         /* for identifying self */
+#endif
+
 #define COMMON_STUFF \
      BASIC_STUFF \
-     int             original;      /* datafile original number */ \
+     element_id      original;      /* datafile original number */ \
      unsigned short  method_count;  /* number of method instances */ 
 
 struct element {
@@ -150,10 +199,10 @@ struct vertex
 #define V_PARAM_ATTR       2
 #define V_FORCE_ATTR       3
 #define V_VELOCITY_ATTR    4
-#define V_BOUNDARY_ATTR    5
-#define V_STAR_ATTR        6
-#define V_CONSTR_LIST_ATTR 7
+#define V_CONSTR_LIST_ATTR 5
 
+extern int V_BOUNDARY_ATTR;
+ 
 /****************************************************
 *
 *  edge structure - will be extended by other attributes.
@@ -172,12 +221,12 @@ struct edge
 
 /* attribute numbers for standard attributes */
 /* be sure these numbers are in same order as allocation in reset_skeleton() */
-#define E_STAR_ATTR          0
-#define E_DENSITY_ATTR       1
-#define E_VERTICES_ATTR      2
-#define E_BOUNDARY_ATTR      3
-#define E_WRAP_ATTR          4
-#define E_CONSTR_LIST_ATTR   5
+#define E_DENSITY_ATTR       0
+#define E_VERTICES_ATTR      1
+#define E_WRAP_ATTR          2
+#define E_CONSTR_LIST_ATTR   3
+
+extern int E_BOUNDARY_ATTR;
 
 /******************************************************
 *
@@ -196,12 +245,13 @@ struct facet
 /* attribute numbers for standard attributes */
 #define F_CONSTR_LIST_ATTR   0
 #define F_VERTICES_ATTR      1
-#define F_BOUNDARY_ATTR      2
-#define F_TAG_ATTR           3
-#define F_BODY_LIST_ATTR     4
-#define F_NEXT_VFACET_ATTR   5
-#define F_NEXT_BFACET_ATTR   6
-#define F_PHASE_ATTR         7
+#define F_BODY_LIST_ATTR     2
+#define F_NEXT_VFACET_ATTR   3
+#define F_NEXT_BFACET_ATTR   4
+
+extern int F_BOUNDARY_ATTR;
+extern int F_TAG_ATTR; 
+extern int F_PHASE_ATTR;
 
 
 /*********************************************************
@@ -214,12 +264,16 @@ struct body
      COMMON_STUFF
      REAL fixvol;         /* volume constraint */
      REAL volume;         /* actual volume      */
+     REAL volume_addends[MAXADDENDS]; /* for binary tree addition */
      REAL oldvolume;      /* volume for restore_coords() */
      REAL actualvolume;   /* set by actual_volume in datafile */
      REAL abstotal;       /* for judging constraint tolerance */
      REAL pressure;       /* internal pressure */
+     REAL oldpressure;    /* for restore_coords() */
      REAL volconst;       /* for body volume constant componenet */
+     REAL oldvolconst;    /* for restore_coords() */
      REAL density;        /* for gravitational potential energy */
+     REAL cm[MAXCOORD];   /* center of mass, unit cell coords */
      facetedge_id fe_id;  /* link to global structure */
      int  volquant;       /* number of named quantity used for volume */
      int  ambquant;       /* number of named quantity used for ambient pressure */
@@ -246,7 +300,6 @@ struct boundary
      struct expnode *convect[MAXCOORD]; /* functions for boundary interior content */
      int compcount;           /* number of components for integrands */
      int energy_method;       /* if using named quantity version */
-     int content_method;   /* if using named quantity version */
   };
 
 
@@ -277,6 +330,7 @@ struct constraint
 #define USURPED_BY_QUANTITY 0x0100
 #define NAMED_THING  0x0200
 #define NONWALL      0x0400
+#define PARTNER_HITTING 0x0800
 
 extern element_id
   NULLVERTEX,
@@ -294,54 +348,129 @@ typedef struct volgrad
   int  qnum;                /* quantity number */
   body_id bb_id;            /* body for real, or which quantity represents */
   struct volgrad *chain;    /* next body for this vertex */
-  REAL grad[MAXCOORD];      /* volume gradient of body for this vertex */
-  REAL normal[MAXCOORD];    /* approx normal of body at this vertex */
-  REAL velocity[MAXCOORD];  /* form converted to vector */
+  REAL *grad;      /* volume gradient of body for this vertex */
+  REAL *normal;    /* approx normal of body at this vertex */
+  REAL *velocity;  /* form converted to vector */
 } volgrad;
 
 struct vgradblock   /* block header */
 { struct volgrad *base;      /* structure space */
+  REAL *values;             /* for variable dimension data */
   int max;                  /* number allocated */
   int top;                  /* number used */
   struct vgradblock *next;  /* chaining */
 };
 
 extern struct vgradblock *vgradbase; /* allocated list header block start */
-extern volgrad **vgptrbase;   /* allocated list pointer for each vertex */
 extern int     vgradtop;      /* number of first free  structure */
 extern long    vgradmax;      /* number allocated */
 extern long    vgradlastused;     /* number actually used last time round */
 
 /* loop macros, for use when lists are not modified in loop */
 
+#ifdef MPI_EVOLVER
+/* MFOR includes imported elements; FOR doesn't */
+/* except FOR_ALL_BODIES includes all, for now */
 #define FOR_ALL_ELEMENTS(type,id) \
 for (id = web.skel[type].used ; valid_id(id) ; id = elptr(id)->forechain) \
-  if ( !valid_element(id) ) continue; else
+  if ( !valid_element(id) || (id_machine(id)!=this_machine) ) continue; else
 
 #define FOR_ALL_VERTICES(v_id) \
 for (v_id = web.skel[VERTEX].used ; valid_id(v_id) ; \
 v_id = vptr(v_id)->forechain) \
-  if ( !valid_element(v_id) ) continue; else
+  if ( !valid_element(v_id) || (id_machine(v_id)!=this_machine) ) continue; else
 
 #define FOR_ALL_EDGES(e_id) \
-for (e_id = web.skel[EDGE].used ; valid_id(e_id) ; \
-e_id = eptr(e_id)->forechain) \
-  if ( !valid_element(e_id) ) continue; else
+for (e_id = web.skel[EDGE].used ; valid_id(e_id) ;\
+ e_id = eptr(e_id)->forechain ) \
+  if ( !valid_element(e_id) || (id_machine(e_id)!=this_machine) )\
+    continue;  else
 
 #define FOR_ALL_FACETS(f_id) \
 for (f_id = web.skel[FACET].used ; valid_id(f_id) ; \
 f_id = fptr(f_id)->forechain) \
-  if ( !valid_element(f_id) ) continue; else
+  if ( !valid_element(f_id) || (id_machine(f_id)!=this_machine) ) continue; else
 
 #define FOR_ALL_BODIES(b_id) \
 for (b_id = web.skel[BODY].used ; valid_id(b_id) ; \
 b_id = bptr(b_id)->forechain) \
-  if ( !valid_element(b_id) ) continue; else
+  if ( !valid_element(b_id) /* || (id_machine(b_id)!=this_machine)*/ ) continue; else
 
 #define FOR_ALL_FACETEDGES(fe_id) \
 for (fe_id = web.skel[FACETEDGE].used ; valid_id(fe_id) ; \
+fe_id = feptr(fe_id)->forechain) \
+  if ( !valid_element(fe_id) || (id_machine(fe_id)!=this_machine) ) continue; else
+
+#define MFOR_ALL_ELEMENTS(type,id) \
+for (id = web.skel[type].used ; valid_id(id) ; id = elptr(id)->forechain) \
+  if ( !valid_element(id) ) continue; else
+
+#define MFOR_ALL_VERTICES(v_id) \
+for (v_id = web.skel[VERTEX].used ; valid_id(v_id) ; \
+v_id = vptr(v_id)->forechain) \
+  if ( !valid_element(v_id) ) continue; else
+
+#define MFOR_ALL_EDGES(e_id) \
+for (e_id = web.skel[EDGE].used ; valid_id(e_id) ; \
+e_id = eptr(e_id)->forechain) \
+  if ( !valid_element(e_id) ) continue; else
+
+#define MFOR_ALL_FACETS(f_id) \
+for (f_id = web.skel[FACET].used ; valid_id(f_id) ; \
+f_id = fptr(f_id)->forechain) \
+  if ( !valid_element(f_id) ) continue; else
+
+#define MFOR_ALL_BODIES(b_id) \
+for (b_id = web.skel[BODY].used ; valid_id(b_id) ; \
+b_id = bptr(b_id)->forechain) \
+  if ( !valid_element(b_id) ) continue; else
+
+#define MFOR_ALL_FACETEDGES(fe_id) \
+for (fe_id = web.skel[FACETEDGE].used ; valid_id(fe_id) ; \
 fe_id = feptr(fe_id)->forechain) \
   if ( !valid_element(fe_id) ) continue; else
+#else
+/* non-MPI */
+#define FOR_ALL_ELEMENTS(type,id) \
+for (id = web.skel[type].used ; valid_id(id) ; id = elptr(id)->forechain) \
+  if ( !(elptr(id)->attr & ALLOCATED) ) continue; else
+
+#define FOR_ALL_VERTICES(v_id) \
+for (v_id = web.skel[VERTEX].used ; valid_id(v_id) ; \
+v_id = vptr(v_id)->forechain) \
+  if ( !(vptr(v_id)->attr & ALLOCATED) ) continue; else
+
+#define FOR_ALL_EDGES(e_id) \
+for (e_id = web.skel[EDGE].used ; valid_id(e_id) ; \
+e_id = eptr(e_id)->forechain) \
+  if ( !(eptr(e_id)->attr & ALLOCATED) ) continue; else
+
+#define FOR_ALL_FACETS(f_id) \
+for (f_id = web.skel[FACET].used ; valid_id(f_id) ; \
+f_id = fptr(f_id)->forechain) \
+  if ( !(fptr(f_id)->attr & ALLOCATED) ) continue; else
+
+#define FOR_ALL_BODIES(b_id) \
+for (b_id = web.skel[BODY].used ; valid_id(b_id) ; \
+b_id = bptr(b_id)->forechain) \
+  if ( !(bptr(b_id)->attr & ALLOCATED) ) continue; else
+
+#define FOR_ALL_FACETEDGES(fe_id) \
+for (fe_id = web.skel[FACETEDGE].used ; valid_id(fe_id) ; \
+fe_id = feptr(fe_id)->forechain) \
+  if ( !(feptr(fe_id)->attr & ALLOCATED) ) continue; else
+
+#define MFOR_ALL_VERTICES(v_id) FOR_ALL_VERTICES(v_id) 
+
+#define MFOR_ALL_EDGES(e_id) FOR_ALL_EDGES(e_id) 
+
+#define MFOR_ALL_FACETS(f_id) FOR_ALL_FACETS(f_id) 
+
+#define MFOR_ALL_BODIES(b_id)  FOR_ALL_BODIES(b_id) 
+
+#define MFOR_ALL_FACETEDGES(fe_id) FOR_ALL_FACETEDGES(fe_id) 
+
+#endif
 
 #ifdef THREADS
 /* Thread version of these loops depend on initialization of
@@ -379,8 +508,6 @@ fe_id = feptr(fe_id)->forechain) \
 /* These access functions are defined as macros, since they involve
     only one evaluation of the arguments */
 
-#define valid_element(id)  (valid_id(id) && (get_attr(id) & ALLOCATED))
-
 #define get_edge_midv(e_id)            (get_edge_vertices(e_id)[2])
 
 #define get_fe_side(fe_id,x)  get_edge_side(get_fe_edge(fe_id),x)
@@ -394,42 +521,59 @@ fe_id = feptr(fe_id)->forechain) \
 #define F_EXTRA(f,attr) ((char*)(fptr(f))+EXTRAS(FACET)[attr].offset)
 
 #define VREAL(v,attr) ((REAL*)((char*)(vptr(v))+EXTRAS(VERTEX)[attr].offset))
+#define VREALP(vp,attr) ((REAL*)((char*)vp+EXTRAS(VERTEX)[attr].offset))
 #define VINT(v,attr) ((int*)((char*)(vptr(v))+EXTRAS(VERTEX)[attr].offset))
+#define VPTR(v,attr) ((char**)((char*)(vptr(v))+EXTRAS(VERTEX)[attr].offset))
+#define V_ELID(v,attr) ((element_id*)((char*)(eptr(e))+EXTRAS(VERTEX)[attr].offset))
+
 #define EREAL(e,attr) ((REAL*)((char*)(eptr(e))+EXTRAS(EDGE)[attr].offset))
 #define EINT(e,attr) ((int*)((char*)(eptr(e))+EXTRAS(EDGE)[attr].offset))
 #define EULONG(e,attr) ((unsigned long*)((char*)(eptr(e))+EXTRAS(EDGE)[attr].offset))
+#define E_ELID(e,attr) ((element_id*)((char*)(eptr(e))+EXTRAS(EDGE)[attr].offset))
+
 #define FREAL(f,attr) ((REAL*)((char*)(fptr(f))+EXTRAS(FACET)[attr].offset))
 #define FINT(f,attr) ((int*)((char*)(fptr(f))+EXTRAS(FACET)[attr].offset))
 #define FULONG(f,attr) ((unsigned long*)((char*)(fptr(f))+EXTRAS(FACET)[attr].offset))
+#define F_ELID(f,attr) ((element_id*)((char*)(fptr(f))+EXTRAS(FACET)[attr].offset))
+
 #define BREAL(b,attr) ((REAL*)((char*)(bptr(b))+EXTRAS(BODY)[attr].offset))
 #define BINT(b,attr) ((int*)((char*)(bptr(b))+EXTRAS(BODY)[attr].offset))
 #define BULONG(b,attr) ((unsinged long*)((char*)(bptr(b))+EXTRA(BODY)[attr].offset))
+#define B_ELID(b,attr) ((element_id*)((char*)(bptr(b))+EXTRA(BODY)[attr].offset))
 #define FEREAL(fe,attr) ((REAL*)((char*)(feptr(fe))+EXTRAS(FACETEDGE)[attr].offset))
 #define FEINT(fe,attr) ((int*)((char*)(feptr(fe))+EXTRAS(FACETEDGE)[attr].offset))
 
 #define get_coord(v_id)  VREAL((v_id),V_COORD_ATTR)
 #define get_oldcoord(v_id)  VREAL((v_id),V_OLDCOORD_ATTR)
 #define get_param(v_id)  VREAL((v_id),V_PARAM_ATTR)
-#define get_boundary(v_id)        (get_vattr(v_id)&BOUNDARY ?\
+#define get_boundary(v_id)        (V_BOUNDARY_ATTR ?\
                      web.boundaries+*VINT(v_id,V_BOUNDARY_ATTR) : NULL)
-#define get_vertex_boundary(v_id)        (get_vattr(v_id)&BOUNDARY ?\
+#define get_vertex_boundary(v_id)        (V_BOUNDARY_ATTR ?\
                      web.boundaries+*VINT(v_id,V_BOUNDARY_ATTR) : NULL)
-#define get_vertex_boundary_num(v_id)        (get_vattr(v_id)&BOUNDARY ?\
+#define get_vertex_boundary_num(v_id)        (V_BOUNDARY_ATTR ?\
                      *VINT(v_id,V_BOUNDARY_ATTR) : 0)
-#define get_edge_boundary_num(e_id)        (get_eattr(e_id)&BOUNDARY ?\
+#define get_edge_boundary_num(e_id)        (E_BOUNDARY_ATTR ?\
                      *EINT(e_id,E_BOUNDARY_ATTR) : 0)
-#define set_boundary_num(v_id,bnum) (get_vattr(v_id)&BOUNDARY ?\
+#define get_facet_boundary_num(f_id)        (F_BOUNDARY_ATTR ?\
+                     *FINT(f_id,F_BOUNDARY_ATTR) : 0)
+#define set_boundary_num(v_id,bnum) (V_BOUNDARY_ATTR ?\
                                     (*VINT(v_id,V_BOUNDARY_ATTR) = (bnum)) : 0)
-#define set_vertex_boundary_num(v_id,bnum) (get_vattr(v_id)&BOUNDARY ?\
+#define set_vertex_boundary_num(v_id,bnum) (V_BOUNDARY_ATTR ?\
                                     (*VINT(v_id,V_BOUNDARY_ATTR) = (bnum)) : 0)
-#define get_edge_boundary(e_id)    (get_eattr(e_id)&BOUNDARY ?\
+#define get_edge_boundary(e_id)    (E_BOUNDARY_ATTR ?\
                       web.boundaries+*EINT(e_id,E_BOUNDARY_ATTR) : NULL)
-#define set_edge_boundary_num(e_id,bnum)  (get_eattr(e_id)&BOUNDARY ?\
+#define set_edge_boundary_num(e_id,bnum)  (E_BOUNDARY_ATTR ?\
                                     (*EINT(e_id,E_BOUNDARY_ATTR) = (bnum)) : 0)
-#define get_facet_boundary(f_id)  (get_fattr(f_id)&BOUNDARY ? \
+#define get_facet_boundary(f_id)  (F_BOUNDARY_ATTR ? \
                      web.boundaries+*FINT(f_id,F_BOUNDARY_ATTR) : 0)
-#define set_facet_boundary_num(f_id,bnum) (get_fattr(f_id)&BOUNDARY ?\
+#define set_facet_boundary_num(f_id,bnum) (F_BOUNDARY_ATTR ?\
                                     (*FINT(f_id,F_BOUNDARY_ATTR) = (bnum)) : 0)
+#define get_vertex_vhead(v_id) (vhead + (*VINT(v_id,vhead_attr)))
+#define get_vertex_v_normal(v_id) (v_normal[*VINT(v_id,vhead_attr)])
+#define set_vertex_vhead(v_id,k) (*VINT(v_id,vhead_attr)=(k))
+#define get_body_vhead(b_id) (vhead + (*BINT(b_id,bhead_attr)))
+#define set_body_vhead(b_id,k) (*BINT(b_id,bhead_attr)=(k))
+#define get_body_cm(b_id)  (bptr(b_id)->cm)
 
 
 /* Constraints */
@@ -456,6 +600,7 @@ extern conmap_t nullcon[2];  /* default empty constraint list */
   (conmap_t*)(F_EXTRA(f_id,F_CONSTR_LIST_ATTR)) : nullcon)
 
 #define get_force(v_id)                   VREAL((v_id),V_FORCE_ATTR)
+#define get_force_p(vp)                   VREALP((vp),V_FORCE_ATTR)
 #define get_velocity(v_id)                VREAL((v_id),V_VELOCITY_ATTR)
 #define get_restore(v_id)                 VREAL((v_id),V_RESTORE_ATTR) 
 #define set_vertex_edge(v_id,e)           (vptr(v_id)->e_id = (e))
@@ -481,14 +626,15 @@ extern conmap_t nullcon[2];  /* default empty constraint list */
 #define set_edge_star(e_id,a)       (eptr(e_id)->star = (a))
 #define add_edge_star(e_id,a)       (eptr(e_id)->star += (a))
 
-#define get_facet_vertices(f_id)   FULONG(f_id,F_VERTICES_ATTR)
-#define get_edge_vertices(e_id)    EULONG(e_id,E_VERTICES_ATTR) 
+#define get_facet_vertices(f_id)   F_ELID(f_id,F_VERTICES_ATTR)
+#define get_edge_vertices(e_id)    E_ELID(e_id,E_VERTICES_ATTR) 
 
 #define get_facet_density(f_id)      (fptr(f_id)->density)
 #define set_facet_density(f_id,den)  (fptr(f_id)->density = (den))
 
 #define get_facet_area(f_id)    (fptr(f_id)->area)
 #define set_facet_area(f_id,a)  (fptr(f_id)->area = (a))
+#define add_facet_area(f_id,a)  (fptr(f_id)->area += inverted(f_id)?-(a):(a))
 
 #define get_facet_flux(f_id)    (fptr(f_id)->flux)
 #define set_facet_flux(f_id,a)  (fptr(f_id)->flux = (a))
@@ -508,6 +654,10 @@ extern conmap_t nullcon[2];  /* default empty constraint list */
 
 #define save_body_volume(b_id) (bptr(b_id)->oldvolume = bptr(b_id)->volume)
 #define restore_body_volume(b_id) (bptr(b_id)->volume = bptr(b_id)->oldvolume)
+#define save_body_pressure(b_id) (bptr(b_id)->oldpressure = bptr(b_id)->pressure)
+#define restore_body_pressure(b_id) (bptr(b_id)->pressure = bptr(b_id)->oldpressure)
+#define save_body_volconst(b_id) (bptr(b_id)->oldvolconst = bptr(b_id)->volconst)
+#define restore_body_volconst(b_id) (bptr(b_id)->volconst = bptr(b_id)->oldvolconst)
 #define get_body_oldvolume(b_id) (bptr(b_id)->oldvolume)
 #define set_body_oldvolume(b_id,vol) (bptr(b_id)->oldvolume = vol)
 
@@ -536,7 +686,7 @@ extern conmap_t nullcon[2];  /* default empty constraint list */
 #define get_f_phase(f_id)  (EXTRAS(FACET)[F_PHASE_ATTR].datacount ? (*FINT(f_id,F_PHASE_ATTR)):0)
 #define get_b_phase(b_id)  (bptr(b_id)->phase)
 
-#define set_original(id,num)  (elptr(id)->original = (num) )
+#define set_original(id,oid)  (elptr(id)->original = (oid) )
 #define get_original(id)  (elptr(id)->original)
 
 #define set_edge_color(e_id,col)    (eptr(e_id)->color = (short)(col))
