@@ -190,7 +190,7 @@ struct method_instance *mi;
      }
 
     FOR_ALL_VERTICES(vv_id) 
-    { struct v_curve_t *vc = v_curve + ordinal(vv_id);
+    { struct v_curve_t *vc = v_curve + loc_ordinal(vv_id);
       if ( !boundary_curvature_flag ) vc->a = vc->area; 
       if ( (get_vattr(vv_id) & CONSTRAINT) && !(sqcurve_ignore_constr) )
       { conmap_t * conmap = get_v_constraint_map(vv_id);
@@ -281,7 +281,7 @@ WRAPTYPE *wraps; /* in case of symmetry group */
 
   area = sqrt(det)/2;
   for ( i = 0 ; i < FACET_VERTS ; i++ )
-  { vc[i] = v_curve + ordinal(v_id[i]);
+  { vc[i] = v_curve + loc_ordinal(v_id[i]);
     vc[i]->area += area;
   }
   if ( boundary_curvature_flag ) /* apportion area differently */
@@ -289,7 +289,7 @@ WRAPTYPE *wraps; /* in case of symmetry group */
     for ( i = 0 ; i < FACET_VERTS ; i++ )
       if ( get_vattr(v_id[i]) & (BOUNDARY|FIXED) ) fixcount++;
     for ( i = 0 ; i < FACET_VERTS ; i++ )
-    { vc[i] = v_curve+ordinal(v_id[i]);
+    { vc[i] = v_curve+loc_ordinal(v_id[i]);
       if ( !(get_vattr(v_id[i]) & (BOUNDARY|FIXED)) )
          vc[i]->a += 3*area/(3-fixcount);
     }
@@ -368,7 +368,7 @@ WRAPTYPE *wraps;
 
   area = sqrt(det)/2;
   for ( i = 0 ; i < FACET_VERTS ; i++ )
-     { vc[i] = v_curve + ordinal(v_id[i]);
+     { vc[i] = v_curve + loc_ordinal(v_id[i]);
        vc[i]->area += area;
      }
   if ( boundary_curvature_flag ) /* apportion area differently */
@@ -380,7 +380,7 @@ WRAPTYPE *wraps;
              vc[i]->a += 3*area/(3-fixcount);
      }
   for ( i = 0 ; i < FACET_EDGES ; i++ )
-     ec[i] = e_curve + ordinal(e_id[i]);
+     ec[i] = e_curve + loc_ordinal(e_id[i]);
 
   memset((char*)force,0,sizeof(force));
   for ( j = 0 ; j < FACET_VERTS ; j++ )
@@ -493,7 +493,7 @@ struct qinfo *v_info;
   vertex_id v_id = v_info->v[0];
   REAL h,venergy;
   ATTR attr = get_vattr(v_id);
-  int ordv = ordinal(v_id);
+  int ordv = loc_ordinal(v_id);
   struct v_curve_t *vc;
   REAL denom,f;
   REAL area; /* curvature normalization area */
@@ -524,9 +524,9 @@ struct qinfo *v_info;
       if ( (fa != fb) && 
                (inverted(get_fe_facet(fa))==inverted(get_fe_facet(fb))) )
       { sprintf(errmsg,
-           "Inconsistent orientation of \nfacets %d and %d at vertex %d.\n",
-            ordinal(get_fe_facet(fa))+1,ordinal(get_fe_facet(fb))+1,
-               ordinal(eid)+1);
+           "Inconsistent orientation of \nfacets %s and %s at vertex %s.\n",
+            ELNAME(get_fe_facet(fa)),ELNAME1(get_fe_facet(fb)),
+               ELNAME(eid));
         strcat(errmsg,"eff_area_sq_mean_curvature and normal_sq_mean_curvature need consistency.\n"); 
         kb_error(2183,errmsg,RECOVERABLE);
       }
@@ -589,7 +589,7 @@ struct qinfo *v_info;
      venergy = h*h;
    }
    else if ( h0_flag )
-   { REAL term,sim;
+   { REAL term=0.0,sim;
      vc->h = h = SDIM_dot(vc->force,vc->normal)/
                           SDIM_dot(vc->normal,vc->normal)*3;
                             /* since vc->normal = 6*volgrad */
@@ -635,12 +635,12 @@ struct qinfo *v_info;
   REAL e,e1,e2,denom;
   REAL f[MAXCOORD];
   REAL fudge1,fudge2,fudge3; /* combinations of values */
-  REAL fudge11,fudge12,fudge13; /* combinations of values */
-  REAL fudge21,fudge22,fudge23; /* combinations of values */
+  REAL fudge11=0.0,fudge12=0.0,fudge13=0.0; /* combinations of values */
+  REAL fudge21=0.0,fudge22=0.0,fudge23=0.0; /* combinations of values */
   REAL h; /* curvature */
   REAL area; /* curvature normalization area */
   REAL a;     /* integral area allocation */
-  struct v_curve_t *vc = v_curve + ordinal(v_id);
+  struct v_curve_t *vc = v_curve + loc_ordinal(v_id);
   ATTR attr = get_vattr(v_id);
   REAL  ad[MAXCOORD];
   REAL venergy = 0.0;
@@ -660,8 +660,9 @@ struct qinfo *v_info;
   if ( !boundary_curvature_flag ) {  area = a = vc->a/3; } 
   else { a = vc->a/3; area = vc->area/3; }
   for ( i = 0 ; i < SDIM  ;i++ ) /* alloc area deriv */
-      if ( boundary_curvature_flag ) ad[i] = vc->star_force[i]/3;
-      else ad[i] = vc->force[i]/3;
+    if ( boundary_curvature_flag ) 
+       ad[i] = vc->star_force[i]/3;
+    else ad[i] = vc->force[i]/3;
 
   /* vertex self-second derivatives */
   if ( METH_INSTANCE(v_info->method)->gen_method==normal_sq_mean_curvature_mi )
@@ -687,11 +688,13 @@ struct qinfo *v_info;
     else 
        for ( i = 0 ; i < SDIM ; i++ ) f[i] =0.0;
   }
-  else if ( METH_INSTANCE(v_info->method)->gen_method==mix_sq_mean_curvature_mi )
+  else
+  if ( METH_INSTANCE(v_info->method)->gen_method==mix_sq_mean_curvature_mi )
   {
     /* normal_sq_mean part */
     e = SDIM_dot(vc->force,vc->force);
     denom = SDIM_dot(vc->force,vc->normal);
+    h = 0.0;
     if ( denom != 0.0 )
     {
        h = 3*e/denom;
@@ -720,17 +723,16 @@ struct qinfo *v_info;
       for ( i = 0 ; i < SDIM  ;i++ )
         fd[i] = SDIM_dot(vc->normal,vc->deriv2[i])/vc->norm*3;
       for ( i = 0 ; i < SDIM  ;i++ )
-        { aread[i] = vc->force[i]/3;
-           if ( boundary_curvature_flag ) ad[i] = vc->star_force[i]/3;
-           else ad[i] = aread[i];
-        }
+      { aread[i] = vc->force[i]/3;
+        if ( boundary_curvature_flag ) ad[i] = vc->star_force[i]/3;
+        else ad[i] = aread[i];
+      }
       h = (h*sq_mean_mix+(1-sq_mean_mix)*vc->term);  /* make combination */
       for ( i = 0  ; i < SDIM ; i++ )
-        {
-           net = 2*h*(f[i]*sq_mean_mix+(1-sq_mean_mix)*fd[i])*a + h*h*ad[i];
-           grad[i] += net;
-        }
+      { net = 2*h*(f[i]*sq_mean_mix+(1-sq_mean_mix)*fd[i])*a + h*h*ad[i];
+        grad[i] += net;
       }
+    }
     venergy = h*h;
   }
   else if ( METH_INSTANCE(v_info->method)->gen_method == eff_area_sq_mean_curvature_mi )
@@ -740,7 +742,7 @@ struct qinfo *v_info;
     { for ( i = 0 ; i < SDIM ; i++ )
         f[i] = 18*SDIM_dot(vc->force,vc->deriv2[i])/denom*a
           + 9*e/denom*ad[i];
-       venergy = 9*e/denom;  /* 9 = 36/4 */
+      venergy = 9*e/denom;  /* 9 = 36/4 */
     }
     else
        for ( i = 0 ; i < SDIM ; i++ ) f[i] =0.0;
@@ -797,7 +799,7 @@ struct qinfo *v_info;
     struct v_curve_t *vc1;
     struct v_curve_t *vc2;
     REAL s[MAXCOORD],cross1[MAXCOORD],cross2[MAXCOORD];
-    REAL denom1,denom2;
+    REAL denom1=0.0,denom2=0.0;
     REAL s0[MAXCOORD];
     REAL wa[MAXCOORD],wb[MAXCOORD],w[MAXCOORD],aw[MAXCOORD],vw[MAXCOORD];
     facetedge_id fe;
@@ -807,9 +809,9 @@ struct qinfo *v_info;
     else e_id = eee_id;
     headv = get_edge_headv(e_id);
     tailv = get_edge_tailv(e_id);
-    ec = e_curve + ordinal(e_id);
-    vc1 = v_curve + ordinal(tailv);
-    vc2 = v_curve + ordinal(headv);
+    ec = e_curve + loc_ordinal(e_id);
+    vc1 = v_curve + loc_ordinal(tailv);
+    vc2 = v_curve + loc_ordinal(headv);
     
     if ( (vc1->area == 0.0) || (vc2->area == 0.0) ) continue;
     if ( METH_INSTANCE(v_info->method)->gen_method == normal_sq_mean_curvature_mi )
@@ -828,13 +830,13 @@ struct qinfo *v_info;
            || (!assume_oriented_flag && triple_prod(vc1->normal,sa,s0) < 0.0 ) )
           for ( i = 0 ; i < SDIM ; i++ ) cross1[i] = -cross1[i];
         if ( fe_b != fe_a ) /* check for single edge on constraint */
-          { get_edge_side(get_fe_edge(fe_b),sb);
-             cross_prod(vc1->force,sb,s);
-             if ( (assume_oriented_flag && inverted(get_fe_facet(fe_b)))
+        { get_edge_side(get_fe_edge(fe_b),sb);
+          cross_prod(vc1->force,sb,s);
+          if ( (assume_oriented_flag && inverted(get_fe_facet(fe_b)))
            || (!assume_oriented_flag && triple_prod(vc1->normal,sb,s0) < 0.0 ) )
-                for ( i = 0 ; i < SDIM ; i++ ) cross1[i] -= s[i];
-             else  for ( i = 0 ; i < SDIM ; i++ ) cross1[i] += s[i];
-          }
+             for ( i = 0 ; i < SDIM ; i++ ) cross1[i] -= s[i];
+          else  for ( i = 0 ; i < SDIM ; i++ ) cross1[i] += s[i];
+        }
   
         e1 = SDIM_dot(vc1->force,vc1->force);
         denom1 = SDIM_dot(vc1->force,vc1->normal);
@@ -894,13 +896,13 @@ struct qinfo *v_info;
              || (!assume_oriented_flag &&  SDIM_dot(cross1,s0) < 0.0 ) )
           for ( i = 0 ; i < SDIM ; i++ ) cross1[i] = -cross1[i];
         if ( fe_b != fe_a ) /* check for single edge on constraint */
-          { get_edge_side(get_fe_edge(fe_b),sb);
-             cross_prod(vc1->normal,sb,s);
-             if ( (assume_oriented_flag && inverted(get_fe_facet(fe_b)))
+        { get_edge_side(get_fe_edge(fe_b),sb);
+          cross_prod(vc1->normal,sb,s);
+          if ( (assume_oriented_flag && inverted(get_fe_facet(fe_b)))
                || (!assume_oriented_flag && SDIM_dot(s,s0) < 0.0 ) )
-                for ( i = 0 ; i < SDIM ; i++ ) cross1[i] -= s[i];
-             else  for ( i = 0 ; i < SDIM ; i++ ) cross1[i] += s[i];
-          }
+             for ( i = 0 ; i < SDIM ; i++ ) cross1[i] -= s[i];
+          else  for ( i = 0 ; i < SDIM ; i++ ) cross1[i] += s[i];
+        }
   
         e1 = SDIM_dot(vc1->force,vc1->force);
         denom1 = SDIM_dot(vc1->normal,vc1->normal);
@@ -928,7 +930,7 @@ struct qinfo *v_info;
     { int tattr = get_vattr(tailv);
       if ( (!(tattr & FIXED) || 
                     (METH_INSTANCE(v_info->method)->flags & IGNORE_FIXED))
-     && (!(tattr & BOUNDARY) || 
+           && (!(tattr & BOUNDARY) || 
                     (METH_INSTANCE(v_info->method)->flags & IGNORE_CONSTR))
       )
       { /* force on head due to curvature at tail */
@@ -994,26 +996,26 @@ struct qinfo *v_info;
       { /* force on tail due to curvature at head */
          if ( METH_INSTANCE(v_info->method)->gen_method == normal_sq_mean_curvature_mi )
           for ( i = 0 ; i < SDIM ; i++ )
-            { f[i] = fudge23*ec->deriv[0][i] - fudge22*cross2[i];
-              for ( j = 0 ; j < SDIM ; j++ )
-                     f[i] += fudge21*vc2->force[j]*ec->deriv2[j][i]
+          { f[i] = fudge23*ec->deriv[0][i] - fudge22*cross2[i];
+            for ( j = 0 ; j < SDIM ; j++ )
+              f[i] += fudge21*vc2->force[j]*ec->deriv2[j][i]
                               - fudge22*vc2->normal[j]*ec->deriv2[j][i];
-            }
+         }
          else if ( METH_INSTANCE(v_info->method)->gen_method == eff_area_sq_mean_curvature_mi )
           for ( i = 0 ; i < SDIM ; i++ )
-            { if ( denom2 != 0.0 )
-                { f[i] =    3*e2/denom2*ec->deriv[0][i] 
+          { if ( denom2 != 0.0 )
+            { f[i] = 3*e2/denom2*ec->deriv[0][i] 
                               - 6*e2/denom2/denom2*cross2[i]*vc2->area;
-                  for ( j = 0 ; j < SDIM ; j++ )
-                     f[i] += 6*vc2->force[j]*ec->deriv2[j][i]/denom2*vc2->area;
-                }
-              else f[i] = 0.0;
+              for ( j = 0 ; j < SDIM ; j++ )
+                f[i] += 6*vc2->force[j]*ec->deriv2[j][i]/denom2*vc2->area;
             }
+            else f[i] = 0.0;
+          }
          else  /* squared curvature */
           for ( i = 0 ; i < SDIM ; i++ )
           { f[i] = -4/3.0*e2*ec->deriv[0][i]/vc2->area*3.0/4;
-             for ( j = 0 ; j < SDIM ; j++ )
-                 f[i] += 2*vc2->force[j]*ec->deriv2[j][i]/vc2->area*3.0/4;
+            for ( j = 0 ; j < SDIM ; j++ )
+              f[i] += 2*vc2->force[j]*ec->deriv2[j][i]/vc2->area*3.0/4;
           }
   
          if ( h0_flag &&
@@ -1123,7 +1125,7 @@ struct qinfo *v_info;
   a2 = (s1 + s2)/2;
   if ( (a1 <= 0.0) || (a2 == 0.0) ) return 0.0;
 
-  hh = 2*a1/a2/a2;
+  hh = (2*a1+12*a1*a1)/a2/a2;
   energy = a2*pow(hh,power/2);
 
   return energy;
@@ -1165,7 +1167,7 @@ struct qinfo *v_info;
   a2 = (s1 + s2)/2;
   if (a2 == 0.0) return 0.0;
 
-  hh = a1/a2/a2;
+  hh = (a1+3*a1*a1)/a2/a2;
   energy = a2*pow(hh,power/2);
 
   side[0] = side1; side[1] = side2; s[0] = s1; s[1] = s2;
@@ -1182,7 +1184,7 @@ struct qinfo *v_info;
      for ( i = 0 ; i < SDIM ; i++ )
      { REAL f;
         f = a2g[k][i]*energy/a2 + term*power/2
-              *(a1g[k][i]/a2/a2 - 2*a1/a2/a2/a2*a2g[k][i]);
+              *(a1g[k][i]*(1+6*a1)/a2/a2 - (2*a1+6*a1*a1)/a2/a2/a2*a2g[k][i]);
         v_info->grad[k+1][i] = f; 
         v_info->grad[0][i] += -f; 
      }
@@ -1228,7 +1230,7 @@ struct qinfo *v_info;
   a2 = (s1 + s2)/2;
   if ( a2 == 0.0 ) return 0.0;
 
-  energy = 2*a1/a2;
+  energy = 2*(a1+6*a1*a1)/a2;
 
   side[0] = side1; side[1] = side2; s[0] = s1; s[1] = s2;
   for ( k = 0 ; k < 2 ; k++ )
@@ -1239,10 +1241,10 @@ struct qinfo *v_info;
         }
   }
   for ( i = 0 ; i < SDIM ; i++ ) v_info->grad[0][i] = 0.0;
-  term = a1/a2/a2;
+  term = (a1+6*a1*a1)/a2/a2;
   for ( k = 0 ; k < 2 ; k++ )
      for ( i = 0 ; i < SDIM ; i++ )
-     { REAL f = 2*(a1g[k][i]/a2 - term*a2g[k][i]);
+     { REAL f = 2*(a1g[k][i]*(1+12*a1)/a2 - term*a2g[k][i]);
         v_info->grad[k+1][i] = f; 
         v_info->grad[0][i] += -f; 
      }
@@ -1279,11 +1281,11 @@ struct qinfo *v_info;
     for ( kk = 0 ; kk < 2 ; kk++ )
      for ( i = 0 ; i < SDIM ; i++ )
       for ( ii = 0 ; ii < SDIM ; ii++ )
-      { REAL f = 2*(a1h[k][kk][i][ii]/a2
-                         - a1g[k][i]*a2g[kk][ii]/a2/a2
-                         - a1g[kk][ii]*a2g[k][i]/a2/a2
-                         - a1*a2h[k][kk][i][ii]/a2/a2
-                         + 2*a1*a2g[k][i]*a2g[kk][ii]/a2/a2/a2);
+      { REAL f = 2*((a1h[k][kk][i][ii]*(1+12*a1)+12*a1g[k][i]*a1g[kk][ii])/a2
+                         - a1g[k][i]*(1+12*a1)*a2g[kk][ii]/a2/a2
+                         - a1g[kk][ii]*(1+12*a1)*a2g[k][i]/a2/a2
+                         - (a1+6*a1*a1)*a2h[k][kk][i][ii]/a2/a2
+                         + 2*(a1+6*a1*a1)*a2g[k][i]*a2g[kk][ii]/a2/a2/a2);
          h[k+1][kk+1][i][ii] += f;
          h[k+1][0][i][ii] += -f;
          h[0][kk+1][i][ii] += -f;
