@@ -14,13 +14,8 @@
 *                
 *        This version has element ids implemented as longs.
 */
-#ifdef WIN32
-#include <windows.h>
-#endif
-
 #include "include.h"
 
-
 struct blocklist_struct *blocklist[NUMELEMENTS]; /* list of allocated blocks */
 int blockcount[NUMELEMENTS];  /* how many blocks allocated */
 int blockmax[NUMELEMENTS];  /* length of blocklist */
@@ -45,19 +40,24 @@ struct webstruct web = {
 
 
 element_id
-  NULLVERTEX  =  VERTEX  << TYPESHIFT, 
-  NULLEDGE     =  EDGE  << TYPESHIFT,
-  NULLFACET    =  FACET  << TYPESHIFT,
-  NULLBODY     =  BODY  << TYPESHIFT,  
-  NULLFACETEDGE =  (unsigned long)FACETEDGE << TYPESHIFT;
-
+  NULLVERTEX  =  (element_id)VERTEX  << TYPESHIFT, 
+  NULLEDGE     =  (element_id)EDGE  << TYPESHIFT,
+  NULLFACET    =  (element_id)FACET  << TYPESHIFT,
+  NULLBODY     =  (element_id)BODY  << TYPESHIFT,  
+  NULLFACETEDGE =  (element_id)FACETEDGE << TYPESHIFT;
 
+#ifdef ELPTRFUNC
 struct element *elptr(id)
 element_id id;
 {
   int type = id_type(id);
+#ifdef MPI_EVOLVER
+  if ( id_machine(id) != this_machine ) 
+    return mpi_remote_elptr(id);
+#endif
   return (struct element *)(web.skel[type].ibase[id & OFFSETMASK]);
 }
+#endif
 
 int oid(id)
 element_id id;
@@ -111,7 +111,17 @@ int newsize; /* new size of structure */
      copysize = (newsize < oldsize) ? newsize : oldsize;
      for ( i = 0, oldptr = (char*)(b->blockptr), newptr = newblock; 
              i < b->count ; i++, oldptr += oldsize, newptr += newsize )
-         memcpy(newptr,oldptr,copysize);
+     { memcpy(newptr,oldptr,copysize);
+       #ifdef MPI_EVOLVER
+       { element_id id =  ((struct element*)oldptr)->self_id;
+         if ( id_machine(id) != this_machine )
+         { /* update pointer in remote_elements */
+           remote_elements[type].ibase[id_machine(id)][ordinal(id)] = 
+               (struct element *)newptr;   
+          }
+        }
+        #endif
+     }
      myfree((char*)b->blockptr);
      b->blockptr = (struct element*)newblock;
      /* update indirect pointers */
@@ -119,8 +129,14 @@ int newsize; /* new size of structure */
         due to reorder command */
      for ( i=0,spot=newblock, iptr=web.skel[type].ibase+b->start_ord ; 
                 i<b->count ; i++,spot+=newsize,iptr++ )
-        web.skel[type].ibase[ordinal(((struct element *)spot)->self_id)] = 
-               (struct element *)spot;
+	 {  
+#ifdef MPI_EVOLVER
+         element_id id = ((struct element *)spot)->local_id;
+#else
+         element_id id = ((struct element *)spot)->self_id;
+#endif
+        web.skel[type].ibase[ordinal(id)] = (struct element *)spot;
+ 	 }
   }
 
   LEAVE_GRAPH_MUTEX;
@@ -139,7 +155,7 @@ void extend(type,mode)
 int type;
 int mode; /* EXTEND_BATCH or EXTEND_FOR_REFINE */
 {
-  int number;  /* of new items to allocate */
+  int number=0;  /* of new items to allocate */
   char *newblock;
   INDIRECT_TYPE *newiblock;
   element_id id,oldfreehead;
@@ -147,17 +163,17 @@ int mode; /* EXTEND_BATCH or EXTEND_FOR_REFINE */
   int k;
   int allocsize;
   long  oldnum = web.skel[type].maxcount;
-  long  newnum;
+  long  newnum=0;
   int neword;
 
+
   ENTER_GRAPH_MUTEX;
   /* Don't mess with structures while graph thread using them */
 
   if ( blockcount[type] >= blockmax[type] )
   { blocklist[type] = (struct blocklist_struct*)
       kb_realloc((char*)(blocklist[type]),
-         (blockcount[type]+BATCHSIZE)*sizeof(struct blocklist_struct),
-         blockcount[type]*sizeof(struct blocklist_struct));
+         (blockcount[type]+BATCHSIZE)*sizeof(struct blocklist_struct));
      blockmax[type] += BATCHSIZE;
   }
   if ( mode == EXTEND_BATCH )
@@ -173,9 +189,15 @@ int mode; /* EXTEND_BATCH or EXTEND_FOR_REFINE */
     number = web.skel[type].count*(1<<web.dimension)
                 - web.skel[type].maxcount + 100;
     newnum = web.skel[type].maxcount + number;
-    if ( number <= 0 ) { LEAVE_GRAPH_MUTEX; return; }/* don't need any more */
+    if ( number <= 0 ) { goto extend_exit; }/* don't need any more */
   }
   else kb_error(2474,"Internal error: bad mode for extend().\n",RECOVERABLE);
+  
+  if ( (web.skel[type].maxcount + number) > OFFSETMASK )
+  { sprintf(errmsg, "Trying to allocate more %s than ID format allows, %Ld\n",
+          typenames[type],OFFSETMASK);
+    kb_error(3712,errmsg,RECOVERABLE);
+  }
 
   newblock = mycalloc(number,web.sizes[type]);
   blocklist[type][blockcount[type]].start_ord = oldnum ;
@@ -190,8 +212,7 @@ int mode; /* EXTEND_BATCH or EXTEND_FOR_REFINE */
     }
     else
     { newiblock = (INDIRECT_TYPE*)kb_realloc((char*)(web.skel[type].ibase),
-        2*web.skel[type].ialloc*sizeof(INDIRECT_TYPE),
-         web.skel[type].ialloc*sizeof(INDIRECT_TYPE));
+        2*web.skel[type].ialloc*sizeof(INDIRECT_TYPE));
       web.skel[type].ialloc *= 2;
     }
     web.skel[type].ibase = newiblock;
@@ -204,11 +225,12 @@ int mode; /* EXTEND_BATCH or EXTEND_FOR_REFINE */
     }
   }
 
-  LEAVE_GRAPH_MUTEX;
-
   /* add to end of freelist */
-  neword = web.skel[type].maxcount;  
-  id = ((long)type << TYPESHIFT) | VALIDMASK | neword; 
+  neword = web.skel[type].maxcount;
+  id = ((element_id)type << TYPESHIFT) | VALIDMASK | neword; 
+#ifdef MPI_EVOLVER
+  id |= this_machine << MACH_ID_SHIFT;  
+#endif
   oldfreehead = web.skel[type].free;
   if ( valid_id(oldfreehead) )
   { element_id nextid = oldfreehead;
@@ -221,14 +243,24 @@ int mode; /* EXTEND_BATCH or EXTEND_FOR_REFINE */
   { newptr = (struct element *)(newblock + k*web.sizes[type]); 
     web.skel[type].ibase[neword] = newptr;
     newptr->self_id = id;
+    #ifdef MPI_EVOLVER
+    newptr->local_id = id;
+    #endif
     if ( k < number-1 )
     { neword++;
-      id = ((long)type << TYPESHIFT) | VALIDMASK | neword; 
+      id = ((element_id)type << TYPESHIFT) | VALIDMASK | neword; 
+#ifdef MPI_EVOLVER
+      id |= this_machine << MACH_ID_SHIFT;  
+#endif
+
       newptr->forechain = id;
     }
   }
 
   web.skel[type].maxcount += number;
+  
+extend_exit:
+  LEAVE_GRAPH_MUTEX;
 
 }
 
@@ -252,7 +284,7 @@ element_id parent; /* for inherited stuff */
   
   if ( !valid_id(newid) ) /* free list empty */
   { if ( aggregate_depth == 0 )
-    { free_discards(DISCARDS_ALL);
+    { free_discards(DISCARDS_SOME);
       newid = web.skel[type].free;
     }
   }
@@ -269,83 +301,132 @@ element_id parent; /* for inherited stuff */
   /* clean out old info */
   ord = ordinal(newid);
   memset((char *)newptr,0,web.sizes[type]);
-  if ( ord > web.skel[type].max_ord ) web.skel[type].max_ord = ord;
-  newptr->original = NO_ORIGINAL;
+  if ( ord > web.skel[type].max_ord ) 
+  { web.skel[type].max_ord = ord;
+  }
+  newptr->original = NULLID;
 
 
   if ( match_id_flag && datafile_flag )
   { /* link in numerical order */
-     int i;
-     struct element *prev,*next;
-     for ( i = ord-1 ; i >= 0 ; i-- )
-     { prev = web.skel[type].ibase[i];
-        if ( prev->attr & ALLOCATED )
-        { if ( valid_id(prev->forechain) )
-          { next = elptr(prev->forechain);
-            next->backchain = newid;
-          }
-          newptr->forechain = prev->forechain;
-          prev->forechain = newid;
-          newptr->backchain = prev->self_id;
-          break;
+    int i;
+    struct element *prev,*next;
+    for ( i = ord-1 ; i >= 0 ; i-- )
+    { prev = web.skel[type].ibase[i];
+      if ( prev->attr & ALLOCATED )
+      { if ( valid_id(prev->forechain) )
+        { next = elptr(prev->forechain);
+          next->backchain = newid;
         }
-     }
-     if ( i < 0 )  /* no predecessor */
-     { newptr->forechain = web.skel[type].used;
-       if ( valid_id(web.skel[type].used) )
-       { next = elptr(web.skel[type].used);
-         next->backchain = newid;
-       }
-       web.skel[type].used = newid;
-     }
-     if ( !valid_id(newptr->forechain) ) web.skel[type].last = newid;
+        newptr->forechain = prev->forechain;
+        prev->forechain = newid;
+        newptr->backchain = prev->self_id;
+        break;
+      }
+    }
+    if ( i < 0 )  /* no predecessor */
+    { newptr->forechain = web.skel[type].used;
+      if ( valid_id(web.skel[type].used) )
+      { next = elptr(web.skel[type].used);
+        next->backchain = newid;
+      }
+      web.skel[type].used = newid;
+    }
+    if ( !valid_id(newptr->forechain) ) web.skel[type].last = newid;
   }
   else
   { /* add to end of in-use chain */
-     newptr->forechain = NULLID;
-     newptr->backchain = web.skel[type].last;
-     if ( valid_id(web.skel[type].last) )
-     {
-        last = elptr(web.skel[type].last);     /* find end of in-use chain */
-        last->forechain = newid;
-     }
-     else
-     {        
-        web.skel[type].used = newid;
-     }
-     web.skel[type].last = newid;
+    newptr->forechain = NULLID;
+    newptr->backchain = web.skel[type].last;
+    if ( valid_id(web.skel[type].last) )
+    {
+       last = elptr(web.skel[type].last);     /* find end of in-use chain */
+       last->forechain = newid;
+    }
+    else
+    {        
+       web.skel[type].used = newid;
+    }
+    web.skel[type].last = newid;
   }
 
   newptr->attr = ALLOCATED | NEWELEMENT;
 
   newptr->self_id = newid;
+  #ifdef MPI_EVOLVER
+  newptr->local_id = newid;
+  #endif
   web.skel[type].count++;  
+  
+  #ifdef MPI_EVOLVER
+  /* kludge to guarantee max_ord gives enough spaces */
+  if ( web.skel[type].count > web.skel[type].max_ord+1 )
+    web.skel[type].max_ord = web.skel[type].count-1;
+  #endif
 
-  /* inherited named methods from parent */
+  /* inherited attributes and named methods from parent */
   if ( valid_id(parent) )
   { struct element *e_ptr = elptr(parent);
-     int *instlist = (int*)((char*)e_ptr + get_meth_offset(id_type(parent)));
-     int i;
-     for ( i = 0 ; i < (int)e_ptr->method_count ; i++ )
-     { if ( METH_INSTANCE(abs(instlist[i]))->type <= type )
-          apply_method_num(newid,instlist[i]);
-     }
+    int *instlist = (int*)((char*)e_ptr + get_meth_offset(id_type(parent)));
+    int i;
+
+    newptr->attr |= e_ptr->attr & (NODISPLAY|FIXED|BOUNDARY|NEGBOUNDARY|
+        BDRY_ENERGY|CONSTRAINT|BDRY_CONTENT|NEGCONSTRAINT|BARE_NAKED);
+
+    if ( e_ptr->attr & CONSTRAINT )
+    { conmap_t *parent_conmap=NULL;
+      switch ( id_type(parent) )
+      { case VERTEX: parent_conmap = get_v_constraint_map(parent); break;
+        case EDGE  : parent_conmap = get_e_constraint_map(parent); break;
+        case FACET : parent_conmap = get_f_constraint_map(parent); break;
+      }
+      switch ( type )
+      { case VERTEX: set_v_conmap(newid,parent_conmap); break; 
+        case EDGE  : set_e_conmap(newid,parent_conmap); break; 
+        case FACET : set_f_conmap(newid,parent_conmap); break; 
+      }
+    }
+    if ( e_ptr->attr & BOUNDARY )
+    { int bnum=0;
+      switch ( id_type(parent) )
+      { case VERTEX: bnum = get_vertex_boundary_num(parent); break;
+        case EDGE  : bnum = get_edge_boundary_num(parent); break;
+        case FACET : bnum = get_facet_boundary_num(parent); break;
+      }
+      switch ( type )
+      { case VERTEX: set_boundary_num(newid,bnum); break;
+        case EDGE  : set_edge_boundary_num(newid,bnum); break;
+        case FACET : set_facet_boundary_num(newid,bnum); break;
+      }
+    }
+
+    for ( i = 0 ; i < (int)e_ptr->method_count ; i++ )
+    { if ( METH_INSTANCE(abs(instlist[i]))->type <= type )
+         apply_method_num(newid,instlist[i]);
+    }
   }
 
   return newid;
-}
+} /* end new_element() */
 
+/**************************************************************************
+*
+* function: free_element()
+*
+* purpose: mark element as unallocated, but leave it in the chain of
+*          used elements.  Moved to freelist later by free_discards.
+*/
 void free_element(id) 
 element_id id;
 {
   struct element *ptr;
   int type = id_type(id);
 
-  if ( !valid_id(id) )
+  if ( !valid_element(id) )
   {
     sprintf(errmsg,
-      "Internal error: Trying to free invalid element type %d id %d \n",
-            type,ordinal(id)+1);
+      "Internal error: Trying to free invalid element type %d id %s \n",
+            type,ELNAME(id));
     kb_error(1311,errmsg,WARNING);
     return;
   }
@@ -357,7 +438,7 @@ element_id id;
     remove_vertex_edge(tailv,id);
     remove_vertex_edge(headv,inverse_id(id));
     if ( web.modeltype == QUADRATIC )
-             free_element(get_edge_midv(id));
+       free_element(get_edge_midv(id));
     if ( web.modeltype == LAGRANGE )
     { vertex_id *v = get_edge_vertices(id);
       for ( i = 1 ; i < web.lagrange_order ; i++ )
@@ -365,23 +446,45 @@ element_id id;
     }
   }
     
-  if ( (type == FACET) && (web.representation==SIMPLEX) )
-  { /* remove from vertex lists */
-    int n,nmax = web.skel[FACET].ctrlpts;
-    vertex_id *v = get_facet_vertices(id);
-    for ( n = 0 ; n < nmax ; n++ )
-      remove_vertex_facet(v[n],id);
+  if ( type == FACET ) 
+  { 
+    /* remove from body facet lists */
+    set_facet_body(id,NULLID);
+    set_facet_body(inverse_id(id),NULLID);
+
+    /* remove from vertex lists */
+    if ( web.representation==SIMPLEX )
+    { int n,nmax = web.skel[FACET].ctrlpts;
+      vertex_id *v = get_facet_vertices(id);
+        for ( n = 0 ; n < nmax ; n++ )
+          remove_vertex_facet(v[n],id);
+    }
   }
   ptr = elptr(id);
   if ( !(ptr->attr & ALLOCATED) )
   { sprintf(errmsg,
-      "Internal error: Trying to free unallocated element type %d id %d \n",
-            type,ordinal(id)+1);
+      "Internal error: Trying to free unallocated element type %d id %s \n",
+            type,ELNAME(id));
     kb_error(1313,errmsg,WARNING);
     return;
   }
   ptr->attr &= ~ALLOCATED;
 
+  #ifdef MPI_EVOLVER
+  if ( id_machine(id) != this_machine )
+  { mpi_remove_remote(id);
+    if ( valid_id(ptr->forechain) )
+      elptr(ptr->forechain)->backchain = ptr->local_id;
+    else
+      web.skel[type].last = ptr->local_id;
+   if ( valid_id(ptr->backchain) )
+      elptr(ptr->backchain)->forechain = ptr->local_id;
+    else
+      web.skel[type].used = ptr->local_id;
+  }
+  #endif
+
+
 #ifdef OLDDISCARD
   /* remove from in-use list */
   if ( valid_id(ptr->forechain) )
@@ -431,6 +534,10 @@ element_id id;
   }
   ptr->attr |= ALLOCATED;
 
+  #ifdef MPI_EVOLVER
+  if ( id_machine(id) != this_machine )
+    mpi_unfree_element(id,ptr);
+  #endif
 
   web.skel[type].discard_count--;
 
@@ -440,14 +547,20 @@ element_id id;
 /* index as id */
 element_id get_ordinal_id(type,ord)
 int type; /* type of element */
-int ord;  /* ordinal of element */
+int ord;  /* ordinal of element, signed */
 { element_id id;
 
   if ( (type < 0) || (type > NUMELEMENTS) ) return NULLID;
-  if ( (ord < 0) || (ord > web.skel[type].max_ord)) return NULLID;
-  id = (type << TYPESHIFT) | VALIDMASK | ord;
-  if ( get_attr(id) & ALLOCATED ) return id;
-  else return NULLID;
+  if ( abs(ord) > web.skel[type].max_ord ) return NULLID;
+  id = ((element_id)type << TYPESHIFT) | VALIDMASK | abs(ord);
+  if ( ord < 0 ) invert(id);
+
+#ifdef MPI_EVOLVER
+  id |= this_machine << MACH_ID_SHIFT;
+#endif
+  if ( !(get_attr(id) & ALLOCATED) )
+     return NULLID;
+ return id;
 }
 
 int generate_all(type,idptr,sentinel)  /* re-entrant */
@@ -461,37 +574,70 @@ element_id *sentinel; /* to record original end of list */
   { *idptr = web.skel[type].used;
     *sentinel = web.skel[type].last;   /* may be a discard */
     if ( !valid_id(*idptr) ) return 0;
+    ptr = elptr(*idptr);
+    if ( (ptr->attr & ALLOCATED)
+    #ifdef MPI_EVOLVER
+     && (id_machine(ptr->self_id) == this_machine)
+    #endif
+     )
+      return 1;
   }
-  else
-  { ptr = elptr(*idptr);
+   
+    ptr = elptr(*idptr);
     do
-    { if ( equal_id(*idptr,*sentinel) ) return 0;
+      { if ( equal_id(*idptr,*sentinel) ) return 0;
       *idptr = ptr->forechain;
       if ( !valid_id(*idptr) ) return 0;
       ptr = elptr(*idptr);
     }
-    while ( !(ptr->attr & ALLOCATED) );
-  }
+    while ( !(ptr->attr & ALLOCATED)
+    #ifdef MPI_EVOLVER
+     || (id_machine(ptr->self_id) != this_machine)
+    #endif
+     );
+
 
   return 1;
 }
 
+/***************************************************************************
+* 
+* function: memory_report()
+*
+* purpose: implement 'c' command memory usage report
+*/
 void memory_report()
 {
     long mem;
     int k;
 
+    #ifdef MPI_EVOLVER
+    if ( this_machine == 0 )
+    { mpi_count_report();
+      return;
+    }
+    #endif
+
     mem = 0;
     for ( k = 0 ; k < NUMELEMENTS ; k++ )
       mem += web.skel[k].count*web.sizes[k];
 
     sprintf(errmsg,
-    "Vertices: %ld  Edges: %ld  Facets: %ld  Bodies: %ld  Facetedges: %ld\nElement memory: %ld  ",
-	web.skel[0].count,web.skel[1].count,web.skel[2].count,
-                web.skel[3].count,web.skel[4].count,
-      mem);
+    "Vertices: %ld  Edges: %ld  Facets: %ld  Bodies: %ld  Facetedges: %ld\nElement memory: %ld\n",
+                web.skel[0].count,web.skel[1].count,web.skel[2].count,
+                web.skel[3].count,web.skel[4].count, mem);
     outstring(errmsg);
 
+    if ( verbose_flag )
+    { /* report element sizes */
+      for ( k = 0 ; k < NUMELEMENTS ; k++ )
+      { sprintf(msg,"%10.10s size: %4d bytes\n",typenames[k],web.sizes[k]);
+        outstring(msg);
+      }
+      sprintf(msg,"quantity size:   %4d bytes\n",sizeof(struct gen_quant));
+      sprintf(msg,"instance size:   %4d bytes\n",sizeof(struct method_instance));
+    }
+
 #if defined(_WIN32) && defined(_HEAPOK)
 #ifndef _HEAPINFO
 #define _HEAPINFO _heapinfo
@@ -502,9 +648,10 @@ void memory_report()
     else if ( verbose_flag )
     { struct _HEAPINFO hinfo;
       int b_use=0,b_free=0;
-      long mem_use=0,mem_free=0;
+      size_t mem_use=0,mem_free=0;
       char * heaptop = NULL; 
       char * heapstart = NULL;
+      MEMORYSTATUSEX memstat;
 
       hinfo._pentry = NULL;
       while ( _heapwalk(&hinfo) == _HEAPOK )
@@ -537,6 +684,12 @@ void memory_report()
       sprintf(errmsg,"Heap size: %4.2f MB\n",
              (heaptop-heapstart)/1024./1024.);
       outstring(errmsg);
+
+      memstat.dwLength = sizeof(memstat);
+      GlobalMemoryStatusEx(&memstat);
+      sprintf(errmsg,"Physical memory size: %I64d bytes   Virtual memory top: %I64X\n",
+           memstat.ullTotalPhys,memstat.ullTotalVirtual);
+      outstring(errmsg);
     }
 #endif
 
@@ -592,7 +745,6 @@ void memory_report()
 void reset_skeleton()
 {
   int i;
-  int type;
   int three = FACET_VERTS;
   int permcount = web.perm_global_count;
   int maxperm   = web.max_perm_globals;
@@ -637,37 +789,56 @@ void reset_skeleton()
   /* have REAL attributes first, so can align on 8-byte */
   /* Be sure the order given here is same as order in skeleton.h */
   /* vertex */
-  add_attribute(VERTEX,"x",REAL_ATTR,1,NULL,0,NULL);
-  add_attribute(VERTEX,"__oldx",REAL_ATTR,1,NULL,0,NULL);
-  add_attribute(VERTEX,"p",REAL_ATTR,1,NULL,0,NULL);
-  add_attribute(VERTEX,"__force",REAL_ATTR,1,NULL,0,NULL);
-  add_attribute(VERTEX,"__velocity",REAL_ATTR,1,NULL,0,NULL);
-  add_attribute(VERTEX,"__v_boundary",INTEGER_ATTR,0,NULL,0,NULL);
-  add_attribute(VERTEX,"star",INTEGER_ATTR,0,NULL,0,NULL);
-  add_attribute(VERTEX,"__v_constraint_list",UINT_ATTR,1,NULL,0,NULL);
+  add_attribute(VERTEX,"x",REAL_TYPE,1,NULL,0,NULL);
+  add_attribute(VERTEX,"__oldx",REAL_TYPE,1,NULL,0,NULL);
+  add_attribute(VERTEX,"p",REAL_TYPE,1,NULL,0,NULL);
+  add_attribute(VERTEX,"__force",REAL_TYPE,1,NULL,0,NULL);
+  add_attribute(VERTEX,"__velocity",REAL_TYPE,1,NULL,0,NULL);
+  add_attribute(VERTEX,"__v_constraint_list",INTEGER_TYPE,1,NULL,0,NULL);
+  web.meth_attr[VERTEX] = 
+        add_attribute(VERTEX,"__v_method_list",INTEGER_TYPE,1,NULL,0,NULL);
   /* edge */
-  add_attribute(EDGE,"star",REAL_ATTR,0,NULL,0,NULL);
-  add_attribute(EDGE,"density",REAL_ATTR,0,NULL,0,NULL);
-  add_attribute(EDGE,"__e_vertices",ELEMENTID_ATTR,1,NULL,0,NULL);
-  add_attribute(EDGE,"__e_boundary",INTEGER_ATTR,0,NULL,0,NULL);
-  add_attribute(EDGE,"__wrap_list",INTEGER_ATTR,1,NULL,0,NULL);
-  add_attribute(EDGE,"__e_constraint_list",UINT_ATTR,1,NULL,0,NULL);
+  add_attribute(EDGE,"density",REAL_TYPE,0,NULL,0,NULL);
+  add_attribute(EDGE,"__e_vertices",ELEMENTID_TYPE,1,NULL,0,NULL);
+  add_attribute(EDGE,"__wrap_list",INTEGER_TYPE,1,NULL,0,NULL);
+  add_attribute(EDGE,"__e_constraint_list",INTEGER_TYPE,1,NULL,0,NULL);
+  web.meth_attr[EDGE] = 
+        add_attribute(EDGE,"__e_method_list",INTEGER_TYPE,1,NULL,0,NULL);
   /* facet */
-  add_attribute(FACET,"__f_constraint_list",UINT_ATTR,1,NULL,0,NULL);
-  add_attribute(FACET,"__f_vertices",ELEMENTID_ATTR,1,&three,0,NULL);
-  add_attribute(FACET,"__f_boundary",INTEGER_ATTR,0,NULL,0,NULL);
-  add_attribute(FACET,"tag",INTEGER_ATTR,0,NULL,0,NULL);
-  add_attribute(FACET,"__body_list",ELEMENTID_ATTR,1,NULL,0,NULL);
-  add_attribute(FACET,"__next_vfacet_list",ELEMENTID_ATTR,0,NULL,0,NULL);
-  add_attribute(FACET,"__next_bfacet_list",ELEMENTID_ATTR,0,NULL,0,NULL);
-  add_attribute(FACET,"phase",INTEGER_ATTR,0,NULL,0,NULL);
+  add_attribute(FACET,"__f_constraint_list",INTEGER_TYPE,1,NULL,0,NULL);
+  add_attribute(FACET,"__f_vertices",ELEMENTID_TYPE,1,&three,0,NULL);
+  add_attribute(FACET,"__body_list",ELEMENTID_TYPE,1,NULL,0,NULL);
+  add_attribute(FACET,"__next_vfacet_list",ELEMENTID_TYPE,0,NULL,0,NULL);
+  add_attribute(FACET,"__next_bfacet_list",ELEMENTID_TYPE,1,NULL,0,NULL);
+  web.meth_attr[FACET] = 
+        add_attribute(FACET,"__f_method_list",INTEGER_TYPE,1,NULL,0,NULL);
   /* body */
-  add_attribute(BODY,"__b_constraint_list",UINT_ATTR,1,NULL,0,NULL);
+  web.meth_attr[BODY] = 
+        add_attribute(BODY,"__b_method_list",INTEGER_TYPE,1,NULL,0,NULL);
+ 
 
-  /* all */
-  for ( type = 0 ; type < NUMELEMENTS ; type++ )
-     web.meth_attr[type] = 
-        add_attribute(type,"__method_list",INTEGER_ATTR,1,NULL,0,NULL);
+  F_TAG_ATTR = F_PHASE_ATTR = V_BOUNDARY_ATTR = E_BOUNDARY_ATTR = 
+    F_BOUNDARY_ATTR = 0;
+
+#ifdef MPI_EVOLVER
+  { int m = MPI_EXPORT_MAX;
+    web.mpi_export_attr[VERTEX] = 
+        add_attribute(VERTEX,"__mpi_export_v",USHORT_TYPE,1,&m,0,NULL);
+    web.mpi_export_attr[EDGE] = 
+        add_attribute(EDGE,"__mpi_export_e",USHORT_TYPE,1,&m,0,NULL);
+    web.mpi_export_attr[FACET] = 
+        add_attribute(FACET,"__mpi_export_f",USHORT_TYPE,1,&m,0,NULL);
+    web.mpi_export_attr[BODY] = 
+        add_attribute(BODY,"__mpi_export_b",USHORT_TYPE,1,&m,0,NULL);
+    web.mpi_export_attr[FACETEDGE] = 
+        add_attribute(FACETEDGE,"__mpi_export_fe",USHORT_TYPE,1,&m,0,NULL);
+    mpi_export_voffset = EXTRAS(VERTEX)[web.mpi_export_attr[VERTEX]].offset;
+    mpi_export_eoffset = EXTRAS(EDGE)[web.mpi_export_attr[EDGE]].offset;
+    mpi_export_foffset = EXTRAS(FACET)[web.mpi_export_attr[FACET]].offset;
+    mpi_export_boffset = EXTRAS(BODY)[web.mpi_export_attr[BODY]].offset;
+    mpi_export_feoffset = EXTRAS(FACETEDGE)[web.mpi_export_attr[FACETEDGE]].offset;
+  }
+#endif
 }
 
 /***********************************************************************
@@ -700,7 +871,11 @@ int mode; /* DISCARDS_ALL or DISCARDS_SOME */
         else web.skel[type].used = ptr->forechain;
         ptr->forechain = web.skel[type].free;
         ptr->backchain = NULLID;
+#ifdef MPI_EVOLVER
+        web.skel[type].free = ptr->local_id;
+#else
         web.skel[type].free = id;
+#endif
       }
       id = next_id;
     }
@@ -884,7 +1059,7 @@ void reorder_storage()
 #define copy_sign(id1,id2)     (((id1)&~SIGNMASK) | ((id2) & SIGNMASK))
 void renumber_all()
 { int type;
-  struct element *ep;
+  struct element *ep=NULL;
   element_id id,newid;
   int k;
   int dim,off,dima,offa,dimb,offb,dimc,offc;
@@ -1027,9 +1202,9 @@ void renumber_all()
 
   /* miscellaneous elements */
   if ( valid_id(web.zoom_v) ) web.zoom_v = vptr(web.zoom_v)->self_id;
-  if ( pickvnum ) pickvnum = ordinal(vibase[pickvnum-1]->self_id)+1;
-  if ( pickenum ) pickenum = ordinal(eibase[pickenum-1]->self_id)+1;
-  if ( pickfnum ) pickfnum = ordinal(fibase[pickfnum-1]->self_id)+1;
+  if ( pickvnum ) pickvnum = loc_ordinal(vibase[pickvnum-1]->self_id)+1;
+  if ( pickenum ) pickenum = loc_ordinal(eibase[pickenum-1]->self_id)+1;
+  if ( pickfnum ) pickfnum = loc_ordinal(fibase[pickfnum-1]->self_id)+1;
 
   /* swap ibase to new */
   for ( type = VERTEX ; type <= FACETEDGE ; type++ )
@@ -1150,13 +1325,127 @@ int inx;
   }
 
   switch ( ext->type )
-  { case REAL_ATTR: return ((REAL*)get_extra_ptr(id,ext))[inx];
-    case INTEGER_ATTR: return (REAL)(((int*)get_extra_ptr(id,ext))[inx]);
-    case ULONG_ATTR: 
-      return (REAL)(((unsigned long*)get_extra_ptr(id,ext))[inx]);
-    case UCHAR_ATTR: 
+  { case REAL_TYPE: return ((REAL*)get_extra_ptr(id,ext))[inx];
+    case INTEGER_TYPE: return (REAL)(((int*)get_extra_ptr(id,ext))[inx]);
+    case UINT_TYPE: return (REAL)(((unsigned int*)get_extra_ptr(id,ext))[inx]);
+    case UCHAR_TYPE: 
       return (REAL)(((unsigned char*)get_extra_ptr(id,ext))[inx]);
+    case CHAR_TYPE: 
+      return (REAL)(((char*)get_extra_ptr(id,ext))[inx]);
+    case USHORT_TYPE: 
+      return (REAL)(((unsigned short int*)get_extra_ptr(id,ext))[inx]);
+    case SHORT_TYPE: 
+      return (REAL)(((short int*)get_extra_ptr(id,ext))[inx]);
+    case ULONG_TYPE: 
+      return (REAL)(((unsigned long int*)get_extra_ptr(id,ext))[inx]);
+    case LONG_TYPE: 
+      return (REAL)(((long int*)get_extra_ptr(id,ext))[inx]);
+    case PTR_TYPE: 
+      return (REAL)(unsigned long int)(((char**)get_extra_ptr(id,ext))[inx]);
+    default: kb_error(3101,
+      "get_extra_attribute_value() trying to get nonumeric type.\n",
+           RECOVERABLE);
+     break;
   }
   return 0.0;  /* shouldn't get here. */
 }
 
+/************************************************************************
+*
+* Function: int32hash()
+*
+* Purpose: hash 32 bits to 32 bits
+*/
+
+unsigned int int32hash(key)
+unsigned int key;
+{
+  key += ~(key << 15);
+  key ^=  (key >> 10);
+  key +=  (key << 3);
+  key ^=  (key >> 6);
+  key += ~(key << 11);
+  key ^=  (key >> 16);
+  return key;
+}
+
+/************************************************************************
+*
+* Function: int64hash()
+*
+* Purpose: hash 64 bits to 64 bits
+* Integer hashing functions from Thomas Wang,
+* http://www.concentric.net/~Ttwang/tech/inthash.htm
+*/
+
+#ifdef NOLONGLONG
+unsigned long int64hash(key)
+unsigned long key;
+#else
+unsigned long long int64hash(key)
+unsigned long long key;
+#endif
+{
+  key += ~(key << 32);
+  key ^= (key >> 22);
+  key += ~(key << 13);
+  key ^= (key >> 8);
+  key += (key << 3);
+  key ^= (key >> 15);
+  key += ~(key << 27);
+  key ^= (key >> 31);
+  return key;
+}
+
+
+
+#ifdef MPI_EVOLVER
+int mpi_remote_present(id)
+element_id id;
+{ struct element *eptr = mpi_remote_elptr(id);
+  if ( eptr == NULL ) 
+    return 0;
+  if ( eptr->attr & ALLOCATED ) 
+    return 1;
+  return 0; 
+}
+#endif
+
+/************************************************************************
+*
+* function: valid_element()
+*
+* purpose: To see if the element referred to by an element id is
+*          actually present and valid.
+*/
+
+int valid_element(id)
+element_id id;
+{ struct element *ep;
+  if ( !valid_id(id) ) return 0;
+  ep = elptr(id);
+  if ( ep == NULL ) return 0;
+  if ( ep->attr & ALLOCATED ) return 1;
+  return 0;
+}
+
+
+/************************************************************************
+*
+* function: binary_tree_add()
+*
+* purpose: implements binary tree addition for accurate sum of large
+*          number of addends.
+*/
+
+void binary_tree_add(addends,term)
+REAL *addends;
+REAL term;
+{ int i;
+  for ( i = 0 ; addends[i] != 0.0 ; i++ )
+  { term += addends[i];
+    addends[i] = 0.0;
+  }
+  addends[i] = term;
+}
+
