@@ -230,19 +230,22 @@ facetedge_id fe_id;
   REAL *xt,*xh;
   REAL area;
   body_id b_id1,b_id2; /* bodies to add area to */
+  facet_id f_id1,f_id2; /* facets to add area to */
   edge_id e_id = get_fe_edge(fe_id);
 
   if ( get_eattr(e_id) & NONCONTENT ) return;
 
   if ( inverted(e_id) ) { fe_id = inverse_id(fe_id); e_id = inverse_id(e_id); }
 
-  b_id1 = get_facet_body(get_fe_facet(fe_id));
-  b_id2 = get_facet_body(facet_inverse(get_fe_facet(fe_id)));
-  if ( !valid_id(b_id1) && !valid_id(b_id2) ) return; /* no bodies */
+  f_id1 = get_fe_facet(fe_id);
+  f_id2 = facet_inverse(get_fe_facet(fe_id));
+  if ( !valid_id(f_id1) && !valid_id(f_id2) ) return; /* no facets */
+  b_id1 = get_facet_body(f_id1);
+  b_id2 = get_facet_body(f_id2);
 
   if ( web.torus_flag )
   { REAL v1,v2,v3,v4;
-    int wx=0,wy=0; /* wraps */
+    int wx,wy; /* wraps */
     WRAPTYPE wrap = get_edge_wrap(e_id);
   
      /* new way, accurate modulo unit cell area */
@@ -255,16 +258,9 @@ facetedge_id fe_id;
     v3 = dot(web.inverse_periods[1],xt,2);
     v4 = dot(web.inverse_periods[1],xh,2);
 
-    switch ( wrap & WRAPMASK )
-    { 
-       case POSWRAP: wx = 1 ; break;
-       case NEGWRAP: wx = -1; break;
-    }
-    switch ( (wrap>>TWRAPBITS) & WRAPMASK )
-    { 
-       case POSWRAP: wy = 1 ; area = v2+wx; break;
-       case NEGWRAP: wy = -1; area = -(v2+wx); break;
-    }
+    wx = WRAPNUM(wrap & WRAPMASK);
+    wy = WRAPNUM((wrap>>TWRAPBITS) & WRAPMASK);
+    area += wy*(v2+wx);
      
     area -= ((v2+wx - v1)*(v4+wy + v3)/2 );
     area *= web.torusv;
@@ -279,6 +275,9 @@ facetedge_id fe_id;
   }
 
   /* add to cell areas */
+  if ( valid_id(f_id1) )
+     add_facet_area(f_id1,area);
+  
   if ( valid_id(b_id1) )
      add_body_volume(b_id1,area);
   
@@ -309,11 +308,6 @@ void string_grad_l()
 
   FOR_ALL_FACETEDGES(fe_id)
   {
-    e_id = get_fe_edge(fe_id);
-    if ( get_eattr(e_id) & NONCONTENT ) continue;
-
-    get_edge_side(e_id,side);
-
     /* see which side has body, if any */
     bi_id = get_facet_body(get_fe_facet(fe_id));
     if ( !valid_id(bi_id) ) 
@@ -322,7 +316,13 @@ void string_grad_l()
     }
     if ( !valid_id(bi_id) ) continue;  /* nothing here */
     if ( !(get_battr(bi_id) & (FIXEDVOL|PRESSURE) ) ) continue;
+   
+	e_id = get_fe_edge(fe_id);
+    if ( get_eattr(e_id) & NONCONTENT ) continue;
 
+    get_edge_side(e_id,side);
+
+ 
     headv = get_fe_headv(fe_id);
     tailv = get_fe_tailv(fe_id);
     xt = get_coord(tailv);
@@ -352,20 +352,11 @@ void string_grad_l()
       v3 = dot(web.inverse_periods[1],xt,2);
       v4 = dot(web.inverse_periods[1],xh,2);
 
-      switch ( wrap & WRAPMASK )
-      { case POSWRAP: wx = 1 ; break;
-        case NEGWRAP: wx = -1; break;
-      }
-      switch ( (wrap>>TWRAPBITS) & WRAPMASK )
-      { case POSWRAP: wy = 1 ; 
-            for ( i = 0 ; i < SDIM ; i++ )
-              hgrad[i] += web.inverse_periods[0][i];
-            break;
-        case NEGWRAP: wy = -1; 
-            for ( i = 0 ; i < SDIM ; i++ )
-              hgrad[i] += -web.inverse_periods[0][i];
-            break;
-      }
+      wx = WRAPNUM(wrap & WRAPMASK);
+      wy = WRAPNUM((wrap>>TWRAPBITS) & WRAPMASK);
+      if ( wy )
+        for ( i = 0 ; i < SDIM ; i++ )
+          hgrad[i] += wy*web.inverse_periods[0][i];
      
       for ( i = 0 ; i < SDIM ; i++ )
       { hgrad[i] -= (v4+wy + v3)/2*web.inverse_periods[0][i];
