@@ -177,8 +177,8 @@ void expand_global_hash ARGS((void));
 int global_hash(name)
 char *name;
 { int hashval = 0;  
-  int len = strlen(name);
-  int i;
+  size_t len = strlen(name);
+  size_t i;
  
   for ( i = 0; i < len ; i++ )
     hashval = (hashval + tolower(name[i])) * HASHSTRIDE;
@@ -544,9 +544,9 @@ int perm_rewind_spot; /* where to rewind to */
    Each list is tree-linked for block scoping.
 **************************************************************************/
 
-struct locallist_t *local_scope_bases[100]; /* for reference during parsing */
+struct locallist_t *local_scope_bases[LOCALSCOPEMAX]; /* for reference during parsing */
 int local_nest_depth;
-struct locallist_t query_locals; /* for unnamed commands */
+struct locallist_t *query_locals; /* for unnamed commands */
 struct locallist_t *localbase;
 
 /**************************************************************************
@@ -560,15 +560,18 @@ void init_local_scope(iid)
 ident_t iid;  /* identifier of procedure to begin new scope */
 {
   struct locallist_t *locals;
-  struct global *proc;  
+  struct global *proc=NULL;  
     
   if ( iid )
   { proc = globals(iid);    
     locals = proc->attr.procstuff.locals; 
   } 
-  else locals = &query_locals;
+  else 
+  { locals = query_locals;
+    /*locals->flags |= LL_PERMANENT; *//* potentially execute a permanent command */
+  }
     
-  if ( locals )
+  if ( locals && valid_id(iid) )
   { /* free old stuff */
     if ( locals->list ) 
     { if ( locals->flags & LL_PERMANENT ) free((char*)locals->list);
@@ -576,23 +579,27 @@ ident_t iid;  /* identifier of procedure to begin new scope */
     }
     memset(locals,0,sizeof(struct locallist_t));
   }
-  else
+  else if ( locals == NULL )
   { locals = (iid & PERMGLOBAL ) ?
        (struct locallist_t *)calloc(1,sizeof(struct locallist_t)) :
-       (struct locallist_t *)kb_calloc(1,sizeof(struct locallist_t)) ;
-    if ( iid ) proc->attr.procstuff.locals = locals;
-    if ( iid & PERMGLOBAL ) locals->flags |= LL_PERMANENT;
+       (struct locallist_t *)mycalloc(1,sizeof(struct locallist_t)) ;
+    if ( iid )   
+      proc->attr.procstuff.locals = locals;
+    if ( iid & PERMGLOBAL ) 
+      locals->flags |= LL_PERMANENT;
   }
 
+  if ( local_nest_depth < 0 ) 
+     local_nest_depth = 0;
   local_scope_bases[local_nest_depth] = locals;
   local_nest_depth++;
   
-  locals->iid = iid;
+  locals->iid = iid; 
   locals->maxlist = 8;
-  locals->list = (iid & PERMGLOBAL ) ?
+  locals->list = (locals->flags & LL_PERMANENT ) ?
       (struct localvar_t*)calloc(locals->maxlist,
                                  sizeof(struct localvar_t)) :
-      (struct localvar_t*)kb_calloc(locals->maxlist,
+      (struct localvar_t*)mycalloc(locals->maxlist,
                                  sizeof(struct localvar_t)) ;
   localbase = locals;
   locals->list[0].prev = -1;
@@ -624,7 +631,9 @@ void begin_local_scope()
  
   if ( local_nest_depth == 0 )
   { /* coming in from unnamed command */
-    init_local_scope(0);
+    /*init_local_scope(0); */
+	  kb_error(3501,"INTERNAL ERROR: local_scope not initialized.\n",
+		   RECOVERABLE);
   }
     
   locals = local_scope_bases[local_nest_depth-1]; 
@@ -644,6 +653,7 @@ void end_local_scope()
   int n;
   
   locals = local_scope_bases[local_nest_depth-1]; 
+  if ( !locals ) return;
   
   if ( locals->list )
   {  
@@ -700,10 +710,9 @@ char *name; /* NULL if just reserving space for anonymous value */
      (struct localvar_t*)realloc((char*)locals->list,
       locals->maxlist*2*sizeof(struct localvar_t))  :
     (struct localvar_t*)kb_realloc((char*)locals->list,
-      locals->maxlist*2*sizeof(struct localvar_t),
-      locals->maxlist*sizeof(struct localvar_t));
+      locals->maxlist*2*sizeof(struct localvar_t));
     locals->maxlist*=2;
-    v = locals->list + locals->count;   // reset dangling pointer
+    v = locals->list + locals->count;   /* reset dangling pointer */
   }
  
   /* move counter up */
@@ -728,14 +737,14 @@ char *name;
   struct localvar_t *v;
   struct locallist_t *locals;
    
-  if ( local_nest_depth == 0 ) return 0; 
+  if ( local_nest_depth <= 0 ) return 0; 
   locals = local_scope_bases[local_nest_depth-1]; 
   if ( locals->list == NULL ) return 0;
   
   v = locals->list + locals->count;
 
   /* search for duplicate in current scope */
-  for ( n = v->prev ; n >= 0 ; n = locals->list[n].prev )
+  for ( n = v->prev ; n >= 0 ; n = ( n ? locals->list[n].prev : -1) )
     if ( strcmp(name,locals->list[n].g.name) == 0 )
     { return LOCALVAR | n;
     }
@@ -761,7 +770,7 @@ void initialize_perm_globals()
   g->value.arrayptr->dim = 1;
   g->attr.varstuff.dim = 1;
   g->value.arrayptr->itemsize = sizeof(int);
-  g->value.arrayptr->datatype = INTEGER_TYPE_;
+  g->value.arrayptr->datatype = INTEGER_TYPE;
   g->value.arrayptr->datasize = 0;
   g->value.arrayptr->size[0] = 0;
   g->value.arrayptr->datastart = 0; 
@@ -775,7 +784,7 @@ void initialize_perm_globals()
   g->value.arrayptr->dim = 3;
   g->attr.varstuff.dim = 3;
   g->value.arrayptr->itemsize = sizeof(REAL);
-  g->value.arrayptr->datatype = REAL_TYPE_;
+  g->value.arrayptr->datatype = REAL_TYPE;
   g->value.arrayptr->datasize = 0;
   g->value.arrayptr->size[0] = 0;
   g->value.arrayptr->size[1] = 0;
@@ -791,7 +800,7 @@ void initialize_perm_globals()
   g->value.arrayptr->dim = 2;
   g->attr.varstuff.dim = 2;
   g->value.arrayptr->itemsize = sizeof(REAL);
-  g->value.arrayptr->datatype = REAL_TYPE_;
+  g->value.arrayptr->datatype = REAL_TYPE;
   g->value.arrayptr->datasize = 0;
   g->value.arrayptr->size[0] = 0;
   g->value.arrayptr->size[1] = 0;
@@ -807,7 +816,7 @@ void initialize_perm_globals()
   g->value.arrayptr->dim = 2;
   g->attr.varstuff.dim = 2;
   g->value.arrayptr->itemsize = sizeof(REAL);
-  g->value.arrayptr->datatype = REAL_TYPE_;
+  g->value.arrayptr->datatype = REAL_TYPE;
   g->value.arrayptr->size[0] = 0;
   g->value.arrayptr->size[1] = 0;
   g->value.arrayptr->datasize = 0;
@@ -821,7 +830,7 @@ void initialize_perm_globals()
   g->value.arrayptr->dim = 2;
   g->attr.varstuff.dim = 2;
   g->value.arrayptr->itemsize = sizeof(REAL);
-  g->value.arrayptr->datatype = REAL_TYPE_;
+  g->value.arrayptr->datatype = REAL_TYPE;
   g->value.arrayptr->datasize = 0;
   g->value.arrayptr->size[0] = 0;
   g->value.arrayptr->size[1] = 0;
@@ -854,7 +863,7 @@ int count;
   g->value.arrayptr->size[0] = count;
   g->value.arrayptr->datasize = count;
   g->value.arrayptr->datastart = count ?
-        (char*)transform_colors - (char*)(g->value.arrayptr) : 0;
+        ((char*)transform_colors - (char*)(g->value.arrayptr)) : 0;
 }
 
 /*************************************************************************
