@@ -12,13 +12,14 @@ int crit_count; /* debugging */
 *  Contents:  main() for Surface Evolver
 */
 
-#ifdef WIN32
+#if defined(WIN32) && !defined(WINVER)
 #define _WIN32_WINNT 0x0500
 #define _WIN32_WINDOWS 0x0410
 #define WINVER 0x0400
-#include <windows.h>
 #endif
+#include "include.h"
 
+int just_lex_flag = 0;  /* for -l option */
 
 /***********************************************************************/
 /* System-specific thread stuff is all in this file.                   */
@@ -60,7 +61,7 @@ int spincount = 0;  /* times to try spinlock before sleeping */
 /* End of thread stuff.                                                */
 /***********************************************************************/
 
-#include "include.h"
+
 
 /* Separate thread for periodically printing state; useful when deadlocked */
 int web_locks;
@@ -78,7 +79,7 @@ void * reporter_thread(void * arg)
  for (;;)
  {
 #ifdef WIN32
-	 Sleep(10000);
+     Sleep(10000);
 #else
   sleep(10);
 #endif
@@ -120,22 +121,17 @@ void m_set_idlist()
 { proc_ids[GET_THREAD_ID] = getpid(); }
 #endif
 
-#if defined(PVM) || defined(MPI)
+#if defined(MPI_EVOLVER)
 #define main old_main 
 #endif
-
-int main(argc,argv) 
+ 
+int main(argc,argv)  
 int argc;
-char *argv[];
-
+char *argv[]; 
 { 
-  msgmax = 200;
-  msg = list_calloc(1,msgmax,ETERNAL_BLOCK); 
-
-#ifdef CLK_TCKq
- */ 
-#endif         
-
+  msgmax = 200; 
+  msg = my_list_calloc(1,msgmax,ETERNAL_BLOCK); 
+  set_ctypes();
 
   outfd = stdout;  
   sprintf(msg,"Surface Evolver %s\n\n",VERSION);
@@ -145,15 +141,25 @@ char *argv[];
   outstring(msg);
 #endif
 
+ if ( sizeof(element_id) > sizeof(REAL) )
+    kb_error(3102,"Bad datatype sizes: element_id is %d bytes, but REAL is %d.\n",
+      UNRECOVERABLE);
+      
 #ifdef _HISTORY_H_
  /* readline history initialization */
- using_history();
+ using_history(); 
 #endif
 
 #ifdef SGI_MULTI
   procs_requested = 1;  /* default at John's request.  m_get_numprocs(); */
 #endif
 
+#ifdef LONG_ID
+id_hash = int64hash;
+#else 
+id_hash = int32hash;
+#endif
+
 #ifdef WIN32
 /* Mutex for use between graph thread and main thread. */
 /* Nameless to prevent interference between separate processes. */
@@ -172,6 +178,8 @@ char *argv[];
     machine_eps = 2.0*eps;
   }
 
+  find_cpu_speed();
+
   initialize_perm_globals(); /* put some internal variable names in
      permanent symbol table */
   
@@ -182,9 +190,14 @@ char *argv[];
      while (  argc && (argv[0] != NULL) && (argv[0][0] == '-') )
      { switch ( argv[0][1] )
        {  
+          case 'E': err_tok_gen_flag = 1;
+                    break;
           case 'a': auto_convert_flag = (argv[0][2]=='-') ? 0 : 1; break;
           case 'q': option_q = (argv[0][2]=='-') ? 0 : 1; break; 
           case 'e': echo_flag = 1; break;
+#ifdef MPI_EVOLVER
+          case 'z': mpi_debug = 1; break;
+#endif
           case 't': tty_flag = 1;
                 break;
           case 'u': tty_flag = 1; 
@@ -229,6 +242,7 @@ char *argv[];
           case 'w' : exit_after_warning = exit_after_error = 1; break;
           case 'y' : break_after_warning = 1; break;
           case 'm' : memdebug = 1; break;
+          case 'l' : just_lex_flag = 1; break;
           default:
                  sprintf(msg,"Illegal option: %s\n",argv[0]); outstring(msg);
           case 'h' :
@@ -306,9 +320,9 @@ char *argv[];
       InitializeCriticalSection(&element_cs); 
       InitializeCriticalSection(&web_cs); 
     }
-    threadlist = (HANDLE*)list_calloc(procs_requested,sizeof(HANDLE),
+    threadlist = (HANDLE*)my_list_calloc(procs_requested,sizeof(HANDLE),
                        ETERNAL_BLOCK);
-    thread_data_ptrs = (struct thread_data**)list_calloc(procs_requested,
+    thread_data_ptrs = (struct thread_data**)my_list_calloc(procs_requested,
                       sizeof(struct thread_data*),ETERNAL_BLOCK);
     busythreads = procs_requested;
     barrier_count = procs_requested;
@@ -316,7 +330,7 @@ char *argv[];
     for ( i = 0 ; i < procs_requested ; i++ )
     { int tid;  /* dummy for thread id */
       thread_data_ptrs[i] = 
-           (struct thread_data *)list_calloc(1,sizeof(struct thread_data),
+           (struct thread_data *)my_list_calloc(1,sizeof(struct thread_data),
                  ETERNAL_BLOCK);
       thread_data_ptrs[i]->worker_id = i;
       threadlist[i] = CreateThread(NULL,0,winthread_worker,
@@ -431,8 +445,15 @@ char *argv[];
   }
   datafile_flag = 0;
 
+#ifdef MPI_EVOLVER
+  MPI_Barrier(MPI_COMM_WORLD);
+  if ( this_machine > 0 )
+    return 0;   /* return to mpi main() */
+  calc_energy();  /* initial energy */
+#endif
+
   while ( setjmp(jumpbuf) != 0 );    /* return here after commandfile  errors */
-  exec_commands(NULL);    /* command read and execute loop */
+  exec_commands(NULL,"Enter command: ");  /* command read and execute loop */
   my_exit(0);
 
   return 0; /* success return code */
@@ -461,12 +482,8 @@ int code;
      }
 #endif
 
-#if defined(PVM)
-  my_pvm_exit();
-#endif
-
-#if defined(MPI)
-  my_pvm_exit();
+#if defined(MPI_EVOLVER)
+  mpi_my_exit();
 #endif
 
 
@@ -481,24 +498,42 @@ int code;
 *              pops command file stack whenever end of file.
 */
 
-void exec_commands(basefd)
+void exec_commands(basefd,promptstring)
 FILE *basefd;  /* stop when get back to this input source */
+char *promptstring;  /* prompt to use */
 { /* main event loop of program */
   while ( commandfd && (commandfd != basefd))
   {
     char response[200];
 
     datafile_flag = 0;  /* in case of error bailout sometimes */
-    temp_free_all(); /* stray memory blocks */
-    free_discards(DISCARDS_ALL); /* from previous cycle */
-    memset(response,0,sizeof(response));
-    if ( prompt("Enter command: ",response,sizeof(response)) == EOF ) 
+    if ( !subshell_flag )
+    {
+      temp_free_all(); /* stray memory blocks */
+      free_discards(DISCARDS_ALL); /* from previous cycle */
+      memset(response,0,sizeof(response));
+    }
+    if ( prompt(promptstring,response,sizeof(response)) == EOF ) 
       pop_commandfd();
     else 
-      old_menu(response);
+      if ( old_menu(response) == END_COMMANDS )
+         break;
   }
 }
 
+/********************************************************************
+*
+* function: mac_exec_commands()
+*
+* purpose: wrapper for exec_commands so Mac main thread (which is
+*          the draw thread) can call with one argument.
+*/
+void *mac_exec_commands(arg)
+FILE *arg;
+{ exec_commands(arg,"Enter command: ");
+  return NULL;
+}
+
 /********************************************************************
 *
 *  function: exec_file()
@@ -551,6 +586,9 @@ char *file_name;  /* NULL if need to ask for name */
 file_retry:
   if ( name == NULL )
   { char *c;
+#ifdef MPI_EVOLVER
+printf("Task %d: ",this_machine);
+#endif
     prompt("Enter new datafile name (none to continue, q to quit): ",
        response,sizeof(response));
     c = strchr(response,'\n');
@@ -576,7 +614,11 @@ file_retry:
      return; /* continue with old */
   }
 #ifdef WIN32
+#ifdef MPI_EVOLVER
+  sprintf(msg,"Surface Evolver MPI - %s",name);
+#else
   sprintf(msg,"Surface Evolver - %s",name);
+#endif
   SetConsoleTitle(msg);
 #endif
 
@@ -594,8 +636,17 @@ if ( heapcheck() < 0 )
 #endif
 
   datafile_flag = 1;  /* so parser knows */
+  datafile_input_flag = 1;  /* so lex input knows */
   cmdptr = 0;
 
+  if ( just_lex_flag )
+  { int tokencount = 0;
+    while ( kb_yylex(NULL) )
+      tokencount++;
+    printf("tokencount %d\n",tokencount);
+    exit(0);
+  }
+
   initialize();
 
   datafile_flag = 0;
@@ -609,12 +660,12 @@ if ( heapcheck() < 0 )
     for ( sum = 0.0, i = 0 ; i < 3 ; i++ )
       for ( j = 0 ; j < 3 ; j++ ) sum += fabs(view[i][j]);
     overall_size = 3/sum;
+    dont_resize_flag = 1;
   }
   else
   { 
     resize(); 
   }
-  LEAVE_GRAPH_MUTEX;
 
   if ( parse_errors ) 
   { while ( (read_depth > 1) ) pop_commandfd();
@@ -622,13 +673,16 @@ if ( heapcheck() < 0 )
   }
 
   run_checks();
-  calc_content(Q_FIXED|Q_INFO|Q_RENORMALIZE);
+  
+  #ifndef MPI_EVOLVER
+  calc_content(Q_ENERGY|Q_FIXED|Q_INFO|Q_RENORMALIZE);
   if ( web.torus_flag ) fix_volconst();
   calc_pressure();
   calc_energy();  /* just to get initial total area */
   target_length = web.total_area; /* for square curvature string model */
   if ( OOGL_flag ) ask_wrap_display();
   update_display();
+  #endif 
 }
 
 #ifdef THREADS
@@ -651,10 +705,16 @@ int thread_tasknum;
 { 
   struct thread_data *data = GET_THREAD_DATA;
 __int32 task_caller_elapsed_time[2];
-char *taskname;
+__int32 now[2];
 task_caller_elapsed_time[0] = 0;
 task_caller_elapsed_time[1] = 0;
 data->task_state = 1;
+
+PROF_NOW(now);
+#ifdef _MSC_VER
+data->stagestart[0] = *(__int64*)now;
+#endif
+
 PROF_START(task_caller)
   switch ( thread_tasknum )
     {
@@ -668,12 +728,15 @@ PROF_START(task_caller)
       case TH_MULTI_QUANT_GRADS: m_calc_quant_grads(m_type); break;
       case TH_FIX_GRADS: m_fix_grads(); break;
       case TH_MULTI_QUANT_HESS: m_calc_quant_hess(m_type,m_mode,m_rhs); break;
+      case TH_MOVE_VERTICES: thread_move_vertices(); break;
       default: sprintf(errmsg,"Invalid thread task: %d\n",thread_tasknum);
                kb_error(2192,errmsg,UNRECOVERABLE);
     }
 PROF_FINISH(task_caller)
+
+#ifdef  ZZZZZ
 if(verbose_flag)
-{
+{ char *taskname;
   switch ( thread_tasknum )
     {
       case TH_PROJECT_ALL_TEST: taskname="thread_project_all"; break;
@@ -690,7 +753,10 @@ if(verbose_flag)
 printf("%30s",taskname);
 PROF_PRINT(task_caller)
 }
-data->task_state = 2;
+#endif
+
+  data->task_state = 2;
+
 }
 #endif
 
@@ -710,7 +776,7 @@ DWORD WINAPI winthread_worker( void * arg )
   /* set per-thread data */
   TlsSetValue(thread_data_key,(void*)data);
  
-  /* Set affinity mask.  Actually seems to hurt. */
+  /* Set affinity mask.  Actually seems to hurt. */ 
   /*
   SetThreadAffinityMask(threadlist[data->worker_id],1<<data->worker_id);
   */
@@ -825,45 +891,45 @@ void thread_stage_setup()
   v_partition_coord_attr = find_attribute(VERTEX,"v_partition_coord");
   if ( v_partition_coord_attr < 0 )
   { v_partition_coord_attr = add_attribute(VERTEX,"v_partition_coord",
-        REAL_ATTR,0,&one,0,NULL);
+        REAL_TYPE,0,&one,0,NULL);
   }
 
-  /* default for now: use z coordinate as partition coordinate */
+  /* default for now: use x coordinate as partition coordinate */
   FOR_ALL_VERTICES(v_id)
-  { *((REAL*)(get_extra(v_id,v_partition_coord_attr))) = get_coord(v_id)[SDIM-1];
+  { *((REAL*)(get_extra(v_id,v_partition_coord_attr))) = get_coord(v_id)[0];
   }
   
   v_partition_stage_attr = find_attribute(VERTEX,"v_partition_stage");
   if ( v_partition_stage_attr < 0 )
   { v_partition_stage_attr = add_attribute(VERTEX,"v_partition_stage",
-        INTEGER_ATTR,0,&one,0,NULL);
+        INTEGER_TYPE,0,&one,0,NULL);
   }
   v_partition_proc_attr = find_attribute(VERTEX,"v_partition_proc");
   if ( v_partition_proc_attr < 0 )
   { v_partition_proc_attr = add_attribute(VERTEX,"v_partition_proc",
-        INTEGER_ATTR,0,&one,0,NULL);
+        INTEGER_TYPE,0,&one,0,NULL);
   }
   
   e_partition_stage_attr = find_attribute(EDGE,"e_partition_stage");
   if ( e_partition_stage_attr < 0 )
   { e_partition_stage_attr = add_attribute(EDGE,"e_partition_stage",
-        INTEGER_ATTR,0,&one,0,NULL);
+        INTEGER_TYPE,0,&one,0,NULL);
   }
   e_partition_proc_attr = find_attribute(EDGE,"e_partition_proc");
   if ( e_partition_proc_attr < 0 )
   { e_partition_proc_attr = add_attribute(EDGE,"e_partition_proc",
-        INTEGER_ATTR,0,&one,0,NULL);
+        INTEGER_TYPE,0,&one,0,NULL);
   }
   
   f_partition_stage_attr = find_attribute(FACET,"f_partition_stage");
   if ( f_partition_stage_attr < 0 )
   { f_partition_stage_attr = add_attribute(FACET,"f_partition_stage",
-        INTEGER_ATTR,0,&one,0,NULL);
+        INTEGER_TYPE,0,&one,0,NULL);
   }
   f_partition_proc_attr = find_attribute(FACET,"f_partition_proc");
   if ( f_partition_proc_attr < 0 )
   { f_partition_proc_attr = add_attribute(FACET,"f_partition_proc",
-        INTEGER_ATTR,0,&one,0,NULL);
+        INTEGER_TYPE,0,&one,0,NULL);
   }
   
   /* Find percentiles, for load balancing */
@@ -909,8 +975,7 @@ void thread_stage_setup()
          { int newalloc = th->allocated[VERTEX][stage] + 10
                      + 2*web.skel[VERTEX].count/(nprocs*max_thread_stages);
            th->blocks[VERTEX][stage] = (element_id *)kb_realloc(
-                (char*)(th->blocks[VERTEX][stage]),newalloc*sizeof(element_id),
-                    th->allocated[VERTEX][stage]*sizeof(element_id));
+                (char*)(th->blocks[VERTEX][stage]),newalloc*sizeof(element_id));
            th->allocated[VERTEX][stage] = newalloc;
          }
          th->blocks[VERTEX][stage][th->counts[VERTEX][stage]++] = v_id;
@@ -948,8 +1013,7 @@ void thread_stage_setup()
          { int newalloc = th->allocated[EDGE][stage] + 10
                      + 2*web.skel[EDGE].count/(nprocs*max_thread_stages);
            th->blocks[EDGE][stage] = (element_id *)kb_realloc(
-                (char*)(th->blocks[EDGE][stage]),newalloc*sizeof(element_id),
-                    th->allocated[EDGE][stage]*sizeof(element_id));
+                (char*)(th->blocks[EDGE][stage]),newalloc*sizeof(element_id));
            th->allocated[EDGE][stage] = newalloc;
          }
          th->blocks[EDGE][stage][th->counts[EDGE][stage]++] = e_id;
@@ -995,8 +1059,7 @@ void thread_stage_setup()
          { int newalloc = th->allocated[FACET][stage] + 10
                      + 2*web.skel[FACET].count/(nprocs*max_thread_stages);
            th->blocks[FACET][stage] = (element_id *)kb_realloc(
-                (char*)(th->blocks[FACET][stage]),newalloc*sizeof(element_id),
-                    th->allocated[FACET][stage]*sizeof(element_id));
+                (char*)(th->blocks[FACET][stage]),newalloc*sizeof(element_id));
            th->allocated[FACET][stage] = newalloc;
          }
          th->blocks[FACET][stage][th->counts[FACET][stage]++] = f_id;
@@ -1019,7 +1082,11 @@ void thread_stage_setup()
 */
 
 void thread_launch(int task, int element_type)
-{ int i,proc;
+{ int i,j,proc;
+  __int32 now[2];
+  
+  PROF_NOW(now);
+  thread_launch_start = *(__int64*)now;
 
   global_id = web.skel[element_type].used;  /* in case no threads */
 
@@ -1045,6 +1112,27 @@ void thread_launch(int task, int element_type)
       }
       break;
   }
+
+  PROF_NOW(now);
+  thread_launch_end = *(__int64*)now;
+
+if ( verbose_flag )
+{
+/* print detailed clock timing */
+printf("thread_launch_start   %12I64X\n",thread_launch_start-thread_launch_start);
+for (i = 0 ; i <= max_thread_stages ; i++ )
+  printf("task 0 stage %d start: %12I64X end: %12I64X\n",i,
+    thread_data_ptrs[0]->stagestart[i]-thread_launch_start,
+    thread_data_ptrs[0]->stageend[i]-thread_launch_start);
+for ( j = 1; j < nprocs ; j++ )
+{ for (i = 0 ; i < max_thread_stages ; i++ )
+    printf("task 1 stage %d start: %12I64X end: %12I64X\n",i,
+      thread_data_ptrs[j]->stagestart[i]-thread_launch_start,
+      thread_data_ptrs[j]->stageend[i]-thread_launch_start);
+}
+printf("thread_launch_end     %12I64X\n",thread_launch_end-thread_launch_start);
+}
+
 }
 #endif
 
@@ -1138,10 +1226,11 @@ element_id thread_next_element()
   int proc,nextproc;
   int eltype;
   struct thread_stages_data *th;
-
-  /* skipping along */
   struct thread_data *data;
 
+  __int32 now[2];  /* for timing */
+
+
   /* First, in case multithreading not being used */
   if ( threadflag == 0 )
   {
@@ -1166,6 +1255,10 @@ element_id thread_next_element()
     }
     else
     { th->stage++;  /* signify done */
+#ifdef _MSC_VER
+PROF_NOW(now);
+data->stageend[th->stage-1] = *(__int64*)now;
+#endif
       th->spot = 0;
       if ( th->stage == ((proc==0) ? max_thread_stages+1 : max_thread_stages) )
       { id = NULLID;
@@ -1179,7 +1272,11 @@ element_id thread_next_element()
         else /* proc 0 to do big elements */
         { for ( nextproc = 1 ; nextproc < nprocs ; nextproc++ )
            while ( thread_stages[nextproc].stage < th->stage ) ;  /* wait */
-        } 
+        }  
+#ifdef _MSC_VER
+PROF_NOW(now);
+data->stagestart[th->stage] = *(__int64*)now;
+#endif
       }
     }
   } 
@@ -1247,3 +1344,35 @@ int mylock_element(element_id id)
 }
 #endif
 
+/**************************************************************************
+*
+* function: find_cpu_speed()
+* 
+* purpose: tries to find out current CPU speed for profiling reports.
+*/
+void find_cpu_speed()
+{
+#if defined(_MSC_VER) && defined(PROFILING)
+  { /* use high-performance counter to estimate frequency */
+    LARGE_INTEGER freq,start,finish;
+    __int32 profstart[2],profnow[2];
+    QueryPerformanceFrequency(&freq);
+    PROF_NOW(profstart);
+    PROF_NOW(profnow);
+    QueryPerformanceCounter(&start);
+    do {
+      if ( *(__int64*)profnow - *(__int64*)profstart <= 0 )
+      { /* something wrong; maybe hibernated in middle of loop! */
+        return;
+      }
+      PROF_NOW(profnow);
+    }
+    while ( *(__int64*)profnow - *(__int64*)profstart < 100000000 );
+    QueryPerformanceCounter(&finish);
+    cpu_speed = (*(__int64*)profnow-*(__int64*)profstart)/
+             ((finish.QuadPart-start.QuadPart)/(double)freq.QuadPart);
+  }
+#else
+   cpu_speed = 3e9;  /* rough guess */
+#endif
+}
