@@ -14,7 +14,6 @@
 #ifdef WIN32
 #undef FIXED
 #undef DOUBLE
-#include <windows.h>
 #endif
 #include "include.h"
 
@@ -91,7 +90,7 @@ FILE *stream;
     c = fgetc(stream);
     if ( (c == EOF) && ( k == 0 ) ) return NULL;
     if ( c == EOF ) { *p = 0; return s; }
-    *p = c;
+    *p = (char)c;
     if ( c == '\n' ) { *(++p) = 0; return s; }
     if ( c == '\r' )
      { *p = '\n'; *(++p) = 0;
@@ -126,7 +125,7 @@ int max;
     {
       if ( !topflag) outstring(inmsg); /* echo */
       if ( (int)strlen(inmsg) == max-1 ) 
-	  {inmsg[max-1] = MOREIN; inmsg[max] = 0; }
+      {inmsg[max-1] = MOREIN; inmsg[max] = 0; }
       return;
     }
   }
@@ -211,7 +210,7 @@ int read_pick()
                 do
                 { 
                    if ( equal_element(e_id,e_id2) )
-                   { ednum = ordinal(e_id) + 1; break; }
+                   { ednum = loc_ordinal(e_id) + 1; break; }
                    if ( get_vattr(v_id2) & (Q_MIDPOINT | Q_MIDEDGE) )
                     break;
                    e_id2 = get_next_tail_edge(e_id2);
@@ -226,7 +225,7 @@ int read_pick()
            { e1 = e_id = get_vertex_edge(v_id1);
              do 
              { if ( equal_id(v_id2,get_edge_headv(e_id)) )
-                { ednum = ordinal(e_id) + 1; break; }
+                { ednum = loc_ordinal(e_id) + 1; break; }
                 e_id = get_next_tail_edge(e_id);
              } while ( !equal_id(e1,e_id) );
            }
@@ -239,7 +238,7 @@ int read_pick()
      strcpy(msg,"Picked ");
      if ( vnum >= 0 )
      { if ( prim > 0 ) vnum += gv_vect_start;
-       pickvnum = ordinal(vpicklist[vnum]) + 1;
+       pickvnum = loc_ordinal(vpicklist[vnum]) + 1;
        sprintf(msg+strlen(msg),"vertex %d  ",pickvnum);
        if ( geomview_bug_flag == 1 )
        { strcat(msg,
@@ -265,7 +264,7 @@ int read_pick()
          strcat(msg," (probably wrong)");
      } 
      if ( fnum >= 0 )
-     { pickfnum = ordinal(fpicklist[fnum]) + 1;
+     { pickfnum = loc_ordinal(fpicklist[fnum]) + 1;
         sprintf(msg+strlen(msg),"facet %d  ",pickfnum);
      } 
      strcat(msg,"\n");
@@ -304,19 +303,21 @@ int check_pick()
   pp[1].fd = gv_pipe[0]; /* geomview pick */
   pp[1].events = POLLIN;
   poll(pp,2,-1);
-  if ( pp[1].revents )  return read_pick();
+  if ( pp[1].revents ) 
+     return read_pick();
 #elif !defined(WIN32)
 /* try using select() from time.h */
   fd_set fdset;
   struct timeval timeout;
 
-  timeout.tv_sec = 5; timeout.tv_usec = 0;
-  FD_ZERO(&fdset);
   do
-  { FD_SET(0,&fdset);
-     FD_SET(gv_pipe[0],&fdset);
+  { timeout.tv_sec = 5; timeout.tv_usec = 0; /* linux select() modifies */
+    FD_ZERO(&fdset);
+    FD_SET(0,&fdset);
+    FD_SET(gv_pipe[0],&fdset);
   } while (  select(FD_SETSIZE,&fdset,NULL,NULL,&timeout) == 0 );
-  if ( FD_ISSET(gv_pipe[0],&fdset) ) return read_pick();
+  if ( FD_ISSET(gv_pipe[0],&fdset) ) 
+     return read_pick();
   else return 0;
 #endif
   return 0;
@@ -382,12 +383,12 @@ if ( threadflag )
     if (my_fgets(inmsg,max,commandfd) == NULL) 
     { return EOF; }
     else 
-    { oldquiet = quiet_flag; quiet_flag = 0;
+    { oldquiet = quiet_flag; /* quiet_flag = 0; */
       outstring(promptmsg);  /* so appears on screen also */
       outstring(inmsg); /* echo */
       if ( (c = strchr(inmsg,'\n')) != NULL ) 
-      { 
-        *c = 0;
+      { if ( strcmp(promptmsg,"Enter command: ") != 0 )
+          *c = 0;
       }
       else if ( (int)strlen(inmsg) == max-1 ) 
         inmsg[max-1] = MOREIN; 
@@ -404,16 +405,25 @@ if ( threadflag )
   read_line_from_console(inmsg);
 #else
 #if defined(_READLINE_H_)
-   { char * response = readline(promptmsg);
+  if ( !geomview_flag || geompipe_flag )
+   { /* strange interference between picking and readline */
+     char * response = readline(promptmsg);
      strncpy(inmsg,response,max);
      add_history(inmsg);
      free(response);  /* since readline() allocated it */
    }
+  else
 #else
+  {
   /* from stdin */
   oldquiet = quiet_flag; quiet_flag = 0;
   outstring(promptmsg);
   quiet_flag = oldquiet;
+
+#ifdef WIN32
+  signal(SIGINT,SIG_IGN); /* no interrupt during input (esp. WIN32) */
+#endif
+
 #ifdef OOGL
   /* check for geomview pick */
   if ( geomview_flag && !geompipe_flag )
@@ -421,14 +431,11 @@ if ( threadflag )
         outstring(promptmsg);
 #endif
 
-#ifdef WIN32
-  signal(SIGINT,SIG_IGN); /* no interrupt during input (esp. WIN32) */
-#endif
-
   if ( my_fgets(inmsg,max,stdin) == NULL )
      return EOF;
   if ( echo_flag ) { outstring(inmsg); outstring("\n"); }
   signal(SIGINT,catcher);
+  }
 #endif
 #endif
 
@@ -529,12 +536,12 @@ char *name;  /* name of file */
      cmdfile_stack[read_depth-1].line = line_no; /* save previous line number */
   line_no = 1;
   cmdfile_stack[read_depth].fd = commandfd = cfd;
-  if ( file_no_used >= file_no_max )
+  if ( file_no_used >= file_no_max ) 
   { file_names = (char**)kb_realloc((char*)file_names,
-        (2*file_no_max+10)*sizeof(char*),
-        file_no_max*sizeof(char*));
+        (2*file_no_max+10)*sizeof(char*));
+    file_no_max = 2*file_no_max+10;
   } 
-  file_names[file_no_used] = kb_calloc(strlen(name)+4,1);
+  file_names[file_no_used] = mycalloc(strlen(name)+4,1);
   strcpy(file_names[file_no_used],name);
   if ( read_depth > 0 )
   { file_no = file_no_used;
@@ -559,8 +566,9 @@ void pop_commandfd()
 #ifdef WIN32
   if ( (commandfd == stdin) && !echo_flag ) return; /* Ctrl-C gives spurious EOF */
 #endif
-  if ( datafile_flag & !cmdfile_stack[read_depth-1].datafile_flag )
-  { datafile_flag = 0;
+  if ( datafile_input_flag & !cmdfile_stack[read_depth-1].datafile_flag )
+  { datafile_input_flag = 0;
+    datafile_flag = 0; /* safe since doing one-char read-ahead */
     return;  /* lex needs to keep reading EOF for datafile */
   }
   if ( read_depth <= 1 )
@@ -588,10 +596,9 @@ void pop_commandfd()
 
 void add_warning_message(message)
 char *message;
-{ int needed = (warning_messages ? strlen(warning_messages):0) + strlen(message);
+{ size_t needed = (warning_messages ? strlen(warning_messages):0) + strlen(message);
   if ( warning_messages_max < needed )
-  { warning_messages = (char*)kb_realloc(warning_messages,
-             needed+1000,warning_messages_max);
+  { warning_messages = (char*)kb_realloc(warning_messages,needed+1000);
     warning_messages_max = needed+1000;
   }
   strcat(warning_messages,message);
@@ -614,7 +621,11 @@ int mode;
   extern int line_no;
   char *fullmsg;
   int size;
+  int prev_read_depth;
 
+#ifdef MPI_EVOLVER
+fprintf(stderr,"Task %d: ",this_machine);
+#endif
  
   if ( emsg == NULL ) emsg = "";  /* just in case */
   if ( emsg == errmsg ) { fullmsg = msg; size = msgmax; }
@@ -622,8 +633,10 @@ int mode;
 
   last_error = errnum;
 
-  if ( read_depth > 1 ) { sprintf(fullmsg,"%s Line %d:\n",
-      cmdfile_stack[read_depth-1].filename,line_no); }
+  if ( read_depth > 1 ) 
+  { sprintf(fullmsg,"%s Line %d:\n",
+      cmdfile_stack[read_depth-1].filename,line_no); 
+  }
   else fullmsg[0] = 0;
 
   reading_comp_quant_flag = 0;  /* just in case */
@@ -665,12 +678,16 @@ int mode;
      case Q_ERROR:
         sprintf(fullmsg+strlen(fullmsg),"ERROR %d: ",errnum);
         strcat(fullmsg,emsg); 
+        dump_buff(fullmsg+strlen(fullmsg),size-strlen(fullmsg));
         /* pop stack of command files */
         if ( read_depth > 0 ) cmdfile_stack[read_depth-1].line = line_no;
+        prev_read_depth = read_depth+1;
         while ( commandfd && (commandfd  != stdin) )
-          { sprintf(fullmsg+strlen(fullmsg),"file %s at line %d\n",
+          { if ( read_depth < prev_read_depth )
+               sprintf(fullmsg+strlen(fullmsg),"file %s at line %d\n",
                   cmdfile_stack[read_depth-1].filename,
-                  cmdfile_stack[read_depth-1].line);             
+                  cmdfile_stack[read_depth-1].line); 
+            prev_read_depth = read_depth; /* pop_commandfd() doesn't always pop */            
              if ( datafile_flag )
              { parse_error_flag = 1;
                recovery_flag = 1; 
@@ -692,7 +709,7 @@ int mode;
         erroutstring(fullmsg);
         add_warning_message(fullmsg);
         if ( exit_after_warning ) my_exit(1);
-        if ( break_after_warning ) goto bailout;
+        if ( break_after_warning ) breakflag = BREAKAFTERWARNING;
         return;
 
     case EXPRESSION_ERROR:
@@ -715,19 +732,28 @@ int mode;
 
      case SYNTAX_ERROR:
         strncat(fullmsg,emsg,size); 
+        /*
         dump_buff(fullmsg+strlen(fullmsg),size-strlen(fullmsg));
+        */
         rewind_globals(old_global_count);
         perm_rewind_globals(old_perm_global_count);
+        local_nest_depth = 0;
         /* pop stack of command files */
-        if ( commandfd && (commandfd  != stdin) ) pop_commandfd();
-        while ( commandfd && (commandfd  != stdin) )
+        if ( !function_kludge_flag )
+        { FILE *oldcommandfd = commandfd;
+          if ( commandfd && (commandfd  != stdin) ) pop_commandfd();
+          while ( commandfd && (commandfd  != stdin) )
           { char c[100];
-             sprintf(c,"called from file %s at line %d\n",
+            if ( commandfd != oldcommandfd )
+            {
+              sprintf(c,"called from file %s at line %d\n",
                   cmdfile_stack[read_depth-1].filename,
                   cmdfile_stack[read_depth-1].line);
-             strncat(fullmsg,c,size-strlen(fullmsg));
-             pop_commandfd();
+              strncat(fullmsg,c,size-strlen(fullmsg));
+            }
+            pop_commandfd();
           }
+        }
         strcat(fullmsg,"\n");
         erroutstring(fullmsg); 
         const_expr_flag = 0;
@@ -756,6 +782,16 @@ int mode;
           }
         strcat(fullmsg,"\n");
         erroutstring(fullmsg);
+        #ifdef PTHREADS
+        if ( pthread_equal(locking_thread,pthread_self()) )
+        {  ABORT_GRAPH_MUTEX
+        }
+        #endif
+        #ifdef WINTHREADS
+        if ( GetCurrentThreadId() == locking_thread )
+        {  ABORT_GRAPH_MUTEX
+        }
+        #endif
         temp_free_all();
         if ( exit_after_error ) my_exit(1);
         FPRESET;
@@ -801,6 +837,7 @@ int mode;
         parse_error_flag = 1;
         recovery_flag = 1;
         if ( exit_after_error ) my_exit(1);
+        local_nest_depth = 0;
         return;
      }
 
@@ -808,33 +845,54 @@ int mode;
   return;
 
 bailout:
+
+  #ifdef MPI_EVOLVER
+  if ( this_machine != 0 )
+  { erroutstring("ALL SLAVE NODE ERRORS FATAL! KILLING ALL PROCESSES!\n");
+    MPI_Abort(MPI_COMM_WORLD,1);
+  }
+  #endif
+
   if ( outfd != stdout )
-    { if ( outfd != NULL )
-      {
-         fclose(outfd);
-      }
-      outfd = stdout;    /* in case of previous piping */
+  { if ( outfd != NULL )
+    {
+       fclose(outfd);
     }
+    outfd = stdout;    /* in case of previous piping */
+  }
   breakflag = 0; iterate_flag = 0;
   if ( saved.coord ) 
-     { restore_coords(&saved,SAVE_IN_ATTR); 
-       unsave_coords(&saved,SAVE_IN_ATTR); 
-       LEAVE_GRAPH_MUTEX;
-       recalc();
-     }
+  { restore_coords(&saved,SAVE_IN_ATTR); 
+    unsave_coords(&saved,SAVE_IN_ATTR); 
+    ABORT_GRAPH_MUTEX;
+    recalc();
+  }
   clear_symtable();     /* clear symbol table */
+  vgrad_end();          /* clear up volume gradients */
+  #ifdef PTHREADS
+  if ( pthread_equal(locking_thread,pthread_self()) )
+  {  ABORT_GRAPH_MUTEX
+  }
+  #endif
+  #ifdef WINTHREADS
+  if ( GetCurrentThreadId() == locking_thread )
+  {  ABORT_GRAPH_MUTEX
+  }
+  #endif
   temp_free_all();     
   if ( list && (list != permlist) )
     { myfree((char*)list); list = NULL; } /* plug memory leak */
   quiet_flag = 0;
+  gocount = 1;
+  local_nest_depth = 0;
   if ( exit_after_error ) my_exit(1);
   FPRESET;
-  LEAVE_GRAPH_MUTEX;
+  ABORT_GRAPH_MUTEX;
 #ifdef WIN32
   if ( draw_thread_id == GetCurrentThreadId() )
       longjmp(graphjumpbuf,1);
 #elif defined(PTHREADS)
-  if ( draw_thread_id == pthread_self() )
+  if ( draw_thread_id == pthread_self() ) 
       longjmp(graphjumpbuf,1);
 #endif
   longjmp(jumpbuf,1);
@@ -943,7 +1001,7 @@ int rows;
     kb_error(2428,"Minimum scroll buffer size is 1.  Command ignored.\n",
          WARNING);
   size.X = 80;
-  size.Y = rows;
+  size.Y = (short)rows;
   SetConsoleScreenBufferSize(GetStdHandle(STD_OUTPUT_HANDLE),size);
 #else
   kb_error(2429,"ScrollBufferSize not implemented on this system.\n",
