@@ -13,27 +13,24 @@
 
 #include "include.h"
 
-char *evolver_version = "2.20";
+char *evolver_version = "2.23";
 char needed_version[30]; 
 
-#ifdef MPI
-char *VERSION = "Version 2.20, August 20, 2003; compiled for MPI.";
-#else
-#ifdef PVM
-char *VERSION = "Version 2.20, August 20, 2003; compiled for PVM.";
+#ifdef MPI_EVOLVER
+char *VERSION = "Version 2.23, June 20, 2004; compiled for MPI.";
 #else
 #ifdef SGI_MULTI
-char *VERSION = "Version 2.20, August 20, 2003; SGI multiprocessing";
+char *VERSION = "Version 2.23, June 20, 2004; SGI multiprocessing"; 
 #else
 #ifdef OPENGL
-char *VERSION = "Version 2.20, August 20, 2003; Windows 95/98/Me/NT/2000/XP OpenGL";
+char *VERSION = "Version 2.23, June 20, 2004; Windows 95/98/Me/NT/2000/XP OpenGL";
 #else
-char *VERSION = "Version 2.20, August 20, 2003";
-#endif
+char *VERSION = "Version 2.23, June 20, 2004";
 #endif
 #endif
 #endif
 
+
 char *typenames[NUMELEMENTS] = {"vertex","edge","facet","body","facetedge"};
 
 REAL factorial[20] = {1.,1.,2.,6.,24.,120.,720.,7*720.,8*7*720.,9*8*7*720.,
@@ -43,16 +40,30 @@ REAL factorial[20] = {1.,1.,2.,6.,24.,120.,720.,7*720.,8*7*720.,9*8*7*720.,
   9.*8*7*7200*11*12*13*14*15*16*17*18,9.*8*7*7200*11*12*13*14*15*16*17*18*19};
 
 int broken_pipe_flag; /* so output routines will know to quit */
+int function_kludge_flag; /* handling errors in function def in datafile */
 
-int match_id_flag; /* to make id match datafile number, option -i */
+int this_machine = 1; /* machine identifier for MPI; really task number */
+  /* 1 works as default when not using MPI */
+
+#ifdef MPI_EVOLVER
+int match_id_flag = 1; /* so remote references to locals work */
+#else
+int match_id_flag = 0; /* to make id match datafile number, option -i */
+#endif
 int echo_flag; /* whether to echo stdin; for piped input */
 struct optparam_t optparam[MAXOPTPARAM]; 
 int optparamcount;  /* number thereof */
 REAL **optparam_congrads;  /* constraint gradients */
 
+/* my own ctypes */
+char kb_upper_array[256];
+char kb_lower_array[256];
+
 REAL machine_eps;  /* smallest machine resolution at 1.0  */
 char loadfilename[200]; /* for LOAD command */
 jmp_buf loadjumpbuf;  /* for LOAD command */
+int exit_flag;  /* whether to end this command interpreter */
+int subshell_flag; /* whether in subshell, so can't reload */
 
 int file_no; /* number of current source file */
 int file_no_used; /* number in list */
@@ -73,17 +84,17 @@ int macro_count;  /* number of macros defined */
 char  *macro_subs;  /* string space for substitution strings */
 struct macro  *macros;  /* dynamically allocated */
 char *warning_messages;  /* for storing warning messages */
-int warning_messages_max; /* allocated bytes for warning_messages */
+size_t warning_messages_max; /* allocated bytes for warning_messages */
 int warning_messages_new; /* number of new warning messages since last looked */
 int shadow_warn_flag; /* warn if locals shadow existing names */
 
+int dont_resize_flag; /* set if datafile specified view matrix */
 REAL brightness = 0.65; /* midlevel gray for screen display */
 int markedgedrawflag = 0; /* for single-drawing edges */
 
 struct constraint null_constraint; /* non-constraint placeholder */
 
 struct vgradblock *vgradbase;    /* allocated list block header start */
-volgrad **vgptrbase;    /* allocated list of chain start pointers */
 int    vgradtop;     /* number of first free  structure */
 long      vgradmax;     /* number allocated */
 long    vgradlastused;     /* number actually used last time round */
@@ -134,7 +145,7 @@ int comp_quant_stamp; /* coordination with eval_all and eval_sec */
 vertex_id comp_quant_vi; /* during calc_quant_hess */
 vertex_id comp_quant_vj; /* during calc_quant_hess */
 
-int gocount = 0;          /* number of iterations left */ 
+int gocount = 1;          /* number of iterations left */ 
 #if defined(MAC_APP) || defined(MAC_CW)
 int go_display_flag = 1; /* for displaying each change */
 #else
@@ -146,6 +157,9 @@ int rgb_colors_flag;  /* enabling rgb color scheme */
 int edge_rgb_color_attr; /* number of rgb color attribute */
 int facet_rgb_color_attr; /* number of rgb color attribute */
 int facet_rgb_backcolor_attr; /* number of rgb color attribute */
+int smooth_graph_flag; /* whether to smoothly plot in lagrange mode */
+int graph_capabilities;  /* bits to describe what current graphics device
+                               can do */
 int edge_alpha_flag; /* whether edges have alpha channel */
 int facet_alpha_flag; /* whether facets have alpha channel */
 int facetback_alpha_flag; /* whether facetbacks have alpha channel */
@@ -160,12 +174,14 @@ int gv_binary_flag = 0;  /* whether to do geomview in binary */
 int self_similar_flag;    /* for self-similar motion */
 int labelflag; /* whether ps doing labels */
 /* Following PostScript line width variables relative to page size */
+REAL ps_labelsize;  /* default label size */
 REAL ps_stringwidth;  /* default edge width */
 REAL ps_fixededgewidth;
 REAL ps_tripleedgewidth;
 REAL ps_conedgewidth;
 REAL ps_bareedgewidth;
 REAL ps_gridedgewidth;
+int full_bounding_box_flag; /* for window bounding box */
 int geomview_bug_flag; /* for geomview 1.6.1 picking bug */
 int gv_pipe[2]; /* for pipe for reading geomview pick commands */
 vertex_id *vpicklist; /* for geomview picking */
@@ -174,6 +190,7 @@ int pickvnum,pickenum,pickfnum,pickbnum; /* geomview picks */
 int new_vertex_id,new_edge_id,new_facet_id,new_body_id; /* just created elements */
 int gv_vect_start; /* vector vertex start in vpicklist */
 int circular_arc_flag; /* whether to draw edges as arcs */
+int spherical_arc_flag; /* whether to draw edges as spherical arcs */
 
 element_id junk; /* for MSC bug */
 #ifndef PARALLEL_MACHINE
@@ -225,11 +242,12 @@ int  history_offsets[MAXHISTORY];
 int  history_number;    /* number of current command */
 int history_count;  /* number in list */
 char fulltext[MAXCMDSIZE+5]; /* for full text of commands */
-int  fulltextsize;              /* length of command */
+size_t  fulltextsize;              /* length of command */
 char datafilename[PATHSIZE];  /* current datafile name */
 char filename[PATHSIZE];  /* file name in command */
 int datafile_flag;  /* 1 for datafile, 0 for command so expression
                                         parser knows what's up */
+int datafile_input_flag; /* whether further input available from datafile */
 int topflag;  /* 1 while in datafile top section */
 int backquote_flag; /* 1 while in backquoted command; kludge */
 int lists_flag;  /* set when parsing space-separated lists */
@@ -255,6 +273,9 @@ int in_function;                /* toggle for lexer */
 int in_comment;                /* toggle for lexer */
 int in_control_structure;                /* toggle for lexer */
 int autopop_flag;     /* whether to do autopopping */
+int pop_disjoin_flag;  /* whether cones to be disjoined rather than merged */
+int pop_to_edge_flag;  /* control which way popping goes */
+int pop_to_face_flag;  /* control which way popping goes */
 int autochop_flag;     /* whether to do autochopping */
 REAL autochop_size;  /* max edge length for autochop */
 int autopop_count;  /* number of edges found */
@@ -288,8 +309,11 @@ int hess_move_con_flag=1; /* whether to project to global constraints in move */
 REAL last_hessian_scale; /* from hessian_line_seek() */
 REAL last_eigenvalue;    /* from eigenprobe and stuff */
 struct hess_entry *hashtable;  /* the table */
- int table_size;  /* hashtable size */
+int table_size;  /* hashtable size */
 int hash_per_row = 5;  /* for estimating table size */
+int vhead_attr;  /* number of vertex attribute for vhead index */
+int bhead_attr;  /* number of body attribute for vhead index */
+
 
 /* for controlling steps */
 int rhs_flag;     /* filling in right hand side */
@@ -297,6 +321,17 @@ int hess_flag;    /* filling in hessian matrix */
 int negdiag;      /* C index of most negative entry on diagonal */
 struct linsys Met; /* vector-to-form  metric */
 
+#if defined(USEYSMP) || defined(MPI_EVOLVER)
+int ysmp_flag=YSMP_FACTORING;  /* set if doing Yale Sparse Matrix version */
+/* factor matrix */
+void (*sp_factor_func)ARGS((struct linsys *)) = ysmp_factor;
+/* solve given rhs */
+void (*sp_solve_func)ARGS((struct linsys *,REAL *,REAL *)) = ysmp_solve;
+/* solve multiple given rhs */
+void (*sp_solve_multi_func)ARGS((struct linsys*,REAL**,REAL**,int)) = ysmp_solve_multi;
+/* matrix inner product with hessian inverse as metric */
+void (*sp_CHinvC_func)ARGS((struct linsys *)) = sp_CHinvC;
+#else
 /* use mindeg since doing sparse_constraints */
 int ysmp_flag=MINDEG_FACTORING;  /* Hessian problem in Borland LONGDOUBLE version */
 /* factor matrix */
@@ -306,24 +341,21 @@ void (*sp_solve_func)ARGS((struct linsys *,REAL *,REAL *)) = xmd_solve;
 /* solve multiple given rhs */
 void (*sp_solve_multi_func)ARGS((struct linsys*,REAL**,REAL**,int)) = xmd_solve_multi;
 /* matrix inner product with hessian inverse as metric */
-void (*sp_CHinvC_func)ARGS((struct linsys *)) = xmd_CHinvC;
+void (*sp_CHinvC_func)ARGS((struct linsys *)) = sp_CHinvC;
 
-#ifdef USEYSMP
-int ysmp_flag=YSMP_FACTORING;  /* set if doing Yale Sparse Matrix version */
-/* factor matrix */
-void (*sp_factor_func)ARGS((struct linsys *)) = ysmp_factor;
-/* solve given rhs */
-void (*sp_solve_func)ARGS((struct linsys *,REAL *,REAL *)) = ysmp_solve;
-/* solve multiple given rhs */
-void (*sp_solve_multi_func)ARGS((struct linsys*,REAL**,REAL**,int)) = ysmp_solve_multi;
-/* matrix inner product with hessian inverse as metric */
-void (*sp_CHinvC_func)ARGS((struct linsys *)) = NULL;
 #endif
 
+/* for mindeg control */
+int mindeg_debug_level;
+int mindeg_margin;
+int mindeg_min_region_size;
+
 int sparse_constraints_flag = 0; /* whether to store constraint gradients
                in sparse format */
 int blas_flag; /* whether to use BLAS */
-int augmented_hessian_flag = 0; /* whether to use augmented Hessian */
+int augmented_hessian_flag = -1; /* whether to use augmented Hessian */
+                                /* -1 for unset */
+int augmented_hessian_mode; /* set during factoring */
 REAL BKalpha = 0.525; /* single/REAL pivot ratio */
 int BK_flag=0;  /* for enabling Bunch-Kaufman version of sparse factoring */
 /* sparse matrix function pointers, for easy switching among algorithms */
@@ -379,7 +411,17 @@ char last_name[50]; /* name of last element generator */
 char idname[35]; /* for saving yytext */
 char set_extra_name[100]; /* for saving name */
 
+/* useful for debugging */
+struct extra *Extras[NUMELEMENTS];
+
+int F_TAG_ATTR; 
+int F_PHASE_ATTR;
+int V_BOUNDARY_ATTR;
+int E_BOUNDARY_ATTR;
+int F_BOUNDARY_ATTR;
+
 struct expnode torus_period_expr[MAXCOORD][MAXCOORD];
+struct expnode torus_display_period_expr[MAXCOORD][MAXCOORD];
 int torus_display_mode;  /* default, raw, connected, clipped */
 
 /* squared curvature as part of energy */
@@ -402,6 +444,7 @@ int eff_area_sq_mean_curvature_mi = -1;
 int sq_mean_curvature_mi = -1;
 int mix_sq_mean_curvature_mi = -1;
 int star_normal_sq_mean_curvature_mi;
+int star_perp_sq_mean_curvature_mi;
 int star_eff_area_sq_mean_curvature_mi;
 int star_sq_mean_curvature_mi;
 int h0_attr;  /* attribute number for h_zero, if any */
@@ -431,7 +474,7 @@ jmp_buf graphjumpbuf;  /* for errors during MS graphing */
 #ifdef PTHREADS
 pthread_t draw_thread_id; /* for graphics thread */
 #else
-int draw_thread_id; /* for graphics thread */
+unsigned int draw_thread_id; /* for graphics thread */
 #endif
 int parse_error_flag;  /* set when parser hits error */
 int recovery_flag;      /* set while recovering from parsing error */
@@ -455,7 +498,8 @@ long top_timestamp;    /* timestamp for topology changes */
 long vedge_timestamp = -1; /* for vertex edgelist currency */
 long vfacet_timestamp = -1; /* for vertex facetlist currency */
 long bfacet_timestamp = -1; /* for body facetlist currency */
-long volume_timestamp; /* for volume calculation */
+long fixed_volume_timestamp; /* for volume calculation */
+long info_volume_timestamp; /* for volume calculation */
 long reset_timestamp;  /* when surface loaded */
 int need_fe_reorder_flag; /* setting causes recalc to fe_reorder */
 int parallel_update_flag[NUMELEMENTS];  /* set when element info changed */
@@ -576,6 +620,8 @@ int movie_init;        /* whether movie routine initialized */
 int view_matrix_global; /* global id number */
 int torus_periods_global;
 int inverse_periods_global;
+REAL window_aspect_ratio;  /* 0 if not set */
+
 
 REAL wulff_vector[MAXWULFF][MAXCOORD];  /* the vector components */
 #ifdef NOPROTO
@@ -664,6 +710,12 @@ REAL **conical_x;
 REAL *conical_w;
 
 int bezier_flag; /* whether to do Bezier basis polynomials in Lagrange */
+REAL **bezier1invert[MAXLAGRANGE];
+REAL **bezier1revert[MAXLAGRANGE];
+REAL **bezier_refine_1d[MAXLAGRANGE];
+REAL **bezier2invert[MAXLAGRANGE];
+REAL **bezier2revert[MAXLAGRANGE];
+REAL **bezier_refine_2d[MAXLAGRANGE];
 
 /* new version for gauss-lagrange elements */
 struct gauss_lag *gauss_lagrange[MAXCOORD];
@@ -1176,12 +1228,17 @@ int pmax[MAXPROCS];    /* available per calculator */
 #ifdef WIN32
 void * graphmutex;
 void * mem_mutex;
+unsigned int locking_thread; /* so we can tell who has it */
 #elif defined(PTHREADS)
 pthread_mutex_t graphmutex;
 pthread_mutex_t mem_mutex;
+pthread_t locking_thread;
 #else
 #endif
 
+/* element id hashing */
+element_id (*id_hash) ARGS((element_id));
+
 /* Multithreading with worker threads */
 int threadflag;   /* 1 if -p option used and worker threads in effect */
 int busythreads;  /* number of threads that have started on a task */
@@ -1217,5 +1274,33 @@ int fe_partition_stage_attr;  /* which stage element is in */
 int fe_partition_proc_attr;   /* which processor element is assigned to */
 long partition_timestamp;  /* last time partitioning done */
 
+/* temp storage for element id name strings */
+char elnames[10][30];
+
 /* profiling */
 int q_facet_setup_elapsed_time[2];
+int calc_quants_elapsed_time[2];
+int calc_quant_grads_elapsed_time[2];
+int calc_quant_hess_elapsed_time[2];
+int element_setup_elapsed_time[2];
+int exparse_elapsed_time[2];
+int yyparse_elapsed_time[2];
+int yylex_elapsed_time[2];
+int kblex_elapsed_time[2];
+int hessian_solve_elapsed_time[2];
+int hessian_mul_elapsed_time[2];
+int hessian_AIJ_setup_elapsed_time[2];
+int hessian_constraint_setup_elapsed_time[2];
+int hessian_project_setup_elapsed_time[2];
+int hessian_factor_elapsed_time[2];
+int hessian_CHinvC_elapsed_time[2];
+double cpu_speed;
+
+int mpi_debug;  /* to enable verbose MPI messages */
+int mpi_show_corona_flag; /* whether to display imported elements */
+
+#ifdef _MSC_VER
+__int64 thread_launch_start;
+__int64 thread_launch_end;
+#endif
+
