@@ -59,12 +59,12 @@ int verpop_str()
 int pop_string_vertex(v_id)
 vertex_id v_id;
 { int valence = get_vertex_evalence(v_id);
-  int popcount;
+  int popcount = 0;
   if ( valence == 0 ) popcount = dissolve_vertex(v_id);
   else  if ( valence > 3 ) 
      popcount = poponest(v_id,valence);
   else if ( (valence >= 2) && (get_vattr(v_id) & (CONSTRAINT|BOUNDARY)) )
-     popcount = poponest(v_id,valence);
+     popcount = poponest(v_id,valence); 
   return popcount;
 } 
   
@@ -111,7 +111,7 @@ int edges;  /* number of edges at this vertex */
   for ( i = 0 ; i < edges ; i++ )
   { 
     side[i].e_id = e_id;
-    get_edge_side(e_id,side[i].vec);
+    get_edge_tail_tangent(e_id,side[i].vec);
     side[i].norm = sqrt(SDIM_dot(side[i].vec,side[i].vec));
     e_id = get_next_tail_edge(e_id);
   }
@@ -262,7 +262,7 @@ int edges;  /* number of edges at this vertex */
   REAL *x;
   facet_id f1,f2;
   REAL ff,f[MAXCOORD],fproj[MAXCOORD],ffmax;
-  REAL new_density;
+  REAL new_density=0.0;
   struct side_t *side;
   REAL bestmove[MAXCOORD];
   facetedge_id fe_id,new_fe1,new_fe2,other_fe;
@@ -304,8 +304,8 @@ int edges;  /* number of edges at this vertex */
   for ( i = 0 ; i < edges ; i++  )
   { 
     if  ( !valid_id(e_id) )
-    { sprintf(errmsg,"Pop vertex: Invalid edge at vertex %d.\n",
-             ordinal(v_id)+1);
+    { sprintf(errmsg,"Pop vertex: Invalid edge at vertex %s.\n",
+             ELNAME(v_id));
       kb_error(1382,errmsg,RECOVERABLE);
     }
 
@@ -331,12 +331,13 @@ int edges;  /* number of edges at this vertex */
       if ( valid_id(f1) && inverted(f1) )
          side[i].fe = get_next_facet(side[i].fe);
     }
-    get_edge_side(side[i].e_id,side[i].vec);
+    get_edge_tail_tangent(side[i].e_id,side[i].vec);
     side[i].norm = sqrt(SDIM_dot(side[i].vec,side[i].vec));
     if ( side[i].norm <= 0.0 )
-    { sprintf(errmsg,"verpop: edge %d length zero. \n",
-        ordinal(side[i].e_id)+1);
+    { sprintf(errmsg,"verpop: edge %s length zero. \n",
+        ELNAME(side[i].e_id));
       kb_error(1383,errmsg,WARNING);
+      temp_free((char *)side);
       return 0;
     }
     side[i].density = get_edge_density(side[i].e_id);
@@ -355,12 +356,16 @@ int edges;  /* number of edges at this vertex */
     if ( ( (side[0].degfree <= degfree) ? 1:0 ) 
         + ( (side[1].degfree <= degfree) ? 1:0 )
         + ( (side[2].degfree <= degfree) ? 1:0 ) >= 2 )
+    {  temp_free((char *)side);
        return 0;
+    }
         
   /* order edges around vertex geometrically */
+  /*
   if ( SDIM == 2 )
      qsort((char*)side,edges,sizeof(struct side_t),FCAST anglecomp);
   else
+  */
   { /* see if we can use facet info */
     facetedge_id fe;
     int top,bottom;
@@ -410,35 +415,35 @@ order_fail:
     bottom = 0;
     while ( top - bottom > 1 )
     { REAL topgap,bottomgap,topcos,bottomcos;
-      int topbest,bottombest;
+      int topbest=0,bottombest=0;
       bottomgap = topgap = -1e30;
       for ( i = bottom+1 ; i < top ; i++ )
       { bottomcos = SDIM_dot(side[i].vec,side[bottom].vec)/
                        side[i].norm/side[bottom].norm;
         if ( bottomcos > bottomgap )
-	{ bottomgap = bottomcos; bottombest = i; }
+        { bottomgap = bottomcos; bottombest = i; }
         topcos = SDIM_dot(side[i].vec,side[top].vec)/
                        side[i].norm/side[top].norm;
         if ( topcos > topgap )
-	{ topgap = topcos; topbest = i; }
+        { topgap = topcos; topbest = i; }
       }
       if ( topgap > bottomgap )
       { /* add at top */
         top--;
-	if ( topbest != top )
-	{ struct side_t temp = side[top];
-	  side[top] = side[topbest];
+        if ( topbest != top )
+        { struct side_t temp = side[top];
+          side[top] = side[topbest];
           side[topbest] = temp;
-	} 
+        } 
       }
       else
       { /* add at bottom */
         bottom++;
-	if ( bottombest != bottom )
-	{ struct side_t temp = side[bottom];
-	  side[bottom] = side[bottombest];
+        if ( bottombest != bottom )
+        { struct side_t temp = side[bottom];
+          side[bottom] = side[bottombest];
           side[bottombest] = temp;
-	} 
+        } 
       }
     }
 order_succeed: ;
@@ -449,7 +454,13 @@ order_succeed: ;
   if ( edges == 2 )
   { /* must be constraint or boundary or fixed */
     if ( (side[0].degfree <= degfree) || (side[1].degfree <= degfree ) )
+    { temp_free((char *)side);
       return 0;
+    }
+    if ( (side[0].degfree <= 1 ) && (side[1].degfree <= 1 ) )
+    { temp_free((char *)side);
+      return 0;
+    }
     besti = 0;
     /* find angle between edges to see which way we want to pop */
     cosa = SDIM_dot(side[0].vec,side[1].vec);
@@ -459,7 +470,7 @@ order_succeed: ;
           (degfree >= 1) && (cosa < 0.0)  )
     { /* split apart along constraint rather than make Y */
       if ( verbose_flag )
-      { sprintf(msg,"Popping vertex %d.\n",ordinal(v_id)+1);
+      { sprintf(msg,"Popping vertex %s.\n",ELNAME(v_id));
         outstring(msg);
       }
       new_v = dup_vertex(v_id);  /* new vertex in same spot, to be pulled out */
@@ -474,6 +485,100 @@ order_succeed: ;
     }
   }
 
+  if ( pop_disjoin_flag && (edges == 4) )
+  { /* see if exactly two opposite wedges have same body */
+    facetedge_id fe_id[4];
+    facet_id f_id[4];
+    body_id b_id[4];
+    int same02,same13;
+    REAL *xnew,*xold,*x0,*x1,*x2,*x3;
+
+    for ( i = 0 ; i < edges ; i++ )
+    { /* assume positive facets on bodies */
+      fe_id[i] = get_edge_fe(side[i].e_id);
+      if ( !valid_id(fe_id[i]) )
+      { f_id[i] = b_id[i] = NULLID;
+        continue;
+      }
+      f_id[i] = get_fe_facet(fe_id[i]);
+      if ( inverted(f_id[i]) )
+      { fe_id[i] = get_next_facet(fe_id[i]);
+        f_id[i] = get_fe_facet(fe_id[i]);
+        if ( inverted(f_id[i]) )
+        { f_id[i] = b_id[i] = fe_id[i] = NULLID;
+          continue;
+        }
+      }
+      b_id[i] = get_facet_body(f_id[i]);
+    }
+    same02 = equal_id(b_id[0],b_id[2]) && valid_id(b_id[0]);
+    same13 = equal_id(b_id[1],b_id[3]) && valid_id(b_id[1]);
+    if ( same02 != same13 )
+    { /* do the disjoin */
+      if ( same13 )
+      { /* shuffle things so same02 */
+        element_id temp;
+        struct side_t tempside = side[0];
+        side[0] = side[1]; side[1] = side[2]; side[2] = side[3]; 
+        side[3] = tempside;
+        temp = fe_id[0]; fe_id[0] = fe_id[1]; fe_id[1] = fe_id[2];
+        fe_id[2] = fe_id[3]; fe_id[3] = temp;
+        temp = f_id[0]; f_id[0] = f_id[1]; f_id[1] = f_id[2];
+        f_id[2] = f_id[3]; f_id[3] = temp;
+        temp = b_id[0]; b_id[0] = b_id[1]; b_id[1] = b_id[2];
+        b_id[2] = b_id[3]; b_id[3] = temp;
+      }
+      /* now the real action */
+      new_v = dup_vertex(v_id);  /* new vertex in same spot */
+      remove_vertex_edge(v_id,side[1].e_id);
+      set_edge_tailv(side[1].e_id,new_v);
+      remove_vertex_edge(v_id,side[2].e_id);
+      set_edge_tailv(side[2].e_id,new_v);
+      if ( valid_id(f_id[0]) )
+      { /* have to merge facets */
+        facetedge_id fe_a = fe_id[0];
+        facetedge_id fe_b = inverse_id(get_prev_edge(fe_a));
+        facetedge_id fe_c = fe_id[2];
+        facetedge_id fe_d = inverse_id(get_prev_edge(fe_c));
+        set_prev_edge(fe_a,inverse_id(fe_d));
+        set_prev_edge(fe_b,inverse_id(fe_c));
+        set_prev_edge(fe_c,inverse_id(fe_b));
+        set_prev_edge(fe_d,inverse_id(fe_a));
+        if ( !equal_id(f_id[0],f_id[2]) )
+        { /* merge f_id[2] into f_id[1] */
+          facetedge_id fe = fe_c;
+          do
+          { set_fe_facet(fe,f_id[0]);
+            fe = get_next_edge(fe);
+          } while ( valid_id(fe) && !equal_element(f_id[0],get_fe_facet(fe)) );
+          fe = inverse_id(fe_d);
+          do
+          { set_fe_facet(fe,f_id[0]);
+            fe = get_prev_edge(fe);
+          } while ( valid_id(fe) && !equal_element(f_id[0],get_fe_facet(fe)) );
+          set_facet_body(f_id[2],NULLID);
+          free_element(f_id[2]);
+        } 
+      }
+    
+      /* spread the vertices a bit */
+      xold = get_coord(v_id);
+      xnew = get_coord(new_v);
+      x0 = side[0].vec;
+      x1 = side[1].vec;
+      x2 = side[2].vec;
+      x3 = side[3].vec;
+      for ( i = 0 ; i < SDIM ; i++ )
+      { xold[i] += 0.05*x0[i] - 0.05*x1[i] - 0.05*x2[i] + 0.05*x3[i];
+        xnew[i] += -0.05*x0[i] + 0.05*x1[i] + 0.05*x2[i] - 0.05*x3[i];
+      }
+ 
+      temp_free((char *)side);
+      return 1;
+    }
+    /* else fall through and do ordinary wedge splitting */
+  }
+
   /* If here, want to do Y pull out */
   /* find two edges that pull apart the most */
   ffmax = 0.01; /* margin to prevent looping with autopop */
@@ -548,7 +653,7 @@ order_succeed: ;
   }
   
   if ( verbose_flag )
-  { sprintf(msg,"Popping vertex %d.\n",ordinal(v_id)+1);
+  { sprintf(msg,"Popping vertex %s.\n",ELNAME(v_id));
     outstring(msg);
   }
 
@@ -578,6 +683,7 @@ order_succeed: ;
 
   /* want to be careful here; new vertex and edge should get union of 
      properties of the two wedge edges */
+  unset_attr(new_v,FIXED);
   if ( attrv & CONSTRAINT )
   { conmap_t *map = get_v_constraint_map(new_v);
     for ( i = map[0] ; i >= 0 ; i-- ) map[i] = 0;
@@ -760,14 +866,15 @@ REAL scale;  /* scale factor for motion */
         REAL *headf = get_velocity(headv);
         vertex_id tailv = get_edge_tailv(e_id);
         REAL *tailf = get_velocity(tailv);
-        REAL side[MAXCOORD];
+        REAL tailtan[MAXCOORD],headtan[MAXCOORD];
         REAL length,dx;
 
-        get_edge_side(e_id,side);
-        length = sqrt(SDIM_dot(side,side));
+        get_edge_tail_tangent(e_id,tailtan);
+        get_edge_tail_tangent(inverse_id(e_id),headtan);
+        length = sqrt(SDIM_dot(tailtan,tailtan));
         if ( length > 0.0 )
-          dx = scale*(SDIM_dot(headf,side)
-                          - SDIM_dot(tailf,side))/length;
+          dx = -scale*(SDIM_dot(headf,headtan)
+                          + SDIM_dot(tailf,tailtan))/length;
         else dx = 0.0;
         if ( autopop_flag && (dx <= 0.0) )
          { if ( length + dx <= (runge_kutta_flag ? 0.52*length : 0.0) )
@@ -802,13 +909,21 @@ void autopop_pop()
   if ( autopop_list == NULL ) return;
 
   for ( k = 0 ; k < autopop_count ; k++ )
-  { /*  printf("popping edge %d\n",ordinal(autopop_list[k])+1);  */
+  { /*  printf("popping edge %s\n",ELNAME(autopop_list[k]));  */
     if ((get_eattr(autopop_list[k])&ALLOCATED)&&eliminate_edge(autopop_list[k]))
         free_element(autopop_list[k]);
+#ifdef _DEBUG
+if ( verbose_flag ) run_checks();
+#endif
   }
   if ( autopop_count )
+  {
       popped = verpop_str();
-
+ #ifdef _DEBUG
+ if ( verbose_flag ) run_checks();
+ #endif
+  }
+  
   if ( (autopop_count > 0) || (popped > 0) )
   { sprintf(msg,"Autopopped %d edges, %d vertices.\n",autopop_count,popped);
     outstring(msg);
@@ -837,7 +952,7 @@ void autochop_chop()
   if ( autochop_list == NULL ) return;
 
   for ( k = 0 ; k < autochop_count ; k++ )
-    { /* printf("chopping edge %d\n",ordinal(autochop_list[k])+1); */
+    { /* printf("chopping edge %s\n",ELNAME(autochop_list[k])); */
       edge_refine(autochop_list[k]);
     }
 
